/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda;

import fdda.algorithm.Algorithms;
import fdda.core.Core;
import fdda.core.impl.CoreImplementation;

import fdda.dialog.DialogFactory;
import fdda.dialog.java2d.Java2DFactory;
import fdda.dialog.swing.MainFrame;

import fdda.graphics.ComponentFactory;
import fdda.graphics.java2d.Java2dFactory;

import fdda.utils.Notification;
import fdda.utils.Localization;
import fdda.utils.Logging;

import javax.swing.UIManager;

import java.util.Locale;
import java.util.prefs.Preferences;
import java.util.prefs.InvalidPreferencesFormatException;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;

/**
 * This is the main class of the Framework for the Demonstration of Distributed
 * Algorithms (fdda). It initializes the various components, loads the user
 * preferences and then shows the main application frame.
 * 
 * @author Jrg Rdenauer
 */
public final class Fdda {

  /**
   * This is just the starting class. It needs no instances.
   */
  private Fdda() {
  }  

  /**
   * The main function, starts the application.
   * 
   * @param args command line arguments. Currently one argument is supported:
   * --locale <em>Locale</em>, e.g. --locale DE or --locale EN, to set the 
   * locale for the application.
   */
  public static void main(String[] args) {
    // initialize components
    Locale locale = null;
    if (args.length > 1) {
      if (args[0].equals("--locale")) {
        locale = new Locale(args[1]);
      }
    }
    if (locale == null) {
      locale = Locale.getDefault();
    }
    Localization.setLocale(locale);

    Logging.initialize();
    
    Logging.getPackageLogger(fdda.Fdda.class).fine("fdda_start");
    
    
    Notification.setNotification(new fdda.dialog.swing.utils.Notification());
    Core.setCoreImplementation(new CoreImplementation());
    ComponentFactory.setFactory(new Java2dFactory());
    DialogFactory.setFactory(new Java2DFactory());
    Algorithms.getInstance().researchForAlgorithms();
    
    
    // load the user preferences
    String errorMessage = Localization.getLocalizedString(Fdda.class, 
      "preferences_load_error", "Konnte Einstellungen nicht laden: {0}");
   
    try {
      Preferences.importPreferences(new FileInputStream(new File(
        System.getProperty("user.home") + File.separator 
        + "fdda" + File.separator + "fdda_options.xml")));
    }
    catch (FileNotFoundException e) {
      Logging.exceptionCaught(e);
      Logging.getPackageLogger(Fdda.class).warning(
        Localization.formatMessage(errorMessage, e.getLocalizedMessage()));
    }
    catch (IOException e) {
      Logging.exceptionCaught(e);
      Logging.getPackageLogger(Fdda.class).warning(
        Localization.formatMessage(errorMessage, e.getLocalizedMessage()));
    }
    catch (InvalidPreferencesFormatException e) {
      Logging.exceptionCaught(e);
      Logging.getPackageLogger(Fdda.class).warning(
        Localization.formatMessage(errorMessage, e.getLocalizedMessage()));
    }
    
    // set Look & Feel to default    
    try {
      UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
    }
    catch (Exception e) {
      e.printStackTrace();
    }
    
    // start
    MainFrame mainFrame = new MainFrame();
    mainFrame.setVisible(true);
    mainFrame.performStartAction();
  }


}
