/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.algorithm;

import java.net.URL;
import java.net.URLClassLoader;

import java.util.HashMap;
import java.util.Set;

import fdda.utils.Localization;
import fdda.utils.Logging;
import fdda.utils.Notification;
import fdda.utils.ObjectCache;

/**
 * Instances of this class are the algorithms that are demonstrated. When you 
 * program an algorithm you need not subclass this. It suffices to implement a
 * subclass of {@link fdda.algorithm.Node Node} and write an algorithm 
 * description file.<br /> Each 
 * algorithm must have an unique name by which it can be identified. It further
 * stores prototypes for all types of nodes contained in this algorithm. 
 * The algorithm can have an associated class loader to specify where its
 * implementation classes are found. Finally, there's a default algorithm that
 * does nothing for use with new projects.
 * @author Jrg Rdenauer
 */
public final class Algorithm {
  
  /** the default algorithm */
  private static Algorithm defaultAlgorithm = null;

  /** the prototypes of the nodes */
  private HashMap nodePrototypes;
  
  /** the name of the algorithm */
  private String name;
  
  /** a description of the algorithm */
  private String description;
  
  /** the default type for new nodes */
  private String defaultType;
  
  /** the class loader. null if the standard class loader is used */
  private URLClassLoader classLoader;
  
  /**
   * Returns the default algorithm. Singleton-pattern.
   * @return the default algorithm.
   */
  public static Algorithm getDefaultAlgorithm() {
    if (defaultAlgorithm == null) {
      defaultAlgorithm = new Algorithm(null);
      defaultAlgorithm.setName("Default");
      defaultAlgorithm.setDescription(Localization.getLocalizedString(
        Algorithm.class, "default_alg_desc", "Does nothing."));
    }
    return defaultAlgorithm;
  }
    
  /**
   * Constructor. Give null as a class base to indicate that the default class
   * loader shall be used exclusively
   * @param classBase the base URL of the classes that implement the nodes of 
   * this algorithm. This could be a directory or a jar file.
   */
  public Algorithm(URL classBase) {
    nodePrototypes = new HashMap();
    // add standard types that are used in every algorithm
    Node passivePrototype = new PassiveNode();
    nodePrototypes.put(passivePrototype.getType(), passivePrototype);
    setDefaultType(passivePrototype.getType());
    Node repeaterPrototype = new Repeater();
    nodePrototypes.put(repeaterPrototype.getType(), repeaterPrototype);
    if (classBase != null) {
      classLoader = URLClassLoader.newInstance(new URL[] {classBase});
    }
    else {
      classLoader = null;
    }
  }
  
  /**
   * Adds a new node type. Also generates a prototype of this type of node.
   * @param type the name of the type
   * @param className the name of the class that implements the type
   * @throws IllegalNodeException if the prototype couldn't be instantiated (for
   * whatever reason) or the type name is already used
   */
  public void addNodeType(String type, String className) 
  throws IllegalNodeException {
    if (nodePrototypes.containsKey(type)) {
      throw new IllegalNodeException("Type name " + type + " already used!");
    }
    try {
      Node prototype = createPrototype(className);
      prototype.setType(type);
      nodePrototypes.put(type, prototype);
    }
    // here a *very* global catch. Thrown errors could include NoClassDefFound,
    // InstantiationException, ClassNotFoundException etc.
    catch (Exception e) { 
      Logging.exceptionCaught(e);
      throw new 
        IllegalNodeException("Couldn't initialize node " + type + ": ", e);
    }
    catch (Error e) {
      Logging.errorCaught(e);
      throw new 
        IllegalNodeException("Couldn't initialize node " + type + ": ", e);
    }
  }
  
  /**
   * Removes a node type.
   * @param type the type
   */
  public void removeNodeType(String type) {
    nodePrototypes.remove(type);
  }
  
  /**
   * Removes all node types apart from the standard ones
   */
  public void clearNodeTypes() {
    nodePrototypes.clear();
    // re-add the standard types
    Node passivePrototype = new PassiveNode();
    nodePrototypes.put(passivePrototype.getType(), passivePrototype);
    Node repeaterPrototype = new Repeater();
    nodePrototypes.put(repeaterPrototype.getType(), repeaterPrototype);
  }
  
  /**
   * Returns the names of the known node types for this algorithm
   * @return the known node types
   */
  public Set getKnownNodeTypes() {
    return nodePrototypes.keySet();
  }
  
  /**
   * Creates a node by cloning its prototype.
   * @param type the type name of the node
   * @param modelNode the model node that contains the algorithmic node
   * @return the newly created node
   * @throws IllegalArgumentException if the type is not known
   */
  public Node createNode(String type, fdda.model.Node modelNode) {
    Node prototype = (Node) nodePrototypes.get(type);
    if (prototype == null) {
      throw new IllegalArgumentException("Nodetype not known!");
    }
    Node newNode = (Node) prototype.clone();
    newNode.setModelNode(modelNode);
    // initialize the node, since constructor must be simple
    try {
      newNode.initialize();
    }
    catch (Exception e) {
      Logging.exceptionCaught(e);
      Notification.message(Notification.M_ERROR, Notification.S_PROCESSING, 
        Localization.getLocalizedString(getClass(), "algorithm", "Algorithm"),
        Localization.formatMessage(Localization.getLocalizedString(getClass(),
          "node_init_error", "Exception occured while initializing " +
            "algorithmic node. Error message:\n{0}"), e.getLocalizedMessage()));
    }
    return newNode;
  }

  /**
   * Returns the description.
   * @return the description.
   */
  public String getDescription() {
    return description;
  }

  /**
   * Returns the name of the algorithm.
   * @return the name
   */
  public String getName() {
    return name;
  }

  /**
   * Sets the description
   * @param string the description
   */
  public void setDescription(String string) {
    description = string;
  }

  /**
   * Sets the name of the algorithm
   * @param string the name
   */
  public void setName(String string) {
    name = string;
  }
  
  /**
   * Returns the name of the default node type for this algorithm
   * @return the default node type
   */
  public String getDefaultType() {
    return defaultType;
  }
  
  /**
   * Sets the name of the default node type for this algorithm. No checks 
   * whether that type does exist.
   * @param type the default type
   */
  public void setDefaultType(String type) {
    defaultType = type;
  }
  
  /**
   * Creates a prototype of a node type. Loads the class dynamically.
   * @param className the name of the implementing class
   * @return the newly created prototype
   * @throws InstantiationException if the class can't be instantiated
   * @throws ClassNotFoundException if the class is not found
   * @throws IllegalAccessException if access to the class isn't allowed
   */
  private Node createPrototype(String className)
  throws InstantiationException, ClassNotFoundException,IllegalAccessException {
    Class nodeClass;
    if (classLoader == null) {
      nodeClass = ObjectCache.getInstance().loadClass(className);
    }
    else {
      // use that special class loader
      nodeClass = Class.forName(className, true, classLoader);
    }
    Node prototype = (Node) nodeClass.newInstance();
    return prototype;
  }

}
