/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.algorithm;

import java.util.HashMap;
import java.util.Set;
import java.util.prefs.Preferences;

import fdda.plugins.AlgorithmLoader;
import fdda.plugins.DefaultErrorHandler;
import fdda.utils.Localization;
import fdda.utils.Notification;
import fdda.utils.ObjectCache;

/**
 * Manages the known algorithms. Also searches for algorithms in the file system
 * and parses the algorithm definition files. This is a singleton.
 * @author Jrg Rdenauer
 */
public final class Algorithms {
  
  /** the instance of the class */
  private static Algorithms instance = null;

  /** the algorithms */
  private HashMap algorithms = null;
  
  /**
   * Returns the instance (singleton)
   * @return the instance
   */  
  public static Algorithms getInstance() {
    if (instance == null) {
      instance = new Algorithms();
    }
    return instance;
  }
  
  /**
   * Constructor.
   */
  private Algorithms() {
    algorithms = new HashMap();
  }
  
  /**
   * Returns the names of the known algorithms.
   * @return the names of the known algorithms
   */
  public Set getAvailableAlgorithms() {
    return algorithms.keySet();
  }
  
  /**
   * Returns an algorithm.
   * @param name the name of the algorithm
   * @return the algorithm
   * @throws IllegalArgumentException if no algorithm with this name is known
   */
  public Algorithm getAlgorithm(String name) {
    if (!algorithms.containsKey(name)) {
      throw new IllegalArgumentException("Algorithm " + name + " not known!");
    }
    return (Algorithm) algorithms.get(name);
  }
  
  /**
   * Adds an algorithm to the known algorithms. Returns whether the operation
   * was successful.
   * @param algorithm the algorithm
   * @return false, iff the name is already used
   */
  public boolean addAlgorithm(Algorithm algorithm) {
    String name = algorithm.getName();
    if (algorithms.containsKey(name)) {
      return false;
    }
    else {
      algorithms.put(name, algorithm);
      return true;
    }
  }
  
  /**
   * Clears the algorithm map and searches again in the file system for 
   * algorithm definition files. The starting directory is 
   * $USER_HOME/fdda/algorithms; the files must have the ending '.xml'. They 
   * can be in any subdirectory or jar archive.
   */
  public void researchForAlgorithms() {
    algorithms.clear();
    AlgorithmLoader reader = null;
    String className = Preferences.userRoot().node("fdda/plugins").get(
      "currentAlgorithmReader", "fdda.plugins.xmlreader.AlgorithmReader"); 
     
    try {
      reader = (AlgorithmLoader) ObjectCache.getInstance().getObject(className);
      DefaultErrorHandler handler = new DefaultErrorHandler(
        Localization.getLocalizedString(getClass(), 
          "load_of_alg", "Laden der Algorithmen"));
      reader.searchForAlgorithms(handler);
      handler.printWarnings();
      handler.printErrors();
      algorithms.put("Default", Algorithm.getDefaultAlgorithm());
    }
    catch (ClassNotFoundException e) {
      Notification.message(Notification.M_ERROR, Notification.S_PROCESSING,
        Localization.formatMessage(Localization.getLocalizedString(getClass(), 
          "plugin_not_found", "Plug-in-Klasse {0} wurde nicht gefunden!"), 
          className));
    }
    catch (InstantiationException e) {
      Notification.message(Notification.M_ERROR, Notification.S_PROCESSING,
        Localization.formatMessage(Localization.getLocalizedString(getClass(),
         "plugin_instantiation", 
          "Plug-in-Klasse {0} konnte nicht instanziiert werden!\n"
        + "Prfen sie, ob die Klasse einen Konstruktor ohne Parameter hat."), 
        className));
    }
    catch (IllegalAccessException e) {
      Notification.message(Notification.M_ERROR, Notification.S_PROCESSING,
        Localization.formatMessage(Localization.getLocalizedString(getClass(),
          "plugin_instantiation2", 
          "Plug-in-Klasse {0} konnte nicht instanziiert werden!\n"
        + "Prfen sie, ob der Konstruktor public ist."), className));
    }
    catch (ClassCastException e) {
      Notification.message(Notification.M_ERROR, Notification.S_PROCESSING,
        Localization.formatMessage(Localization.getLocalizedString(getClass(),
          "plugin_class_reader", 
          "Plug-in-Klasse {0} implementiert nicht das Interface "
        + "fdda.plugins.AlgorithmLoader!"), className));
    }
  }
  
}
