/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.algorithm;

import java.awt.Color;

import fdda.core.demonstration.Demonstrator;

/**
 * <p>
 * A message that is sent from one node to another node during the demonstration
 * of an algorithm. A message can 'transport' an Object as its payload. 
 * The color of the message and a (short) label can also be customized.<br />
 * <br />
 * A message has two states: sent and not yet sent. Payload, color and label
 * can only be changed before the message is sent.
 * </p>
 * @author Jrg Rdenauer
 */
public class Message implements Cloneable {

  /** payload */
  private Object payload;
  /** sender */
  private Node sender;
  /** receiver */
  private Node receiver;
  /** label */
  private String label;
  /** Object that manages the demonstration */
  private Demonstrator demonstrator;
  /** the graphical representation of the message */
  private fdda.graphics.Message graphicalRepresentation;
  /** has the message already been sent? */ 
  private boolean sent;
  /** nr of cycles the message is on its way */
  private int cycles;

  /**
   * Constructor. Do not construct messages directly; use the sendMessage(...)
   * or createMessage(...) methods of Node instead.
   * @param sender the sender of the message
   * @param receiver the receiver of the message
   * @param payload the payload of the message
   * @param demo the object that manages the demonstration
   */
  public Message(Node sender, Node receiver, Object payload, 
                 Demonstrator demo) {
    this.payload = payload;
    this.sender = sender;
    this.receiver = receiver;
    this.demonstrator = demo;
    this.sent = false;
    this.graphicalRepresentation = null;
  }
  
  /**
   * Clones the message. <emph>For framework internal use only.</emph>
   * @see java.lang.Object#clone()
   */
  public Object clone() throws CloneNotSupportedException {
    Message clone = (Message) super.clone();
    return clone;
  }
  
  /**
   * Makes a copy of the important attributes of a message - i.e. payload, label
   * and color. Convenience method.
   * @param original the message to be copied.
   * @throws MessageAlreadySentException if this message has already been sent 
   * (the parameter, OTOH, can be sent already with no problems)
   */
  public final void copyContents(Message original) 
  throws MessageAlreadySentException {
    if (sent) throw new MessageAlreadySentException();
    this.payload = original.payload;
    this.label = original.label;
    if (graphicalRepresentation != null) {
      fdda.graphics.Message originalGraphic = 
        original.getGraphicalRepresentation();
      if (originalGraphic != null) {
        setColor(originalGraphic.getColor());
      }
    }
  }

  /**
   * Returns the payload of the message.
   * @return the payload
   */
  public final Object getPayload() {
    return payload;
  }
  
  /**
   * Sets the payload of the message.
   * @param payload the payload
   * @throws MessageAlreadySentException if the message has already been sent
   */
  public final void setPayload(Object payload) 
  throws MessageAlreadySentException {
    if (sent) {
      throw new MessageAlreadySentException();
    }
    this.payload = payload;
  }

  /**
   * Returns the sender of the message. This is how you can also determine 
   * over which connection the message was sent.
   * @return the sender of the message
   */
  public final Node getSender() {
    return sender;
  }

  /**
   * Returns the receiver of the message. 
   * @return the receiver
   */
  public final Node getReceiver() {
    return receiver;
  }
  
  /**
   * Sets the graphical representation of the message. <emph>For framework 
   * internal use only.</emph>
   * @param message the graphical message
   */
  public final void setGraphicalRepresentation(fdda.graphics.Message message) {
    graphicalRepresentation = message;
  }
  
  /**
   * Returns the graphical representation of the message. <emph>For framework
   * internal use only.</emph>
   * @return the graphical message
   */
  public final fdda.graphics.Message getGraphicalRepresentation() {
    return graphicalRepresentation;
  }

  /**
   * Returns the color of the message.
   * @return the color
   */
  public final Color getColor() {
    return graphicalRepresentation.getColor();
  }
  
  /**
   * Sets the color of the message.
   * @param color the new color
   * @throws MessageAlreadySentException if the message has already been sent
   */
  public final void setColor(Color color) throws MessageAlreadySentException {
    if (sent) {
      throw new MessageAlreadySentException();
    }
    graphicalRepresentation.setColor(color);
  }
  
  /**
   * Returns the label of the message.
   * @return the label
   */
  public final String getLabel() {
    return label;
  }
  
  /**
   * Sets the label of the message. This should be quite short, not more than
   * a few characters.
   * @param label the new label
   * @throws MessageAlreadySentException if the message has already been sent
   */
  public final void setLabel(String label) throws MessageAlreadySentException {
    if (sent) {
      throw new MessageAlreadySentException();
    }
    this.label = label;
  }
  
  /**
   * Sends the message on its way to the receiver. After this, it can't be
   * changed anymore. If the message can't be sent - because the connection
   * is broken or even because there is no connection - the method returns
   * false and the state isn't changed.
   * @return whether the send operation was successful
   */
  public final boolean send() {
    if (sender.canSendMessageTo(receiver)) {
      this.sent = true;
      this.cycles = 0;
      demonstrator.addActiveMessage(this);
      return true;
    }
    else {
      return false;
    }
  }
  
  /**
   * Adds a cycle to the count of cylces since sending the message. <emph>For
   * framework internal use only!</emph>
   */
  public final void addCycle() {
    cycles += 1;
  }
  
  /**
   * Returns the number of cycles since sending the message. <emph>For framework
   * internal use only!<emph>
   * @return the number of cycles since the message has been sent
   */
  public final int getCyclesSinceSending() {
    return cycles;
  }
}