/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.algorithm;

import fdda.core.Core;
import fdda.core.demonstration.Demonstrator;
import fdda.model.Property;

import java.awt.Color;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

/**
 * <p>
 * A node in the algorithm. All specialized nodes must be subclasses of this 
 * class. The framework calls several methods during the demonstration so the
 * algorithm can work.<br />
 * The graphical representation of the node can be customized by changing the
 * border color and the fill color and by setting a small label (only a few
 * characters). The node can also have properties.<br />
 * This class also provides methods that can be used to send messages to a
 * neighbour. Important: you can only send messages to the immediately 
 * adjacent nodes. Routing is not provided ;-)
 * </p> 
 * @author Jrg Rdenauer
 */
public abstract class Node implements Cloneable {

  /** the model node that contains this algorithmic node */
  private fdda.model.Node modelNode;
  
  /** the type of the Node */
  private String type;

  /**
   * Constructor. Do not create nodes on your own. The framework does that. Do 
   * not use framework method in your constructor, since the node may not be
   * initialized completely.
   */
  public Node() {
  }
  
  /**
   * Clones the node. <emph>For framework internal use only.</emph> New nodes
   * are created by cloning a prototype. That means that you must override this
   * method in a subclass if you have to make deep copies of some attributes.
   * @see java.lang.Object#clone()
   */
  public Object clone() {
    try {
      Node clone = (Node) super.clone();
      return clone;
    }
    catch (CloneNotSupportedException e) {
      // can't happen
      throw new InternalError();
    }
  }
  
  /**
   * Initializes the node. This method is called immediately after the framework
   * is done with its initialization of the node. It is safe to call framework
   * methods here (apart from the ones that only work while the demonstration
   * is running).
   */
  public abstract void initialize();

  /**
   * Prepares the node for a start of the demonstration. This method is called
   * immediatly before the demonstration begins. Do not send messages here yet.
   */
  protected abstract void prepareForStart();

  /**
   * If the node is marked 'active' in the property window, this method is 
   * called once at the start of the demonstration. If the node then shall 
   * remain active, you must call activate() or sleepTime(). The method is also
   * called if a node returns to 'working' state during a demo and is marked
   * as 'active'.
   * @throws WrongStateException don't bother about this
   */
  protected abstract void start() throws WrongStateException;

  /**
   * This method is called once per cycle for each node that is active during
   * the demonstration. Since a cycle is very short, it is usually a good idea
   * to use the sleepTime() method at the end of this method.<br />
   * <emph>Do not program an 'endless' loop here!<emph> 'run' is called by the
   * framework once each cycle.
   * @throws WrongStateException don't bother about this
   */
  protected abstract void run() throws WrongStateException;

  /**
   * This method is called if the node receives a message during the 
   * demonstration.
   * @param sender the sender of the message (redundant, but more clear)
   * @param message the message
   * @throws WrongStateException don't bother about this
   */
  protected abstract void messageReceived(Node sender, Message message)
  throws WrongStateException;
  
  /**
   * Returns the type name of the node. The type name should be the same as the
   * one given in the algorithm description file. It must be unique per 
   * node type in each algorithm.
   * @return the name of the type of the node
   */
  public String getType() {
    return type;
  }
  
  /**
   * Sets the type of the node. For internal use only.
   * @param type the type
   */
  public final void setType(String type) {
    this.type = type;
  }
  
  /**
   * Creates a simple message.
   * @param receiver the receiver of the message
   * @return the new message
   * @throws WrongStateException if no demo is running
   * @throws IllegalArgumentException if the receiver is not a neighbour
   */
  protected final Message createMessage(Node receiver) 
  throws WrongStateException {
    return createMessage(receiver, null);
  }
  
  /**
   * Creates a message with a payload
   * @param receiver the receiver
   * @param payload the payload
   * @return the new message
   * @throws WrongStateException if no demo is running
   * @throws IllegalArgumentException if the receiver is not a neighbour
   */
  protected final Message createMessage(Node receiver, Object payload) 
  throws WrongStateException {
    return getDemonstrator().createMessage(this, receiver, payload);
  }
  
  /**
   * Sends a simple message.
   * @param receiver the receiver
   * @return whether the send operation was successful
   * @throws WrongStateException if no demo is running
   * @throws IllegalArgumentException if the receiver is not a neighbour
   */
  protected final boolean sendMessage(Node receiver) 
  throws WrongStateException {
    return sendMessage(receiver, null, "");
  }
  
  /**
   * Sends a message with a payload
   * @param receiver the receiver
   * @param payload the payload
   * @return whether the send operation of successful
   * @throws WrongStateException if no demo is running
   * @throws IllegalArgumentException if the receiver is not a neighbour
   */
  protected final boolean sendMessage(Node receiver, Object payload) 
  throws WrongStateException {
    return sendMessage(receiver, payload, "");
  }
  
  /**
   * Sends a message with a payload and a label
   * @param receiver the receiver
   * @param payload the payload
   * @param label the label
   * @return whether the send operation was successful
   * @throws WrongStateException if no demo is running
   * @throws IllegalArgumentException if the receiver is not a neighbour
   */
  protected final boolean sendMessage(Node receiver, Object payload, 
                                      String label) throws WrongStateException {
    Message message = createMessage(receiver, payload);
    message.setLabel(label);
    return message.send();
  }

  /**
   * Sets the primary color of the node.
   * @param color the new color
   */
  protected final void setPrimaryColor(Color color) {
    getModelNode().getGraphicalRepresentation().setPrimaryColor(color);
  }

  /**
   * Sets the secondary color of the node.
   * @param color the new color
   */
  protected final void setSecondaryColor(Color color) {
    getModelNode().getGraphicalRepresentation().setSecondaryColor(color);
  }

  /**
   * Sets the label of the node.
   * @param label the new label
   */
  protected final void setLabel(String label) {
    getModelNode().setLabel(label);
  }

  /**
   * Deactivates the node. Its run()-method won't be called any more.
   * @throws WrongStateException if no demo is running
   */
  protected final void sleep() throws WrongStateException {
    getDemonstrator().removeActiveNode(this);
  }
  
  /**
   * Deactivates the node for a certain time. It's run()-method won't be 
   * called for this time; then it will become active again.
   * @param timeInMillis time to sleep
   * @throws WrongStateException if no demo is running
   */
  protected final void sleepTime(long timeInMillis) throws WrongStateException {
    getDemonstrator().pauseActiveNode(this, timeInMillis);
  }
  
  /**
   * Activates the node. It's run()-method will be called once per cycle.
   * @throws WrongStateException if no demo is running
   */
  protected final void activate() throws WrongStateException {
    getDemonstrator().addActiveNode(this);
  }
  
  /**
   * Returns the property with the specified name (or null if no such property
   * exists).
   * @param name the name of the property
   * @return the property
   */
  protected final Property getProperty(String name) {
    return modelNode.getProperty(name);
  }

  /**
   * Returns the first value of a property (or null if it doesn't have values).
   * @param name the name of the property
   * @return the value of the property
   * @throws NullPointerException if no such property exists
   */
  protected final Object getPropertyValue(String name) {
    return modelNode.getProperty(name).getValue(0);
  }
  
  /**
   * Returns the neighbours of the node. The neighbours are instances of this
   * class.
   * @return all neighbours of this node
   */
  protected final List getNeighbours() {
    List modelNeighbours = getModelNode().getNeighbours();
    LinkedList myNeighbours = new LinkedList();
    for (Iterator it = modelNeighbours.iterator(); it.hasNext(); ) {
      myNeighbours.addLast(((fdda.model.Node) it.next()).getAlgorithmicNode());
    }
    return myNeighbours;
  }
  
  /**
   * Returns whether the node can send a message to a neighbour. This is only
   * the case if a working connection between the nodes exists and allows 
   * message transfer in the right direction.
   * @param neighbour the neighbour
   * @return true, iff a send operation to the neighbour would succeed
   */
  protected final boolean canSendMessageTo(Node neighbour) {
    return getModelNode().canSendMessageDirectlyTo(neighbour.getModelNode());
  }

  /**
   * Sets the model node. <emph>For framework internal use only.</emph>
   * @param modelNode the model node
   */
  public final void setModelNode(fdda.model.Node modelNode) {
    this.modelNode = modelNode;
  }
  
  /**
   * Returns the model node. In most cases, you won't need this. You can use
   * it, though, to attach property listeners and to add your own properties.
   * @return the model node that contains this algorithmic node
   */
  public final fdda.model.Node getModelNode() {
    return modelNode;
  }

  /**
   * Called by the framework when a message arrives. <emph>For framework 
   * internal use only.<emph>
   * @param sender the sender
   * @param msg the message
   */
  public final void FWMessageArrived(Node sender, Message msg) {
    try {
      messageReceived(sender, msg);
    }
    catch (WrongStateException e) {
      throw new 
        RuntimeException("WrongStateException? Did you do that on purpose?");
    }
  }
  
  /**
   * Called by the framework to prepare for the start. <emph>For framework
   * internal use only.</emph>
   */
  public final void FWPrepareForStart() {
    prepareForStart();
  }
  
  /**
   * Called by the framework to start an 'active' node. <emph>For framework
   * internal use only.</emph>
   */
  public final void FWStart() {
    try {
      start();
    }
    catch (WrongStateException e) {
      throw new 
        RuntimeException("WrongStateException? Did you do that on purpose?");
    }
  }
  
  /**
   * Called by the framework once per cycle if the node is active. <emph>For
   * framework internal use only.</emph>
   */
  public final void FWRun() {
    try {
      run();
    }
    catch (WrongStateException e) {
      throw new 
        RuntimeException("WrongStateException? Did you do that on purpose?");
    }
  }      
  
  /**
   * Returns the object that manages the current demonstration.
   * @return the demonstrator
   * @throws WrongStateException if no demo is running
   */
  private Demonstrator getDemonstrator() throws WrongStateException {
    return Core.getDemonstration().getDemonstrator(modelNode.getProject());
  }

}