/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.core.demonstration;

import fdda.core.projects.Project;
import fdda.dialog.GraphicsControl;

import fdda.algorithm.WrongStateException;

/**
 * Manages the demonstration of projects. Though currently only one project at a
 * time can be demonstrated, the managing capabilities for more are available 
 * here. 
 * Also checks for state errors (e.g. stopping a demo that isn't started).
 * @author Jrg Rdenauer
 */
public interface Demonstration {
  
  /**
   * Starts the demonstration of a project.
   * 
   * @param project the project
   * @param graphicsControl object that controls the graphics of this project
   * @throws WrongStateException if the project is already being demonstrated
   */
  void start(Project project, GraphicsControl graphicsControl) 
  throws WrongStateException;
  
  /**
   * Pauses the demonstration of a project. If the demo is already paused, 
   * continues the demonstration (i.e. it works like a switch).
   * 
   * @param project the project
   * @throws WrongStateException if the project isn't being demonstrated
   */
  void pause(Project project) throws WrongStateException;
  
  /**
   * Stops the demonstration of a project. Since the demonstration runs in 
   * another thread, it may not yet be stopped after this method returns. OTOH, 
   * at most the time for one cycle plus a small fixed time will run until the 
   * demo is stopped.
   * 
   * @param project the project
   * @throws WrongStateException if the project isn't being demonstrated
   */
  void stop(Project project) throws WrongStateException;
  
  /**
   * Returns whether a project has already been demonstrated
   * @param project the project
   * @return true, iff the project has already been demonstrated
   */
  boolean wasDemonstrated(Project project);
  
  /**
   * Returns the demonstrator for a project.
   * 
   * @param project the project
   * @return the demonstrator
   * @throws WrongStateException if the project wasn't demonstrated yet
   */
  Demonstrator getDemonstrator(Project project) throws WrongStateException;

}
