/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.core.demonstration;

import fdda.algorithm.Message;
import fdda.algorithm.Node;

/**
 * Manages the demonstration of the algorithm of a project. Also provides the 
 * link to the graphical visualization. The demonstration will run in an extra
 * thread.
 * @author Jrg Rdenauer
 */
public interface Demonstrator {
  
  /**
   * Returns whether the demo is stopped.
   * 
   * @return whether the demo is stopped
   */
  boolean isStopped();
  
  /**
   * Returns whether the demo is paused.
   * 
   * @return whether the demo is paused
   */
  boolean isPaused();
  
  /**
   * Returns whether the demo is running (and not paused).
   * 
   * @return whether the demo is paused
   */
  boolean isRunning();
  
  /**
   * Adds a node to the active nodes. That means that the run()-method of this 
   * node will be called once each cycle.
   * 
   * @param node the node
   */
  void addActiveNode(Node node);
  
  /**
   * Removes a node from the active nodes for a certain time. After that time, 
   * the node becomes active again. The algorithm isn't terminated while there 
   * are still nodes waiting for activation at a fixed point of time.
   * 
   * @param node the node
   * @param timeInMillis time in milliseconds after which the node will be 
   * activated
   */
  void pauseActiveNode(Node node, long timeInMillis);
  
  /**
   * Removes a node from the active nodes. That means that its run()-method 
   * won't be called anymore.
   * 
   * @param node the node
   */
  void removeActiveNode(Node node);
  
  /**
   * Adds a message to the active messages. That means that this message is on 
   * its way to a receiving node. Once it reaches its destination, it is 
   * automatically removed.
   * 
   * @param message the message
   */
  void addActiveMessage(Message message);
  
  /**
   * Creates a new message. 
   * 
   * @param sender the node that sends the message
   * @param receiver the node that shall receive the message
   * @param payload possible payload that the message carries
   * @throws IllegalArgumentException if the nodes are not neighboured
   * @return the newly created message
   */
  Message createMessage(Node sender, Node receiver, Object payload);
  
  /**
   * Returns up-to-date statistics about the demo. These statistics even 
   * continue to get updated.
   * @return statistics for the demo
   */
  Statistics getCurrentStatistics();
  
}