/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.core.impl;

import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Timer;
import java.util.TimerTask;
import java.util.prefs.Preferences;

import fdda.core.Core;
import fdda.core.demonstration.Demonstrator;
import fdda.core.demonstration.Statistics;

import fdda.algorithm.Message;
import fdda.algorithm.Node;

import fdda.core.projects.Project;

import fdda.dialog.GraphicsControl;
import fdda.dialog.MessageControl;

import fdda.graphics.ComponentFactory;
import fdda.graphics.Line;

import fdda.model.Connection;
import fdda.model.PropertyEvent;
import fdda.model.PropertyListener;

import fdda.utils.Localization;
import fdda.utils.Logging;
import fdda.utils.Notification;

/**
 * Manages the demonstration of a project. Therefore, it runs an extra thread
 * that moves the messages and calls the methods of active nodes in an 'endless'
 * loop. Registers itself as a PropertyListener with each node and connection to
 * be able to react to changes of the 'working' property.
 * @author Jrg Rdenauer
 */
class DemonstratorImpl implements Runnable, Demonstrator, PropertyListener {
  
  /** State: demo is running */
  private static final int RUNNING = 0;
  /** State: demo is paused */
  private static final int PAUSED = 1;
  /** State: demo is stopped */
  private static final int STOPPED = 2;
  /** State: demo is not yet started */
  private static final int NOT_STARTED = 3;

  /** Algorithm shall continue */
  private static final int CONTINUE = 0;
  /** Algorithm has terminated */
  private static final int TERMINATED = 1;
  /** Algorithm has thrown an exception */
  private static final int EXCEPTION = 2;
  /** Algorithm has been stopped */
  private static final int STOP = 3;
  
  /** shall the demonstration be paused? */
  private boolean pause;
  /** shall the demonstration be stopped? */
  private boolean stop;
  /** the current state of the demonstration: one of the constants */
  private int state;
  
  /** the project that is demonstrated */
  private Project project;
  
  /** the currently active nodes */
  private HashSet activeNodes;
  /** the nodes that are currently waiting for activation (after some time) */
  private HashSet pausedNodes;
  /** the currently active messages */
  private HashSet activeMessages;
  
  /** the control object for the graphical messages */
  private MessageControl msgControl;
  
  /** timer to manage the waiting nodes */
  private Timer timer = null;
  
  /** stores an exception thrown by the algorithm */
  private Exception lastException = null;
  
  /** stores the statistics */
  private StatisticsImpl statistics;
  
  /**
   * Constructor
   * @param project the project that is demonstrated
   * @param graphicsControl the graphics controlling object
   */
  public DemonstratorImpl(Project project, GraphicsControl graphicsControl) {
    pause = false;
    stop = false;
    state = NOT_STARTED;
    this.project = project;
    // note that the project must have a graphical system for the demo to work..
    this.msgControl = graphicsControl.getMessageControl();
    activeNodes = new HashSet();
    pausedNodes = new HashSet();
    activeMessages = new HashSet();
    statistics = new StatisticsImpl();
  }
  
  /**
   * Resets the demonstrator for a new run. Also resets the statistics.
   */
  public synchronized void reset() {
    state = NOT_STARTED;
    pause = false;
    stop = false;
    statistics.reset();
  }
  
  /**
   * Starts the demonstration thread.
   */
  public synchronized void start() {
    (new Thread(this)).start();
  }
  
  /**
   * Pauses the demonstration. Pause will not be in effect immediately after the
   * return of the method.
   */
  public synchronized void pause() {
    pause = true;
  }
  
  /**
   * Continues the demonstration. Continuation will not be in effect immediately
   * after the return of the method.
   */
  public synchronized void endPause() {
    pause = false;
  }
  
  /**
   * Stops the demonstration. Demonstration will not be stopped immediately 
   * after the return of the method.
   */
  public synchronized void stop() {
    stop = true;
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.core.demonstration.Demonstrator#isStopped()
   */
  public synchronized boolean isStopped() {
    return state == STOPPED;
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.core.demonstration.Demonstrator#isPaused()
   */
  public synchronized boolean isPaused() {
    return state == PAUSED;
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.core.demonstration.Demonstrator#isRunning()
   */
  public synchronized boolean isRunning() {
    return state == RUNNING;
  }
  
  /**
   * Runs the demonstration. First registers itself as listener; than 
   * demonstrates (the actual work is done by the method doLoop); than cleans 
   * up, changes the state of the project back and informs the user. Only than, 
   * the demo counts as stopped.
   * 
   * @see java.lang.Runnable#run()
   */
  public void run() {
    // private variable for pause to avoid long synchronized blocks
    boolean shallPause = false;
    long minCycleTime = Preferences.userNodeForPackage(getClass()).getLong(
      "minCycleTime", 30);
    synchronized (this) {
      timer = new Timer();
      lastException = null;
    }
    // retCode stores whether the demo shall continue & the reason for stopping
    int retCode = CONTINUE;
    int nrOfNodes = 0; int nrOfConnections = 0;
    // register listener at all nodes and connections. Also starts active nodes.
    for (Iterator it = project.nodes(); it.hasNext(); ) {
      fdda.model.Node node = (fdda.model.Node) it.next();
      try {
        node.getAlgorithmicNode().FWPrepareForStart();
      }
      catch (Exception e) {
        Logging.exceptionCaught(e);
        lastException = e;
        retCode = EXCEPTION;
        break;
      }
      node.addPropertyListener(this);
      if (((Boolean)node.getProperty(fdda.model.Node.ACTIVE_PROPERTY_NAME).
            getValue()).booleanValue()) {
        // active nodes are started here
        if (node.isWorking()) {
          try {
            node.getAlgorithmicNode().FWStart();
          }
          catch (Exception e) {
            Logging.exceptionCaught(e);
            lastException = e;
            retCode = EXCEPTION;
            break;
          }
          if (!(node.isRepeater())) {
            statistics.addUsedNode(node.getAlgorithmicNode());
            statistics.addStartedNode();
          }
        }
      }
      if (!(node.isRepeater())) {
        nrOfNodes++;
      }
      if (!node.isWorking()) statistics.addNodeFailure();
    }
    if (retCode == CONTINUE) {
      statistics.setPresentNodes(nrOfNodes);
      for (Iterator it = project.connections(); it.hasNext(); ) {
        Connection connection = (Connection) it.next();
        connection.addPropertyListener(this);
        nrOfConnections++;
        if (!connection.isWorking()) {
          statistics.addConnectionFailure();
        }
      }
      statistics.setPresentConnections(nrOfConnections);
    }
    while (retCode == CONTINUE) {
      // check for pause
      synchronized (this) {
        shallPause = pause;
        if (shallPause) state = PAUSED; else state = RUNNING;
      }
      if (!shallPause) {
        long startTime = System.currentTimeMillis();
        
        retCode = doLoop();
        
        long endTime = System.currentTimeMillis();
        // ensure that one loop takes at least minCycleTime
        if ((endTime - startTime) < minCycleTime) {
          try {
            Thread.sleep(minCycleTime - endTime + startTime);
          }
          catch (InterruptedException e) { }
        }
      }
      else {
        msgControl.startCycle();
        // Pause: wait 200 milliseconds (5 continuation checks / sec.)
        try {
          Thread.sleep(200);
        }
        catch (InterruptedException e) { }
        msgControl.cycleDone();
      }
      // check whether the user stopped the demo
      synchronized(this) {
        if (stop) retCode = STOP;
      }
    }
    // clean up
    msgControl.startCycle();
    msgControl.removeAllMessages();
    msgControl.cycleDone();
    synchronized (this) {
      activeNodes.clear();
      activeMessages.clear();
      pausedNodes.clear();
      timer.cancel();
    }
    statistics.setCurrentlyActiveNodes(0);
    statistics.setCurrentlyPausedNodes(0);
    statistics.setCurrentlyActiveMessages(0);
    // remove the listener
    for (Iterator it = project.nodes(); it.hasNext(); ) {
      fdda.model.Node node = (fdda.model.Node) it.next();
      node.removePropertyListener(this);
    }
    for (Iterator it = project.connections(); it.hasNext(); ) {
      Connection connection = (Connection) it.next();
      connection.removePropertyListener(this);
    }
    // notify the user
    if (retCode == STOP) {
      Notification.message(Notification.M_CONFIRM, Notification.S_ALGORITHM,
        Localization.getLocalizedString(getClass(), "algorithm_stopped",
        "Algorithmus wurde gestoppt."));
    }
    else if (retCode == TERMINATED) {
      Notification.message(Notification.M_CONFIRM, Notification.S_ALGORITHM,
        Localization.getLocalizedString(getClass(), "algorithm_terminated",
        "Algorithmus hat terminiert."));
    }
    else if (retCode == EXCEPTION) {
      StackTraceElement ste = lastException.getStackTrace()[0];
      Notification.message(Notification.M_ERROR, Notification.S_ALGORITHM,
        Localization.formatMessage(Localization.getLocalizedString(getClass(),
        "algorithm_error",
        "Algorithmus ist fehlerhaft!\nAusgelster Fehler: {0}" +
        "\nin {1}, Methode {2}, Zeile {3}."), new Object[] { 
        lastException.getClass().getName(), ste.getFileName(), 
        ste.getMethodName(), new Integer(ste.getLineNumber())}));
    }
    else {
      Notification.message(Notification.M_ERROR, Notification.S_ALGORITHM,
        Localization.getLocalizedString(getClass(), "algorithm_stopped2",
        "Algorithmus wurde irregulr beendet."));
    }
    // return to normal state
    Core.getProjectManager().setModState();
    synchronized (this) {
      state = STOPPED;
    }
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.core.demonstration.Demonstrator#addActiveNode(fdda.algorithm.Node)
   */
  public synchronized void addActiveNode(Node node) {
    activeNodes.add(node);
    statistics.addActiveNode(node);
    statistics.addUsedNode(node);
    statistics.setCurrentlyActiveNodes(activeNodes.size());
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.core.demonstration.Demonstrator#removeActiveNode(fdda.algorithm.Node)
   */
  public synchronized void removeActiveNode(Node node) {
    activeNodes.remove(node);
    statistics.setCurrentlyActiveNodes(activeNodes.size());
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.core.demonstration.Demonstrator#addActiveMessage(fdda.algorithm.Message)
   */
  public synchronized void addActiveMessage(Message message) {
    activeMessages.add(message);
    msgControl.addMessage(message.getGraphicalRepresentation());
    statistics.setCurrentlyActiveMessages(activeMessages.size());
    statistics.addUsedConnection(
      message.getGraphicalRepresentation().getLine().getConnection());
    statistics.addSentMessage();
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.core.demonstration.Demonstrator#pauseActiveNode(fdda.algorithm.Node, long)
   * @see java.util.Timer
   */
  public synchronized void pauseActiveNode(Node node, long timeInMillis) {
    // note that this works also if the node wasn't active...
    activeNodes.remove(node);
    pausedNodes.add(node);
    // using java.util.Time to schedule the readding of the node
    timer.schedule(new NodeActivator(node), timeInMillis);    
    statistics.setCurrentlyActiveNodes(activeNodes.size());
    statistics.setCurrentlyPausedNodes(pausedNodes.size());
    statistics.addUsedNode(node);
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.core.demonstration.Demonstrator#createMessage(fdda.algorithm.Node, fdda.algorithm.Node, java.lang.Object)
   */
  public Message createMessage(Node sender, Node receiver, Object payload) {
    Message message = new Message(sender, receiver, payload, this);
    // find the line and determine direction
    List connections = sender.getModelNode().getConnections();
    Line line = null; boolean forward = false;
    for (Iterator it = connections.iterator(); 
         (line == null) && it.hasNext(); ) {
      Connection connection = (Connection) it.next();
      if ((connection.getStartNode() == sender.getModelNode()) &&
          (connection.getEndNode() == receiver.getModelNode())) {
        line = connection.getGraphicalRepresentation();
        forward = true;
      }
      else if ((connection.getStartNode() == receiver.getModelNode()) &&
               (connection.getEndNode() == sender.getModelNode())) {
        line = connection.getGraphicalRepresentation();
        forward = false;
      }
    }
    if (line == null) 
      throw new IllegalArgumentException("No suitable line found!");
    fdda.graphics.Message graphMessage = 
      ComponentFactory.getInstance().createMessage(line, forward);
    message.setGraphicalRepresentation(graphMessage);
    graphMessage.setRealMessage(message);
    return message;
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.core.demonstration.Demonstrator#getCurrentStatistics()
   */
  public Statistics getCurrentStatistics() {
    return statistics;
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.model.PropertyListener#propertyChanged(fdda.model.PropertyEvent)
   */
  public void propertyChanged(PropertyEvent pe) {
    // is it the 'working' property?
    if (pe.getProperty().getName().equals(
           fdda.model.WorkingPart.WORKING_PROPERTY_NAME)) {
      // Nodes: if it gets working and is marked 'active', start it
      // if it breaks down and is currently active, remove the active node
      if (pe.getSource() instanceof fdda.model.Node) {
        fdda.model.Node node = (fdda.model.Node) pe.getSource();
        if (((Boolean) pe.getProperty().getValue()).booleanValue()) {
          if (((Boolean) node.getProperty(fdda.model.Node.ACTIVE_PROPERTY_NAME)
              .getValue()).booleanValue()) {
            node.getAlgorithmicNode().FWStart();
            if (!(node.isRepeater())) {
              statistics.addStartedNode();
              statistics.addUsedNode(node.getAlgorithmicNode());
            }
          }
          statistics.addNodeRecovery();
        }
        else {
          removeActiveNode(node.getAlgorithmicNode());
          synchronized (this) {
            pausedNodes.remove(node.getAlgorithmicNode());
            statistics.setCurrentlyActiveNodes(activeNodes.size());
            statistics.setCurrentlyPausedNodes(pausedNodes.size());
          }
          statistics.addNodeFailure();
        }
      }
      else { // pe.getSource() instanceof Connection
        // if the connection breaks down, remove all its messages
        if (!((Boolean) pe.getProperty().getValue()).booleanValue()) {
          // copy messages to another structure to avoid 
          // ConcurrentModification errors
          LinkedList oldMessages = new LinkedList();
          synchronized (this) {
            for (Iterator it = activeMessages.iterator(); it.hasNext(); ) {
              oldMessages.addLast(it.next());
            }
            for (Iterator it = oldMessages.iterator(); it.hasNext(); ) {
              Message message = (Message) it.next();
              if (message.getGraphicalRepresentation().getLine().getConnection()
                  == pe.getSource()) {
                // Line broke down. Message is lost.
                activeMessages.remove(message);
                msgControl.removeMessage(message.getGraphicalRepresentation());
              }
            }
          } // synch.
          statistics.addConnectionFailure();
        } // not working?
        else {
          statistics.addConnectionRecovery();
        }
      } // connection
    } // working property    
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.model.PropertyListener#propertyAdded(fdda.model.PropertyEvent)
   */
  public void propertyAdded(PropertyEvent pe) {
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.model.PropertyListener#propertyRemoved(fdda.model.PropertyEvent)
   */
  public void propertyRemoved(PropertyEvent pe) {
  }
  
  /**
   * Sets a node to 'active' status again. 
   * @param node the node
   */
  private synchronized void removePausedNode(Node node) {
    if (pausedNodes.contains(node)) {
      pausedNodes.remove(node);
      activeNodes.add(node);
      statistics.addActiveNode(node);
      statistics.setCurrentlyActiveNodes(activeNodes.size());
      statistics.setCurrentlyPausedNodes(pausedNodes.size());
    }
  }
  
  /**
   * Performs one cycle of the demonstration. First, the active nodes are 
   * called, then the messages are moved. If a message reaches its destination,
   * the node is informed.
   * @return return code, one of the constants
   */
  private int doLoop() {
    int retCode = CONTINUE;
    msgControl.startCycle();
    // check for termination (global view)
    synchronized (this) {
      if (activeNodes.isEmpty() && pausedNodes.isEmpty() && 
          activeMessages.isEmpty()) {
        retCode = TERMINATED;
      }
    }
    try {
      // copy nodes and messages to another temporary structure. This is to 
      // avoid long synchronized blocks and ConcurrentModification errors.
      LinkedList nodes = new LinkedList();
      LinkedList messages = new LinkedList();
      synchronized (this) {
        for (Iterator it = activeNodes.iterator(); it.hasNext(); ) {
          nodes.addLast(it.next());
        }
        for (Iterator it = activeMessages.iterator(); it.hasNext(); ) {
          messages.addLast(it.next());
        }
      }
      // Run active nodes
      for (Iterator it = nodes.iterator(); it.hasNext(); ) {
        Node node = (Node) it.next();
        if (node.getModelNode().isWorking()) node.FWRun();
      }
      // move messages forward (logically, not graphically)
      for (Iterator it = messages.iterator(); it.hasNext(); ) {
        Message message = (Message) it.next();
        Line line = message.getGraphicalRepresentation().getLine();
        message.addCycle();
        // has message reached its destination?
        if (message.getCyclesSinceSending() * line.getMessageDistancePerCycle()
            >= line.getLength()) {
          synchronized (this) {
            activeMessages.remove(message);
            statistics.setCurrentlyActiveMessages(activeMessages.size());
          }
          msgControl.removeMessage(message.getGraphicalRepresentation());
          statistics.addReceivedMessage();
          Node receiver = message.getReceiver();
          if (!(receiver.getModelNode().isRepeater())) {
            statistics.addUsedNode(receiver);
          }
          // notify the node (if it is working)
          if (receiver.getModelNode().isWorking()) {
            receiver.FWMessageArrived(message.getSender(), message);
          }
        }
      }
      // move the remaining message graphically, too
      msgControl.moveMessages(activeMessages);
    }
    catch (Exception e) {
      lastException = e;
      retCode = EXCEPTION;
      Logging.exceptionCaught(e);
      // e.printStackTrace();
    }
    msgControl.cycleDone();
    return retCode;
  }    
  
  /**
   * Activates a node again that was paused.
   */
  private class NodeActivator extends TimerTask {
    /** the node */
    private Node node = null;
    /**
     * Constructor
     * @param node the node
     */
    public NodeActivator(Node node) {
      this.node = node;
    }
    /*
     *  (non-Javadoc)
     * @see java.lang.Runnable#run()
     */
    public void run() {
      removePausedNode(node);
    }
  }

}
