/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.core.impl;

import fdda.core.Core;
import fdda.core.demonstration.Demonstration;
import fdda.core.demonstration.Demonstrator;
import fdda.core.projects.ProjectEvent;
import fdda.core.projects.ProjectListener;

import fdda.graphics.ComponentFactory;

import fdda.utils.Localization;
import fdda.utils.Notification;
import fdda.utils.ObjectCache;

import fdda.plugins.ProjectInformation;
import fdda.plugins.SystemReader;
import fdda.plugins.SystemWriter;
import fdda.plugins.DefaultErrorHandler;

import java.util.prefs.Preferences;

import java.util.HashSet;
import java.util.Iterator;

import javax.swing.event.EventListenerList;

/**
 * Implementation of the project manager.
 * @author Jrg Rdenauer
 */
public class ProjectManagerImpl implements fdda.core.projects.ProjectManager {

  /** the listeners for project events */
  private EventListenerList projectListeners;

  /** the projects */
  private HashSet projects;

  /** the currently active project */
  private fdda.core.projects.Project activeProject;

  /**
   * Constructor
   */
  public ProjectManagerImpl() {
    this.projectListeners = new EventListenerList();
    this.projects = new HashSet(5);
  }

  /*
   *  (non-Javadoc)
   * @see fdda.core.projects.ProjectManager#newProject()
   */
  public fdda.core.projects.Project newProject() {
    Project project = new Project();
    project.setName(Localization.getLocalizedString(getClass(), 
      "new_project_name",  "<Neues Projekt>"));
    project.setGraphicalSystem(
      ComponentFactory.getInstance().createGraphicalSystem());
    projects.add(project);
    this.fireProjectOpened(new ProjectEvent(project));
    return project;
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.core.projects.ProjectManager#openProject(fdda.core.projects.Project, java.lang.String)
   */
  public fdda.core.projects.Project openProject(final String location) {
    Project project = new Project();
    project.setGraphicalSystem(
      ComponentFactory.getInstance().createGraphicalSystem());
    boolean success = false;
    SystemReader reader = null;
    String className = Preferences.userRoot().node("fdda/plugins").get(
      "currentProjectReader", "fdda.plugins.xmlreader.ProjectReader"); 
     
    try {
      reader = (SystemReader) ObjectCache.getInstance().getObject(className);
    }
    catch (ClassNotFoundException e) {
      Notification.message(Notification.M_ERROR, Notification.S_PROCESSING,
        Localization.formatMessage(Localization.getLocalizedString(getClass(), 
          "plugin_not_found", "Plug-in-Klasse {0} wurde nicht gefunden!"), 
          className));
    }
    catch (InstantiationException e) {
      Notification.message(Notification.M_ERROR, Notification.S_PROCESSING,
        Localization.formatMessage(Localization.getLocalizedString(getClass(),
         "plugin_instantiation", 
          "Plug-in-Klasse {0} konnte nicht instanziiert werden!\n"
        + "Prfen sie, ob die Klasse einen Konstruktor ohne Parameter hat."), 
        className));
    }
    catch (IllegalAccessException e) {
      Notification.message(Notification.M_ERROR, Notification.S_PROCESSING,
        Localization.formatMessage(Localization.getLocalizedString(getClass(),
          "plugin_instantiation2", 
          "Plug-in-Klasse {0} konnte nicht instanziiert werden!\n"
        + "Prfen sie, ob der Konstruktor public ist."), className));
    }
    catch (ClassCastException e) {
      Notification.message(Notification.M_ERROR, Notification.S_PROCESSING,
        Localization.formatMessage(Localization.getLocalizedString(getClass(),
          "plugin_class_reader", 
          "Plug-in-Klasse {0} implementiert nicht das Interface "
        + "fdda.plugins.SystemReader!"), className));
    }
    // loading is possible if we have a reader and either a file is selected
    // or the reader can use a saved location
    if (reader != null &&
       ((location != null && reader.setSystemLocation(location)) ||
        reader.performUI())) {
      try {
        DefaultErrorHandler deh = new DefaultErrorHandler("Laden");
        reader.loadProject(deh, project);
        deh.printWarnings();
        if (deh.hadError()) {
          deh.printErrors();
        }
        else {
          success = true;
          project.setLocation(null);
          ProjectInformation[] infos = 
            reader.getLastLoadedProjectsInformation();
          if (infos != null && infos.length > 0 &&
              infos[0] != null) {
            // try to pass the location to the SystemWriter
            className = Preferences.userRoot().node("fdda/plugins").get(
              "currentProjectWriter", "fdda.plugins.xmlwriter.ProjectWriter");
            try {
              SystemWriter projectWriter = 
                (SystemWriter) ObjectCache.getInstance().getObject(className);
              if (projectWriter.setSystemLocation(infos[0].getLocation())) {
                project.setLocation(infos[0].getLocation());
              }
              else {
                project.setLocation(null);
              }
            }
            // These Exceptions aren't shown to the user: they could occur if no
            // writer plug-in is selected or something like that
            catch (ClassNotFoundException e) {
            }
            catch (InstantiationException e) {
            }
            catch (IllegalAccessException e) {
            }
            catch (ClassCastException e) {
            }
          }
        }
      }
      catch (fdda.plugins.MissingOptionsException e) {
        Notification.message(Notification.M_ERROR, Notification.S_PLUGIN,
          Localization.formatMessage(Localization.getLocalizedString(getClass(),
        "plugin_error", "Beim Aufruf des Plugins ist ein Fehler aufgetreten. \n"
          + "Fehlermeldung: {0}"), e.getMessage()));
      }
    }
    if (success) {
      projects.add(project);
      this.fireProjectOpened(new ProjectEvent(project));
    }
    else {
      project = null;
    }
    return project;
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.core.projects.ProjectManager#saveProject(fdda.core.projects.Project, boolean)
   */
  public void saveProject(fdda.core.projects.Project project, 
                           boolean newName) {
    String oldLocation = project.getLocation();
    if (newName) {
      project.setLocation(null);
    }
    String className = Preferences.userRoot().node("fdda/plugins").get(
      "currentProjectWriter", "fdda.plugins.xmlwriter.ProjectWriter");
    SystemWriter projectWriter = null;
    try {
      projectWriter = (SystemWriter) ObjectCache.getInstance().getObject(
        className);
    }
    catch (ClassNotFoundException e) {
      Notification.message(Notification.M_ERROR, Notification.S_PROCESSING,
        Localization.formatMessage(Localization.getLocalizedString(getClass(), 
          "plugin_not_found", "Plug-in-Klasse {0} wurde nicht gefunden!"), 
          className));
    }
    catch (InstantiationException e) {
      Notification.message(Notification.M_ERROR, Notification.S_PROCESSING,
        Localization.formatMessage(Localization.getLocalizedString(getClass(),
         "plugin_instantiation", 
         "Plug-in-Klasse {0} konnte nicht instanziiert werden!\n"
        + "Prfen sie, ob die Klasse einen Konstruktor ohne Parameter hat."), 
        className));
    }
    catch (IllegalAccessException e) {
      Notification.message(Notification.M_ERROR, Notification.S_PROCESSING,
        Localization.formatMessage(Localization.getLocalizedString(getClass(),
          "plugin_instantiation2", 
          "Plug-in-Klasse {0} konnte nicht instanziiert werden!\n"
        + "Prfen sie, ob der Konstruktor public ist."), className));
    }
    catch (ClassCastException e) {
      Notification.message(Notification.M_ERROR, Notification.S_PROCESSING,
        Localization.formatMessage(Localization.getLocalizedString(getClass(),
          "plugin_class_writer", 
          "Plug-in-Klasse {0} implementiert nicht das Interface "
        + "fdda.plugins.SystemWriter!"), className));
    }
    // save is possible if we have a writer and either a file is selected or a 
    // stored location can be used
    if (projectWriter != null && ((project.isLocationKnown() &&
         projectWriter.setSystemLocation(project.getLocation())) ||
         projectWriter.performUI())) {
      try {
        DefaultErrorHandler deh = new DefaultErrorHandler("Speichern");
        projectWriter.writeProject(project, deh);
        deh.printWarnings();
        deh.printErrors();
        if (!deh.hadError()) {
          Notification.message(Notification.M_CONFIRM, Notification.S_PLUGIN,
         Localization.getLocalizedString(getClass(), "save_title", "Speichern"), 
         Localization.formatMessage(Localization.getLocalizedString(getClass(),
              "save_message", "Projekt {0} erfolgreich gespeichert."), 
            project.getName()));
        }
        else {
          project.setLocation(oldLocation);
        }
      }
      catch (fdda.plugins.MissingOptionsException e) {
        Notification.message(Notification.M_ERROR, Notification.S_PLUGIN,
          Localization.formatMessage(Localization.getLocalizedString(getClass(),
        "plugin_error", "Beim Aufruf des Plugins ist ein Fehler aufgetreten. \n"
          + "Fehlermeldung: {0}"), e.getMessage()));
        project.setLocation(oldLocation);
      }
    }
    else {
      project.setLocation(oldLocation);
    }
  }

  /*
   *  (non-Javadoc)
   * @see fdda.core.projects.ProjectManager#closeProject(fdda.core.projects.Project)
   */
  public boolean closeProject(fdda.core.projects.Project project) {
     int result = -1; // RC from user question
     // Does the project need to be saved?
     if (project.wasChanged()) {
       result = Notification.message(Notification.M_QUESTION,
                Notification.S_MOD,
          Localization.formatMessage(Localization.getLocalizedString(getClass(),
              "save_quit_question", "Das Projekt {0} ist nicht gespeichert. "
              + "Projekt trotzdem schlieen?"), project.getName()), new String[]
              {Localization.getLocalizedString(getClass(), "not_save_quit", 
                "Schlieen ohne Speichern"), 
              Localization.getLocalizedString(getClass(), "save_quit", 
                "Speichern und Schlieen"),
              Localization.getLocalizedString(getClass(), "not_quit",
               "Nicht schlieen")});
     }
     // save and close
     if (result == 1) {
       saveProject(project, false);
       // check whether save was successful
       if (project.wasChanged()) {
         return false; // don't close since save didn't work
       }
     }
     // don't close
     if (result == 2) {
       return false;
     }
     // check whether a demo must be stopped
     if (project.getState() == Project.STATE_DEMONSTRATION) {
       result = Notification.message(Notification.M_QUESTION,
                Notification.S_ALGORITHM,
          Localization.formatMessage(Localization.getLocalizedString(getClass(),
                "demo_quit_question", "Demo des Projektes {0} ist aktiv. " +
                "Projekt trotzdem schlieen?"), project.getName()), new String[]
                {Localization.getLocalizedString(getClass(), "stop_quit", 
                   "Stoppen und schlieen"), 
                 Localization.getLocalizedString(getClass(), "not_stop_quit",
                   "Nicht stoppen / schlieen")});
       if (result == 1) {
         return false;
       }
       final fdda.core.projects.Project projectToClose = project;
       Runnable stopper = new Runnable() {
         
         public void run() {
           Notification.indicateOccupationState();
           Demonstration demo = Core.getDemonstration();
           try {
             Demonstrator demonstrator = demo.getDemonstrator(projectToClose);
             demo.stop(projectToClose);
             // wait for the real stop
             while (!demonstrator.isStopped()) {
               try {
                 Thread.sleep(150);
               }
               catch (InterruptedException e) {}
             }
           }
           catch (fdda.algorithm.WrongStateException e) {
             throw new InternalError();
           }
           projects.remove(projectToClose);
           if (projectToClose == activeProject) {
             setActiveProject(null);
           }
           fireProjectClosed(new ProjectEvent(projectToClose));
           Notification.indicateNormalState();
         }
       };
       (new Thread(stopper)).start();
       return true;
     }
     else {
       projects.remove(project);
       if (project == activeProject) {
         setActiveProject(null);
       }
       fireProjectClosed(new ProjectEvent(project));
       return true;
     }
  }

  /*
   *  (non-Javadoc)
   * @see fdda.core.projects.ProjectManager#closeAll()
   */
  public boolean closeAll() {
    boolean success = true;
    // copy to an array to avoid ConcurrentModification errors
    fdda.core.projects.Project[] projectArray 
      = new fdda.core.projects.Project[projects.size()];
    projects.toArray(projectArray);
    for (int i = 0; i < projectArray.length; i++) {
      success = success && this.closeProject(projectArray[i]);
    }
    return success;
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.core.projects.ProjectManager#getActiveProject()
   */
  public fdda.core.projects.Project getActiveProject() {
    return activeProject;
  }

  /*
   *  (non-Javadoc)
   * @see fdda.core.projects.ProjectManager#setActiveProject(fdda.core.projects.Project)
   */
  public void setActiveProject(fdda.core.projects.Project project) {
    if ((project == null || projects.contains(project))
      && project != activeProject) {
      activeProject = project;
      if (project != null) {
        this.fireActiveProjectChanged(new ProjectEvent(project));
      }
    }
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.core.projects.ProjectManager#changeProjectName(fdda.core.projects.Project, java.lang.String)
   */
  public void changeProjectName(fdda.core.projects.Project project, 
                                 String newName) {
    ((Project) project).setName(newName);
    fireProjectNameChanged(new ProjectEvent(project));
  }

  /*
   *  (non-Javadoc)
   * @see fdda.core.projects.ProjectManager#getNrOfProjects()
   */
  public int getNrOfProjects() {
    return projects.size();
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.core.projects.ProjectManager#projects()
   */
  public Iterator projects() {
    return projects.iterator();
  }

  /*
   *  (non-Javadoc)
   * @see fdda.core.projects.ProjectManager#setModState()
   */
  public void setModState() {
    Iterator it = projects.iterator();
    boolean found = false;
    while (!found && it.hasNext()) {
      fdda.core.projects.Project project = 
        (fdda.core.projects.Project) it.next();
      if (project.getState() == Project.STATE_DEMONSTRATION) {
        found = true;
        ((Project) project).setState(fdda.core.projects.Project.STATE_MOD);
        this.fireProjectStateChanged(new ProjectEvent(project));
      }
    }
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.core.projects.ProjectManager#isInDemoState()
   */
  public boolean isInDemoState() {
    Iterator it = projects.iterator();
    boolean found = false;
    while (!found && it.hasNext()) {
      fdda.core.projects.Project project = 
        (fdda.core.projects.Project) it.next();
      if (project.getState() == Project.STATE_DEMONSTRATION) {
        found = true;
      }
    }
    return found;
  }

  /*
   *  (non-Javadoc)
   * @see fdda.core.projects.ProjectManager#setDemoStateForActiveProject()
   */
  public void setDemoStateForActiveProject() {
    ((Project) activeProject).setState(Project.STATE_DEMONSTRATION);
    this.fireProjectStateChanged(new ProjectEvent(activeProject));
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.core.projects.ProjectManager#addProjectListener(fdda.core.projects.ProjectListener)
   */
  public void addProjectListener(ProjectListener projectListener) {
    projectListeners.add(ProjectListener.class, projectListener);
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.core.projects.ProjectManager#removeProjectListener(fdda.core.projects.ProjectListener)
   */
  public void removeProjectListener(ProjectListener projectListener) {
    projectListeners.remove(ProjectListener.class, projectListener);
  }

  /**
   * Informs the listeners that a project has been opened
   * @param projectEvent the event
   */
  protected final void fireProjectOpened(ProjectEvent projectEvent) {
    // Guaranteed to return a non-null array
    Object[] listeners = this.projectListeners.getListenerList();
    // Process the listeners last to first, notifying
    // those that are interested in this event
    for (int i = listeners.length - 2; i >= 0; i -= 2) {
      if (listeners[i] == ProjectListener.class) {
        // Lazily create the event:
        if (projectEvent == null) {
          projectEvent = new ProjectEvent(activeProject);
        }
        ((ProjectListener)listeners[i + 1]).projectOpened(projectEvent);
      }
    }
  }

  /**
   * Informs the listeners that a project has been closed
   * @param projectEvent the event
   */
  protected final void fireProjectClosed(ProjectEvent projectEvent) {
    // Guaranteed to return a non-null array
    Object[] listeners = this.projectListeners.getListenerList();
    // Process the listeners last to first, notifying
    // those that are interested in this event
    for (int i = listeners.length - 2; i >= 0; i -= 2) {
      if (listeners[i] == ProjectListener.class) {
          // Lazily create the event:
        if (projectEvent == null) {
          projectEvent = new ProjectEvent(activeProject);
        }
        ((ProjectListener) listeners[i + 1]).projectClosed(projectEvent);
      }
    }
  }

  /**
   * Informs the listeners that a project has changed its state
   * @param projectEvent the event
   */
  protected final void fireProjectStateChanged(ProjectEvent projectEvent) {
    // Guaranteed to return a non-null array
    Object[] listeners = this.projectListeners.getListenerList();
    // Process the listeners last to first, notifying
    // those that are interested in this event
    for (int i = listeners.length - 2; i >= 0; i -= 2) {
      if (listeners[i] == ProjectListener.class) {
        // Lazily create the event:
        if (projectEvent == null) {
          projectEvent = new ProjectEvent(activeProject);
        }
        ((ProjectListener) listeners[i + 1]).projectStateChanged(projectEvent);
      }
    }
  }
  
  /**
   * Informs the listeners that another project is active now
   * @param projectEvent the event
   */
  protected final void fireActiveProjectChanged(ProjectEvent projectEvent) {
    // Guaranteed to return a non-null array
    Object[] listeners = this.projectListeners.getListenerList();
    // Process the listeners last to first, notifying
    // those that are interested in this event
    for (int i = listeners.length - 2; i >= 0; i -= 2) {
      if (listeners[i] == ProjectListener.class) {
        // Lazily create the event:
        if (projectEvent == null) {
          projectEvent = new ProjectEvent(activeProject);
        }
        ((ProjectListener) listeners[i + 1]).activeProjectChanged(projectEvent);
      }
    }
  }
  
  /**
   * Informs the listeners that a project name has been changed
   * @param projectEvent the event
   */
  protected final void fireProjectNameChanged(ProjectEvent projectEvent) {
    // Guaranteed to return a non-null array
    Object[] listeners = this.projectListeners.getListenerList();
    // Process the listeners last to first, notifying
    // those that are interested in this event
    for (int i = listeners.length - 2; i >= 0; i -= 2) {
      if (listeners[i] == ProjectListener.class) {
        // Lazily create the event:
        if (projectEvent == null) {
          projectEvent = new ProjectEvent(activeProject);
        }
        ((ProjectListener) listeners[i + 1]).projectNameChanged(projectEvent);
      }
    }
  }

}
