/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.core.impl;

import java.util.HashSet;

import fdda.algorithm.Node;
import fdda.core.demonstration.Statistics;
import fdda.model.Connection;

/**
 * Implementation of statistics interface. One instance of this class is used by
 * each Demonstrator.
 * @author Jrg Rdenauer
 */
class StatisticsImpl implements Statistics {
  
  private int sentMessages;
  private int receivedMessages;
  private int startedNodes;
  private HashSet activeNodes;
  private int currentlyActiveNodes;
  private int currentlyPausedNodes;
  private int currentlyActiveMessages;
  private HashSet usedNodes;
  private int presentNodes;
  private HashSet usedConnections;
  private int presentConnections;
  private int nodeFailures;
  private int connectionFailures;
  private int nodeRecoveries;
  private int connectionRecoveries;

  /**
   * Constructor
   */
  public StatisticsImpl() {
    activeNodes = new HashSet();
    usedNodes = new HashSet();
    usedConnections = new HashSet();
    reset();
  }
  
  /**
   * Resets the statistics. All counts are set to 0.
   */
  public synchronized void reset() {
    sentMessages = 0; 
    receivedMessages = 0;
    startedNodes = 0;
    activeNodes.clear();
    currentlyActiveNodes = 0;
    currentlyPausedNodes = 0;
    usedNodes.clear();
    presentNodes = 0;
    usedConnections.clear();
    presentConnections = 0;
    nodeFailures = 0;
    connectionFailures = 0;
    nodeRecoveries = 0;
    connectionRecoveries = 0;
  }

  /* (non-Javadoc)
   * @see fdda.core.demonstration.Statistics#getSentMessages()
   */
  public synchronized int getSentMessages() {
    return sentMessages;
  }
  
  /**
   * Adds 1 to count of sent messages
   */
  public synchronized void addSentMessage() {
    sentMessages++;
  }

  /* (non-Javadoc)
   * @see fdda.core.demonstration.Statistics#getReceivedMessages()
   */
  public synchronized int getReceivedMessages() {
    return receivedMessages;
  }
  
  /**
   * Adds 1 to the count of received messages
   */
  public synchronized void addReceivedMessage() {
    receivedMessages++;
  }

  /* (non-Javadoc)
   * @see fdda.core.demonstration.Statistics#getStartedNodes()
   */
  public synchronized int getStartedNodes() {
    return startedNodes;
  }
  
  /**
   * Adds 1 to the count of started nodes
   */
  public synchronized void addStartedNode() {
    startedNodes++;
  }

  /* (non-Javadoc)
   * @see fdda.core.demonstration.Statistics#getActiveNodes()
   */
  public synchronized int getActiveNodes() {
    return activeNodes.size();
  }
  
  /**
   * Adds an active node. Checks whether the node was already there.
   * @param node the node
   */
  public synchronized void addActiveNode(Node node) {
    activeNodes.add(node);
  }

  /* (non-Javadoc)
   * @see fdda.core.demonstration.Statistics#getCurrentlyActiveNodes()
   */
  public synchronized int getCurrentlyActiveNodes() {
    return currentlyActiveNodes;
  }
  
  /**
   * Sets the number of currently active nodes
   * @param number the number
   */
  public synchronized void setCurrentlyActiveNodes(int number) {
    currentlyActiveNodes = number;
  }

  /* (non-Javadoc)
   * @see fdda.core.demonstration.Statistics#getCurrentlyPausedNodes()
   */
  public synchronized int getCurrentlyPausedNodes() {
    return currentlyPausedNodes;
  }
  
  /**
   * Sets the number of currently paused nodes
   * @param number the number
   */
  public synchronized void setCurrentlyPausedNodes(int number) {
    currentlyPausedNodes = number;
  }

  /* (non-Javadoc)
   * @see fdda.core.demonstration.Statistics#getCurrentlyActiveMessages()
   */
  public synchronized int getCurrentlyActiveMessages() {
    return currentlyActiveMessages;
  }
  
  /**
   * Sets the number of currently active messages
   * @param number the number
   */
  public synchronized void setCurrentlyActiveMessages(int number) {
    currentlyActiveMessages = number;
  }

  /* (non-Javadoc)
   * @see fdda.core.demonstration.Statistics#getUsedNodes()
   */
  public synchronized int getUsedNodes() {
    return usedNodes.size();
  }
  
  /**
   * Adds a node to the used nodes. Checks whether it was already there.
   * @param node the node
   */
  public synchronized void addUsedNode(Node node) {
    usedNodes.add(node);
  }

  /* (non-Javadoc)
   * @see fdda.core.demonstration.Statistics#getPresentNodes()
   */
  public synchronized int getPresentNodes() {
    return presentNodes;
  }
  
  /**
   * Sets the number of present nodes
   * @param number the number
   */
  public synchronized void setPresentNodes(int number) {
    presentNodes = number;
  }

  /* (non-Javadoc)
   * @see fdda.core.demonstration.Statistics#getUsedConnections()
   */
  public synchronized int getUsedConnections() {
    return usedConnections.size();
  }
  
  /**
   * Adds a connection to the used connections. Checks whether it was already 
   * there.
   * @param connection the connection
   */
  public synchronized void addUsedConnection(Connection connection) {
    usedConnections.add(connection);
  }

  /* (non-Javadoc)
   * @see fdda.core.demonstration.Statistics#getPresentConnections()
   */
  public synchronized int getPresentConnections() {
    return presentConnections;
  }
  
  /**
   * Sets the number of present connections
   * @param number the number
   */
  public synchronized void setPresentConnections(int number) {
    presentConnections = number;
  }

  /* (non-Javadoc)
   * @see fdda.core.demonstration.Statistics#getNodeFailures()
   */
  public synchronized int getNodeFailures() {
    return nodeFailures;
  }
  
  /**
   * Adds 1 to the count of node failures
   */
  public synchronized void addNodeFailure() {
    nodeFailures++;
  }
  
  /**
   * Sets the number of node failures
   * @param number the number
   */
  public synchronized void setNodeFailures(int number) {
    nodeFailures = number;
  }

  /* (non-Javadoc)
   * @see fdda.core.demonstration.Statistics#getConnectionFailures()
   */
  public synchronized int getConnectionFailures() {
    return connectionFailures;
  }
  
  /**
   * Adds 1 to the count of connection failures
   */
  public synchronized void addConnectionFailure() {
    connectionFailures++;
  }
  
  /**
   * Sets the number of connection failures
   * @param number the number
   */
  public synchronized void setConnectionFailures(int number) {
    connectionFailures = number;
  }

  /* (non-Javadoc)
   * @see fdda.core.demonstration.Statistics#getNodeRecoveries()
   */
  public synchronized int getNodeRecoveries() {
    return nodeRecoveries;
  }
  
  /**
   * Adds 1 to the count of node recoveries
   */
  public synchronized void addNodeRecovery() {
    nodeRecoveries++;
  }

  /* (non-Javadoc)
   * @see fdda.core.demonstration.Statistics#getConnectionRecoveries()
   */
  public synchronized int getConnectionRecoveries() {
    return connectionRecoveries;
  }
  
  /**
   * Adds 1 to the count of connection recoveries
   */
  public synchronized void addConnectionRecovery() {
    connectionRecoveries++;
  }

}
