/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.core.projects;

import fdda.algorithm.Algorithm;
import fdda.graphics.GraphicalSystem;

import fdda.model.Node;
import fdda.model.Connection;

import java.util.Iterator;

/**
 * A project. A project has (currently) exactly one frame with exactly one model
 * and corresponding graphical system. It has two states: the modelling state, 
 * where nodes and connections may be added or removed etc., and the 
 * demonstration state, where the algorithm is demonstrated.
 * @author Jrg Rdenauer
 */
public interface Project {

  /** modelling state */
  int STATE_MOD = 1;
  
  /** demonstration state */
  int STATE_DEMONSTRATION = 2;

  /**
   * Sets the location where the project is stored, e.g. a file path. Set to 
   * null if the location is not known or the project wasn't stored yet.
   * @param location the location
   */
  void setLocation(String location);
  
  /**
   * Returns the location where the project is stored, e.g. a file path
   * @return the location (or null if it is not known).
   */
  String getLocation();

  /**
   * Returns whether a storage location for the project is known. This is not 
   * the case for projects the user has newly created.
   * @return whether the storage location is known
   */
  boolean isLocationKnown();

  /**
   * Returns the state of the project (modelling or demonstration)
   * @return the state
   */
  int getState();
  
  // setState is not in this interface. It is only used by the ProjectManager
  // void setState(int projectState);

  /**
   * Returns whether options specific to the project are used.
   * @return whether project-specific options are used.
   */
  // boolean areOptionsUsed();

  /**
   * Sets whether options specific to the project are used. 
   * @param used whether project-specific options are used.
   */
  // void setOptionsUsed(boolean used);

  /**
   * Returns the graphical system of the project.
   * @return the graphical system
   */
  GraphicalSystem getGraphicalSystem();

  /**
   * Sets the graphical system of the project.
   * @param system the graphical system
   */
  void setGraphicalSystem(GraphicalSystem system);

  /**
   * Returns the name of the project.
   * @return the name
   */
  String getName();

  // setName is not in this interface. It is only used by the ProjectManager
  //void setName(String name);

  /**
   * Adds a node to the project
   * @param node the node
   */
  void addNode(Node node);

  /**
   * Removes a node from the project
   * @param node the node
   */
  void removeNode(Node node);

  /**
   * Adds a connection to the project
   * @param connection the connection
   */
  void addConnection(Connection connection);

  /**
   * Removes a connection from the project
   * @param connection the connection
   */
  void removeConnection(Connection connection);

  /**
   * Removes all nodes and connections.
   */
  void clear();

  /**
   * Returns an iterator over the nodes in the project.
   * @return iterator over the nodes
   */
  Iterator nodes();

  /**
   * Returns an iterator over the connections in the project.
   * @return iterator over the connections
   */
  Iterator connections();

  /**
   * Returns whether the project was changed since the last save operation.
   * @return whether the project was changed
   */
  boolean wasChanged();

  /**
   * Sets whether the project was changed since the last save operation.
   * @param changed whether the project was changed
   */
  void setChanged(boolean changed);
  
  /**
   * Returns the algorithm that is currently assigned to the project.
   * @return the algorithm
   */
  Algorithm getAlgorithm();
  
  /**
   * Assigns an algorithm to the project.
   * @param algorithm the algorithm
   */
  void setAlgorithm(Algorithm algorithm);

}