/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.core.projects;

import java.util.Iterator;

/**
 * Project manager interface. Only one instance exists. Generates events for 
 * each (important) action; listeners can register for these events according to 
 * the observer pattern. All manipulations concerning whole projects run through 
 * this class.
 * The manager also keeps track of whether a project is currently being 
 * demonstrated. Only one project can be in demonstration mode at each point of 
 * time. 
 * @author Jrg Rdenauer
 */
public interface ProjectManager {

  /**
   * Creates a new project.
   * 
   * @return a new, empty project
   */
  Project newProject();
  
  /**
   * Opens a project from a certain location. The current plugin is used to 
   * perform this operation. The new project is returned.
   * 
   * @param location storage location of the project, e.g. a file path
   * @return the loaded project or null if an error occured
   */
  Project openProject(final String location);
  
  /**
   * Saves a project. The current plugin is used to perform 
   * this operation (and also to determine the storage location).
   * 
   * @param project the project to be saved
   * @param newName whether the project shall be saved in a new location 
   * ('save as')
   */
  void saveProject(Project project, boolean newName);

  /**
   * Closes a project. Checks whether it should be saved and whether a demo must
   * be stopped before closing.
   * 
   * @param project the project to be closed
   * @return true, iff the project was closed
   */
  boolean closeProject(Project project);

  /**
   * Closes all projects.
   * 
   * @return true, iff all projects were closed
   */
  boolean closeAll();
  
  /**
   * Changes the name of a project.
   * 
   * @param project the project
   * @param name the new name
   */
  void changeProjectName(Project project, String name);

  /**
   * Returns the currently active project (the user activates projects by 
   * selecting the frame).
   * 
   * @return the project that is active or null if no project is active
   */
  Project getActiveProject();

  /**
   * Sets the currently active project. This doesn't change the active frame but
   * follows from changing the active frame.
   * 
   * @param project the project that is active now
   */
  void setActiveProject(Project project);

  /**
   * Returns the number of projects that are open.
   * 
   * @return the number of open projects
   */
  int getNrOfProjects();

  /**
   * Returns an iterator over the open projects.
   * 
   * @return Iterator over the open projects
   */
  Iterator projects();

  /**
   * Returns to the modelling state. Since there can only be one project in demo
   * state, no project must be given.
   */
  void setModState();

  /**
   * Returns whether a project is currently being demonstrated.
   * @return whether a project is currently in the demo state
   */
  boolean isInDemoState();

  /**
   * Turns the currently active project into demo state.
   */
  void setDemoStateForActiveProject();

  /**
   * Adds a project listener.
   * 
   * @param listener the listener
   */
  void addProjectListener(ProjectListener listener);

  /**
   * Removes a project listener.
   * 
   * @param listener the listener
   */
  void removeProjectListener(ProjectListener listener);
  
}