/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.dialog;

import fdda.plugins.AlgorithmLoader;
import fdda.plugins.ComponentNotSupportedException;
import fdda.plugins.SystemReader;
import fdda.plugins.SystemWriter;
import fdda.utils.Localization;
import fdda.utils.ObjectCache;
import fdda.utils.Notification;

import java.util.HashMap;
import java.util.Iterator;
import java.util.prefs.Preferences;

/**
 * Controls the option dialog
 * @author Jrg Rdenauer
 */
public class OptionsDialogControl implements DialogControl {
  
  /** the dialog */
  private Dialog optionsDialog;
  
  /** the possible system writer plug-ins and their class names */
  private HashMap writers;
  /** the possible system reader plug-ins and their class names */
  private HashMap readers;
  /** the possible algorithm reader plug-ins and their class names */
  private HashMap algReaders;
  /** error message if wrong data was entered */
  private String errors;
  
  /**
   * Constructor.
   * @param dialog the options dialog
   */
  public OptionsDialogControl(Dialog dialog) {
    optionsDialog = dialog;
    Preferences pluginPrefs = Preferences.userRoot().node("fdda/plugins");
    Options options = new Options();
    String systemWriter = pluginPrefs.get(
      "currentSystemWriter", "fdda.plugins.xmlwriter.ProjectWriter");
    String systemReader =  pluginPrefs.get(
      "currentSystemReader", "fdda.plugins.xmlreader.ProjectReader");
    int readerCount = pluginPrefs.getInt("readerCount", 1);
    readers = new HashMap((int) (readerCount * 1.5));
    for (int i = 0; i < readerCount; i++) {
      String reader = pluginPrefs.get("reader" + i, 
        "fdda.plugins.xmlreader.ProjectReader");
      if (reader != null && !reader.equals("")) {
        try {
          SystemReader readerObject = 
            (SystemReader) ObjectCache.getInstance().getObject(reader);
          readers.put(readerObject.getName(), reader);
          if (reader.equals(systemReader)) {
            options.setSystemReader(readerObject.getName());
          }
        }
        catch (ClassNotFoundException e) {
          Notification.message(Notification.M_ERROR, Notification.S_PROCESSING,
          Localization.formatMessage(Localization.getLocalizedString(getClass(), 
              "plugin_not_found", "Plug-in-Klasse {0} wurde nicht gefunden!"), 
              reader));
        }
        catch (InstantiationException e) {
          Notification.message(Notification.M_ERROR, Notification.S_PROCESSING,
          Localization.formatMessage(Localization.getLocalizedString(getClass(),
             "plugin_instantiation", 
             "Plug-in-Klasse {0} konnte nicht instanziiert werden!\n"
           + "Prfen sie, ob die Klasse einen Konstruktor ohne Parameter hat."), 
            reader));
        }
        catch (IllegalAccessException e) {
          Notification.message(Notification.M_ERROR, Notification.S_PROCESSING,
          Localization.formatMessage(Localization.getLocalizedString(getClass(),
              "plugin_instantiation2", 
              "Plug-in-Klasse {0} konnte nicht instanziiert werden!\n"
            + "Prfen sie, ob der Konstruktor public ist."), reader));
        }
        catch (ClassCastException e) {
          Notification.message(Notification.M_ERROR, Notification.S_PROCESSING,
          Localization.formatMessage(Localization.getLocalizedString(getClass(),
              "plugin_class_reader", 
              "Plug-in-Klasse {0} implementiert nicht das Interface "
            + "fdda.plugins.SystemReader!"), reader));
        }
      }
    }      
    options.setPossibleReaders(readers.keySet());         
    
    String algorithmReader =  pluginPrefs.get(
      "currentAlgorithmReader", "fdda.plugins.xmlreader.AlgorithmReader");
    int algReaderCount = pluginPrefs.getInt("algReaderCount", 1);
    algReaders = new HashMap((int) (algReaderCount * 1.5));
    for (int i = 0; i < algReaderCount; i++) {
      String algReader = pluginPrefs.get("algReader" + i, 
        "fdda.plugins.xmlreader.AlgorithmReader");
      if (algReader != null && !algReader.equals("")) {
        try {
          AlgorithmLoader algReaderObject = 
            (AlgorithmLoader) ObjectCache.getInstance().getObject(algReader);
          algReaders.put(algReaderObject.getName(), algReader);
          if (algReader.equals(algorithmReader)) {
            options.setAlgorithmReader(algReaderObject.getName());
          }
        }
        catch (ClassNotFoundException e) {
          Notification.message(Notification.M_ERROR, Notification.S_PROCESSING,
          Localization.formatMessage(Localization.getLocalizedString(getClass(), 
              "plugin_not_found", "Plug-in-Klasse {0} wurde nicht gefunden!"), 
              algReader));
        }
        catch (InstantiationException e) {
          Notification.message(Notification.M_ERROR, Notification.S_PROCESSING,
          Localization.formatMessage(Localization.getLocalizedString(getClass(),
             "plugin_instantiation", 
             "Plug-in-Klasse {0} konnte nicht instanziiert werden!\n"
           + "Prfen sie, ob die Klasse einen Konstruktor ohne Parameter hat."), 
            algReader));
        }
        catch (IllegalAccessException e) {
          Notification.message(Notification.M_ERROR, Notification.S_PROCESSING,
          Localization.formatMessage(Localization.getLocalizedString(getClass(),
              "plugin_instantiation2", 
              "Plug-in-Klasse {0} konnte nicht instanziiert werden!\n"
            + "Prfen sie, ob der Konstruktor public ist."), algReader));
        }
        catch (ClassCastException e) {
          Notification.message(Notification.M_ERROR, Notification.S_PROCESSING,
          Localization.formatMessage(Localization.getLocalizedString(getClass(),
              "plugin_class_algreader", 
              "Plug-in-Klasse {0} implementiert nicht das Interface "
            + "fdda.plugins.AlgorithmLoader!"), algReader));
        }
      }
    }      
    options.setPossibleAlgorithmReaders(algReaders.keySet());         
    
    int writerCount = pluginPrefs.getInt("writerCount", 1);
    writers = new HashMap((int) (writerCount * 1.5));
    for (int i = 0; i < writerCount; i++) {
      String writer = pluginPrefs.get("writer" + i, 
        "fdda.plugins.xmlwriter.ProjectWriter");
      if (writer != null && !writer.equals("")) {
        try {
          SystemWriter writerObject = 
            (SystemWriter) ObjectCache.getInstance().getObject(writer);
          writers.put(writerObject.getName(), writer);
          if (writer.equals(systemWriter)) {
            options.setSystemWriter(writerObject.getName());
          }
        }
        catch (ClassNotFoundException e) {
          Notification.message(Notification.M_ERROR, Notification.S_PROCESSING,
          Localization.formatMessage(Localization.getLocalizedString(getClass(), 
              "plugin_not_found", "Plug-in-Klasse {0} wurde nicht gefunden!"), 
              writer));
        }
        catch (InstantiationException e) {
          Notification.message(Notification.M_ERROR, Notification.S_PROCESSING,
          Localization.formatMessage(Localization.getLocalizedString(getClass(),
             "plugin_instantiation", 
             "Plug-in-Klasse {0} konnte nicht instanziiert werden!\n"
           + "Prfen sie, ob die Klasse einen Konstruktor ohne Parameter hat."), 
            writer));
        }
        catch (IllegalAccessException e) {
          Notification.message(Notification.M_ERROR, Notification.S_PROCESSING,
          Localization.formatMessage(Localization.getLocalizedString(getClass(),
              "plugin_instantiation2", 
              "Plug-in-Klasse {0} konnte nicht instanziiert werden!\n"
            + "Prfen sie, ob der Konstruktor public ist."), writer));
        }
        catch (ClassCastException e) {
          Notification.message(Notification.M_ERROR, Notification.S_PROCESSING,
          Localization.formatMessage(Localization.getLocalizedString(getClass(),
              "plugin_class_writer", 
              "Plug-in-Klasse {0} implementiert nicht das Interface "
            + "fdda.plugins.SystemWriter!"), writer));
        }
      }
    }               
    options.setPossibleWriters(writers.keySet());
    options.setGridWidth(Preferences.userRoot().node("fdda/graphics/java2d").
      getInt("GridXWidth", 10));
    options.setStartAction(Preferences.userRoot().node("fdda/dialog/swing").
      getInt("startFileAction", 0));
    options.setAskAtClosing(Preferences.userRoot().node("fdda/dialog/swing")
      .getBoolean("askAtClosing", true));
    options.setMinCycleTime("" + Preferences.userRoot().node("fdda/core/impl")
      .getLong("minCycleTime", 30));
    options.setMessageSpeed("" + Preferences.userRoot().node(
      "fdda/graphics/java2d").getDouble("messageSpeed", 3.0));
    options.setBrowserCommand(
      Preferences.userRoot().node("fdda").get("browser", ""));
    optionsDialog.updateObject(options);
  }

  /* (non-Javadoc)
   * @see fdda.dialog.DialogControl#executeCommand(int)
   */
  public void executeCommand(int command) throws NoSuchCommandException {
    Options options = (Options) optionsDialog.getData();
    if (command == OptionsCommands.SHOW_SYSTEM_READER_CONFIG) {
      SystemReader reader = getSystemReader(options.getSystemReader());
      if (reader != null && reader.hasOptions()) {
        reader.configure();
      }
    }
    else if (command == OptionsCommands.SHOW_SYSTEM_WRITER_CONFIG) {
      SystemWriter writer = getSystemWriter(options.getSystemWriter());
      if (writer != null && writer.hasOptions()) {
        writer.configure();
      }
    }
    else if (command == OptionsCommands.SHOW_ALG_READER_CONFIG) {
      AlgorithmLoader loader = getAlgorithmLoader(options.getAlgorithmReader());
      if (loader != null && loader.hasOptions()) {
        loader.configure();
      }
    }
    else if (command == OptionsCommands.CHANGE_SYSTEM_READER) {
      try {
        optionsDialog.disableCommand(OptionsCommands.SHOW_SYSTEM_READER_CONFIG);
      }
      catch (NoSuchCommandException e) {
        assert false;
      }
      String oldReaderClass = Preferences.userRoot().node("fdda/plugins").get(
        "currentProjectReader", "fdda.plugins.xmlreader.ProjectReader");
      Iterator it = readers.keySet().iterator();
      String oldReader = null;
      while (oldReader == null && it.hasNext()) {
        String name = (String) it.next();
        String test = (String) readers.get(name);
        if (test.equals(oldReaderClass)) {
          oldReader = name;
        }
      }
      SystemReader reader = getSystemReader(options.getSystemReader());
      if (reader != null) {
        try {
          reader.useDialog(DialogFactory.getInstance().getDialogComponent());
          reader.useGraphic(DialogFactory.getInstance().getGraphicsComponent());
        }
        catch (ComponentNotSupportedException e) {
          Notification.message(Notification.M_ERROR, Notification.S_PROCESSING,
            Localization.getLocalizedString(getClass(), "plugin_config_error",
            "Plugin in dieser Konfiguration nicht benutzbar!"));
          options.setSystemReader(oldReader);
          optionsDialog.updateObject(options);
        }
      }
      else {
        options.setSystemReader(oldReader);
        optionsDialog.updateObject(options);
      }
      reader = getSystemReader(options.getSystemReader());
      if (reader != null && reader.hasOptions()) {
        try {
         optionsDialog.enableCommand(OptionsCommands.SHOW_SYSTEM_READER_CONFIG);
        }
        catch (NoSuchCommandException e) {
          assert false;
        }
      }
    }
    else if (command == OptionsCommands.CHANGE_ALGORITHM_READER) {
      try {
        optionsDialog.disableCommand(OptionsCommands.SHOW_ALG_READER_CONFIG);
      }
      catch (NoSuchCommandException e) {
        assert false;
      }
      String oldReaderClass = Preferences.userRoot().node("fdda/plugins").get(
        "currentAlgorithmReader", "fdda.plugins.xmlreader.AlgorithmReader");
      Iterator it = algReaders.keySet().iterator();
      String oldReader = null;
      while (oldReader == null && it.hasNext()) {
        String name = (String) it.next();
        String test = (String) algReaders.get(name);
        if (test.equals(oldReaderClass)) {
          oldReader = name;
        }
      }
      AlgorithmLoader loader = getAlgorithmLoader(options.getAlgorithmReader());
      if (loader != null) {
        try {
          loader.useDialog(DialogFactory.getInstance().getDialogComponent());
        }
        catch (ComponentNotSupportedException e) {
          Notification.message(Notification.M_ERROR, Notification.S_PROCESSING,
            Localization.getLocalizedString(getClass(), "plugin_config_error",
            "Plugin in dieser Konfiguration nicht benutzbar!"));
          options.setAlgorithmReader(oldReader);
          optionsDialog.updateObject(options);
        }
      }
      else {
        options.setAlgorithmReader(oldReader);
        optionsDialog.updateObject(options);
      }
      loader = getAlgorithmLoader(options.getAlgorithmReader());
      if (loader != null && loader.hasOptions()) {
        try {
          optionsDialog.enableCommand(OptionsCommands.SHOW_ALG_READER_CONFIG);
        }
        catch (NoSuchCommandException e) {
          assert false;
        }
      }
    }
    else if (command == OptionsCommands.CHANGE_SYSTEM_WRITER) {
      try {
        optionsDialog.disableCommand(OptionsCommands.SHOW_SYSTEM_WRITER_CONFIG);
      }
      catch (NoSuchCommandException e) {
        assert false;
      }
      String oldWriterClass = Preferences.userRoot().node("fdda.plugins").get(
        "currentProjectWriter", "fdda.plugins.xmlwriter.ProjectWriter");
      String oldWriter = null;
      Iterator it = writers.keySet().iterator();
      while (oldWriter == null && it.hasNext()) {
        String name = (String) it.next();
        String test = (String) writers.get(name);
        if (test.equals(oldWriterClass)) {
          oldWriter = name;
        }
      }
      SystemWriter writer = getSystemWriter(options.getSystemWriter());
      if (writer != null) {
        try {
          writer.useDialog(DialogFactory.getInstance().getDialogComponent());
          writer.useGraphic(DialogFactory.getInstance().getGraphicsComponent());
        }
        catch (ComponentNotSupportedException e) {
          Notification.message(Notification.M_ERROR, Notification.S_PROCESSING,
            Localization.getLocalizedString(getClass(), "plugin_config_error",
            "Plugin in dieser Konfiguration nicht benutzbar!"));
          options.setSystemWriter(oldWriter);
          optionsDialog.updateObject(options);
        }
      }
      else {
        options.setSystemWriter(oldWriter);
        optionsDialog.updateObject(options);
      }
      writer = getSystemWriter(options.getSystemWriter());
      if (writer != null && writer.hasOptions()) {
        try {
          optionsDialog.enableCommand(OptionsCommands.SHOW_SYSTEM_WRITER_CONFIG);
        }
        catch (NoSuchCommandException e) {
          assert false;
        }
      }
    }
    else {
      throw new NoSuchCommandException();
    }
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.dialog.DialogControl#canClose(boolean)
   */
  public boolean canClose(boolean closedByOK) {
    if (!closedByOK) return true;
    Options options = (Options) optionsDialog.getData();
    int gridWidth = options.getGridWidth();
    Preferences.userRoot().node("fdda/graphics/java2d").putInt(
      "GridXWidth", gridWidth); 
    int startAction = options.getStartAction();
    Preferences.userRoot().node("fdda/dialog/swing").putInt(
      "startFileAction", startAction);
    Preferences.userRoot().node("fdda/dialog/swing").putBoolean(
      "askAtClosing", options.shallAskAtClosing());
    String systemReader = (String) readers.get(options.getSystemReader());
    Preferences.userRoot().node("fdda/plugins").put(
      "currentSystemReader", systemReader);
    String algReader = (String) algReaders.get(options.getAlgorithmReader());
    Preferences.userRoot().node("fdda/plugins").put(
      "currentAlgorithmReader", algReader);
    String systemWriter = (String) writers.get(options.getSystemWriter());
    Preferences.userRoot().node("fdda/plugins").put(
      "currentSystemWriter", systemWriter);
    errors = "";
    String minCycleTimeString = options.getMinCycleTime();
    long minCycleTime = -1;
    try {
      minCycleTime = Long.parseLong(minCycleTimeString);
      if ((minCycleTime < 5) || (minCycleTime > 1000)) {
        errors += Localization.getLocalizedString(getClass(), "min_cycle_time", 
          "Min. CycleTime should be between 5 and 1000. ");
      }
    }
    catch (NumberFormatException e) {
      errors += Localization.getLocalizedString(getClass(), "cycle_time_format",
         "Min. CycleTime must be a positive integer. ");
    }
    String messageSpeedString = options.getMessageSpeed();
    double messageSpeed = -1.0;
    try {
      messageSpeed = Double.parseDouble(messageSpeedString);
      if ((messageSpeed < 0.2) || (messageSpeed > 25.0)) {
        errors += Localization.getLocalizedString(getClass(), "message_speed", 
         "MessageSpeed should be between 0.2 and 25.0. ");
      }
    }
    catch (NumberFormatException e) {
      errors += Localization.getLocalizedString(getClass(), "speed_format", 
        "MessageSpeed must be a positive real number. ");
    }
    if (errors.length() > 0) {
      Notification.message(Notification.S_PROCESSING, Notification.M_ERROR,
        "Errors", errors);
      return false;
    }
    Preferences.userRoot().node("fdda/core/impl").putLong(
      "minCycleTime", minCycleTime);
    Preferences.userRoot().node("fdda/graphics/java2d").putDouble(
      "messageSpeed", messageSpeed);
    Preferences.userRoot().node("fdda").put("browser", 
      options.getBrowserCommand());
    return true;
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.dialog.DialogControl#getErrors()
   */
  public String getErrors() {
    return errors;
  }
  
  /**
   * Loads a system reader
   * @param readerName the name of the reader
   * @return the loaded reader or null if an error occured
   */
  private SystemReader getSystemReader(String readerName) {
    String className = (String) readers.get(readerName);
    if (className == null || className.equals("")) {
      return null;
    }
    try {
      return (SystemReader) ObjectCache.getInstance().getObject(className);
    }
    catch (ClassNotFoundException e) {
      Notification.message(Notification.M_ERROR, Notification.S_PROCESSING,
      Localization.formatMessage(Localization.getLocalizedString(getClass(), 
          "plugin_not_found", "Plug-in-Klasse {0} wurde nicht gefunden!"), 
          className));
    }
    catch (InstantiationException e) {
      Notification.message(Notification.M_ERROR, Notification.S_PROCESSING,
      Localization.formatMessage(Localization.getLocalizedString(getClass(),
         "plugin_instantiation", 
         "Plug-in-Klasse {0} konnte nicht instanziiert werden!\n"
       + "Prfen sie, ob die Klasse einen Konstruktor ohne Parameter hat."), 
        className));
    }
    catch (IllegalAccessException e) {
      Notification.message(Notification.M_ERROR, Notification.S_PROCESSING,
      Localization.formatMessage(Localization.getLocalizedString(getClass(),
          "plugin_instantiation2", 
          "Plug-in-Klasse {0} konnte nicht instanziiert werden!\n"
        + "Prfen sie, ob der Konstruktor public ist."), className));
    }
    catch (ClassCastException e) {
      Notification.message(Notification.M_ERROR, Notification.S_PROCESSING,
      Localization.formatMessage(Localization.getLocalizedString(getClass(),
          "plugin_class_reader", 
          "Plug-in-Klasse {0} implementiert nicht das Interface "
        + "fdda.plugins.SystemReader!"), className));
    }
    return null;
  }

  /**
   * Loads a system writer
   * @param writerName the name of the writer
   * @return the loaded writer or null if an error occured
   */
  private SystemWriter getSystemWriter(String writerName) {
    String className = (String) writers.get(writerName);
    if (className == null || className.equals("")) {
      return null;
    }
    try {
      return (SystemWriter) ObjectCache.getInstance().getObject(className);
    }
    catch (ClassNotFoundException e) {
      Notification.message(Notification.M_ERROR, Notification.S_PROCESSING,
      Localization.formatMessage(Localization.getLocalizedString(getClass(), 
          "plugin_not_found", "Plug-in-Klasse {0} wurde nicht gefunden!"), 
          className));
    }
    catch (InstantiationException e) {
      Notification.message(Notification.M_ERROR, Notification.S_PROCESSING,
      Localization.formatMessage(Localization.getLocalizedString(getClass(),
         "plugin_instantiation", 
         "Plug-in-Klasse {0} konnte nicht instanziiert werden!\n"
       + "Prfen sie, ob die Klasse einen Konstruktor ohne Parameter hat."), 
        className));
    }
    catch (IllegalAccessException e) {
      Notification.message(Notification.M_ERROR, Notification.S_PROCESSING,
      Localization.formatMessage(Localization.getLocalizedString(getClass(),
          "plugin_instantiation2", 
          "Plug-in-Klasse {0} konnte nicht instanziiert werden!\n"
        + "Prfen sie, ob der Konstruktor public ist."), className));
    }
    catch (ClassCastException e) {
      Notification.message(Notification.M_ERROR, Notification.S_PROCESSING,
      Localization.formatMessage(Localization.getLocalizedString(getClass(),
          "plugin_class_writer", 
          "Plug-in-Klasse {0} implementiert nicht das Interface "
        + "fdda.plugins.SystemWriter!"), className));
    }
    return null;
  }

  /**
   * Loads an algorithm loader
   * @param readerName the name of the loader
   * @return the loader or null if an error occured
   */
  private AlgorithmLoader getAlgorithmLoader(String readerName) {
    String className = (String) algReaders.get(readerName);
    if (className == null || className.equals("")) {
      return null;
    }
    try {
      return (AlgorithmLoader) ObjectCache.getInstance().getObject(className);
    }
    catch (ClassNotFoundException e) {
      Notification.message(Notification.M_ERROR, Notification.S_PROCESSING,
      Localization.formatMessage(Localization.getLocalizedString(getClass(), 
          "plugin_not_found", "Plug-in-Klasse {0} wurde nicht gefunden!"), 
          className));
    }
    catch (InstantiationException e) {
      Notification.message(Notification.M_ERROR, Notification.S_PROCESSING,
      Localization.formatMessage(Localization.getLocalizedString(getClass(),
         "plugin_instantiation", 
         "Plug-in-Klasse {0} konnte nicht instanziiert werden!\n"
       + "Prfen sie, ob die Klasse einen Konstruktor ohne Parameter hat."), 
        className));
    }
    catch (IllegalAccessException e) {
      Notification.message(Notification.M_ERROR, Notification.S_PROCESSING,
      Localization.formatMessage(Localization.getLocalizedString(getClass(),
          "plugin_instantiation2", 
          "Plug-in-Klasse {0} konnte nicht instanziiert werden!\n"
        + "Prfen sie, ob der Konstruktor public ist."), className));
    }
    catch (ClassCastException e) {
      Notification.message(Notification.M_ERROR, Notification.S_PROCESSING,
      Localization.formatMessage(Localization.getLocalizedString(getClass(),
          "plugin_class_algreader", 
          "Plug-in-Klasse {0} implementiert nicht das Interface "
        + "fdda.plugins.AlgorithmLoader!"), className));
    }
    return null;
  }
}
