/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.dialog.java2d;

import fdda.graphics.Component;
import fdda.graphics.ComponentFactory;
import fdda.graphics.Message;
import fdda.graphics.Node;
import fdda.graphics.Line;
import fdda.graphics.Container;

import fdda.graphics.Selection;
import fdda.graphics.SelectionModifier;
import fdda.graphics.java2d.Point;
import fdda.graphics.java2d.LineSegment;
import fdda.graphics.java2d.Java2DComponentVisitor;

import java.awt.geom.Area;

import java.util.Iterator;

/**
 * Deletes a component (which may be Container, e.g. the selection).
 * @author Jrg Rdenauer
 */
public class Deleter extends AbstractManipulator
    implements Java2DComponentVisitor, SelectionModifier {

  /** the component that will be deleted */
  private Component component;

  /**
   * Constructor
   * @param graphicHolder the graphic holder
   */
  public Deleter(GraphicHolder graphicHolder) {
    super(graphicHolder);
    component = null;
  }

  /*
   *  (non-Javadoc)
   * @see fdda.dialog.java2d.Manipulator#start(fdda.graphics.Component)
   */
  public void start(Component component) throws NoSuitableComponentException {
    if (component == null) {
      throw new NoSuitableComponentException("Keine Komponente bergeben!");
    }
    this.component = component;
  }

  /*
   *  (non-Javadoc)
   * @see fdda.dialog.java2d.Manipulator#start(double, double)
   */
  public void start(double x, double y) throws NoSuitableComponentException {
    component = graphicHolder.getGraphicalSystem().getElementAt(x, y);
    if (component == null) {
      throw new NoSuitableComponentException("Keine Komponente an den "
        + "Koordinaten x=" + x + ", y=" + y + "!");
    }
  }

  /*
   *  (non-Javadoc)
   * @see fdda.dialog.java2d.Manipulator#execute(double, double)
   */
  public void execute(double x, double y) throws NoSuitableComponentException {
    // nothing
  }

  /*
   *  (non-Javadoc)
   * @see fdda.dialog.java2d.Manipulator#finish(double, double)
   */
  public void finish(double x, double y) throws NoSuitableComponentException {
    if (component == null) {
      throw new NoSuitableComponentException("Keine Komponente festgelegt!");
    }
    // use visitor pattern to delete. The vistXX returns the area to repaint
    Area clippingArea = (Area) component.visit(this);
    Selection selection 
      = ComponentFactory.getInstance().getComponentSelection();
    clippingArea.add(new Area(selection.getShape()));
    selection.clear(this);
    graphicHolder.getGraphicalSystem().setChanged(true);
    // repaint
    painter.setClippingShape(clippingArea);
    painter.repaintSystem();
    painter.updateScreen();
  }

  /*
   *  (non-Javadoc)
   * @see fdda.graphics.ComponentVisitor#visitComponent(fdda.graphics.Component)
   */
  public Object visitComponent(Component component) {
    return component.visit(this);
  }

  /*
   *  (non-Javadoc)
   * @see fdda.graphics.java2d.Java2DComponentVisitor#visitLineSegment(fdda.graphics.java2d.LineSegment)
   */
  public Object visitLineSegment(LineSegment lineSegment) {
    return new Area(); // line segments are deleted with their line autom.
  }

  /*
   *  (non-Javadoc)
   * @see fdda.graphics.ComponentVisitor#visitContainer(fdda.graphics.Container)
   */
  public Object visitContainer(Container container) {
    if (container instanceof fdda.graphics.java2d.Line) {
      return visitLine((Line) container);
    }
    Area returnedArea = new Area();
    Iterator it = container.getIterator();
    while (it.hasNext()) {
      returnedArea.add((Area) ((Component) it.next()).visit(this));
    }
    return returnedArea;
  }

  /*
   *  (non-Javadoc)
   * @see fdda.graphics.java2d.Java2DComponentVisitor#visitPoint(fdda.graphics.java2d.Point)
   */
  public Object visitPoint(Point point) {
    Area returnedArea = new Area(point.getShape());
    fdda.graphics.java2d.Line line = point.getLine();
    // inform the line that its point was deleted
    returnedArea.add(new Area(line.getShape()));
    line.removePoint(point);
    returnedArea.add(new Area(line.getShape()));
    return returnedArea;
  }

  /*
   *  (non-Javadoc)
   * @see fdda.graphics.ComponentVisitor#visitLine(fdda.graphics.Line)
   */
  public Object visitLine(Line line) {
    Area returnedArea = new Area(line.getShape());
    fdda.graphics.java2d.Line realLine = (fdda.graphics.java2d.Line) line;
    // since all points will be removed, avoid line.removePoint
    Iterator it = realLine.getIterator();
    while (it.hasNext()) {
      Component component = (Component) it.next();
      if (component instanceof Point) {
        returnedArea.add(new Area(component.getShape()));
      }
    }
    fdda.core.Core.getModelManager().removeConnection(
      graphicHolder.getGraphicalSystem().getProject(), line.getConnection());
    graphicHolder.getGraphicalSystem().removeLine(line);
    return returnedArea;
  }

  /*
   *  (non-Javadoc)
   * @see fdda.graphics.ComponentVisitor#visitNode(fdda.graphics.Node)
   */
  public Object visitNode(Node node) {
    Area returnedArea = new Area(node.getShape());
    // remove all lines (connections) of the node
    Object[] connections = node.getModelNode().getConnections().toArray();
    for (int i = 0; i < connections.length; i++) {
      returnedArea.add((Area) visitLine(((fdda.model.Connection) connections[i])
          .getGraphicalRepresentation()));
    }
    fdda.core.Core.getModelManager().removeNode(
      graphicHolder.getGraphicalSystem().getProject(), node.getModelNode());
    graphicHolder.getGraphicalSystem().removeNode(node);
    return returnedArea;
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.graphics.ComponentVisitor#visitMessage(fdda.graphics.Message)
   */
  public Object visitMessage(Message message) {
    // message aren't deleted by the user!
    assert false;
    return null;
  }

}