/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.dialog.java2d;

import fdda.graphics.Selection;
import fdda.graphics.SelectionModifier;
import fdda.graphics.Component;
import fdda.graphics.ComponentFactory;
import fdda.graphics.Line;

import fdda.graphics.java2d.ComponentSelection;
import fdda.graphics.java2d.Painter;

import fdda.utils.Logging;
import fdda.utils.Notification;

import fdda.dialog.MessageControl;
import fdda.dialog.swing.MainFrame;
import fdda.dialog.swing.SwingGraphicsControl;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.Shape;

import java.awt.geom.Area;
import java.awt.geom.AffineTransform;
import java.awt.geom.NoninvertibleTransformException;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;

import javax.swing.JScrollPane;
import javax.swing.Scrollable;
import javax.swing.JPanel;
import javax.swing.SwingConstants;
import javax.swing.JInternalFrame;

import java.awt.event.ComponentEvent;
import java.awt.event.ComponentAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseMotionAdapter;
import java.awt.event.InputEvent;

import java.util.Iterator;
import java.util.prefs.Preferences;

/**
 * Controls the graphics of the java2d component. That is, it performs the 
 * actions requested by the user through menus or mouse manipulation, partly by
 * delegating them to a manipulator.
 * 
 * Also controls the automatic painting of the graphical system and its zoom.
 * @author Jrg Rdenauer
 */
public class Java2DControl 
    implements SwingGraphicsControl, GraphicHolder, SelectionModifier {

  /** the color grey */
  public final static Color GREY = new Color(211, 201, 177);

  /** scale factor for one zoom out operation. Has to be smaller than 1.0 */
  public final static double ZOOM_OUT_SCALE = 0.8;

  /** panel on which the system is drawn */
  private ZoomablePanel pnlDrawingArea;

  /** scrollpane that contains the graphics*/
  private JScrollPane speDrawingArea;

  /** current background color */
  private Color bgColor = Color.WHITE;

  /** last size of the system */
  private Dimension oldSystemSize = new Dimension(1, 1);

  /** the graphics */
  private java.awt.Graphics2D graphic = null;

  /** the graphical system */
  private fdda.graphics.java2d.GraphicalSystem graphicalSystem = null;

  /** manipulator for an action */
  private Manipulator manipulator = null;
  
  /** action that the user has selected */
  private int selectedAction;

  /** editing mode */
  private int frameMode = S_UNLOCKED;

  /**
   * Constructor.
   */
  public Java2DControl() {
  }
  
  /**
   * Attaches the control instance to a frame. Registers several listeners 
   * with the panel to react to user actions. The panel is added to the frame.
   */
  public void attachToFrame(JInternalFrame frame, final MainFrame mainFrame) {
    // resize-handler
    frame.addComponentListener(new ComponentAdapter() {
      public void componentResized(ComponentEvent e) {
        Point2D p = null;
        double minWidth = Java2DControl.this.speDrawingArea
          .getViewportBorderBounds().getWidth();
        double minHeight = Java2DControl.this
          .speDrawingArea.getViewportBorderBounds().getHeight();
        if (graphicalSystem != null) {
          Rectangle2D size = graphicalSystem.getEnclosingRectangle();
          // calculate with zoom
          p = new Point2D.Double(size.getMaxX(), size.getMaxY());
          p = pnlDrawingArea.getZoomTransform().transform(p, null);
        }
        else {
          p = new Point2D.Double(minWidth, minHeight);
        }
        int preferredWidth = (int) (Math.max(p.getX(), minWidth));
        int preferredHeight = (int) (Math.max(p.getY(), minHeight));
        oldSystemSize = new Dimension(preferredWidth, preferredHeight);
        pnlDrawingArea.setPreferredSize(oldSystemSize);
        pnlDrawingArea.revalidate();
      }
    });

    pnlDrawingArea = new ZoomablePanel();
    pnlDrawingArea.setAutoscrolls(true);
    pnlDrawingArea.addMouseMotionListener(new MouseMotionAdapter() {
      public void mouseDragged(MouseEvent e) {
         Rectangle r = new Rectangle(e.getX(), e.getY(), 1, 1);
         ((JPanel) e.getSource()).scrollRectToVisible(r);
       }
    });

    speDrawingArea = new JScrollPane(pnlDrawingArea);
    frame.getContentPane().add(speDrawingArea);

    // catch mouse event for graphical operations
    this.pnlDrawingArea.addMouseListener(new MouseAdapter() {
      
      // Released mouse button: perhaps finish an operation
      public void mouseReleased(MouseEvent e) {
        Point2D p = null;
        try {
          p =  pnlDrawingArea.getZoomTransform().createInverse().transform(
            new Point2D.Double(e.getX(), e.getY()), null);
        }
        catch (NoninvertibleTransformException ex) {
          Logging.exceptionCaught(ex);
          Notification.message(Notification.M_ERROR, Notification.S_PROCESSING,
            "Interner Fehler",
            "NoninvertibleTransformException in ControlFrame$mouseReleased!");
        }
        if (manipulator != null) {
          try {
            manipulator.finish(p.getX(), p.getY());
            mainFrame.tryToolSwitch(); // revert to pointer
          }
          catch (NoSuitableComponentException e1) {
            // perhaps the line wasn't finished properly
            if ((e1.getReason() == LineCreator.LINE_IS_LOOP)
                || (e1.getReason() == LineCreator.NODES_ALREADY_CONNECTED)) {
              Notification.message(Notification.M_ERROR, Notification.S_MOD,
                             e1.getMessage());
            }
            else {
              // ignore, typical user error
            }
          }
          manipulator = null;
        }
      }
      
      // mouse clicked: selection change, add nodes or points 
      public void mouseClicked(MouseEvent e) {
        Point2D p = null;
        try {
          p =  pnlDrawingArea.getZoomTransform().createInverse().transform(
            new Point2D.Double(e.getX(), e.getY()), null);
        }
        catch (NoninvertibleTransformException ex) {
          Logging.exceptionCaught(ex);
          Notification.message(Notification.M_ERROR, Notification.S_PROCESSING,
            "Interner Fehler",
            "NoninvertibleTransformException in ControlFrame$mouseClicked!");
        }
        // look whether the selection changes
        boolean selectionChanged = false;
        Selection selection = 
          ComponentFactory.getInstance().getComponentSelection();
        Shape oldSelectionShape = selection.getShape();
        Component clickedAt =
          graphicalSystem.getElementAt(p.getX(), p.getY());
        switch (selectedAction) {
          case POINTER:
            // right mouse button is ignored
            if (e.getButton() == MouseEvent.BUTTON1) {
              if (clickedAt != null) {
                // double click on a line: add a point
                if (clickedAt instanceof Line &&
                    (e.getClickCount() > 1 || e.isShiftDown())
                    && frameMode == S_UNLOCKED) {
                  manipulator =
                    new PointCreator((GraphicHolder) Java2DControl.this);
                  try {
                    manipulator.start(clickedAt);
                    manipulator.execute(p.getX(), p.getY());
                    manipulator.finish(p.getX(), p.getY());
                  }
                  catch (NoSuitableComponentException e1) {
                    Logging.exceptionCaught(e1);
                    Notification.message(Notification.M_ERROR,
                      Notification.S_PROCESSING,
                      "Interner Fehler",
                     "NoSuitableComponentException von PointCreator geworfen!");
                  }
                  manipulator = null;
                }
                else {
                  selectionChanged = select(e);
                }
              } // != null
              else if (!e.isControlDown()) {
                // no component and not ctrl => clear selection
                selection.clear(Java2DControl.this);
                selectionChanged = true;
              }
            }
            break; // pointer
          case NODE:
            if (frameMode == S_UNLOCKED) {
              // add a node
              try {
                manipulator = new NodeCreator(Java2DControl.this);
                manipulator.start(p.getX(), p.getY());
                manipulator.execute(p.getX(), p.getY());
                manipulator.finish(p.getX(), p.getY());
                // selection change is made by the NodeCreator
                mainFrame.tryToolSwitch();
              }
              catch (NoSuitableComponentException e1) {
                Logging.exceptionCaught(e1);
                Notification.message(Notification.M_ERROR,
                  Notification.S_PROCESSING,
                  "Interner Fehler",
                  "NoSuitableComponentException von NodeCreator!");
              }
              manipulator = null;
            }
            break;
          case DELETE:
            if (clickedAt != null && frameMode == S_UNLOCKED) {
              // delete the selected component
              manipulator = new Deleter(Java2DControl.this);
              try {
                manipulator.start(clickedAt);
                manipulator.execute(p.getX(), p.getY());
                manipulator.finish(p.getX(), p.getY());
                mainFrame.tryToolSwitch();
              }
              catch (NoSuitableComponentException e1) {
                Logging.exceptionCaught(e1);
                Notification.message(Notification.M_ERROR,  
                  Notification.S_PROCESSING, "Interner Fehler", 
                  "NoSuitableComponentException von Deleter!");
              }
              manipulator = null;
            }
          break;
          default: 
        } // Button1
        if (selectionChanged) {
          // repaint
          Shape newSelectionShape = selection.getShape();
          Area clip = new Area(newSelectionShape);
          clip.add(new Area(oldSelectionShape));
          Painter painter = new Painter(Java2DControl.this);
          painter.setClippingShape(clip);
          painter.repaintSystem();
          painter.updateScreen();
        }
      } // method
    });

    this.pnlDrawingArea.addMouseMotionListener(new MouseMotionAdapter() {

      // dragged: move, select, add point or draw line
      public void mouseDragged(MouseEvent e) {
        final int SAFETY_OFFSET = 4; // close to the picture border
        Point2D p = null;
        try {
          p = pnlDrawingArea.getZoomTransform().createInverse().transform(
            new Point2D.Double(e.getX(), e.getY()), null);
        }
        catch (NoninvertibleTransformException ex) {
          Logging.exceptionCaught(ex);
          Notification.message(Notification.M_ERROR, Notification.S_PROCESSING,
            "Interner Fehler",
            "NoninvertibleTransformException in ControlFrame$mouseDragged!");
        }
        // Point not in the system area? Then ignore
        if (p.getX() < SAFETY_OFFSET || p.getY() < SAFETY_OFFSET) {
          return;
        }
        // no manipulator yet => perhaps create it
        if (manipulator == null) switch (selectedAction) {
          case POINTER: // move, select, point
            Component pressedAt
              = graphicalSystem.getElementAt(p.getX(), p.getY());
            if (pressedAt != null && !e.isPopupTrigger()
                && (e.getModifiers() & InputEvent.BUTTON1_MASK) != 0) {
              // on a line with shift pressed -> add point
              if (pressedAt instanceof Line && e.isShiftDown()
                       && frameMode == S_UNLOCKED) {
                try {
                  manipulator = new PointCreator(Java2DControl.this);
                  manipulator.start(pressedAt);
                  manipulator.execute(p.getX(), p.getY());
                  manipulator.finish(p.getX(), p.getY());
                  // select the point
                  Component newPoint
                    = getGraphicalSystem().getElementAt(p.getX(), p.getY());
                  ComponentFactory.getInstance().getComponentSelection()
                    .replaceWith(newPoint, Java2DControl.this);
                  // Start moving the point (dragging op.!)
                  manipulator = new Mover(Java2DControl.this);
                  manipulator.start(p.getX(), p.getY());
                }
                catch (NoSuitableComponentException ex) {
                  Logging.exceptionCaught(ex);
                  Notification.message(Notification.M_ERROR,
                    Notification.S_PROCESSING,
                    "Interner Fehler",
                    "NoSuitableComponentException von Mover.start()!");
                }
              }
              else { // no line or shift is not pressed
                // check if moving is possible
                Selection selection = 
                    ComponentFactory.getInstance().getComponentSelection();
                Shape oldShape = selection.getShape();
                boolean newSelection = false;
                Iterator it = selection.getSelectionIterator();
                boolean canMove = false;
                // Mouse over selection?
                while (it.hasNext() && !canMove) {
                  Component component
                    = (Component)it.next();
                  if (component.getShape().contains(p.getX(), p.getY())) {
                    canMove = true;
                  }
                }
                // if not, select the component the mouse is over
                if (!canMove) {
                  newSelection = select(e);
                  canMove = newSelection;
                }
                canMove = canMove && ((ComponentSelection)
                  ComponentFactory.getInstance().getComponentSelection()).
                    isMovable();
                canMove = canMove && frameMode == S_UNLOCKED;
                if (canMove) {
                  manipulator = new Mover(Java2DControl.this);
                  // Mover with coordinates uses current selection
                  try {
                    manipulator.start(p.getX(), p.getY());
                  }
                  catch (NoSuitableComponentException e2) {
                    Logging.exceptionCaught(e2);
                    Notification.message(Notification.M_ERROR,
                      Notification.S_PROCESSING,
                      "Interner Fehler",
                      "NoSuitableComponentException von Mover.start()!");
                  } // try
                } // canMove
                if (newSelection) {
                  // Redraw selection
                  Area clip =
                    new Area(ComponentFactory.getInstance().
                      getComponentSelection().getShape());
                  clip.add(new Area(oldShape));
                  Painter painter = new Painter(Java2DControl.this);
                  painter.setClippingShape(clip);
                  painter.repaintSystem();
                  painter.updateScreen();
                }
              } // no shift on line
            }  // over a component
            else if (pressedAt == null &&
                    ((e.getModifiers() & InputEvent.BUTTON1_MASK) != 0)) {
              // start selecting
              manipulator = new Selector(Java2DControl.this);
              try {
                manipulator.start(p.getX(), p.getY());
              }
              catch (NoSuitableComponentException e2) {
                Logging.exceptionCaught(e2);
                Notification.message(Notification.M_ERROR,
                  Notification.S_PROCESSING, "Interner Fehler",
                  "NoSuitableComponentException von Selector.start() geworfen");
              }
            }
            break; // pointer
          case LINE: // draw a line
          case DIRECTED_LINE:
            boolean directed = (selectedAction == DIRECTED_LINE);
            pressedAt = graphicalSystem.getElementAt(p.getX(), p.getY());
            if (pressedAt instanceof fdda.graphics.Node) {
              try {
                manipulator = new LineCreator(Java2DControl.this, directed);
                manipulator.start(pressedAt);
              }
              catch (NoSuitableComponentException e2) {
                Logging.exceptionCaught(e2);
                Notification.message(Notification.M_ERROR,
                  Notification.S_PROCESSING, "Interner Fehler",
                  "NoSuitableComponentException von LineCreator geworfen");
              }
            }
            break;
          default: 
        } // not started already
        if (manipulator != null) {
          try {
            manipulator.execute(p.getX(), p.getY());
          }
          catch (NoSuitableComponentException e1) {
            Logging.exceptionCaught(e1);
            Notification.message(Notification.M_ERROR, 
              Notification.S_PROCESSING, "Interner Fehler",
              "NoSuitableComponentException von Manipulator.execute()!");
          }
        }
      } // method
    }); // end of anonymous class

  }

  /*
   *  (non-Javadoc)
   * @see fdda.dialog.java2d.GraphicHolder#getGraphicalSystem()
   */
  public fdda.graphics.java2d.GraphicalSystem getGraphicalSystem() {
    return graphicalSystem;
  }

  /*
   *  (non-Javadoc)
   * @see fdda.dialog.GraphicsControl#setGraphicalSystem(fdda.graphics.GraphicalSystem)
   */
  public void setGraphicalSystem(fdda.graphics.GraphicalSystem system) {
    this.graphicalSystem = (fdda.graphics.java2d.GraphicalSystem) system;
    graphicalSystem.setGraphicHolder(this);
  }

  /*
   *  (non-Javadoc)
   * @see fdda.dialog.java2d.GraphicHolder#redraw(java.awt.Rectangle, fdda.graphics.java2d.Painter)
   */
  public void redraw(Rectangle clippingRect, Painter painter) {
    Rectangle2D size = graphicalSystem.getEnclosingRectangle();
    // calculate zoom transformation
    Point2D p = new Point2D.Double(size.getMaxX(), size.getMaxY());
    p = pnlDrawingArea.getZoomTransform().transform(p, null);
    if (p.getX() != oldSystemSize.getWidth() ||
        p.getY() != oldSystemSize.getHeight()) {
      int preferredWidth = (int) (Math.max(p.getX(),
        this.speDrawingArea.getViewportBorderBounds().getWidth()));
      int preferredHeight = (int) (Math.max(p.getY(),
        this.speDrawingArea.getViewportBorderBounds().getHeight()));
      oldSystemSize = new Dimension(preferredWidth, preferredHeight);
      pnlDrawingArea.setPreferredSize(oldSystemSize);
      pnlDrawingArea.revalidate();
    }
    pnlDrawingArea.setPainter(painter);
    // also transform the clipping area
    p = new Point2D.Double(clippingRect.getX(), clippingRect.getY());
    Point2D p2 =
      new Point2D.Double(clippingRect.getWidth(), clippingRect.getHeight());
    p = pnlDrawingArea.getZoomTransform().transform(p, null);
    p2 = pnlDrawingArea.getZoomTransform().transform(p2, null);
    pnlDrawingArea.repaint((int)Math.floor(p.getX()), (int)Math.floor(p.getY()),
      (int)Math.ceil(p2.getX()), (int)Math.ceil(p2.getY()));
  }

  /*
   *  (non-Javadoc)
   * @see fdda.dialog.java2d.GraphicHolder#stringWidth(java.lang.String)
   */
  public double stringWidth(String s) {
    if (graphic != null) {
      // transform because of zoom
      AffineTransform oldTransform = graphic.getTransform();
      AffineTransform currentTransform = pnlDrawingArea.getZoomTransform();
      graphic.setTransform(currentTransform);
      double width = graphic.getFontMetrics().stringWidth(s);
      graphic.setTransform(oldTransform);
      return width;
    }
    else {
      return 0;
    }
  }


  /*
   *  (non-Javadoc)
   * @see fdda.dialog.java2d.GraphicHolder#stringHeight()
   */
  public double stringHeight() {
    if (graphic != null) {
      AffineTransform oldTransform = graphic.getTransform();
      AffineTransform currentTransform = pnlDrawingArea.getZoomTransform();
      graphic.setTransform(currentTransform);
      double height = graphic.getFontMetrics().getHeight();
      graphic.setTransform(oldTransform);
      return height;
    }
    else {
      return 0;
    }
  }

  /*
   *  (non-Javadoc)
   * @see fdda.dialog.GraphicsControl#zoomIn()
   */
  public void zoomIn() {
    pnlDrawingArea.zoom(1 / ZOOM_OUT_SCALE);
    Rectangle2D size = graphicalSystem.getEnclosingRectangle();
    // zoom calculation for new panel size
    Point2D p = new Point2D.Double(size.getMaxX(), size.getMaxY());
    p = pnlDrawingArea.getZoomTransform().transform(p, null);
    if (p.getX() != oldSystemSize.getWidth()
        || p.getY() != oldSystemSize.getHeight()) {
      int preferredWidth = (int) (Math.max(p.getX(),
        this.speDrawingArea.getViewportBorderBounds().getWidth()));
      int preferredHeight = (int) (Math.max(p.getY(),
        this.speDrawingArea.getViewportBorderBounds().getHeight()));
      oldSystemSize = new Dimension(preferredWidth, preferredHeight);
      pnlDrawingArea.setPreferredSize(oldSystemSize);
      pnlDrawingArea.revalidate();
    }
    Painter painter = new Painter(this);
    painter.setClippingShape(getGraphicalSystem().getEnclosingRectangle());
    painter.repaintSystem();
    painter.updateScreen();
  }

  /*
   *  (non-Javadoc)
   * @see fdda.dialog.GraphicsControl#zoomOut()
   */
  public void zoomOut() {
    pnlDrawingArea.zoom(ZOOM_OUT_SCALE);
    Rectangle2D size = graphicalSystem.getEnclosingRectangle();
    // zoom calculation for new panel size
    Point2D p = new Point2D.Double(size.getMaxX(), size.getMaxY());
    p = pnlDrawingArea.getZoomTransform().transform(p, null);
    if (p.getX() != oldSystemSize.getWidth()
        || p.getY() != oldSystemSize.getHeight()) {
      int preferredWidth = (int) (Math.max(p.getX(),
        this.speDrawingArea.getViewportBorderBounds().getWidth()));
      int preferredHeight = (int) (Math.max(p.getY(),
        this.speDrawingArea.getViewportBorderBounds().getHeight()));
      oldSystemSize = new Dimension(preferredWidth, preferredHeight);
      pnlDrawingArea.setPreferredSize(oldSystemSize);
      pnlDrawingArea.revalidate();
    }
    Painter painter = new Painter(this);
    painter.setClippingShape(getGraphicalSystem().getEnclosingRectangle());
    painter.repaintSystem();
    painter.updateScreen();
  }

  /*
   *  (non-Javadoc)
   * @see fdda.dialog.GraphicsControl#selectMovables()
   */
  public void selectMovables() {
    if (graphicalSystem != null) {
      Component movables
        = graphicalSystem.getMovableComponents();
      Selection selection = 
        ComponentFactory.getInstance().getComponentSelection();
      Area clippingArea = new Area(selection.getShape());
      if (movables != null) {
        selection.replaceWith(movables, this);
        clippingArea.add(new Area(selection.getShape()));
      }
      else {
        selection.clear(this);
      }
      Painter painter = new Painter(this);
      painter.setClippingShape(clippingArea);
      painter.repaintSystem();
      painter.updateScreen();
    }
  }

  /*
   *  (non-Javadoc)
   * @see fdda.dialog.GraphicsControl#deleteSelection()
   */   
  public void deleteSelection() {
    if (graphicalSystem == null) return;
    ComponentSelection selection = (ComponentSelection) 
      ComponentFactory.getInstance().getComponentSelection();
    if (!selection.getSelectionIterator().hasNext()) return;
    manipulator = new Deleter(this);
    try {
      manipulator.start(selection);
      manipulator.execute(0.0, 0.0);
      manipulator.finish(0.0, 0.0);
    }
    catch (NoSuitableComponentException e1) {
      Logging.exceptionCaught(e1);
      Notification.message(Notification.M_ERROR,  Notification.S_PROCESSING,
        "Interner Fehler", "NoSuitableComponentException von Deleter!");
    }
    manipulator = null;
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.dialog.GraphicsControl#paint(java.awt.Graphics)
   */
  public void paint(Graphics g) {
    if (graphic == null) {
      graphic = (Graphics2D) g;
      this.getGraphicalSystem().recalculateShapes();
    }
    else {
      graphic = (Graphics2D) g;
    }
    Rectangle2D size = graphicalSystem.getEnclosingRectangle();
    // calculate zoom transformation
    Point2D p = new Point2D.Double(size.getMaxX(), size.getMaxY());
    p = pnlDrawingArea.getZoomTransform().transform(p, null);
    if (p.getX() != oldSystemSize.getWidth()
        || p.getY() != oldSystemSize.getHeight()) {
      int preferredWidth = (int) (Math.max(p.getX(),
        this.speDrawingArea.getViewportBorderBounds().getWidth()));
      int preferredHeight = (int) (Math.max(p.getY(),
        this.speDrawingArea.getViewportBorderBounds().getHeight()));
      oldSystemSize = new Dimension(preferredWidth, preferredHeight);
      pnlDrawingArea.setPreferredSize(oldSystemSize);
      pnlDrawingArea.revalidate();
    }
  }

  /*
   *  (non-Javadoc)
   * @see fdda.dialog.GraphicsControl#paintOnImage()
   */   
  public BufferedImage paintOnImage() {
    // get size of image
    Rectangle2D rect = graphicalSystem.getEnclosingRectangle();
    // create the image
    final int BORDER = 10;
    BufferedImage bi = new BufferedImage((int) rect.getMaxX() + BORDER,
                                         (int) rect.getMaxY() + BORDER,
                                         BufferedImage.TYPE_INT_RGB);
    // get graphics object
    Graphics2D biGraphic = bi.createGraphics();
    // turn off grid
    boolean oldGrid = Preferences.userRoot().node(
      "fdda/graphics").getBoolean("showGrid", false);
    Preferences.userRoot().node(
      "fdda/graphics").putBoolean("showGrid", false);
    // paint the system
    biGraphic.setColor(bgColor);
    biGraphic.setRenderingHint(RenderingHints.KEY_RENDERING,
                               RenderingHints.VALUE_RENDER_SPEED);
    biGraphic.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                               RenderingHints.VALUE_ANTIALIAS_ON);
    biGraphic.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING,
                               RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
    Painter myPainter = new Painter(this);
    myPainter.setBGColor(bgColor);
    myPainter.setClippingShape(new Rectangle(0,0,(int)rect.getMaxX() + BORDER,
                                                 (int)rect.getMaxY() + BORDER));
    myPainter.repaintSystem(biGraphic);
    // restore grid
    Preferences.userRoot().node(
      "fdda/graphics").putBoolean("showGrid", oldGrid);
    return bi;
  }     

  /*
   *  (non-Javadoc)
   * @see fdda.dialog.GraphicsControl#setEditingMode(int)
   */
  public void setEditingMode(int mode) {
    if (this.frameMode != mode) {
      this.frameMode = mode;
      if (mode == S_LOCKED) {
        bgColor = GREY;
      }
      else {
        bgColor = Color.WHITE;
      }
    }
  }

  /*
   *  (non-Javadoc)
   * @see fdda.dialog.GraphicsControl#getEditingMode()
   */
  public int getEditingMode() {
    return frameMode;
  }

  /*
   *  (non-Javadoc)
   * @see fdda.dialog.GraphicsControl#setSelectedAction(int)
   */
  public void setSelectedAction(int action) {
    selectedAction = action;
  }

  /*
   *  (non-Javadoc)
   * @see fdda.dialog.GraphicsControl#getSelectedAction()
   */
  public int getSelectedAction() {
    return selectedAction;
  }

  /*
   *  (non-Javadoc)
   * @see fdda.dialog.GraphicsControl#getMessageControl()
   */   
  public MessageControl getMessageControl() {
    return new Java2DMessageControl(this);
  }

  /**
   * Updates the selection after a click
   * @param e the mouse event
   * @return whether the selection has changed
   */   
  private boolean select(MouseEvent e) {
    Point2D p = null;
    try {
      p =  pnlDrawingArea.getZoomTransform().createInverse().transform(
        new Point2D.Double(e.getX(), e.getY()), null);
    }
    catch (NoninvertibleTransformException ex) {
      Logging.exceptionCaught(ex);
      Notification.message(Notification.M_ERROR,
        Notification.S_PROCESSING,
        "Interner Fehler",
        "NonInvertibleTransformException in ControlFrame.select()!");
      return false;
    }
    // find component
    if (graphicalSystem == null) {
      Logging.getPackageLogger(getClass()).fine("Unexpected null value of " +
        "graphicalSystem in Java2DControl.select");
      return false;
    }
    Component clickedAt =
      graphicalSystem.getElementAt(p.getX(), p.getY());
    Selection selection = 
      ComponentFactory.getInstance().getComponentSelection();
    boolean selectionChanged = false;
    // double click or ctrl not pressed => new selection
    if (e.getClickCount() > 1 || !(e.isControlDown())) {
      selection.replaceWith(clickedAt, this);
      selectionChanged = true;
    }
    // else it's a single click with ctrl
    else {
      // if already selected, deselect
      if (selection.hasElement(clickedAt)) {
        selection.removeElement(clickedAt, this);
        selectionChanged = true;
      }
      else {
        // only add if component allows it and old selection allows it
        Iterator it = selection.getSelectionIterator();
        Component trial = null;
        if (it.hasNext()) {
          trial = (Component)it.next();
        }
        if ((trial == null) || ((
           (trial.selectableWith()
                  == Component.SEL_ALL)
           || (trial.selectableWith()
                     == Component.SEL_EQUALS
               && trial.getClass().equals(clickedAt.getClass()))
           || (trial.selectableWith()
                     == Component.SEL_MOVABLES
               && clickedAt.isMovable())
           ) &&
            // check contrary
           ((clickedAt.selectableWith()
                       == Component.SEL_ALL)
            || (clickedAt.selectableWith()
                         == Component.SEL_EQUALS
                && clickedAt.getClass().equals(trial.getClass()))
            || (clickedAt.selectableWith()
                          == Component.SEL_MOVABLES
                && trial.isMovable())
           ))) {
          selection.addElement(clickedAt, this);
          selectionChanged = true;
        }
      } 
    } 
    return selectionChanged;
  }


   /** A panel that can be zoomed using an AffineTransform */
   class ZoomablePanel extends JPanel implements Scrollable {

     /** the scroll increment */
     private final static int SCROLL_INCREMENT = 10;

     /** current zoom factor */
     private double scaleFactor = 1.0;

     /** the painter */
     private Painter myPainter = null;

     /** constructor */
     public ZoomablePanel() {
     }

     /**
      * Sets the painter used to paint the system
      * @param painter the painter
      */
     public void setPainter(Painter painter) {
       myPainter = painter;
     }

     /**
      * Returns the currently used painter
      * @return the painter
      */
     public Painter getPainter() {
       return myPainter;
     }

     /**
      * Zooms by a certain factor.
      * @param factor the factor
      */
     public void zoom(double factor) {
       scaleFactor *= factor;
       this.setSize((int) Math.floor(this.getWidth() * factor),
                    (int) Math.floor(this.getHeight() * factor));
     }

     /**
      * Returns the current affine transformation used to zoom
      * @return the zoom transformation
      */
     public AffineTransform getZoomTransform() {
       AffineTransform result = new AffineTransform();
       result.scale(scaleFactor, scaleFactor);
       return result;
     }

     /*
      *  (non-Javadoc)
      * Overriden to set transformation and paint system
      * @see javax.swing.JComponent#paintComponent(java.awt.Graphics)
      */
     public void paintComponent(Graphics g) {
       super.paintComponent(g);
       graphic = (Graphics2D) g;
       graphic.setColor(bgColor);
       // set rendering hints
       graphic.setRenderingHint(RenderingHints.KEY_RENDERING,
                                RenderingHints.VALUE_RENDER_SPEED);
       graphic.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                                RenderingHints.VALUE_ANTIALIAS_ON);
       graphic.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING,
                                RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
       // set transformation
       AffineTransform oldTransform = graphic.getTransform();
       AffineTransform newTransform = new AffineTransform(oldTransform);
       newTransform.scale(scaleFactor, scaleFactor);
       graphic.setTransform(newTransform);
       if (myPainter == null) {
         myPainter = new Painter(Java2DControl.this);
         myPainter.setClippingShape(graphic.getClip());
       }
       myPainter.setClippingShape(g.getClip());
       myPainter.setBGColor(bgColor);
       myPainter.repaintSystem(graphic);
       myPainter = null;
       graphic.setTransform(oldTransform);
     }
     
     // Methods of Scrollable

     /*
      *  (non-Javadoc)
      * @see javax.swing.Scrollable#getPreferredScrollableViewportSize()
      */
     public Dimension getPreferredScrollableViewportSize() {
       return this.getPreferredSize();
     }

     /*
      *  (non-Javadoc)
      * @see javax.swing.Scrollable#getScrollableUnitIncrement(java.awt.Rectangle, int, int)
      */
     public int getScrollableUnitIncrement(Rectangle visibleRect,
                                           int orientation, int direction) {
       return SCROLL_INCREMENT;
     }

     /*
      *  (non-Javadoc)
      * @see javax.swing.Scrollable#getScrollableTracksViewportHeight()
      */
     public boolean getScrollableTracksViewportHeight() {
       return false;
     }

     /*
      *  (non-Javadoc)
      * @see javax.swing.Scrollable#getScrollableTracksViewportWidth()
      */
     public boolean getScrollableTracksViewportWidth() {
       return false;
     }

     /*
      *  (non-Javadoc)
      * @see javax.swing.Scrollable#getScrollableBlockIncrement(java.awt.Rectangle, int, int)
      */
     public int getScrollableBlockIncrement(Rectangle visibleRect,
                                            int orientation, int direction) {
       int returnValue = 1;
       if (orientation == SwingConstants.VERTICAL) {
         returnValue = (int) (visibleRect.getHeight() / 2);
       }
       else {
         returnValue = (int) (visibleRect.getWidth() / 2);
       }
       return returnValue;
     }

   }

}