/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.dialog.java2d;

import fdda.graphics.*;
import fdda.model.Connection;
import fdda.utils.Localization;

import java.awt.Shape;
import java.awt.geom.Area;

import java.util.Iterator;

/**
 * Adds lines between nodes. The starting component must be a node.
 * @author Jrg Rdenauer
 */
public class LineCreator extends AbstractManipulator 
    implements SelectionModifier {
      
  /** Reason for Exception: line not ended above node */
  public final static int NO_END_NODE = 1;
  
  /** Reason for Exception: nodes already connected */
  public final static int NODES_ALREADY_CONNECTED = 2;
  
  /** Reason for Exception: loop */
  public final static int LINE_IS_LOOP = 3;

  /** shall the line be directed? */
  private boolean directed;
  
  /** the node where the new line starts */
  private Node node;

  /** the new line */
  private Line line;

  /** the last x coordinate */
  private double lastX;

  /** the last y coordinate */
  private double lastY;

  /**
   * Constructor
   * @param graphicHolder the graphic holder
   * @param directedLine whether the line shall be directed
   */
  public LineCreator(GraphicHolder graphicHolder, boolean directedLine) {
    super(graphicHolder);
    this.node = null;
    this.line = null;
    this.directed = directedLine;
  }

  /*
   *  (non-Javadoc)
   * @see fdda.dialog.java2d.Manipulator#start(fdda.graphics.Component)
   */
  public void start(Component component) throws NoSuitableComponentException {
    if (component == null || !(component instanceof Node)) {
      throw new NoSuitableComponentException("Komponente ist kein Knoten!");
    }
    node = (Node) component;
    this.lastX = node.getLineStartX();
    this.lastY = node.getLineStartY();
    // create a 'loose' line without end node
    line = ((fdda.graphics.java2d.Java2dFactory) ComponentFactory.getInstance())
         .createLooseLine(node, directed);
    line.setGraphicalSystem(graphicHolder.getGraphicalSystem());
    graphicHolder.getGraphicalSystem().addLine(line);
  }

  /*
   *  (non-Javadoc)
   * @see fdda.dialog.java2d.Manipulator#start(double, double)
   */
  public void start(double x, double y) throws NoSuitableComponentException {
    Component component = graphicHolder.getGraphicalSystem().getElementAt(x, y);
    if (component == null || !(component instanceof Node)) {
      throw new NoSuitableComponentException("Kein Knoten an den Koordinaten x="
        + x + ", y=" + y + "!");
    }
    this.start(component);
  }

  /*
   *  (non-Javadoc)
   * @see fdda.dialog.java2d.Manipulator#execute(double, double)
   */
  public void execute(double x, double y) throws NoSuitableComponentException {
    if (line == null) {
      throw new NoSuitableComponentException("Noch keine Linie kreiert!");
    }
    // remove old line
    Shape firstClippingArea = line.getShape();
    // change end point
    line.setEndXY(x, y);
    lastX = x;
    lastY = y;
    // repaint
    Shape secondClippingArea = line.getShape();
    Area clippingArea = new Area(firstClippingArea);
    clippingArea.add((new Area(secondClippingArea)));
    painter.setClippingShape(clippingArea);
    painter.repaintSystem();
    painter.updateScreen();
  }

  /*
   *  (non-Javadoc)
   * @see fdda.dialog.java2d.Manipulator#finish(double, double)
   */
  public void finish(double x, double y) throws NoSuitableComponentException {
    if (line == null || node == null) {
      throw new NoSuitableComponentException("Noch keine Linie kreiert!");
    }
    GraphicalSystem graphSystem = graphicHolder.getGraphicalSystem();
    // look for node at the end coordinates
    Component compAtCo = graphSystem.getElementAt(x, y);
    Node endNode = null;
    try {
      if (compAtCo == null || !(compAtCo instanceof Node)) {
        throw new NoSuitableComponentException("Kein Knoten an Koordinaten x="
          + x + ", y=" + y + "!", NO_END_NODE);
      }
      endNode = (Node) compAtCo; // Knoten am Ende der Linie
      if (endNode == node) {
        throw new NoSuitableComponentException(Localization.getLocalizedString(
          getClass(), "node_loop_error", "Ein Knoten kann nicht mit "
          + "sich selbst verbunden werden."), LINE_IS_LOOP);
      }
      if (node.getModelNode().isConnectedTo(endNode.getModelNode())) {
        throw new NoSuitableComponentException(Localization.getLocalizedString(
          getClass(), "double_connection_error", "Zwischen den beiden Knoten "
          + "besteht bereits eine Verbindung."), NODES_ALREADY_CONNECTED);
      }
    }
    catch (NoSuitableComponentException e) {
      // catch to remove the old line
      graphSystem.removeLine(line);
      painter.setClippingShape(line.getShape());
      painter.repaintSystem();
      painter.updateScreen();
      throw(e);
    }
    Area clippingArea = new Area(line.getShape()); // for later union
    line.setEndXY(endNode.getLineStartX(), endNode.getLineStartY());
    if (endNode.getModelNode().isConnectedTo(node.getModelNode())) {
      // there's already a directed connection between the nodes, but in the
      // other direction. Instead of adding a second connection, make the old
      // connection undirected
      graphSystem.removeLine(line);
      Iterator oldConnections =
          endNode.getModelNode().getConnections().iterator();
      while (oldConnections.hasNext()) {
        Connection oldConnection = (Connection) oldConnections.next();
        if (oldConnection.getEndNode() == node.getModelNode()) {
          assert oldConnection.isDirected();
          oldConnection.setDirected(false);
          painter.setClippingShape(
              oldConnection.getGraphicalRepresentation().getShape());
          painter.repaintSystem();
          painter.updateScreen();
        }
      }
    }
    else {
      fdda.core.Core.getModelManager().addConnection(
        graphicHolder.getGraphicalSystem().getProject(), node.getModelNode(),
          endNode.getModelNode(), line, directed);
      clippingArea.add((new Area(line.getShape())));
      // also repaint the nodes, they may be changed
      clippingArea.add(new Area(node.getShape()));
      clippingArea.add(new Area(endNode.getShape()));
      // select the line
      Selection selection = 
        ComponentFactory.getInstance().getComponentSelection();
      clippingArea.add(new Area(selection.getShape()));
      selection.replaceWith(line, this);
      graphSystem.setChanged(true);
      painter.setClippingShape(clippingArea);
      painter.repaintSystem();
      painter.updateScreen();
    }
  }

}
