/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.dialog.java2d;

import fdda.graphics.Component;

/**
 * Interface for the manipulators that use and change graphical components.
 * Command pattern is used here. Usually, a manipulators methods are called in 
 * the order start, execute, finish, though in rare cases execute may be 
 * skipped. These messages can take coordinates, probably the current mouse
 * pointer position (e.g. while dragging).
 * @author Jrg Rdenauer
 */
public interface Manipulator {

  /**
   * Starts the action, using the specified component
   * @param component the component
   * @throws NoSuitableComponentException if the component can't be used for the
   * action of the manipulator
   */
  void start(Component component) throws NoSuitableComponentException;
  
  /**
   * Starts the action. The component is specified by its coordinates.
   * @param x the x coordinate
   * @param y the y coordinate 
   * @throws NoSuitableComponentException if no component that could be used for
   * the action of the manipulator is at that point
   */
  void start(double x, double y) throws NoSuitableComponentException;

  /**
   * Executes the action.
   * @param x the new x coordinate (may not be important)
   * @param y the new y coordinate (may not be important)
   * @throws NoSuitableComponentException if no suitable component was selected
   * first by calling start
   */
  void execute(double x, double y) throws NoSuitableComponentException;

  /**
   * Finishes the action, making the changes permanent.
   * @param x the new x coordinate (may not be important)
   * @param y the new y coordinate (may not be important)
   * @throws NoSuitableComponentException if no suitable component was selected
   * first by calling start
   */
  void finish(double x, double y) throws NoSuitableComponentException;

  /**
   * Sets the object holding the graphics
   * @param graphicHolder the graphics holder
   */
  void setGraphicHolder(GraphicHolder graphicHolder);

  /** 
   * Returns the graphics holder
   * @return the graphics holder
   */
  GraphicHolder getGraphicHolder();

}
