/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.dialog.java2d;

import fdda.graphics.Component;
import fdda.graphics.ComponentFactory;
import fdda.graphics.java2d.*;

import fdda.model.Connection;

import java.util.Iterator;

import java.awt.Shape;
import java.awt.geom.Area;
import java.awt.geom.Rectangle2D;

/**
 * Moves a component (may be a Container, e.g. the selection)
 * @author Jrg Rdenauer
 */
public class Mover extends AbstractManipulator
    implements Java2DComponentVisitor {
  
  /** the component to move. */
  private Component component;

  /** the last x coordinate */
  private double lastx;

  /** the last y coordinate */
  private double lasty;

  /** difference for x coordinate */
  private double xdiff;
  /** difference for y coordinate */
  private double ydiff;

  /**
   * Constructor
   * @param graphicHolder the graphicHolder
   */
  public Mover(GraphicHolder graphicHolder) {
    super(graphicHolder);
    this.component = null;
  }

  /*
   *  (non-Javadoc)
   * @see fdda.dialog.java2d.Manipulator#start(fdda.graphics.Component)
   */
  public void start(Component component) throws NoSuitableComponentException {
    if (component == null) {
      throw new NoSuitableComponentException("Null bergeben!");
    }
    this.component = component;
    this.lastx = component.getX();
    this.lasty = component.getY();
  }

  /**
   * The current selection is used as the component to move.
   * @see fdda.dialog.java2d.Manipulator#start(double, double)
   */
  public void start(double x, double y) throws NoSuitableComponentException {
    component = (ComponentSelection)
      ComponentFactory.getInstance().getComponentSelection();
    if (((ComponentSelection) component).getSize() == 0) {
      throw new NoSuitableComponentException("Keine Komponente selektiert!");
    }
    this.lastx = x;
    this.lasty = y;
  }

  /**
   * Moves the components.
   * @see fdda.dialog.java2d.Manipulator#execute(double, double)
   */
  public void execute(double x, double y) throws NoSuitableComponentException {
    // check that components are not moved out of the picture
    Rectangle2D componentRect = component.getShape().getBounds2D();
    xdiff = x - lastx;
    ydiff = y - lasty;
    if (componentRect.getX() + xdiff < 0.0) {
      xdiff = 0.0;
    }
    if (componentRect.getY() + ydiff < 0.0) {
      ydiff = 0.0;
    }
    // move - using visitor pattern. Each visitXX returns the shape to redraw
    Shape clippingShape = (Shape) component.visit(this);
    lastx = x;
    lasty = y;
    ((ComponentSelection) 
     ComponentFactory.getInstance().getComponentSelection()).recalculateShape();
    // repaint
    painter.setClippingShape(clippingShape);
    painter.repaintSystem();
    painter.updateScreen();
  }

  /*
   *  (non-Javadoc)
   * @see fdda.dialog.java2d.Manipulator#finish(double, double)
   */
  public void finish(double x, double y) throws NoSuitableComponentException {
    double cx = component.getX();
    double newx = moveToGrid(cx, true);
    double cy = component.getY();
    double newy = moveToGrid(cy, false);
    // Moving is easiest by calling execute
    this.execute(newx - cx + lastx, newy - cy + lasty);
    graphicHolder.getGraphicalSystem().setChanged(true);
  }

  /*
   *  (non-Javadoc)
   * @see fdda.graphics.ComponentVisitor#visitLine(fdda.graphics.Line)
   */
  public Object visitLine(fdda.graphics.Line line) {
    // don't move lines
    return new Area();
  }

  /*
   *  (non-Javadoc)
   * @see fdda.graphics.java2d.Java2DComponentVisitor#visitLineSegment(fdda.graphics.java2d.LineSegment)
   */
  public Object visitLineSegment(LineSegment segment) {
    return new Area();
  }

  /*
   *  (non-Javadoc)
   * @see fdda.graphics.ComponentVisitor#visitComponent(fdda.graphics.Component)
   */
  public Object visitComponent(Component component) {
    if (component.isMovable()) return component.visit(this);
    else return new Area();
  }

  /*
   *  (non-Javadoc)
   * @see fdda.graphics.ComponentVisitor#visitNode(fdda.graphics.Node)
   */
  public Object visitNode(fdda.graphics.Node node) {
    Area returnedArea = new Area(node.getShape());
    node.move(xdiff, ydiff);
    // move the starting/ending points of lines at this node
    Iterator it = node.getModelNode().getConnections().iterator();
    while (it.hasNext()) {
      Connection connection = (Connection) it.next();
      Line line = (Line) connection.getGraphicalRepresentation();
      returnedArea.add(new Area(line.getShape()));
      if (connection.getStartNode() == node.getModelNode()) {
        line.moveStartPoint(xdiff, ydiff);
      }
      else {
        line.moveEndPoint(xdiff, ydiff);
      }
      returnedArea.add(new Area(line.getShape()));
    }
    returnedArea.add(new Area(node.getShape()));
    return returnedArea;
  }

  /*
   *  (non-Javadoc)
   * @see fdda.graphics.java2d.Java2DComponentVisitor#visitPoint(fdda.graphics.java2d.Point)
   */
  public Object visitPoint(Point point) {
    Area returnedArea = new Area(point.getShape());
    point.move(xdiff, ydiff);
    // inform the line that its point got moved
    Line line = point.getLine();
    returnedArea.add(new Area(line.getShape()));
    line.pointMoved(point);
    returnedArea.add(new Area(line.getShape()));
    returnedArea.add(new Area(point.getShape()));
    return returnedArea;
  }

  /*
   *  (non-Javadoc)
   * @see fdda.graphics.ComponentVisitor#visitContainer(fdda.graphics.Container)
   */
  public Object visitContainer(fdda.graphics.Container container) {
    Area returnedArea = new Area();
    Iterator it = container.getIterator();
    while (it.hasNext()) {
      returnedArea.add((Area) ((Component) it.next()).visit(this));
    }
    return returnedArea;
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.graphics.ComponentVisitor#visitMessage(fdda.graphics.Message)
   */
  public Object visitMessage(fdda.graphics.Message message) {
    // messages aren't moved by the user!
    assert false;
    return null;
  }

}