/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.dialog.java2d;

import fdda.graphics.*;
import fdda.graphics.java2d.Painter;


import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.Shape;
import java.awt.Stroke;

import java.awt.geom.Area;
import java.awt.geom.Rectangle2D;

/**
 * Selects movable components in a rectangle dragged open by the user.
 * @author Jrg Rdenauer
 */
public class Selector extends AbstractManipulator 
    implements SelectionModifier {

  /** starting point x coordinate */
  private double startx;

  /** starting point y coordinate */
  private double starty;

  /** last x coordinate of end point */
  private double lastx;

  /** last y coordinate of end point*/
  private double lasty;

  /**
   * Constructor
   * @param graphicHolder the graphics holder
   */
  public Selector(GraphicHolder graphicHolder) {
    super(graphicHolder);
    startx = starty = lastx = lasty = 0.0;
  }

  /**
   * Calculates the starting point from the component.
   * @see fdda.dialog.java2d.Manipulator#start(fdda.graphics.Component)
   */
  public void start(Component component) throws NoSuitableComponentException {
    start(component.getX(), component.getY());
  }

  /*
   *  (non-Javadoc)
   * @see fdda.dialog.java2d.Manipulator#start(double, double)
   */
  public void start(double x, double y) throws NoSuitableComponentException {
    startx = lastx = x;
    starty = lasty = y;
  }

  /*
   *  (non-Javadoc)
   * @see fdda.dialog.java2d.Manipulator#execute(double, double)
   */
  public void execute(double x, double y) throws NoSuitableComponentException {
    drawRect(startx, starty, lastx, lasty, false);
    changeSelection(x, y);
    drawRect(startx, starty, x, y, true);
    lastx = x;
    lasty = y;
  }

  /*
   *  (non-Javadoc)
   * @see fdda.dialog.java2d.Manipulator#finish(double, double)
   */
  public void finish(double x, double y) throws NoSuitableComponentException {
    drawRect(startx, starty, lastx, lasty, false);
    changeSelection(x, y);
  }

  /**
   * Changes the selection using the current end point of the rectangle.
   * @param x the x coordinate of the end point
   * @param y the y coordinate of the end point
   */
  private void changeSelection(double x, double y) {
   Selection selection =
        ComponentFactory.getInstance().getComponentSelection();
    Shape oldSelection = selection.getShape();
    GraphicalSystem system = graphicHolder.getGraphicalSystem();
    Rectangle2D rect = createRectangle(startx, starty, x, y);
    Component selectedComponents =
        system.getMovableComponentsInArea(new Area(rect));
    if (selectedComponents != null) {
      selection.replaceWith(selectedComponents, this);
    }
    else {
      selection.clear(this);
    }
    Area clippingArea = new Area(selection.getShape());
    clippingArea.add(new Area(oldSelection));
    painter.setClippingShape(clippingArea);
    painter.repaintSystem();
    painter.updateScreen();
  }

  /**
   * Draws or removes the rectangle.
   * @param x1 x coordinate of first point
   * @param y1 y coordinate of first point
   * @param x2 x coordinate of second point
   * @param y2 y coordinate of second point
   * @param draw whehter the rectangle shall be drawn or removed
   */
  private void drawRect(double x1, double y1, double x2, double y2,
                        final boolean draw) {
    final Rectangle2D rect = createRectangle(x1, y1, x2, y2);
    Painter painter = new Painter(graphicHolder) {

      // Override draw-Method to add the rectangle
      public void drawGraphicalSystem(Graphics2D graphics) {

        super.drawGraphicalSystem(graphics);

        if (draw) {
          // save old drawing properties
          Shape oldClippingShape = graphics.getClip();
          graphics.setClip(rect);
          Color oldColor = graphics.getColor();
          Stroke oldStroke = graphics.getStroke();
          Object oldAntialias =
            graphics.getRenderingHint(RenderingHints.KEY_ANTIALIASING);
          Object oldStrokeControl =
            graphics.getRenderingHint(RenderingHints.KEY_STROKE_CONTROL);

          // set new drawing properties
          // set width to more than 1.0, else upper and lower line are missing
          graphics.setStroke(new BasicStroke(1.05f));
          graphics.setColor(Color.LIGHT_GRAY);
          graphics.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                                    RenderingHints.VALUE_ANTIALIAS_OFF);
          // STROKE_PURE, else upper left is thicker than lower right
          graphics.setRenderingHint(RenderingHints.KEY_STROKE_CONTROL,
                                    RenderingHints.VALUE_STROKE_PURE);

          // draw
          graphics.draw(rect);

          // restore old properties
          graphics.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                                    oldAntialias);
          graphics.setRenderingHint(RenderingHints.KEY_STROKE_CONTROL,
                                    oldStrokeControl);
          graphics.setStroke(oldStroke);
          graphics.setColor(oldColor);
          graphics.setClip(oldClippingShape);
        }
      }
    };
    painter.setClippingShape(rect);
    painter.repaintSystem();
    painter.updateScreen();
  }
  
  /**
   * Cretates a rectangle out of two points
   * @param x1 x coordinate of first point
   * @param y1 y coordinate of first point
   * @param x2 x coordinate of second point
   * @param y2 y coordinate of second point
   * @return new rectangle
   */
  private Rectangle2D createRectangle(double x1, double y1,
                                      double x2, double y2) {
    // leftmost coordinate
    double leftx = x1 < x2 ? x1 : x2;
    // rightmost
    double rightx = x1 < x2 ? x2 : x1;
    // topmost
    double topy = y1 < y2 ? y1 : y2;
    // bottommost
    double bottomy = y1 < y2 ? y2 : y1;
    return new Rectangle2D.Double(leftx, topy, rightx - leftx, bottomy - topy);
  }

}
