/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.dialog.swing;

import fdda.dialog.GraphicsControl;
import fdda.dialog.swing.utils.SimpleFileFilter;

import fdda.core.Core;
import fdda.core.projects.*;

import fdda.utils.Localization;
import fdda.utils.Logging;
import fdda.utils.Notification;

import java.awt.Graphics;
import java.awt.image.BufferedImage;

import javax.swing.JFileChooser;
import javax.swing.JInternalFrame;

import java.io.*;

import com.sun.image.codec.jpeg.JPEGCodec;
import com.sun.image.codec.jpeg.JPEGImageEncoder;

/**
 * A frame containing a graphical system. Delegates core graphical operations
 * to the graphics control.
 * @author Jrg Rdenauer
 */
public class FrameControlProxy extends SystemFrame {

  /** the graphics control */
  private GraphicsControl graphicsControl;

  /**
   * Constructor.
   */
  public FrameControlProxy(String title, GraphicsControl graphicsControl) {
    super(title);
    Core.getProjectManager().addProjectListener(new ProjectAdapter() {
      
      public void projectStateChanged(ProjectEvent e) {
        if (e.getSource() == getProject()) {
          FrameControlProxy.this.graphicsControl.setEditingMode(
            Core.getProjectManager().isInDemoState()
             ? GraphicsControl.S_LOCKED : GraphicsControl.S_UNLOCKED);
        }
      }    
    });
    this.setDefaultCloseOperation(JInternalFrame.DO_NOTHING_ON_CLOSE);
    this.graphicsControl = graphicsControl;
  }

  /**
   * Sets the graphical system. Overriden to inform the graphics control.
   */
  public void setGraphicalSystem(fdda.graphics.GraphicalSystem system) {
    super.setGraphicalSystem(system);
    graphicsControl.setGraphicalSystem(system);
  }

  /*
   *  (non-Javadoc)
   * @see fdda.dialog.swing.SystemFrame#frameClosing()
   */
  protected void frameClosing() {
    Core.getProjectManager().closeProject(graphicalSystem.getProject());
  }

  /**
   * Zooms in.
   */
  public void zoomIn() {
    graphicsControl.zoomIn();
  }

  /**
   * Zooms out.
   */
  public void zoomOut() {
    graphicsControl.zoomOut();
  }

  /**
   * Selects all movable components.
   */
  public void selectMovables() {
    graphicsControl.selectMovables();
  }
  
  /**
   * Deletes the selected components.
   */
  public void deleteSelection() {
    graphicsControl.deleteSelection();
  }

  /*
   *  (non-Javadoc)
   * @see java.awt.Component#paint(java.awt.Graphics)
   */
  public void paint(Graphics g) {
    graphicsControl.paint(g);
    super.paint(g);
  }

  /**
   * Returns the frame mode.
   * @return the frame mode
   */
  public int getFrameMode() {
    return graphicsControl.getEditingMode();
  }

  /**
   * Sets the frame mode. In locked mode, most graphical operations are not
   * allowed.
   * @param frameMode the new mode
   */
  public void setFrameMode(int frameMode) {
    if (frameMode != graphicsControl.getEditingMode()) {
      graphicsControl.setEditingMode(frameMode);
      repaint();
    }
  }

  /**
   * Exports the graphics into a jpeg image file.
   */
  public void saveJPEG() {
    File file = null;
    JFileChooser fileChooser = new JFileChooser();
    fileChooser.setFileFilter(new SimpleFileFilter("jpeg"));
    fileChooser.setDialogType(JFileChooser.SAVE_DIALOG);
    int retVal = 0;
    int retVal2 = 0;
    do {
      retVal2 = 0;
      retVal = fileChooser.showSaveDialog(this);
      if (retVal == JFileChooser.APPROVE_OPTION) {
        file = fileChooser.getSelectedFile();
        if (file != null) {
          if (!file.getName().endsWith(".jpeg")) {
            file = new File(file.getPath() + ".jpeg");
          }
          if (file.exists()) {
            retVal2 = Notification.message(Notification.M_QUESTION,
              Notification.S_SYSTEM, Localization.formatMessage(
              Localization.getLocalizedString(getClass(), "overwrite_file",
              "File {0} does exist. Overwrite it?"), file.getName()), new 
              String[] {Localization.getLocalizedString(getClass(), "yes", 
                "Yes"), Localization.getLocalizedString(getClass(), "no", 
                "No"), Localization.getLocalizedString(getClass(), "cancel", 
                "Cancel")});
          }
        }
      }
    } while (retVal2 == 1);

    if (retVal == JFileChooser.APPROVE_OPTION && retVal2 == 0) {
      
      BufferedImage bi = graphicsControl.paintOnImage();
      
      // Output Stream to encode
      FileOutputStream outputS = null;
      try {
        outputS = new FileOutputStream(file);
        // encode the image and write it to the file
        // JPEGImageEncoder is a special sun class always available
        JPEGImageEncoder enc = JPEGCodec.createJPEGEncoder(outputS);
        enc.encode(bi);
        outputS.flush();
      } 
      catch (java.io.FileNotFoundException e) {
        Logging.exceptionCaught(e);
        Notification.message(Notification.M_ERROR, Notification.S_SYSTEM,
          Localization.formatMessage(Localization.getLocalizedString(getClass(),
        "file_save_error", "Error occured while saving: {0}"), e.getMessage()));
      } 
      catch (IOException e) {
        Logging.exceptionCaught(e);
        Notification.message(Notification.M_ERROR, Notification.S_SYSTEM,
          Localization.formatMessage(Localization.getLocalizedString(getClass(),
        "file_save_error", "Error occured while saving: {0}"), e.getMessage()));
      }
      finally {
        try {
          if (outputS != null) outputS.close();
        }
        catch (IOException e) { }
      }
    }
  }


  /**
   * Sets the action selected by the user
   * @param action the action
   */
  public void setSelectedAction(int action) {
    graphicsControl.setSelectedAction(action);
  }
  
  /**
   * Sets the graphics control
   * @return the graphics control
   */
  public GraphicsControl getGraphicsControl() {
    return graphicsControl;
  }

}