/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.dialog.swing;

import javax.swing.*;
import javax.swing.JFrame;

import fdda.dialog.*;
import fdda.dialog.swing.properties.ObjectInspector;
import fdda.dialog.swing.utils.SizeableToolbar;

import fdda.graphics.*;

import fdda.core.Core;
import fdda.core.demonstration.Demonstration;
import fdda.core.demonstration.Statistics;
import fdda.core.projects.*;

import fdda.algorithm.WrongStateException;

import fdda.utils.Localization;
import fdda.utils.Logging;
import fdda.utils.Notification;

import java.util.ResourceBundle;
import java.util.prefs.Preferences;

import java.awt.*;
import java.awt.event.*;
import java.awt.geom.Rectangle2D;

import java.io.*;

import java.beans.*;

import javax.swing.event.InternalFrameAdapter;
import javax.swing.event.InternalFrameEvent;

/**
 * This is the main window of fdda. It controls functions, presents menu bar,
 * button bar and property editor.
 * @author Jrg Rdenauer
 */
public class MainFrame extends JFrame implements SelectionModifier {

  /** action at start: do nothing */
  public static final int NOTHING = 0;
  /** action at start: load the last used project */
  public static final int LAST_PROJECT = 1;
  /** action at start: open a new project */
  public static final int NEW_PROJECT = 2;

  /** the currently selected tool */
  private int selectedTool = GraphicsControl.POINTER;

  /** the menubar */
  private MenuBar mbrMenuBar;

  /** the button bar */
  private ButtonBar btrButtonBar;

  /** container for the button bar */
  private JPanel pnlToolBars;

  /** toolbar for object inspector */
  private SizeableToolbar tbrProperties; 

  /** MDI area */
  private JDesktopPane dpeMDI;

  /*  Hintergrundbild, das in der Klasse JDesktopPane verwendet wird. */
  // private Image background;

  /**
   * The constructor.
   */
  public MainFrame() {
    WindowManager.getInstance().setMainFrame(this);
    // Splash-Screen anzeigen
    /*Splash sp = null;
    if (GeneralOptions.getInstance().getOption("berlin.options.splash").
      toString().equalsIgnoreCase("true")) {
      sp = new Splash("/splash.gif", this, 8000);
      sp.setVisible(true);
    }*/
    this.setIconImage(
      (new ImageIcon(getClass().getResource("/icon.gif"))).getImage());
    /*if (sp != null) {
      sp.setMessage("Lade Plugins");
    }*/
    /*if (sp != null) {
      sp.setMessage("Erstelle Men");
    }*/
    createMenuBar();
    /*if (sp != null) {
      sp.setMessage("Erstelle Symbolleiste");
    }*/
    createBars();
    /*if (sp != null) {
      sp.setMessage("Erstelle Eigenschaftsfenster");
    }*/
    createOInspector();
    /*if (sp != null) {
      sp.setMessage("Erstelle MDI-Area");
    }*/
    createMDI();
    /*if (sp != null) {
      sp.setMessage("Initialisiere");
    }*/

    String noProject = Localization.getLocalizedString(getClass(), "no_project",
      "no project");
    this.setTitle("FDDA - <" + noProject + ">");

    this.setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);

    // Splash-Screen ausblenden
    /*if (sp != null) {
      sp.setVisible(false);
      sp.dispose();
    }*/

    // restore last dimensions of the window
    Preferences prefs = Preferences.userNodeForPackage(getClass());
    int wheight = prefs.getInt("appHeight", 200);
    int wwidth = prefs.getInt("appWidth", 400);
    int wx = prefs.getInt("xOffset", 40);
    int wy = prefs.getInt("yOffset", 40);

    // don't make window bigger than the screen
    Dimension scrSize = Toolkit.getDefaultToolkit().getScreenSize();
    if (wwidth > scrSize.width) {
      wx -= wwidth - scrSize.width;
      if (wx < 0) {
        wx = 0;
        wwidth = scrSize.width;
      }
    }
    if (wheight > scrSize.height) {
      wy -= wheight - scrSize.height;
      if (wy < 0) {
        wy = 0;
        wheight = scrSize.height;
      }
    }
    this.setSize(wwidth, wheight);
    this.setLocation(wx, wy);
    int extendedState = prefs.getInt("extendedState", getExtendedState());
    this.setExtendedState(extendedState);

    // restore width of property window
    int propwidth = prefs.getInt("propWidth", 100);
    tbrProperties.setPreferredSize(new Dimension(propwidth, 100));

    // restore grid mode
    mbrMenuBar.setGridMEntry(Preferences.userRoot().node("fdda/graphics").
      getBoolean("showGrid", false));

    // Control quit of the application
    WindowListener wndCloser = new WindowAdapter() {
        public void windowClosing(WindowEvent e) {
            quit();
        }
    };
    addWindowListener(wndCloser);

    // use pointer tool
    useTool(fdda.dialog.GraphicsControl.POINTER, false);
  }

  /**
   * Performs the first action after starting the application.
   */  
  public void performStartAction() {
    // which action?
    Preferences prefs = Preferences.userNodeForPackage(getClass());
    int startAction = prefs.getInt("startFileAction", NOTHING); 
    if (NEW_PROJECT == startAction) {
      newProject();
    }
    else if (LAST_PROJECT == startAction) {
      String lastProject = Preferences.userRoot().node("fdda").get(
        "lastProjectLocation", null);
      if (lastProject != null) {
        openProject(lastProject);
      }
    }
  }

  /**
   * Repaints all windows
   */
  public void refreshMDI() {
    JInternalFrame[] frames = dpeMDI.getAllFrames();
    for (int i = 0; i < frames.length; i++) {
      if (frames[i] instanceof FrameControlProxy) {
        ((FrameControlProxy) frames[i]).repaint();
      }
    }
  }

  /** Creates a new project */
  public void newProject() {
    Project newProject = Core.getProjectManager().newProject();
    GraphicsControl control 
      = DialogFactory.getInstance().createGraphicsControl();
    String newProjectS = Localization.getLocalizedString(getClass(), 
      "new_project", "new project");
    FrameControlProxy ifmProject = 
      new FrameControlProxy("<" + newProjectS + ">", control);
    double x = 500;
    double y = 400;
    Dimension mdiSize = dpeMDI.getSize();
    if (x > mdiSize.getWidth()) {
      x = mdiSize.getWidth();
    }
    if (y > mdiSize.getHeight()) {
      y = mdiSize.getHeight();
    }
    ifmProject.setSize((int) Math.ceil(x), (int) Math.ceil(y));
    ((SwingGraphicsControl) control).attachToFrame(ifmProject, this);
    ifmProject.setGraphicalSystem(newProject.getGraphicalSystem());
    ifmProject.setSelectedAction(selectedTool);
    btrButtonBar.setSystemFrameButtons(SystemFrame.SF_PROJECT);
    mbrMenuBar.setSystemFrameMEntrys(SystemFrame.SF_PROJECT);
    WindowManager.getInstance().addProjectFrame(
      ifmProject, newProject.getName(), newProject);
    ifmProject.addInternalFrameListener(new SystemFrameListener(ifmProject));
    ifmProject.addPropertyChangeListener
       (JInternalFrame.IS_MAXIMUM_PROPERTY, new MaximizeListener(ifmProject));
    dpeMDI.add(ifmProject);
    try {
      ifmProject.setSelected(true);
      ifmProject.moveToFront();
    }
    catch (java.beans.PropertyVetoException e) {
      Logging.exceptionCaught(e);
      Notification.message(Notification.M_ERROR, Notification.S_PROCESSING,
        "Interner Fehler", "PropertyVetoException in Berlin.newProject()!");
    }
    Core.getProjectManager().setActiveProject(newProject);
    ifmProject.setVisible(true);
  }

  /**
   * Opens a project
   * @param location the location of the project
   */
  public void openProject(String location) {
    // delegate loading to project manager
    Project project = Core.getProjectManager().openProject(location);
    if (project == null) {
      return;
    }
    GraphicsControl control = 
      DialogFactory.getInstance().createGraphicsControl();
    FrameControlProxy projectFrame = 
      new FrameControlProxy(project.getName(), control);
    projectFrame.setSize(500, 400);
    Rectangle2D systemSize = 
      project.getGraphicalSystem().getEnclosingRectangle();
    double x = systemSize.getWidth() + systemSize.getX() + 60;
    double y = systemSize.getHeight() + systemSize.getY() + 60;
    if (x < 200) x = 200;
    if (y < 100) y = 100;
    Dimension mdiSize = dpeMDI.getSize();
    if (x > mdiSize.getWidth()) {
      x = mdiSize.getWidth();
    }
    if (y > mdiSize.getHeight()) {
      y = mdiSize.getHeight();
    }
    projectFrame.setSize((int) Math.ceil(x), (int) Math.ceil(y));
    ((SwingGraphicsControl) control).attachToFrame(projectFrame, this);
    projectFrame.setGraphicalSystem(project.getGraphicalSystem());
    btrButtonBar.setSystemFrameButtons(SystemFrame.SF_PROJECT);
    mbrMenuBar.setSystemFrameMEntrys(SystemFrame.SF_PROJECT);
    WindowManager.getInstance().addProjectFrame(
      projectFrame, project.getName(), project);
    projectFrame.addInternalFrameListener
      (new SystemFrameListener(projectFrame));
    projectFrame.addPropertyChangeListener
       (JInternalFrame.IS_MAXIMUM_PROPERTY,
         new MaximizeListener(projectFrame));
    dpeMDI.add(projectFrame);
    projectFrame.setSelectedAction(selectedTool);
    projectFrame.setVisible(true);
    try {
      projectFrame.setSelected(true);
      projectFrame.moveToFront();
    }
    catch (java.beans.PropertyVetoException e) {
      Logging.exceptionCaught(e);
      Notification.message(Notification.M_ERROR, Notification.S_PROCESSING,
        "Interner Fehler", "PropertyVetoException in Berlin.openProject()!");
    }
    Core.getProjectManager().setActiveProject(project);
  }

  /**
   * Saves the current project.
   * @param newName whether 'save as' is requested
   */
  public void saveProject(boolean newName) {
    Project project = Core.getProjectManager().getActiveProject();
    // delegate to project manager
    Core.getProjectManager().saveProject(project, newName);
    this.mbrMenuBar.rebuildLastFilesMenu();
  }

  /** Closes the current project */
  public void closeProject() {
    Project project = Core.getProjectManager().getActiveProject();
    Core.getProjectManager().closeProject(project);
  }

  /**
   * Quits the application. Shows warning messages, saves attributes.
   */
  public void quit() {
    int result = 0;
    if (Preferences.userNodeForPackage(getClass()).getBoolean("askAtClosing",
      true)) {
      ResourceBundle bundle = 
        Localization.getResourceBundleForPackage(getClass());
      String quitS = Localization.getLocalizedString(bundle, "quit", "Quit");
      String question = Localization.getLocalizedString(bundle, "quit_question",
        "This closes FDDA. Are you sure?");
      String yes = Localization.getLocalizedString(bundle, "yes", "Yes");
      String no = Localization.getLocalizedString(bundle, "no", "No");
      result = Notification.message(Notification.M_QUESTION,
                        Notification.S_MOD, quitS, question,
                        new String[] {yes, no});
    }
    if (result == 0) {
      result = Core.getProjectManager().closeAll() ? 0 : 1;
    }
    if (result == 0) {
      // save attributes
      Preferences prefs = Preferences.userNodeForPackage(getClass());
      int extendedState = this.getExtendedState();
      prefs.putInt("extendedState", extendedState);
      if ((extendedState & MAXIMIZED_VERT) == 0) {
        prefs.putInt("appHeight", this.getHeight());
        prefs.putInt("yOffset", this.getY());
      }
      if ((extendedState & MAXIMIZED_HORIZ) == 0) {
        prefs.putInt("appWidth", this.getWidth());
        prefs.putInt("xOffset", this.getX());
      }
      prefs.putBoolean("properties", tbrProperties.isVisible());
      if (tbrProperties.isVisible()) {
        prefs.putInt("propWidth", tbrProperties.getWidth());
      }
      // export preferences
      try {
        Preferences.userRoot().node("fdda").exportSubtree(
          new FileOutputStream(new File(System.getProperty("user.home") 
               + File.separator + "fdda" + File.separator + "fdda_options.xml")));
      }
      catch (java.util.prefs.BackingStoreException e) {
        Logging.exceptionCaught(e);
        Notification.message(Notification.M_ERROR, Notification.S_SYSTEM, 
          Localization.formatMessage(Localization.getLocalizedString(getClass(),
            "options_store_error", "Options couldn't be saved. Error message:" +
              "\n{0}"), e.getMessage()));
      }
      catch (IOException e) {
        Logging.exceptionCaught(e);
        Notification.message(Notification.M_ERROR, Notification.S_SYSTEM, 
          Localization.formatMessage(Localization.getLocalizedString(getClass(),
            "options_store_error", "Options couldn't be saved. Error message:" +
              "\n{0}"), e.getMessage()));
      }
      // close application
      this.dispose();
      //System.exit(0);
    }
  }

  /** Zooms in. */
  public void zoomIn() {
    FrameControlProxy selFrame = (FrameControlProxy)dpeMDI.getSelectedFrame();
    if (selFrame != null) {
      selFrame.zoomIn();
    }
  }

  /** Zooms out */
  public void zoomOut() {
    FrameControlProxy selFrame = (FrameControlProxy) dpeMDI.getSelectedFrame();
    if (selFrame != null) {
      selFrame.zoomOut();
    }
  }

  /** Selects all movable components */
  public void selectAll() {
    FrameControlProxy selFrame = (FrameControlProxy) dpeMDI.getSelectedFrame();
    if (selFrame != null) {
      selFrame.selectMovables();
    }
  }

  /**
   * Shows or hides the property editor
   * @param visible whether it shall be shown
   */
  public void showObjectInspector(boolean visible) {
    mbrMenuBar.setOInspectorMEntry(visible);
    btrButtonBar.setOInspectorButton(visible);
    tbrProperties.setVisible(visible);
  }

  /**
   * Selects a modelling tool
   * @param tool the tool
   * @param sticky whether the selection shall be 'sticky'
   */
  public void useTool(int tool, boolean sticky) {
    selectedTool = tool; // for new frames
    JInternalFrame[] frames = dpeMDI.getAllFrames();
    for (int i = 0; i < frames.length; i++) {
      ((FrameControlProxy) frames[i]).setSelectedAction(tool);
    }
    mbrMenuBar.toolSelected(tool, sticky);
    btrButtonBar.toolSelected(tool, sticky);
  }
  
  /**
   * Tries to revert to the default tool. This doesn't work if the current
   * selection of another tool is 'sticky'.
   */
  public void tryToolSwitch() {
    if (btrButtonBar.getToolsButtonGroup().revertToDefaultButton()) {
      useTool(GraphicsControl.POINTER, false);
    }      
  }
  
  /**
   * Deletes the selected component(s).
   */
  public void deleteSelection() {
    FrameControlProxy selFrame = (FrameControlProxy) dpeMDI.getSelectedFrame();
    selFrame.deleteSelection();
  }
  
  /**
   * Shows the algorithm selection dialog.
   */
  public void chooseAlgorithm() {
    DialogFactory.getInstance().showAlgorithmDialog();
  }
  
  /**
   * Starts the demonstration.
   */
  public void startDemonstration() {
    Project project = Core.getProjectManager().getActiveProject();
    GraphicsControl control = 
     ((FrameControlProxy) dpeMDI.getSelectedFrame()).getGraphicsControl();
    Demonstration demo = Core.getDemonstration();
    try {
      demo.start(project, control);
    }
    catch (WrongStateException e) {
      Logging.exceptionCaught(e);
      assert false;
    }
  }
  
  /**
   * Stops the demonstration.
   */
  public void stopDemonstration() {
    Project project = Core.getProjectManager().getActiveProject();
    Demonstration demo = Core.getDemonstration();
    try {
      demo.stop(project);
    }
    catch (WrongStateException e) {
      Logging.exceptionCaught(e);
      assert false;
    }
  }

  /**
   * Pauses or continues the demonstration.
   */
  public void pauseDemonstration() {
    Project project = Core.getProjectManager().getActiveProject();
    Demonstration demo = Core.getDemonstration();
    try {
      boolean paused = demo.getDemonstrator(project).isPaused();
      demo.pause(project);
      btrButtonBar.demoPaused(!paused);
      mbrMenuBar.demoPaused(!paused);
    }
    catch (WrongStateException e) {
      Logging.exceptionCaught(e);
      assert false;
    }
  }
  
  /**
   * Shows the statistics dialog.
   */
  public void showStatistics() {
    try {
      Project project = Core.getProjectManager().getActiveProject();
      Statistics stats = 
        Core.getDemonstration().getDemonstrator(project).getCurrentStatistics();
      DialogFactory.getInstance().showStatisticsDialog(stats);
    }
    catch (WrongStateException e) {
      Logging.exceptionCaught(e);
      assert false;
    }
  }    

  /**
   * Sets the visibility of the grid.
   * @param visible whether the grid shall be visible
   */
  public void showGrid(boolean visible) {
    Preferences.userRoot().node("fdda/graphics").putBoolean("showGrid",visible);
    refreshMDI();
  }

  /** Shows the options dialog */
  public void showOptionsDialog() {
    int oldGridWidth = 
      Preferences.userRoot().node("fdda/graphics/java2d").getInt("GridXWidth", 
      10);
    DialogFactory.getInstance().showOptionsDialog();
    if (Preferences.userRoot().node("fdda/graphics/java2d").getInt(
        "GridXWidth", 10) != oldGridWidth) {
      refreshMDI();
    }
  }
  
  /** Shows the about dialog */
  public void showInfo() {
    DialogFactory.getInstance().showAboutDialog();
  }

  /** tiles the windows */
  public void tile() {
    // Get all the desktop's frames
    JInternalFrame[] frames = dpeMDI.getAllFrames();
    // count frames that aren't iconized
    int frameCount = 0;
    for (int i = 0; i < frames.length; i++) {
      if (!frames[i].isIcon()) {
        frameCount++;
      }
    }
    if (frameCount == 0) {
      return;
    }
    int height = dpeMDI.getHeight() / frameCount;
    int c = 0;
    for (int i = 0; i < frames.length; i++) {
      if (!frames[i].isIcon()) {
        if (frames[i] instanceof FrameControlProxy) {
          FrameControlProxy cf = (FrameControlProxy) frames[i];
          cf.repaint();
        }
        try {
          frames[i].setMaximum(false);
          frames[i].reshape(0, c * height, dpeMDI.getWidth(), height);
          c++;
        }
        catch (PropertyVetoException pe) {
          Logging.exceptionCaught(pe);
          Notification.message(Notification.M_ERROR, Notification.S_PROCESSING,
            "Interner Fehler", "PropertyVetoException in Berlin.tile()!");
       }
      }
    }
  }

  /** cascades the windows */
  public void cascade() {
    JInternalFrame[] frames = dpeMDI.getAllFrames();
    int x = 0;
    int y = 0;
    int width = dpeMDI.getWidth() / 2;
    int height = dpeMDI.getHeight() / 2;
    int frameDistance = 22;
    for (int i = 0; i < frames.length; i++) {
      if (!frames[i].isIcon()) {
        try {
          frames[i].setMaximum(false);
          frames[i].reshape(x, y, width, height);
          frames[i].setSelected(true);
          x += frameDistance;
          y += frameDistance;
          if (x + width > dpeMDI.getWidth()) {
            x = 0;
          }
          if (y + height > dpeMDI.getHeight()) {
            y = 0;
          }
        }
        catch (PropertyVetoException pe) {
          Logging.exceptionCaught(pe);
          Notification.message(Notification.M_ERROR, Notification.S_PROCESSING,
            "Interner Fehler", "PropertyVetoException in Berlin.cascade()!");
        }
      }
    }
  }

  /**
   * Shows help pages.
   * @param page code for the page to be shown
   */
  public void help(int page) {
    String url = Preferences.userRoot().node("fdda").get("helpIndex", 
      "http://www.ruedenauer.net/Software/fdda/help.html");
    if (page == 1) {
      url = Preferences.userRoot().node("fdda").get("helpTips",
        "http://www.ruedenauer.net/Software/fdda/tips.html");
    }
    String cmd = Preferences.userRoot().node("fdda").get("browser", "");
    if (cmd.equals("")) {
      Notification.message(Notification.M_CONFIRM, Notification.S_PROCESSING, 
        Localization.getLocalizedString(getClass(), "no_browser_title", "Help"),
        Localization.getLocalizedString(getClass(), "no_browser_msg", 
          "You have to specify a browser in the preferences first."));
    }
    else {
      if (cmd.indexOf("%u") != -1) {
        cmd = cmd.replaceAll("%u", url);
      }
      else {
        cmd += " " + url;
      }
      try {
        Runtime.getRuntime().exec(cmd);
      }
      catch (IOException e) {
        Notification.message(Notification.M_ERROR, Notification.S_SYSTEM,
        Localization.getLocalizedString(getClass(), "no_browser_title", "Help"),
        Localization.getLocalizedString(getClass(), "browser_call_error", 
          "The browser couldn't be started. Please check the preferences."));
      }
    }
  }

  /** Saves current modelling window to a jpeg file */
  public void saveJPEG() {
    FrameControlProxy selFrame = (FrameControlProxy) dpeMDI.getSelectedFrame();
    selFrame.saveJPEG();
  }

  /** Creates the button bar */
  private void createButtonBar() {
    btrButtonBar = new ButtonBar(this);
  }

  /** Creates the menu bar */
  private void createMenuBar() {
    mbrMenuBar = new MenuBar(this);
    setJMenuBar(mbrMenuBar);
  }

  /**
   * Creates button bar, menu bar etc.
   */
  private void createBars() {
    createButtonBar();
    JSeparator sprHLine = new JSeparator();
    pnlToolBars = new JPanel(); 
    pnlToolBars.setPreferredSize(new Dimension(30, 32));
    pnlToolBars.setLayout(new java.awt.BorderLayout(0, 0));
    pnlToolBars.add(btrButtonBar, BorderLayout.NORTH);
    pnlToolBars.add(sprHLine, BorderLayout.SOUTH);
    this.getContentPane().add(pnlToolBars, BorderLayout.NORTH);
  }

  /**
   * Creates the property editor
   */
  private void createOInspector() {
    tbrProperties = new SizeableToolbar(); // put into a tool bar
    tbrProperties.add(ObjectInspector.getInstance()); 
    tbrProperties.setFloatable(false); 
    tbrProperties.setSide(1); // at the right side
    this.getContentPane().add(tbrProperties, BorderLayout.EAST);
    tbrProperties.setPreferredSize(new Dimension(150, 60)); 
    if (Preferences.userNodeForPackage(getClass()).getBoolean("properties", 
                                                              true)) {
      tbrProperties.setVisible(true);
      btrButtonBar.setOInspectorButton(true);
      mbrMenuBar.setOInspectorMEntry(true);
    }
    else {
      tbrProperties.setVisible(false); 
      btrButtonBar.setOInspectorButton(false);
      mbrMenuBar.setOInspectorMEntry(false);
    }
  }

  /**
   * Creates the MDI area
   */
  private void createMDI() {
    /*background =
     Toolkit.getDefaultToolkit().getImage(getClass().getResource("/backg.jpg"));
    MediaTracker mt = new MediaTracker(this);
    mt.addImage(background, 0);
    try {
        mt.waitForAll();
    }
    catch (Exception e) {
    }*/
    dpeMDI = new JDesktopPane() /*{
      public void paintComponent(Graphics g) {
          super.paintComponent(g);
          Rectangle rec = getBounds();
          g.drawImage(background, 0, 0, rec.width, rec.height, null);
      }
    }*/;
    getContentPane().add(dpeMDI, BorderLayout.CENTER);
    dpeMDI.setVisible(true);
  }

  /**
   * Internal listener for changes of the windows.
   */
  class SystemFrameListener extends InternalFrameAdapter {

    /** the system frame */
    private SystemFrame newSystemFrame;

    /**
     * Constructor
     * @param newSF the system frame
     */
    public SystemFrameListener(SystemFrame newSF) {
      super();
      newSystemFrame = newSF;
    }

    /** 
     * Returns the system frame
     * @return the system frame
     */
    public SystemFrame getSystemFrame() {
      return newSystemFrame;
    }

    /*
     *  (non-Javadoc)
     * @see javax.swing.event.InternalFrameListener#internalFrameActivated(javax.swing.event.InternalFrameEvent)
     */
    public void internalFrameActivated(InternalFrameEvent e) {
      // clear selection
      ComponentFactory.getInstance().
        getComponentSelection().clear(MainFrame.this);
      btrButtonBar.setSystemFrameButtons(SystemFrame.SF_PROJECT);
      mbrMenuBar.setSystemFrameMEntrys(SystemFrame.SF_PROJECT);
    }

    /*
     *  (non-Javadoc)
     * @see javax.swing.event.InternalFrameListener#internalFrameDeactivated(javax.swing.event.InternalFrameEvent)
     */
    public void internalFrameDeactivated(InternalFrameEvent e) {
      if (e.getInternalFrame() instanceof FrameControlProxy) {
        FrameControlProxy deactivatedFrameControlProxy =
          (FrameControlProxy) e.getInternalFrame();
        // clear selection
        ComponentFactory.getInstance().getComponentSelection().
          clear(MainFrame.this);
        // repaint
        deactivatedFrameControlProxy.repaint(deactivatedFrameControlProxy.
          getGraphicalSystem().getEnclosingRectangle().getBounds());
      }
    }

    /*
     *  (non-Javadoc)
     * @see javax.swing.event.InternalFrameListener#internalFrameClosed(javax.swing.event.InternalFrameEvent)
     */
    public void internalFrameClosed(InternalFrameEvent ife) {
      SystemFrame closedSystemFrame = (SystemFrame) ife.getInternalFrame();
      // if closed frame was active: clear selection
      if (newSystemFrame == dpeMDI.getSelectedFrame()) {
        ComponentFactory.getInstance().getComponentSelection().
          clear(MainFrame.this);
      }
      WindowManager.getInstance().deleteProjectFrame(closedSystemFrame, 
        closedSystemFrame.getGraphicalSystem().getProject());
      JInternalFrame[] frames = dpeMDI.getAllFrames();
      for (int i = 0; i < frames.length; i++) {
        if (frames[i] instanceof SystemFrame) {
          // close all other project frames
          if (((SystemFrame) frames[i]).getGraphicalSystem().getProject()
              == closedSystemFrame.getGraphicalSystem().getProject()) {
            frames[i].dispose();
          }
        }
      }
      // activate another frame, if possible
      frames = dpeMDI.getAllFrames();
      if (frames.length > 0) {
        frames[0].show();
        try {
          frames[0].setSelected(true);
          frames[0].moveToFront();
        }
        catch (java.beans.PropertyVetoException e) {
          Logging.exceptionCaught(e);
          Notification.message(Notification.M_ERROR, Notification.S_PROCESSING,
            "Interner Fehler: PropertyVetoException in SystemFrameListener." +
            "frameClosed!");
        }
      }
      else {
        btrButtonBar.setSystemFrameButtons(SystemFrame.SF_NONE);
        mbrMenuBar.setSystemFrameMEntrys(SystemFrame.SF_NONE);
      }
    }
  };

  /**
   * Workaround for Java lack: no resize-Event is fired if a frame is 
   * maximized. So catch the maximization event and fire the resize event
   * manually.
   */
  class MaximizeListener implements java.beans.PropertyChangeListener {

    /** the frame */
    private SystemFrame frame;

    /**
     * Constructor
     * @param frame the frame
     */
    public MaximizeListener(SystemFrame frame) {
      super();
      this.frame = frame;
    }
    
    /*
     *  (non-Javadoc)
     * @see java.beans.PropertyChangeListener#propertyChange(java.beans.PropertyChangeEvent)
     */
    public void propertyChange(PropertyChangeEvent e) {
      frame.dispatchEvent(
        new ComponentEvent(frame, ComponentEvent.COMPONENT_RESIZED));
    }
  }

}