/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.dialog.swing;

import fdda.core.Core;
import fdda.core.projects.*;
import fdda.utils.Localization;
import fdda.utils.Logging;

import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import java.util.Vector;
import java.util.HashMap;
import java.util.HashSet;

import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuItem;

/**
 * Manages the internal frames and the window menu
 * @author Jrg Rdenauer
 */
public class WindowManager extends ProjectAdapter {
  
  /** the instance (singleton) */
  private static WindowManager theInstance = null;

  /** the frames */
  private Vector frames;

  /** the main frame */
  private MainFrame mainFrame = null;

  /** the window menu */
  private JMenu windowMenu = null;

  /** number of fixed items in the window menu */
  private int windowItemCount;

  /** the frames/project match */
  private HashMap projectFrameList;

  /**
   * Returns the window manager (singleton).
   * @return the window manager
   */
  public static WindowManager getInstance() {
    if (theInstance == null) {
      theInstance = new WindowManager();
    }
    return theInstance;
  }

  /**
   * Adds a new project frame. Gives it an unique title. Adds the entry to 
   * the window menu.
   * @param newFrame the new frame
   * @param frameName suggestion for the title
   * @param project the project of the frame
   */
  public void addProjectFrame(SystemFrame newFrame, String frameName, 
                               Project project) {
    // Unique title
    String title = getUniqueProjectTitle(project, frameName);
    newFrame.setTitle(title);

    // menu entry
    final JMenuItem mnuProjectItem = new JMenuItem(title);
    Font font = mnuProjectItem.getFont();
    mnuProjectItem.setFont(new Font(font.getFamily(),Font.BOLD,font.getSize()));

    // ActionListener for the entry
    final SystemFrame actFrame = newFrame;
    mnuProjectItem.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        activateFrame(actFrame);
      }
    });

    frames.add(new FrameMenu(newFrame, newFrame, project, title, mnuProjectItem,
                             SystemFrame.SF_PROJECT));
    HashSet frameSet = null;
    if (projectFrameList.containsKey(project)) {
      frameSet = (HashSet) projectFrameList.get(project);
    }
    else {
      frameSet = new HashSet(15);
      projectFrameList.put(project, frameSet);
    }
    frameSet.add(newFrame);
    rebuildWindowMenu();
  }

  /*
  public void addSubsystemFrame(SystemFrame newFrame,
                                SystemFrame projectFrame, String frameName) {
    // Eindeutigen Titel holen und setzen
    String title = getUniqueSubsysTitle(newFrame, projectFrame, frameName);
    newFrame.setTitle(title + " (" + projectFrame.getTitle() + ")");

    // Fge Meneintrag dem Men hinzu
    final JMenuItem mnuProjectItem = new JMenuItem(title);

    // Fge dem Eintrag ActionListener hinzu
    final SystemFrame actFrame = newFrame;
    mnuProjectItem.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        // bei Klick soll ControlFrame aktiviert werden
        activateFrame(actFrame);
      }
    });

    // Einfgestelle suchen
    int j = frames.size();
    for (int i = 0; i < frames.size(); i++) {
      FrameMenu curFM = (FrameMenu)frames.elementAt(i);
      if (curFM.getProjectFrame() == projectFrame) {
        // Eintrge zum Projekt gefunden, innerhalb dieser weitersuchen
        // Subsysteme befinden sich zwischen Projekt und Datalogger
        for (j = i + 1; j < frames.size(); j++) {
          curFM = (FrameMenu)frames.elementAt(j);
          if (curFM.getProjectFrame() != projectFrame
              || curFM.getFrameType() != SystemFrame.SF_SUBSYS) {
            break;
          }
        }
        break;
      }
    }

    FrameMenu newFM = new FrameMenu(newFrame, projectFrame, title,
                                    mnuProjectItem, SystemFrame.SF_SUBSYS);
    if (j == frames.size()) {
      frames.add(newFM);
    }
    else {
      frames.add(j, newFM);
    }

    rebuildWindowMenu();
  }
  */

  /*
  public void addLoggerFrame(SystemFrame newFrame,
                             SystemFrame projectFrame, String frameName) {
    // Eindeutigen Titel holen und setzen
    String title = getUniqueLoggerTitle(newFrame, projectFrame, frameName);
    newFrame.setTitle(title + " (" + projectFrame.getTitle() + ")");

    // Fge Meneintrag dem Men hinzu
    final JMenuItem mnuProjectItem = new JMenuItem(title);
    // Kursiv (italic) setzen
    Font font = mnuProjectItem.getFont();
    mnuProjectItem.setFont(new Font(
      font.getFamily(),Font.ITALIC,font.getSize()));

    // Fge dem Eintrag ActionListener hinzu
    final SystemFrame actFrame = newFrame;
    mnuProjectItem.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        // bei Klick soll ControlFrame aktiviert werden
        activateFrame(actFrame);
      }
    });

    // Einfgestelle suchen
    int j = frames.size();
    for (int i = 0; i < frames.size(); i++) {
      FrameMenu curFM = (FrameMenu)frames.elementAt(i);
      if (curFM.getProjectFrame() == projectFrame) {
        // Eintrge zum Projekt gefunden, deren Ende suchen, da sich
        // Datalogger Eintrge nach den Subsystemen befinden
        for (j = i + 1; j < frames.size(); j++) {
          curFM = (FrameMenu)frames.elementAt(j);
          if (curFM.getProjectFrame() != projectFrame) {
            break;
          }
        }
        break;
      }
    }

    FrameMenu newFM = new FrameMenu(newFrame, projectFrame, title,
                                    mnuProjectItem, SystemFrame.SF_LOGGER);
    if (j == frames.size()) {
      frames.add(newFM);
    }
    else {
      frames.add(j, newFM);
    }

    rebuildWindowMenu();
  }
  */

  /**
   * Removes the entry for a project frame (and all subentries).
   * @param projectFrame the frame
   * @param project the project
   */
  public void deleteProjectFrame(SystemFrame projectFrame, Project project) {
    int i = 0;
    while (i < frames.size()) {
      FrameMenu curFM = (FrameMenu) frames.elementAt(i);
      if (curFM.getProjectFrame() == projectFrame) {
        frames.remove(i);
      } 
      else {
        i++;
      }
    }
    HashSet frameSet = (HashSet) projectFrameList.get(project);
    frameSet.remove(projectFrame);
    if (frameSet.isEmpty()) {
      projectFrameList.remove(project);
    }
    rebuildWindowMenu();
  }

  /*
  public void deleteSubFrame(SystemFrame subFrame) {
    for (int i = 0; i < frames.size(); i++) {
      FrameMenu curFM = (FrameMenu)frames.elementAt(i);
      if (curFM.getFrame() == subFrame) {
        frames.remove(i);
        break;
      }
    }
    rebuildWindowMenu();
  }
  */

  /*
  public void setSubName(SystemFrame sFrame, String name) {
    // Frame suchen
    for (int i = 0; i < frames.size(); i++) {
      FrameMenu curFM = (FrameMenu)frames.elementAt(i);
      if (curFM.getFrame() == sFrame) {
        // Eindeutigen Namen holen
        String title = name;
        if (curFM.getFrameType() == SystemFrame.SF_SUBSYS) {
          title = getUniqueSubsysTitle(sFrame, curFM.getProjectFrame(), name);
        } else if (curFM.getFrameType() == SystemFrame.SF_LOGGER) {
          title = getUniqueLoggerTitle(sFrame, curFM.getProjectFrame(), name);
        }
        // Titel und Meneintrag ndern
        sFrame.setTitle(title + " ("
                        + curFM.getProjectFrame().getTitle() + ")");
        curFM.setFrameName(title);
        curFM.getMenuItem().setText(title);
        // und zurck, mehr als einmal kommt das Frame nicht vor
        break;
      }
    }
    rebuildWindowMenu();
  }
  */

  /**
   * Tells the WindowManager about the window menu and its current number of 
   * entries.
   * @param windowMenu the window menu
   * @param windowItemCount the current number of entries (without windows)
   */
  public void setWindowMenu(JMenu windowMenu, int windowItemCount) {
    this.windowMenu = windowMenu;
    this.windowItemCount = windowItemCount;
  }

  /**
   * Sets the main frame
   * @param mainFrame the main frame
   */
  public void setMainFrame(MainFrame mainFrame) {
    this.mainFrame = mainFrame;
  }

  /**
   * Returns the main frame
   * @return the main frame
   */
  public JFrame getMainFrame() {
    return mainFrame;
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.core.projects.ProjectListener#activeProjectChanged(fdda.core.projects.ProjectEvent)
   */
  public void activeProjectChanged(ProjectEvent e) {
    mainFrame.setTitle("FDDA - " + ((Project) e.getSource()).getName());
  }

  /*
   *  (non-Javadoc)
   * @see fdda.core.projects.ProjectListener#projectNameChanged(fdda.core.projects.ProjectEvent)
   */  
  public void projectNameChanged(ProjectEvent e) {
    Project p = (Project) e.getSource();
    setProjectName(p, p.getName());
    if (p == Core.getProjectManager().getActiveProject()) {
      mainFrame.setTitle("FDDA - " + p.getName());
    }
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.core.projects.ProjectListener#projectStateChanged(fdda.core.projects.ProjectEvent)
   */
  public void projectStateChanged(ProjectEvent e) {
    mainFrame.refreshMDI();
  }

  /*
   *  (non-Javadoc)
   * @see fdda.core.projects.ProjectListener#projectClosed(fdda.core.projects.ProjectEvent)
   */
  public void projectClosed(ProjectEvent e) {
    closeProjectFrames((Project) e.getSource());
    String noProject = Localization.getLocalizedString(getClass(), "no_project",
      "no project");
    if (Core.getProjectManager().getNrOfProjects() == 0) {
      mainFrame.setTitle("FDDA - <" + noProject + ">");      
    }
  }

  /**
   * Closes all frames of a project
   * @param project the project
   */
  private void closeProjectFrames(Project project) {
    HashSet frameSet = (HashSet) projectFrameList.get(project);
    if (projectFrameList.containsKey(project)) {
      SystemFrame[] frames = (SystemFrame[]) 
        frameSet.toArray(new SystemFrame[frameSet.size()]);
      for (int i = 0; i < frames.length; i++) {
        //deleteProjectFrame(frames[i], project);
        frames[i].dispose();
      }
    }
  }

  /** Constructor */
  protected WindowManager() {
    frames = new Vector();
    projectFrameList = new HashMap(10);
    Core.getProjectManager().addProjectListener(this);
  }

  /** Rebuilds the window menu */
  private void rebuildWindowMenu() {
    if (windowMenu == null) {
      return;
    }

    // Remove all variable entries
    for (int i = windowMenu.getItemCount() - 1; i >= windowItemCount; i--) {
      windowMenu.remove(i);
    }

    // Rebuild the menu
    SystemFrame oldProject = null;
    for (int i = 0; i < frames.size(); i++) {
      FrameMenu curFM = (FrameMenu) frames.elementAt(i);
      if (curFM.getProjectFrame() != oldProject) {
        // for a new project, add a separator
        oldProject = curFM.getProjectFrame();
        windowMenu.addSeparator();
      }
      windowMenu.add(curFM.getMenuItem());
    }
  }

  /**
   * Changes the name of a project frame
   * @param project the project
   * @param name the new name
   */
  private void setProjectName(Project project, String name) {
    if (!projectFrameList.containsKey(project)) {
      return;
    }
    // unique title
    String title = getUniqueProjectTitle(project, name);

    // change all subframes, too
    for (int i = 0; i < frames.size(); i++) {
      FrameMenu curFM = (FrameMenu) frames.elementAt(i);
      if (curFM.getProject() == project) {
        if (curFM.getFrameType() == SystemFrame.SF_PROJECT) {
          curFM.setFrameName(title);
          curFM.getFrame().setTitle(title);
          curFM.getMenuItem().setText(title);
        } 
        else {
          curFM.getFrame().setTitle(curFM.getFrameName() + " (" + title + ")");
        }
      }
    }
    rebuildWindowMenu();
  }

  /**
   * Returns an unique title for a frame
   * @param project the project
   * @param suggestedTitle a suggestion for the title 
   * @return the new, unique title
   */
  private String getUniqueProjectTitle(Project project,
                                       String suggestedTitle) {
    boolean ok = false;
    String newTitle = suggestedTitle;
    int nameCnt = 1;
    while (!ok) {
      ok = true;
      for (int i = 0; i < frames.size(); i++) {
        FrameMenu curFM = (FrameMenu) frames.elementAt(i);
        if (curFM.getFrameType() == SystemFrame.SF_PROJECT
            && curFM.getProject() != project
            && curFM.getFrameName().equals(newTitle)) {
          ok = false;
          newTitle = suggestedTitle + " (" + nameCnt + ")";
          nameCnt++;
          break;
        }
      }
    }
    return newTitle;
  }

  /*
  private String getUniqueSubsysTitle(SystemFrame subFrame,
                                      SystemFrame projFrame,
                                      String suggestedTitle) {
    boolean ok = false;
    String newTitle = suggestedTitle;
    int nameCnt = 1;
    while (!ok) {
      ok = true;
      for (int i = 0; i < frames.size(); i++) {
        FrameMenu curFM = (FrameMenu)frames.elementAt(i);
        if (curFM.getFrameType() == SystemFrame.SF_SUBSYS
            && curFM.getProjectFrame() == projFrame
            && curFM.getFrame() != subFrame
            && curFM.getFrameName().equals(newTitle)) {
          ok = false;
          newTitle = suggestedTitle + " (" + nameCnt + ")";
          nameCnt++;
          break;
        }
      }
    }
    return newTitle;
  }
  */

  /*
  private String getUniqueLoggerTitle(SystemFrame subFrame,
                                      SystemFrame projFrame,
                                      String suggestedTitle) {
    boolean ok = false;
    String newTitle = suggestedTitle;
    int nameCnt = 1;
    while (!ok) {
      ok = true;
      for (int i = 0; i < frames.size(); i++) {
        FrameMenu curFM = (FrameMenu)frames.elementAt(i);
        if (curFM.getFrameType() == SystemFrame.SF_LOGGER
            && curFM.getProjectFrame() == projFrame
            && curFM.getFrame() != subFrame
            && curFM.getFrameName().equals(newTitle)) {
          ok = false;
          newTitle = suggestedTitle + " (" + nameCnt + ")";
          nameCnt++;
          break;
        }
      }
    }
    return newTitle;
  }
  */

  /**
   * Activates a frame. I.e., brings it to the front and restores it, if it
   * was iconified
   */
  private void activateFrame(SystemFrame frame) {
    try {
      if (frame.isIcon()) {
        frame.setIcon(false);
      }
      frame.setSelected(true);
    } 
    catch (java.beans.PropertyVetoException pe) {
      Logging.exceptionCaught(pe); 
    }
  }

  /**
   * Stores a frame together with its project and its menu entry
   */
  private class FrameMenu {

    /** the frame */
    private SystemFrame frame;

    /** the project frame */
    private SystemFrame projectFrame;

    /** the frame title */
    private String frameName;

    /** the menu entry */
    private JMenuItem menuItem;

    /** the type */
    private int frameType;
    
    /** the project */
    private Project project;
    
    /**
     * Constructor
     * @param frame the frame
     * @param projectFrame the main project frame for the project
     * @param project the project
     * @param frameName the title of the frame
     * @param menuItem the menu item
     * @param frameType the type of the frame
     */
    public FrameMenu(SystemFrame frame, SystemFrame projectFrame, 
                     Project project, String frameName, JMenuItem menuItem,
                     int frameType) {
      this.frame = frame;
      this.projectFrame = projectFrame;
      this.project = project;
      this.frameName = frameName;
      this.menuItem = menuItem;
      this.frameType = frameType;
    }

    /**
     * Returns the frame
     * @return the frame
     */
    public SystemFrame getFrame() {
      return frame;
    }
    
    /**
     * Returns the project
     * @return the project
     */
    public Project getProject() {
      return project;
    }

    /**
     * Returns the project frame
     * @return the project frame
     */
    public SystemFrame getProjectFrame() {
      return projectFrame;
    }

    /**
     * Sets the name of the frame
     * @param newName the new name
     */
    public void setFrameName(String newName) {
      frameName = newName;
    }

    /**
     * Returns the name of the frame
     * @return the name
     */
    public String getFrameName() {
      return frameName;
    }

    /**
     * Returns the menu entry
     * @return the menu entry
     */
    public JMenuItem getMenuItem() {
      return menuItem;
    }

    /**
     * Returns the frame type
     * @return the frame type
     */
    public int getFrameType() {
      return frameType;
    }

  }

}
