/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.dialog.swing.properties;

import fdda.utils.Localization;
import fdda.utils.Notification;

import fdda.model.Property;
import fdda.model.PropertyFactory;

import javax.swing.AbstractCellEditor;
import javax.swing.event.CellEditorListener;
import javax.swing.event.ChangeEvent;
import javax.swing.JScrollPane;
import javax.swing.JDialog;
import javax.swing.JButton;
import javax.swing.JTable;
import javax.swing.JPanel;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import java.awt.BorderLayout;
import java.awt.GridLayout;

import java.util.LinkedHashMap;

/**
 * Editor for values of properties, when the property has multiple values. As 
 * soon as the property is changed, editngStopped is fired, though that doesn't
 * necessarily mean that the dialog is closed.
 * Works just like the ObjectInspector.
 * @author Jrg Rdenauer
 */
class MultiValueEditor extends AbstractCellEditor {
  
  /**
   * the property
   */
  private Property theProperty = null;

  /**
   * the dialog to edit the values
   */
  private JDialog multiValueDialog = null;

  /**
   * the scroll pane
   */
  private JScrollPane myScrollPane = null;

  /** the table model, containing fake properties */
  private ObjTableModel otm;

  /** Fake-Properties */
  private LinkedHashMap fakeProps = new LinkedHashMap();

  /**
   * Constructor
   * @param aProperty the property
   * @param title title for the dialog
   */
  public MultiValueEditor (Property aProperty, String title) {
    super();
    theProperty = aProperty;

    multiValueDialog =
      new JDialog(fdda.dialog.swing.WindowManager.getInstance().getMainFrame());
    multiValueDialog.setModal(true);
    multiValueDialog.setSize(200, 250);
    multiValueDialog.setLocationRelativeTo(ObjectInspector.getInstance());
    multiValueDialog.setResizable(true);
    multiValueDialog.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);

    multiValueDialog.setTitle(title + "  (" + theProperty.getType() + ")");
    java.awt.Container contentPane = multiValueDialog.getContentPane();
    contentPane.setLayout(new BorderLayout());

    contentPane.add(getScrollPane(), BorderLayout.CENTER);

    contentPane.add(createButtonPanel(), BorderLayout.SOUTH);
  }

  /**
   * Creates the Scrollpane that contains a two-columned table.
   * @return the scroll pane
   */
  private JScrollPane getScrollPane() {
    otm = new ObjTableModel();
    JTable myAttributeTable = new JTable(otm);
    myAttributeTable.setDefaultRenderer(Object.class,
                                        new PanelRenderer(null));
    PanelEditor editor = new PanelEditor();
    editor.addCellEditorListener(new CellEditorListener() {

      public void editingStopped(ChangeEvent e) {
        Property changedProp = ((PanelEditor) e.getSource()).getProperty();
        int nr = ((Integer) changedProp.getFacet("__NR__")).intValue();
        MultiValueEditor.this.theProperty.setValueAt(nr,  
          changedProp.getValue(0));
        fireEditingStopped();
      }

      public void editingCanceled(ChangeEvent e) {
      }

    });

    myAttributeTable.setDefaultEditor(Object.class, editor);
    addAllValues();
    myScrollPane = new JScrollPane(myAttributeTable);
    return myScrollPane;
  }

  /*
   *  (non-Javadoc)
   * @see javax.swing.CellEditor#getCellEditorValue()
   */
  public Object getCellEditorValue() {
    return null;
  }

  /**
   * Returns the cell editing component - in _this_ case, shows the dialog
   * @return null
   */
  public java.awt.Component getTableCellEditorComponent() {
    multiValueDialog.show();
    return null;
  }

  /**
   * Returns the property
   * @return the property
   */
  public Property getProperty() {
    return theProperty;
  }

  /*
   *  (non-Javadoc)
   * @see javax.swing.AbstractCellEditor#fireEditingStopped()
   */
  protected void fireEditingStopped() {
  }
  
  /*
   *  (non-Javadoc)
   * @see javax.swing.AbstractCellEditor#fireEditingCanceled()
   */
  protected void fireEditingCanceled() {
  }

  /**
   * Creates the panel with the buttons
   * @return the panel
   */
  private JPanel createButtonPanel() {
    // only enable buttons if LINE_COUNT_LINK isn't set
    JPanel buttonPanel = new JPanel(new GridLayout());
    boolean buttonsEnabled =
      (theProperty.getFacet(Property.LINE_COUNT_LINK) == null);

    JButton addButton = new JButton(Localization.getLocalizedString(getClass(),
      "add_button_text", "Add"));
    addButton.setEnabled(buttonsEnabled);
    if (buttonsEnabled) {
      // only add listener if button is enabled
      ActionListener addListener = new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          // check whether the maximum count of values is reached
          Integer maxCount = (Integer) 
            theProperty.getFacet(Property.MAX_COUNT_VALUES);
          if (maxCount == null) {
            maxCount = new Integer(Integer.MAX_VALUE);
          }
          if (theProperty.getCountOfValues() < maxCount.intValue()) {
            // myMultiValueDialog.hide();
            Object defaultValue = defaultValue();
            if (defaultValue != null) {
              addValue(defaultValue, theProperty.getCountOfValues());
              addToProperty(defaultValue);
            }
            fireEditingStopped();
            // myMultiValueDialog.show();
          } 
          else {
            Notification.message(Notification.M_ERROR, Notification.S_MOD,
              Localization.formatMessage(Localization.getLocalizedString(
                getClass(), "max_value_count", "You can't have more than {0}" +
                  " values for this property."), maxCount));
          }
        }
      };
      addButton.addActionListener(addListener);
    }
    addButton.setMnemonic(Localization.getLocalizedString(getClass(), 
      "add_button_mnemonic", "A").charAt(0));
    // einen Button um Eintrge zu lschen.
    JButton remButton = new JButton(Localization.getLocalizedString(getClass(),
      "delete_button_text", "Remove"));
    remButton.setEnabled(buttonsEnabled);
    if (buttonsEnabled) {
      // only add listener if button is enabled
      ActionListener remListener = new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          // check whether the minimum count of values is reached
          Integer minCount = (Integer) theProperty.getFacet(Property.MIN_COUNT_VALUES);
          if (minCount == null) {
            minCount = new Integer(0);
          }
          if (theProperty.getCountOfValues() > minCount.intValue()) {
            // myMultiValueDialog.hide();
            removeFromProperty();
            removeValue();
            fireEditingStopped();
            // myMultiValueDialog.show();
          } 
          else {
            Notification.message(Notification.M_ERROR, Notification.S_MOD,
              Localization.formatMessage(Localization.getLocalizedString(
                getClass(), "min_value_count", "You can't have less than {0}" +
                  " values for this property."), minCount));
          }
        }
      };
      remButton.addActionListener(remListener);
      remButton.setMnemonic(Localization.getLocalizedString(getClass(), 
        "delete_button_mnemonic", "D").charAt(0));
    }
    buttonPanel.add(addButton);
    buttonPanel.add(remButton);
    return buttonPanel;
  }

  /**
   * Adds all values when the editor is started
   */
  private void addAllValues() {
    for (int i = 0; i < theProperty.getCountOfValues(); i++) {
      addValue(theProperty.getValue(i), i);
    }
  }

  /**
   * Returns the default value of the property
   * @return the default value
   */
  private Object defaultValue() {
    if (theProperty != null) {
      return theProperty.getFacet(Property.DEFAULT_VALUE);
    }
    else {
      return null;
    }
  }

  /**
   * Adds a value to the editor
   * @param value the value
   * @param nr the number of the new value
   */
  private void addValue(Object value, int nr) {
    // trick: create a temporary fake property for the value
    Property newProp = PropertyFactory.createProperty(theProperty.getName()
      + " (" + nr + ")", theProperty.getType(), theProperty.getName() + " ("
      + nr + ")", Localization.formatMessage(Localization.getLocalizedString(
      getClass(), "valueDesc", "The {0}. value of the property"), new
      Integer(nr)));
    newProp.addValue(value);
    newProp.setModifiable(true);
    newProp.setVisible(true);
    Object facet;
    facet = theProperty.getFacet(Property.MAX_VALUE);
    if (facet != null) {
      newProp.setFacet(Property.MAX_VALUE, facet);
    }
    facet = theProperty.getFacet(Property.MIN_VALUE);
    if (facet != null) {
      newProp.setFacet(Property.MIN_VALUE, facet);
    }
    facet = theProperty.getFacet(Property.DEFAULT_VALUE);
    if (facet != null) {
      newProp.setFacet(Property.DEFAULT_VALUE, facet);
    }
    newProp.setFacet("__NR__", "" + nr); // for internal use
    fakeProps.put(newProp.getName(), newProp);
    otm.setAllValues(fakeProps.values().iterator());
  }

  /**
   * Removes the last value from the editor
   */
  private void removeValue() {
    String name = theProperty.getName() + " ("
      + theProperty.getCountOfValues() + ")";
    fakeProps.remove(name);
    otm.setAllValues(fakeProps.values().iterator());
  }


  /**
   * Adds a value to the property
   * @param value
   */
  private void addToProperty(Object value) {
    theProperty.addValue(value);
  }

  /**
   * Removes the last value from the property
   */
  private void removeFromProperty() {
    if (theProperty.getCountOfValues() > 0) {
      theProperty.deleteValue(theProperty.getCountOfValues() - 1);
    }
  }

}
