/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.dialog.swing.properties;

import javax.swing.AbstractCellEditor;
import java.awt.Component;

import fdda.model.Property;
import fdda.propertyTypes.TypeGenerator;
import fdda.propertyTypes.WrongFacetException;

import fdda.utils.Localization;
import fdda.utils.Notification;

/**
 * An abstract class, extending the cell editor. Superclass of the special 
 * editors for each property type.
 * @author Jrg Rdenauer
 */
public abstract class OiCellEditor extends AbstractCellEditor {

  /** the property */  
  protected Property myProperty = null;

  /** the value */
  protected Object value = null;

  /** Generator for the type */
  protected TypeGenerator generator;

  /** 
   * Default constructor
   * @param property the property
   * @param value the value
   * @param generator TypeGenerator for the type
   */
  public OiCellEditor(Property property, Object value, TypeGenerator generator){
    myProperty = property;
    this.value = value;
    this.generator = generator;
  }

  /*
   *  (non-Javadoc)
   * @see javax.swing.CellEditor#getCellEditorValue()
   */
  public Object getCellEditorValue() {
    return value;
  }

  /**
   * Returns the real editor.
   * @return the editor
   */
  public abstract Component getTableCellEditorComponent();

  /**
   * Checks for errors after editing.
   * @param value the value
   * @return whether the value is ok
   */
  protected boolean checkValue(Object value) {
    boolean correct = false;
    try {
      int returnCode = generator.correctFacets(myProperty); // must be WELL_DONE
      value = generator.correctValue(value, myProperty);
      returnCode = generator.getlastCorrectAction();
      switch (returnCode) {
        case TypeGenerator.WELL_DONE:
          correct = true;
          break;
        case TypeGenerator.WRONG_CLASS_OBJECT:
          Notification.message(Notification.M_ERROR, Notification.S_PROCESSING,
            Localization.formatMessage(Localization.getLocalizedString(
            OiCellEditor.class, "wrong_class_object", 
            "Faulty implementation of {0}: parseString() returns object of " +
            "wrong type."), generator.getClass().getName())); 
          break;
        case TypeGenerator.VALUE_SET_TO_CLASS_DEFAULT:
        case TypeGenerator.VALUE_SET_TO_USER_DEFAULT:
          Notification.message(Notification.M_ERROR, Notification.S_MOD,
            Localization.getLocalizedString(OiCellEditor.class, 
            "value_must_exist", "This property has to have a value."));
          break;
        case TypeGenerator.VALUE_SET_TO_MAX:
          Notification.message(Notification.M_ERROR, Notification.S_MOD,
            Localization.formatMessage(Localization.getLocalizedString(
            OiCellEditor.class, "more_than_maximum", 
            "The value mustn't be higher than {0}."), 
            myProperty.getFacet(Property.MAX_VALUE)));
          break;
        case TypeGenerator.VALUE_SET_TO_MIN:
        Notification.message(Notification.M_ERROR, Notification.S_MOD,
          Localization.formatMessage(Localization.getLocalizedString(
          OiCellEditor.class, "less_than_minimum", 
          "The value mustn't be lower than {0}."), 
          myProperty.getFacet(Property.MIN_VALUE)));
          break;
        default:
          Notification.message(Notification.M_ERROR, Notification.S_MOD,
            Localization.getLocalizedString(OiCellEditor.class, "value_wrong",
            "This value can't be used for this property."));
          break;
      }
    }
    catch (WrongFacetException e) {
      Notification.message(Notification.M_ERROR, Notification.S_PROCESSING,
        Localization.formatMessage(Localization.getLocalizedString(
        OiCellEditor.class,
        "wrong_facet", "Wrong facet in property {0}:\n{1}"), 
        myProperty.getName(), e.getMessage()));
    }
    return correct;
  }

  /** 
   * Returns a string representation of the value
   * @return value as string
   */
  protected abstract String getValueAsString();

  /*
   *  (non-Javadoc)
   * Overriden to check the value
   * @see javax.swing.AbstractCellEditor#fireEditingStopped()
   */
  protected void fireEditingStopped() {
    try {
      value = generator.parseString(getValueAsString());
      if (checkValue(value)) {
        super.fireEditingStopped();
      }
      else {
        fireEditingCanceled();
      }
    }
    catch (java.text.ParseException e) {
      Notification.message(Notification.M_ERROR, Notification.S_MOD,
        Localization.formatMessage(Localization.getLocalizedString(
        OiCellEditor.class,
        "parse_exception", "Value is not of the type {0}."), 
        myProperty.getType()));
      fireEditingCanceled();
    }
  }

}