/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.dialog.swing.utils;

import java.util.LinkedList;
import java.util.ListIterator;

import javax.swing.AbstractButton;

/**
 * Groups several ToggleButtons together so that only one of them is active at
 * each point of time. There may be a 'default' button to revert to. In this 
 * case, reverting may also be blocked by selecting the current button 'sticky'.
 * @author Jrg Rdenauer
 */
public class ToggleButtonGroup {
  
  /** the buttons in the group */  
  private LinkedList buttons;
  /** the default button */
  private AbstractButton defaultButton = null;
  /** 'sticky' selection? */
  private boolean sticky = false;

  /**
   * Constructor.
   */
  public ToggleButtonGroup() {
    buttons = new LinkedList();
  }
  
  /**
   * Adds a button to the group. No check for doubles.
   * @param button the button
   */
  public void addButton(AbstractButton button) {
    buttons.add(button);
  }
  
  /**
   * Removes a button from the group. No effect if button isn't in the group. 
   * If the button was the default button, no default button remains.
   * @param button the button
   */
  public void removeButton(AbstractButton button) {
    ListIterator it = buttons.listIterator();
    while (it.hasNext()) {
      Object buttonInList = it.next();
      if (buttonInList == button) {
        it.remove();   
      }
    }
    if (button == defaultButton) defaultButton = null;
  }
  
  /**
   * Sets the default button. If set to null, no default button is set.
   * @param button the button or null
   */
  public void setDefaultButton(AbstractButton button) {
    defaultButton = button;
  }
  
  /**
   * Selects a button. Selection is not sticky. All other buttons are 
   * deselected.
   * @param button the button
   * @throws IllegalArgumentException if button isn't in the group
   */
  public void selectButton(AbstractButton button) {
    selectButton(button, false);
  }
  
  /**
   * Selects a button. Selection is sticky. All other buttons are deselected.
   * @param button the button
   * @throws IllegalArgumentException if button isn't in the group
   */
  public void selectButtonSticky(AbstractButton button) {
    selectButton(button, true);
  }
  
  /**
   * Selects a button. All other buttons are deselected
   * @param button the button
   * @param sticky whether the selection shall be sticky
   * @throws IllegalArgumentException if button isn't in the group
   */
  public void selectButton(AbstractButton button, boolean sticky) {
    if (!buttons.contains(button)) 
      throw new IllegalArgumentException("Button not in group!");
    ListIterator it = buttons.listIterator();
    while (it.hasNext()) {
      AbstractButton atb = (AbstractButton) it.next();
      atb.setSelected(atb == button);
    }
    this.sticky = sticky;
  }

  /**
   * If a default button is specified and the current selection isn't sticky, 
   * selects the default button. Otherwise, does nothing.
   * @return whether the default button was selected
   */  
  public boolean revertToDefaultButton() {
    if (!sticky && (defaultButton != null)) {
      if (!defaultButton.isSelected()) selectButton(defaultButton);
      return true;
    }
    else return false;
  }
  
  /**
   * Same as revertToDefaultButton, except that the sticky bit is ignored.
   * @param button the button
   * @return whether the default button was selected
   */
  public boolean unselectCurrentButton(AbstractButton button) {
    if (defaultButton != null) {
      if (!defaultButton.isSelected()) selectButton(defaultButton);
      return true;
    }
    else return false;
  }
  
  /**
   * Returns whether the current selection is 'sticky'.
   * @return true, iff sticky
   */
  public boolean isSticky() {
    return (defaultButton == null) || sticky;
  }
  
}
