/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.graphics;

import java.awt.Shape;

/**
 * Abstract superclass of all graphical components in the model. Subclasses 
 * include Nodes and Lines.
 * @author Jrg Rdenauer
 */
public interface Component extends Cloneable {

  /**
   * Indicates that this component may only be selected alone.
   */
  int SEL_NOTHING = 1;

  /**
   * Indicates that this component may only be selected with components of the
   * same type.
   */
  int SEL_EQUALS = 2;

  /**
   * Indicates that this component may be selected together with all movable 
   * components.
   */
  int SEL_MOVABLES = 3;

  /**
   * Indicates that this component may be selected together with any other 
   * component.
   */
  int SEL_ALL = 4;

  /** 
   * Can this component be moved directly by the user?
   * @return true, iff the user can move the component
   */
  boolean isMovable();

  /**
   * Move the component (or its most prominent point).
   * @param xOffset offset for move in x direction
   * @param yOffset offset for move in y direction
   */
  void move(double xOffset, double yOffset);

  /**
   * Returns with which class of components this component can be selected. One 
   * of the SEL_XXX constants.
   * @return constant indicating the selection class
   */
  int selectableWith();

  /**
   * Selects the component.
   */
  void select();

  /**
   * Deselects the component.
   */
  void deselect();

  /**
   * Returns whether the component is currently selected.
   * @return the selection state.
   */
  boolean isSelected();

  /**
   * Returns the x coordinate of the component (or its most prominent point).
   * @return the x coordinate
   */
  double getX();

  /**
   * Returns the y coordinate of the component (or its most prominent point).
   * @return the y coordinate
   */
  double getY();

  /**
   * Sets the coordinates of the component (or of its most prominent point).
   * @param x the x coordinate
   * @param y the y coordinate
   */
  void setXY(double x, double y);

  /**
   * Returns the shape of the component. This may actually be bigger than the 
   * shape seen on screen. It is the shape that is redrawn by the graphic 
   * system.
   * @return the shape of the component
   */
  Shape getShape();

  /**
   * Recalculates the shape of the component. Needs not be called explicitly if
   * the change of the shape is made by other member functions.
   */
  void recalculateShape();

  /**
   * Returns the system which includes the component.
   * @return the graphical system
   */
  GraphicalSystem getGraphicalSystem();

  /**
   * Sets the system which includes the component
   * @param graphicalSystem the graphical system
   */
  void setGraphicalSystem(GraphicalSystem graphicalSystem);

  /**
   * Visitor-pattern: visits this component.
   * @param visitor the visitor
   * @return specific to subclass
   */
  Object visit(ComponentVisitor visitor);

  /**
   * Clones the component
   * @return the clone
   */
  Object clone();

}