/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.graphics;

import fdda.utils.ComponentNotInitializedDisaster;

/**
 * Interface of the factory that produces graphical components. For different 
 * types of graphics, different factories may be used.
 * @author Jrg Rdenauer
 */
public abstract class ComponentFactory {

  /** the concrete factory */
  private static ComponentFactory factory = null;

  /** 
   * Sets the factory that shall be used.
   * @param aFactory the factory
   */
  public static final synchronized void setFactory(ComponentFactory aFactory) {
    factory = aFactory;
  }

  /**
   * Returns the concrete component factory.
   * @return the factory
   */
  public static final ComponentFactory getInstance() {
    synchronized (ComponentFactory.class) {
      if (factory == null) {
        throw new ComponentNotInitializedDisaster("ComponentFactory");
      }
    }
    return factory;
  }

  /**
   * Creates a new node.
   * @param x the x coordinate of the center point of the node
   * @param y the y coordinate of the center point of the node
   * @param selected whether the node shall be selected
   * @return the newly created Node
   */
  public abstract Node createNode(double x, double y, boolean selected);

  /**
   * Creates a new line
   * @param startNode the start node for the line
   * @param endNode the end node for the line
   * @param selected whether the line shall be selected
   * @param directed whether the line shall be directed
   * @return the newly created Line
   */
  public abstract Line createLine(Node startNode, Node endNode,
                                  boolean selected, boolean directed);
  
  /**
   * Creates a new container.
   * @return the newly created Container
   */
  public abstract Container createContainer();
  
  /**
   * Creates a new message.
   * @param line the line which transports the message
   * @param forward whether the message runs in the designated direction of the 
   * line
   * @return the newly created message
   */
  public abstract Message createMessage(Line line, boolean forward);

  /**
   * Creates a new selection. Note that this needs not be used often.
   * @return the newly created Selection
   */
  public abstract Selection getComponentSelection();
  
  /**
   * Creates a new graphical system.
   * @return the newly created GraphicalSystem
   */
  public abstract GraphicalSystem createGraphicalSystem();
}