/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.graphics;

import java.util.ListIterator;

/**
 * Container-component which contains other graphical components. Also 
 * superclass for specialized containers. Composite pattern is used here.
 * @author Jrg Rdenauer
 */
public interface Container extends Component {

  /**
   * Adds an element to the container. No checks for doubles are performed.
   * @param element the element
   */
  void addElement(Component element);

  /**
   * Removes an element from the container. If the element doesn't exist, the 
   * container is unchanged.
   * @param element the element
   */
  void removeElement(Component element);

  /**
   * Checks whether the container contains a certain element.
   * @param element the element
   * @return true, iff the container includes this element
   */
  boolean hasElement(Component element);

  /**
   * Removes all elements from the container.
   */
  void clear();

  /**
   * Returns an iterator over the container.
   * @return a list iterator
   */
  ListIterator getIterator();

  /**
   * Returns the current number of elements
   * @return the number of elements
   */
  int getSize();

  /**
   * Returns whether this container allows a 'flattening' operation
   * @return whether 'flattening' is allowed
   * @see fdda.graphics.Container#flatten()
   */
  boolean isFlatteningAllowed();

  /**
   * Returns a 'flattened' version of the container. This means two things: 
   * firstly, if an element of this container is a container itself and allows 
   * flattening, it will be removed from the container and replaced by its 
   * elements. Secondly, if the size of the container is 1, not the container, 
   * but its element is returned. If the container doesn't allow flattening, it 
   * is returned unchanged
   * @return the flattened version of the container
   */
  Component flatten();

}