/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.graphics;

import fdda.core.projects.Project;

import java.awt.geom.Area;
import java.awt.geom.Rectangle2D;

/**
 * A graphical system, consisting (at this level of abstraction) of nodes,  
 * lines and (while in demo mode) messages.
 * @author Jrg Rdenauer
 */
public interface GraphicalSystem {

  /**
   * Adds a node to the system. No checks for doubles are performed.
   * @param node the node
   */
  void addNode(Node node);

  /**
   * Removes a node from the system. If the node doesn't exist, nothing happens.
   * @param node the node
   */
  void removeNode(Node node);

  /**
   * Adds a line to the system. No checks for doubles are performed.
   * @param line the line
   */
  void addLine(Line line);

  /**
   * Removes a line from the system. If the line doesn't exist, nothing happens.
   * @param line the line
   */
  void removeLine(Line line);

  /**
   * Removes all nodes and all lines from the system.
   */
  void clear();

  /**
   * Returns the 'topmost' component at a certain point. Lines are always 
   * 'higher' than nodes; otherwise components are higher if they were added 
   * later.
   * @param x the x coordinate of the point
   * @param y the y coordinate of the point
   * @return the topmost component at the point - or null, if no component is 
   * currently at that point
   */
  Component getElementAt(double x, double y);

  /**
   * Returns all movable components that lie in the specified area. They need 
   * not lie completely inside, only touching is enough.
   * @param area the area
   * @return all movable components in the area
   */
  Component getMovableComponentsInArea(Area area);

  /**
   * Returns all movable components in the graphical system.
   * @return all movable components
   */
  Component getMovableComponents();

  /**
   * Returns a rectangle that just encloses all components in the system.
   * @return an enclosing rectangle
   */
  Rectangle2D getEnclosingRectangle();

  /**
   * Recalculates the shapes of all components in the system.
   */
  void recalculateShapes();

  /**
   * Returns whether the graphical system was changed since the last save 
   * operation.
   * @return whether the system was changed
   */
  boolean wasChanged();

  /**
   * Sets whether the system was changed since the last save operation.
   * @param changed whether the system was changed
   */
  void setChanged(boolean changed);

  /**
   * Returns the project which includes this graphical system.
   * @return the project
   */
  Project getProject();
  
  /**
   * Sets the project which includes this graphical system.
   * @param project the project
   */
  void setProject(Project project);
  
  /**
   * Adds a message to the system
   * @param message the message
   */
  void addMessage(Message message);
  
  /**
   * Removes a message from the system
   * @param message the message
   */
  void removeMessage(Message message);
  
  /**
   * Removes all messages from the system
   */
  void clearMessages();
  
  /**
   * Returns all messages in the system. The returned Collection is not 
   * modifiable (read only).
   * @return all messsages in the system
   */
  java.util.Collection getAllMessages();
  
}