/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.graphics;

import fdda.model.PropertyHolder;

import java.awt.Color;

/**
 * A line in the system. Each line corresponds to a connection in the model. 
 * The most prominent point of a line is its starting point, but the 'move'
 * operation actually moves both the start and the end point.
 * @author Jrg Rdenauer
 */
public interface Line extends Component, WorkingPart, PropertyHolder {

  /**
   * Returns the x coordinate of the end point of the line
   * @return the x coordinate of the end point of the line
   */
  double getEndX();

  /**
   * Returns the y coordinate of the end point of the line
   * @return the y coordinate of the end point of the line
   */
  double getEndY();
  
  /**
   * Returns the length of the line
   * @return the length of the line
   */
  double getLength();
  
  /**
   * Sets the end point of the line
   * @param x the x coordinate
   * @param y the y coordinate
   */
  void setEndXY(double x, double y);

  /**
   * Moves the start point of the line.
   * @param xOffset the x offset
   * @param yOffset the y offset
   */
  void moveStartPoint(double xOffset, double yOffset);

  /**
   * Moves the end point of the line
   * @param xOffset the x offset
   * @param yOffset the y offset
   */
  void moveEndPoint(double xOffset, double yOffset);

  /**
   * Returns the color of the line.
   * @return the color of the line
   */
  Color getColor();

  /**
   * Sets the color of the line. May be ignored by the implementation.
   * @param color the color
   */
  void setColor(Color color);

  /**
   * Returns whether the line is directed. A directed line corresponds to a 
   * connection which can transfer messages only in one direction.
   * @return whether the line is directed
   */
  boolean isDirected();

  /**
   * Sets whether the line is directed.
   * @param directed whether the line is directed
   */
  void setDirected(boolean directed);

  /**
   * Returns the connection that is symbolized by this line.
   * @return the connection
   */
  fdda.model.Connection getConnection();

  /**
   * Sets the connection that is symbolized by this line.
   * @param connection the connection
   */
  void setConnection(fdda.model.Connection connection);
  
  /**
   * Returns the distance a message proceeds along the line in one cycle.
   * @return distance / cycle -- i.e. message speed
   */
  double getMessageDistancePerCycle();

}