/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.graphics;

import java.awt.Shape;
import java.util.Iterator;

/**
 * The interface of the object which manages selected objects. If a class wants 
 * to modify the selection, it has to implement the tagging interface 
 * 'SelectionModifier'. Thus, a certain control over who modifies it can be 
 * maintained.
 * 
 * Listeners can register themselves to be notified of selection changes 
 * (observer pattern).
 * @author Jrg Rdenauer
 */
public interface Selection {

  /**
   * Adds a listener
   * @param l the listener
   */
  void add(SelectionListener l);
  
  /**
   * Removes a listener
   * @param l the listener
   */
  void remove(SelectionListener l);
  
  /**
   * Returns an iterator over the components in the selection. The iterator is a
   * constant iterator, i.e., you can't modify the selection through it.
   * @return an iterator
   */
  Iterator getSelectionIterator();

  /**
   * If exactly one component is selected, this component is returned, else 
   * null.
   * @return the selected component or null
   */
  Component getSelectedComponent();

  /**
   * Returns the shape of the selection
   * @return the shape
   */
  Shape getShape();
  
  /**
   * Adds a component to the selection
   * @param element the component
   * @param sm the object that wants to modify the selection (a small 'proof')
   */
  void addElement(Component element, SelectionModifier sm);
  
  /**
   * Removes a component from the selection
   * @param element the component
   * @param sm the object that wants to modify the selection (a small 'proof')
   */
  void removeElement(Component element, SelectionModifier sm);
  
  /**
   * Replaces the current selection with another component
   * @param element the component
   * @param sm the object that wants to modify the selection (a small 'proof')
   */
  void replaceWith(Component element, SelectionModifier sm);
  
  /**
   * Removes all components from the selection.
   * @param sm the object that wants to modify the selection (a small 'proof')
   */
  void clear(SelectionModifier sm);
  
  /**
   * Returns whether a certain component is contained in the selection
   * @param element the component
   * @return true, iff the component is currently in the selection
   */
  boolean hasElement(Component element);

}