/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/

package fdda.graphics.java2d;

import java.awt.Shape;
import java.awt.Graphics2D;

/**
 * Implementation of the component interface. Provides several standard 
 * implementations of abstract methods. Most notably implements a mechanism
 * for the calculation of the shape: calcShape() is _only_ called by getShape(),
 * and _only_ if the shape has changed. A change of the shape is signalled by
 * the attribute dirtyShape.
 * @author Jrg Rdenauer
 */
public abstract class Component implements fdda.graphics.Component {

  /** the x coordinate */
  protected double x;

  /** the y coordinate */
  protected double y;

  /** the shape of the component */
  protected Shape shape = null;

  /** whether the component is selected */
  protected boolean selected;

  /** the graphical system that contains the component */
  private fdda.graphics.GraphicalSystem graphicalSystem;

  /** normal drawing color */
  protected final static java.awt.Color COLOR_NORMAL = java.awt.Color.BLACK;

  /** color for selected elements */
  protected final static java.awt.Color COLOR_SELECTED =
    new java.awt.Color(117, 191, 217);

  /** must the shape be recalculated? */
  protected boolean dirtyShape;

  /**
   * Default constructor. Component is not selected.
   * @param x the x coordinate
   * @param y the y coordinate
   */
  public Component(double x, double y) {
    this.x = x;
    this.y = y;
    this.selected = false;
    dirtyShape = true;
  }
  
  /**
   * Constructor.
   * @param x the x coordinate
   * @param y the y coordinate
   * @param selected whether the component is selected
   */
  public Component(double x, double y, boolean selected) {
    this.x = x;
    this.y = y;
    this.selected = selected;
    dirtyShape = true;
  }

  /*
   *  (non-Javadoc)
   * @see java.lang.Object#clone()
   */
  public Object clone() {
    try {
      Component clone = (Component) super.clone();
      clone.deselect();
      clone.dirtyShape = true;
      return clone;
    }
    catch (CloneNotSupportedException e) {
      assert false;
      throw new InternalError();
    }
  }

  /**
   * Sets the selected flag. Doesn't repaint the component. This method should
   * be overriden by subclasses if the shape changes.
   * @see fdda.graphics.Component#select()
   */
  public void select() {
    this.selected = true;
  }

  /**
   * Unsets the selected flag. Doesn't repaint the component. This method should 
   * be overriden by subclasses if the shape changes.
   * @see fdda.graphics.Component#deselect()
   */
  public void deselect() {
    this.selected = false;
  }

  /*
   *  (non-Javadoc)
   * @see fdda.graphics.Component#getX()
   */
  public double getX() {
    return this.x;
  }

  /*
   *  (non-Javadoc)
   * @see fdda.graphics.Component#getY()
   */
  public double getY() {
    return this.y;
  }

  /*
   *  (non-Javadoc)
   * @see fdda.graphics.Component#setXY(double, double)
   */
  public void setXY(double x, double y) {
    this.x = x;
    this.y = y;
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.graphics.Component#getShape()
   */
  public Shape getShape() {
    if (dirtyShape) {
      calcShape();
    }
    return shape;
  }

  /*
   *  (non-Javadoc)
   * @see fdda.graphics.Component#recalculateShape()
   * Note that the shape is _not_ recalculated now, but only when that's needed.
   */
  public void recalculateShape() {
    dirtyShape = true;
  }

  /**
   * Recalculates the shape. Is called by getShape if the flag dirtyShape is 
   * set. Must unset the flag after calculation.
   */
  protected abstract void calcShape();

  /*
   *  (non-Javadoc)
   * @see fdda.graphics.Component#isSelected()
   */
  public boolean isSelected() {
    return this.selected;
  }
  
  /**
   * Draws the component, using the given graphics. The graphics should have the 
   * same attributes (color, pen, etc.) after the call as before the call.
   * @param graphContext the graphics
   */
  public abstract void draw(Graphics2D graphContext);
  
  /*
   *  (non-Javadoc)
   * @see fdda.graphics.Component#getGraphicalSystem()
   */
  public fdda.graphics.GraphicalSystem getGraphicalSystem() {
    return graphicalSystem;
  }

  /*
   *  (non-Javadoc)
   * @see fdda.graphics.Component#setGraphicalSystem(fdda.graphics.GraphicalSystem)
   */
  public void setGraphicalSystem(fdda.graphics.GraphicalSystem graphicalSystem){
    this.graphicalSystem = graphicalSystem;
  }

}
