/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/

package fdda.graphics.java2d;

import fdda.graphics.Selection;
import fdda.graphics.SelectionListener;
import fdda.graphics.SelectionEvent;
import fdda.graphics.SelectionModifier;

import java.util.ListIterator;
import java.util.Iterator;

import javax.swing.event.EventListenerList;

/**
 * Implementation of the selection interface as a Container subclass.
 * @author Jrg Rdenauer
 */
public class ComponentSelection extends Container implements Selection {

  /** the listeners */
  private EventListenerList listeners = new EventListenerList();

  /** 
   * Constructor. The selection is initially empty.
   */
  public ComponentSelection() {
    super();
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.graphics.Selection#add(fdda.graphics.SelectionListener)
   */
  public void add(SelectionListener l) {
    listeners.add(SelectionListener.class, l);
  }

  /*
   *  (non-Javadoc)
   * @see fdda.graphics.Selection#remove(fdda.graphics.SelectionListener)
   */
  public void remove(SelectionListener l) {
    listeners.remove(SelectionListener.class, l);
  }
  
  /**
   * Informs the listeners that the selection has changed
   */
  protected final void fireSelectionChanged() {
    SelectionEvent selectionEvent = null;
    Object[] theListeners = this.listeners.getListenerList();
    for (int i = theListeners.length - 2; i >= 0; i -= 2) {
      if (theListeners[i] == SelectionListener.class) {
        // 'lazy' creation of event
        if (selectionEvent == null) {
          selectionEvent = new SelectionEvent(this);
        }
        ((SelectionListener)
          theListeners[i + 1]).selectionChanged(selectionEvent);
      }
    }
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.graphics.Selection#getSelectionIterator()
   */
  public Iterator getSelectionIterator() {
    return new SelectionIterator(this.getIterator());
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.graphics.Selection#getSelectedComponent()
   */
  public fdda.graphics.Component getSelectedComponent() {
    if (this.getSize() != 1) {
      return null;
    } 
    else {
      ListIterator iter = this.getIterator();
      return (fdda.graphics.Component) iter.next();
    }
  }

  /*
   *  (non-Javadoc)
   * @see fdda.graphics.Selection#addElement(fdda.graphics.Component, fdda.graphics.SelectionModifier)
   */
  public void addElement(fdda.graphics.Component element, 
                          SelectionModifier sm) {
    element.select();
    super.addElement(element);
    dirtyShape = true;
    this.fireSelectionChanged();
  }

  /*
   *  (non-Javadoc)
   * @see fdda.graphics.Selection#removeElement(fdda.graphics.Component, fdda.graphics.SelectionModifier)
   */
  public void removeElement(fdda.graphics.Component element, 
                             SelectionModifier sm) {
    element.deselect();
    super.removeElement(element);
    dirtyShape = true;
    this.fireSelectionChanged();
  }

  /*
   *  (non-Javadoc)
   * @see fdda.graphics.Selection#clear(fdda.graphics.SelectionModifier)
   */
  public void clear(SelectionModifier sm) {
    // deselect the components
    ListIterator it = this.getIterator();
    while (it.hasNext()) {
      ((fdda.graphics.Component) it.next()).deselect();
    }
    super.clear();
    dirtyShape = true;
    this.fireSelectionChanged();
  }

  /*
   *  (non-Javadoc)
   * @see fdda.graphics.Selection#replaceWith(fdda.graphics.Component, fdda.graphics.SelectionModifier)
   */
  public void replaceWith(fdda.graphics.Component element, 
                           SelectionModifier sm) {
    // do not use clear() etc. because they would generate additional events
    // deselect the components
    ListIterator it = this.getIterator();
    while (it.hasNext()) {
      ((fdda.graphics.Component) it.next()).deselect();
    }
    super.clear();
    element.select();
    super.addElement(element);
    this.flatten(); // the new element could be a container!
    dirtyShape = true;
    this.fireSelectionChanged();
  }

  /**
   * An iterator on the selection, not allowing any modifications.
   */
  class SelectionIterator implements Iterator {
  
    /** the real iterator */
    private Iterator it;

    /**
     * Constructor
     * @param it the real iterator
     */
    public SelectionIterator(Iterator it) {
      this.it = it;
    }

    /*
     *  (non-Javadoc)
     * @see java.util.Iterator#hasNext()
     */
    public boolean hasNext() {
      return it.hasNext();
    }
    
    /*
     *  (non-Javadoc)
     * @see java.util.Iterator#next()
     */
    public Object next() {
      return it.next();
    }

    /**
     * Isn't supported and always throws the exception
     * @throws UnsupportedOperationException always
     * @see java.util.Iterator#remove()
     */
    public void remove() throws UnsupportedOperationException {
      throw new UnsupportedOperationException("Don't delete in the selection!");
    }
    
  }
}