/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/

package fdda.graphics.java2d;

import java.awt.Graphics2D;
import java.awt.geom.Area;

import java.util.LinkedList;
import java.util.ListIterator;

/**
 * Implementation of the container interface. Uses a linked list to store the
 * elements; so the order of the elements is fixed to the insertion order. 
 * The Shape is the addition of all element shapes.
 * @author Jrg Rdenauer
 */
public class Container extends Component implements fdda.graphics.Container {

  /** the elements in the container */
  protected LinkedList elements;

  /**
   * Constructor. Container is empty.
   */
  public Container() {
    super(0.0, 0.0);
    this.elements = new LinkedList();
    this.shape = new Area();
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.graphics.Container#addElement(fdda.graphics.Component)
   */
  public void addElement(fdda.graphics.Component element) {
    this.elements.addLast(element);
    ((Area) this.shape).add(new Area(element.getShape()));
  }

  /*
   *  (non-Javadoc)
   * @see fdda.graphics.Container#removeElement(fdda.graphics.Component)
   */
  public void removeElement(fdda.graphics.Component element) {
    this.elements.remove(element);
    dirtyShape = true;
  }

  /*
   *  (non-Javadoc)
   * @see fdda.graphics.Container#hasElement(fdda.graphics.Component)
   */
  public boolean hasElement(fdda.graphics.Component element) {
    return this.elements.contains(element);
  }

  /*
   *  (non-Javadoc)
   * @see fdda.graphics.Container#getIterator()
   */
  public ListIterator getIterator() {
    return this.elements.listIterator(0);
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.graphics.Container#clear()
   */
  public void clear() {
    this.elements.clear();
    this.shape = new Area();
  }
  
  /**
   * Moves all elements in the container.
   * 
   * @see fdda.graphics.Component#move(double, double)
   */
  public void move(double x, double y) {
    ListIterator iter = this.getIterator();
    while (iter.hasNext()) {
      fdda.graphics.Component comp = (fdda.graphics.Component) iter.next();
      comp.move(x, y);
    }
    dirtyShape = true;
  }
  
  /**
   * Selects all elements in the container.
   * @see fdda.graphics.Component#select()
   */
  public void select() {
    ListIterator iter = this.getIterator();
    while (iter.hasNext()) {
      ((fdda.graphics.Component) iter.next()).select();
    }
  }
  
  /**
   * Deselects all elements in the container.
   * @see fdda.graphics.Component#deselect()
   */
  public void deselect() {
    ListIterator iter = this.getIterator();
    while (iter.hasNext()) {
      ((fdda.graphics.Component) iter.next()).deselect();
    }
  }

  /**
   * Draws all elements in the container. They are drawn in the order of their
   * appearance in the container.
   * @see fdda.graphics.java2d.Component#draw(java.awt.Graphics2D)
   */
  public void draw(Graphics2D graphContext) {
    ListIterator iter = this.getIterator();
    while (iter.hasNext()) {
      Object component = iter.next();
      if (component instanceof Component) {
        ((Component) component).draw(graphContext);
      }
    }
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.graphics.java2d.Component#calcShape()
   */
  protected void calcShape() {
    this.shape = new Area();
    ListIterator iter = this.getIterator();
    while (iter.hasNext()) {
      fdda.graphics.Component comp = (fdda.graphics.Component) iter.next();
      ((Area) this.shape).add(new Area(comp.getShape()));
    }
    dirtyShape = false;
  }

  /*
   *  (non-Javadoc)
   * @see java.lang.Object#clone()
   */
  public Object clone() {
    Container newCont = new Container();
    ListIterator iter = this.getIterator();
    while (iter.hasNext()) {
      fdda.graphics.Component comp = (fdda.graphics.Component) iter.next();
      newCont.addElement((fdda.graphics.Component) comp.clone());
    }
    newCont.setGraphicalSystem(this.getGraphicalSystem());
    return newCont;
  }

  /*
   *  (non-Javadoc)
   * @see fdda.graphics.Container#getSize()
   */
  public int getSize() {
    return this.elements.size();
  }
  
  /**
   * Returns the leftmost x coordinate of all the components in the container.
   * @see fdda.graphics.Component#getX()
   */
  public double getX() {
    double x = Double.MAX_VALUE;
    ListIterator it = this.getIterator();
    while (it.hasNext()) {
      double cx = ((fdda.graphics.Component) it.next()).getX();
      if (cx < x) {
        x = cx;
      }
    }
    return x == Double.MAX_VALUE ? 0.0 : x;
  }

  /**
   * Returns the topmost y coordinate of all the components in the container
   * @see fdda.graphics.Component#getY()
   */
  public double getY() {
    double y = Double.MAX_VALUE;
    ListIterator it = this.getIterator();
    while (it.hasNext()) {
      double cy = ((fdda.graphics.Component) it.next()).getY();
      if (cy < y) {
        y = cy;
      }
    }
    return y == Double.MAX_VALUE ? 0.0 : y;
  }
  
  /**
   * Returns true iff all components in the container are movable.
   * @see fdda.graphics.Component#isMovable()
   */
  public boolean isMovable() {
    ListIterator it = this.getIterator();
    boolean movable = true;
    while (movable && it.hasNext()) {
      if (!((fdda.graphics.Component) it.next()).isMovable()) {
        movable = false;
      }
    }
    return movable;
  }

  /**
   * Returns the minimum of the SEL_XX values of all the components in the 
   * container. It is: SEL_NOTHING < SEL_EQUALS < SEL_MOVABLES < SEL_ALL
   * @see fdda.graphics.Component#selectableWith()
   */
  public int selectableWith() {
    ListIterator it = this.getIterator();
    int selectableWith = fdda.graphics.Component.SEL_ALL;
    while (selectableWith != fdda.graphics.Component.SEL_NOTHING && 
           it.hasNext()) {
      int compSel = ((fdda.graphics.Component) it.next()).selectableWith();
      if ((selectableWith == fdda.graphics.Component.SEL_ALL) ||
          (selectableWith == fdda.graphics.Component.SEL_MOVABLES &&
            (compSel == fdda.graphics.Component.SEL_NOTHING ||
             compSel == fdda.graphics.Component.SEL_EQUALS)
            ) ||
          (selectableWith == fdda.graphics.Component.SEL_EQUALS &&
            compSel == fdda.graphics.Component.SEL_NOTHING) ||
          (compSel == fdda.graphics.Component.SEL_NOTHING)) {
        selectableWith = compSel;
      }
    }
    return selectableWith;
  }
  
  /**
   * Sets the graphical system for all elements.
   * @see fdda.graphics.Component#setGraphicalSystem(fdda.graphics.GraphicalSystem)
   */
  public void setGraphicalSystem(GraphicalSystem graphicalSystem) {
    // alle Elemente ebenso
    super.setGraphicalSystem(graphicalSystem);
    ListIterator it = this.getIterator();
    while (it.hasNext()) {
      ((fdda.graphics.Component) it.next()).setGraphicalSystem(graphicalSystem);
    }
  }

  /*
   *  (non-Javadoc)
   * @see fdda.graphics.Container#flatten()
   */
  public fdda.graphics.Component flatten() {
    if (isFlatteningAllowed()) {
      // Container innen flachmachen
      fdda.graphics.Component component = null;
      ListIterator it = elements.listIterator(this.getSize());
      while (it.hasPrevious()) {
        component = (fdda.graphics.Component) it.previous();
        if (component instanceof Container &&
           ((Container) component).isFlatteningAllowed()) {
          it.remove();
          ListIterator it2 = ((Container) component).getIterator();
          while (it2.hasNext()) {
            it.add(it2.next());
          }
        }
      }
      return (this.getSize() == 1 ? component : this);
    }
    else {
      return this;
    }
  }
  
  /**
   * Returns true.
   * @see fdda.graphics.Container#isFlatteningAllowed()
   */
  public boolean isFlatteningAllowed() {
    return true;
  }

  /**
   * Calls the visitContainer method according to the pattern
   * @see fdda.graphics.Component#visit(fdda.graphics.ComponentVisitor)
   */
  public Object visit(fdda.graphics.ComponentVisitor visitor) {
    return visitor.visitContainer(this);
  }
  
}
