/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/

package fdda.graphics.java2d;

import fdda.dialog.java2d.GraphicHolder;

import fdda.core.projects.Project;

import java.awt.Graphics2D;
import java.awt.Shape;

import java.awt.geom.Rectangle2D;
import java.awt.geom.Area;

import java.util.Collection;
import java.util.Collections;
import java.util.LinkedList;
import java.util.TreeSet;
import java.util.Iterator;
import java.util.Comparator;
import java.util.HashMap;

/**
 * Represents a graphical system - that means mostly nodes and lines. These 
 * are stored both in a TreeSet (so the order of their adding is retained) and 
 * a HashMap (wrapped in a special class), which allows a fast search.
 * Messages are just stored in a LinkedList, since they must mostly only be run
 * through to move and draw them.
 * @author Jrg Rdenauer
 */
public class GraphicalSystem implements fdda.graphics.GraphicalSystem {

  /** The nodes. Each node is wrapped in a ComponentWrapper. */
  protected TreeSet nodes;

  /** The lines. Each line is wrapped in a ComponentWrapper. */
  protected TreeSet lines;

  /** map for fast finding of nodes */
  protected HashMap nodeMap;

  /** map for fast finding of lines */
  protected HashMap lineMap;
  
  /** the messages */
  protected LinkedList messages;

  /** was the system changed since the last save operation? */
  protected boolean changed;

  /** ID for the the next node */
  protected int nodeID;

  /** ID for the next line */
  protected int lineID;

  /** Comparator to compare two wrapped components */
  protected Comparator comparator;

  /** the project that contains the system */
  private Project project = null;

  /** the graphic holder of the system */
  private GraphicHolder graphicHolder;
  
  /**
   * Constructor
   */
  public GraphicalSystem() {
    this.comparator = new WrapperComparator();
    this.nodes = new TreeSet(comparator);
    this.lines = new TreeSet(comparator);
    this.nodeMap = new HashMap();
    this.lineMap = new HashMap();
    this.messages = new LinkedList();
    this.nodeID = 0;
    this.lineID = 0;
    setChanged(false);
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.graphics.GraphicalSystem#setProject(fdda.core.projects.Project)
   */
  public void setProject(Project project) {
    this.project = project;
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.graphics.GraphicalSystem#getProject()
   */
  public Project getProject() {
    return project;
  }
  

  /*
   *  (non-Javadoc)
   * @see fdda.graphics.GraphicalSystem#addNode(fdda.graphics.Node)
   */
  public void addNode(fdda.graphics.Node node) {
    Integer newID = new Integer(nodeID++);
    ComponentWrapper cw = new ComponentWrapper(node, newID);
    nodes.add(cw);
    nodeMap.put(new Integer(node.hashCode()), cw);
  }

  /*
   *  (non-Javadoc)
   * @see fdda.graphics.GraphicalSystem#addLine(fdda.graphics.Line)
   */
  public void addLine(fdda.graphics.Line line) {
    Integer newID = new Integer(lineID++);
    ComponentWrapper cw = new ComponentWrapper(line, newID);
    lines.add(cw);
    lineMap.put(new Integer(line.hashCode()), cw);
  }

  /*
   *  (non-Javadoc)
   * @see fdda.graphics.GraphicalSystem#removeNode(fdda.graphics.Node)
   */
  public void removeNode(fdda.graphics.Node node) {
    // Searching for a node: calculate hashCode, find the node in the map
    Integer code = new Integer(node.hashCode());
    ComponentWrapper cw = (ComponentWrapper) nodeMap.get(code);
    nodes.remove(cw);
    nodeMap.remove(code);
  }

  /*
   *  (non-Javadoc)
   * @see fdda.graphics.GraphicalSystem#removeLine(fdda.graphics.Line)
   */
  public void removeLine(fdda.graphics.Line line) {
    Integer code = new Integer(line.hashCode());
    ComponentWrapper cw = (ComponentWrapper) lineMap.get(code);
    lines.remove(cw);
    lineMap.remove(code);
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.graphics.GraphicalSystem#getElementAt(double, double)
   */
  public fdda.graphics.Component getElementAt(double x, double y) {
    fdda.graphics.Component element = null;  // the element
    // search nodes first (they are on top)
    // make an array, since we must search backwards
    Object[] nodesArray = nodes.toArray();
    for (int i = nodesArray.length; element == null && i > 0; i--) {
      ComponentWrapper cw = (ComponentWrapper) nodesArray[i-1];
      fdda.graphics.Node node = (fdda.graphics.Node) cw.getComponent();
      if (node.getShape().contains(x, y)) {
        element = node;
      }
    }
    // no node found? Look in the lines (and also their points)
    Object[] linesArray = lines.toArray();
for_loop:
    for (int i = linesArray.length; element == null && i > 0; i--) {
      ComponentWrapper cw = (ComponentWrapper) linesArray[i-1];
      Line line = (Line) cw.getComponent();
      // first check the points
      Iterator it = line.getIterator();
      while (it.hasNext()) {
        Component c = (Component) it.next();
        if ((c instanceof Point) && c.getShape().contains(x, y)) {
          element = c;
          break for_loop;
        }
      }
      // then the shape of the line itself
      if (line.getShape().contains(x, y)) {
        element = line;
        break;
      }
    }
    return element;
  }

  /*
   *  (non-Javadoc)
   * @see fdda.graphics.GraphicalSystem#getMovableComponentsInArea(java.awt.geom.Area)
   */
  public fdda.graphics.Component getMovableComponentsInArea(Area area) {
    // take all movable components and then remove those not in the area
    // that's perhaps a bit slower, but it's also more easy
    fdda.graphics.Component movables = getMovableComponents();
    if (movables == null) {
      return null;
    }
    if (!(movables instanceof fdda.graphics.Container)) { // only 1 component
      Area componentShape = new Area(movables.getShape());
      componentShape.intersect(area);
      return componentShape.isEmpty() ? null : movables;
    }
    // copy to an array to avoid ConcurrentModification errors
    fdda.graphics.Component[] movablesArray = new Component[
                                ((fdda.graphics.Container) movables).getSize()];
    Iterator it = ((fdda.graphics.Container) movables).getIterator();
    for (int i = 0; i < movablesArray.length; i++) {
      assert it.hasNext();
      movablesArray[i] = (fdda.graphics.Component) it.next();
    }
    for (int i = 0; i < movablesArray.length; i++) {
      fdda.graphics.Component component = movablesArray[i];
      Area componentShape = new Area(component.getShape());
      componentShape.intersect(area);
      if (componentShape.isEmpty()) {
        ((fdda.graphics.Container) movables).removeElement(component);
      }
    }
    return movables;
  }

  /**
   * Draws the system - first the lines (and their points), than the nodes, 
   * finally the messages.
   * @param graphContext the graphics
   */
  public void draw(Graphics2D graphContext) {
    Iterator it;  // Iterator to run through all the data structures
    // lines
    it = lines.iterator();
    while (it.hasNext()) {
      fdda.graphics.Component c = ((ComponentWrapper) it.next()).getComponent();
      if (c instanceof Component) ((Component) c).draw(graphContext);
    }
    // dnodes
    it = nodes.iterator();
    while (it.hasNext()) {
      fdda.graphics.Component c = ((ComponentWrapper) it.next()).getComponent();
      if (c instanceof Component) ((Component) c).draw(graphContext);
    }
    // messages - synchronized since the demo thread may be adding or removing
    synchronized(this) {
      it = messages.iterator();
      while (it.hasNext()) {
        fdda.graphics.Message message = (fdda.graphics.Message) it.next();
        if (message instanceof Component) 
          ((Component) message).draw(graphContext);
      }
    }
  }

  /*
   *  (non-Javadoc)
   * @see fdda.graphics.GraphicalSystem#getEnclosingRectangle()
   */
  public Rectangle2D getEnclosingRectangle() {
    Iterator it; // Iterator to run through the data structures
    Shape componentShape; // shape of a component
    Area area = new Area();
    // nodes
    it = nodes.iterator();
    while (it.hasNext()) {
      componentShape = ((ComponentWrapper) it.next()).getComponent().getShape();
      area.add(new Area(componentShape));
    }
    // lines
    it = lines.iterator();
    while (it.hasNext()) {
      componentShape = ((ComponentWrapper) it.next()).getComponent().getShape();
      area.add(new Area(componentShape));
    }
    // messages - synchronized since the demo thread may be adding or removing
    synchronized (this) {
      it = messages.iterator();
      while (it.hasNext()) {
        componentShape = ((fdda.graphics.Message) it.next()).getShape();
        area.add(new Area(componentShape));
      }
    }
    return area.getBounds2D();
  }
  
  /**
   * Returns the graphic holder for the system
   * @return the graphic holder
   */
  public GraphicHolder getGraphicHolder() {
    return graphicHolder;
  }
  
  /**
   * sets the graphic holder for the system
   * @param holder the graphic holder
   */
  public void setGraphicHolder(GraphicHolder holder) {
    this.graphicHolder = holder;
  }

  /*
   *  (non-Javadoc)
   * @see fdda.graphics.GraphicalSystem#recalculateShapes()
   */
  public void recalculateShapes() {
    Iterator it = nodes.iterator();
    while (it.hasNext()) {
      ((ComponentWrapper) it.next()).getComponent().recalculateShape();
    }
    it = lines.iterator();
    while (it.hasNext()) {
      ((ComponentWrapper) it.next()).getComponent().recalculateShape();
    }
    // messages - synchronized since the demo thread may be adding or removing
    synchronized (this) {
      it = messages.iterator();
      while (it.hasNext()) {
        ((fdda.graphics.Message) it.next()).recalculateShape();
      }
    }
  }

  
  /*
   *  (non-Javadoc)
   * @see fdda.graphics.GraphicalSystem#clear()
   */
  public void clear() {
    this.nodes.clear();
    this.lines.clear();
    this.nodeMap.clear();
    this.lineMap.clear();
    this.nodeID = 0;
    this.lineID = 0;
    clearMessages();
  }

  /*
   *  (non-Javadoc)
   * @see fdda.graphics.GraphicalSystem#getMovableComponents()
   */
  public fdda.graphics.Component getMovableComponents() {
    final Container everything = new Container();
    Java2DComponentVisitor visitor = new Java2DComponentVisitor() {
      public Object visitComponent(fdda.graphics.Component c) {
        if (c.isMovable()) everything.addElement(c);
        return null;
      }
      
      public Object visitNode(fdda.graphics.Node node) {
        everything.addElement(node);
        return null;
      }
      
      public Object visitLine(fdda.graphics.Line line) {
        return null;
      }
      
      public Object visitPoint(Point point) {
        everything.addElement(point);
        return null;
      }
      
      public Object visitLineSegment(LineSegment lineSegment) {
        return null;
      }
      
      public Object visitContainer(fdda.graphics.Container c) {
        Iterator it = c.getIterator();
        while (it.hasNext()) {
          ((Component) it.next()).visit(this);
        }
        return null;
      }
      
      public Object visitMessage(fdda.graphics.Message message) {
        return null;
      }
    };
    Iterator it = nodes.iterator();
    while (it.hasNext()) {
      ((ComponentWrapper) it.next()).getComponent().visit(visitor);
    }
    it = lines.iterator();
    while (it.hasNext()) {
      ((ComponentWrapper) it.next()).getComponent().visit(visitor);
    }
    return (everything.getSize() > 0 ? everything.flatten() : null);
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.graphics.GraphicalSystem#wasChanged()
   */
  public boolean wasChanged() {
    return changed;
  }

  /*
   *  (non-Javadoc)
   * @see fdda.graphics.GraphicalSystem#setChanged(boolean)
   */
  public void setChanged(boolean changed) {
    this.changed = changed;
    // graphic changed -> project changed
    if (changed) getProject().setChanged(true);
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.graphics.GraphicalSystem#addMessage(fdda.graphics.Message)
   */
  public synchronized void addMessage(fdda.graphics.Message message) {
    messages.addLast(message);
    message.setGraphicalSystem(this);
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.graphics.GraphicalSystem#removeMessage(fdda.graphics.Message)
   */
  public synchronized void removeMessage(fdda.graphics.Message message) {
    messages.remove(message);
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.graphics.GraphicalSystem#clearMessages()
   */
  public synchronized void clearMessages() {
    messages.clear();
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.graphics.GraphicalSystem#getAllMessages()
   */
  public Collection getAllMessages() {
    return Collections.unmodifiableCollection(messages);
  }
  
  /**
   * Wraps a component: adds an attribute ID, enables comparing of the 
   * components so the TreeSet can be used to store them.
   */
  static class ComponentWrapper {

    /** the wrapped component */
    private fdda.graphics.Component component;

    /** the ID */
    private Integer id;

    /**
     * Constructor
     * @param component the component
     * @param id the ID
     */
    public ComponentWrapper(fdda.graphics.Component component, Integer id) {
      this.component = component;
      this.id = id;
    }

    /**
     * Returns the wrapped component
     * @return the component
     */
    public fdda.graphics.Component getComponent() {
      return component;
    }
  
    /**
     * Sets the wrapped component
     * @param component the component
     */
    public void setComponent(fdda.graphics.Component component) {
      this.component = component;
    }
  
    /**
     * Returns the ID
     * @return the ID
     */
    public Integer getID() {
      return id;
    }
  
    /**
     * Sets the ID
     * @param id the ID
     */
    public void setID(Integer id) {
      this.id = id;
    }
  
  }

  /**
   * Compares two ComponentWrappers by using their IDs.
   */
  static class WrapperComparator implements Comparator {

    /*
     *  (non-Javadoc)
     * @see java.util.Comparator#compare(java.lang.Object, java.lang.Object)
     */
    public int compare(Object o1, Object o2) {
      if (o1.equals(o2)) {
        return 0;
      }
      else if (((ComponentWrapper) o1).getID().intValue() <
               ((ComponentWrapper) o2).getID().intValue()) {
        return -1;
      }
      else if (((ComponentWrapper) o1).getID().intValue() >
               ((ComponentWrapper) o2).getID().intValue()) {
        return 1;
      }
      else {
        // different components, but the same ID - can't happen!
        assert false;
        return 0;
      }
    }
  }

}
