/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/

package fdda.graphics.java2d;

import fdda.dialog.java2d.GraphicHolder;

import fdda.model.Connection;
import fdda.model.Property;
import fdda.model.PropertyListener;

import java.util.Iterator;
import java.util.ListIterator;
import java.util.prefs.Preferences;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.geom.Area;
import java.awt.geom.Line2D;

/**
 * A line between two nodes. The line consists of one or more segments, leading
 * from the start node through one ore more intermediate points to the end node. 
 * Implemented is the line as a container in which both the segments and the 
 * points (which are components) are stored. Since the points must be drawn 
 * after the segments (to be above them), the storage order is LineSegment
 * (LineSegment Point)*; i.e. a point is always directly after the two segments
 * it connects.
 * The coordinates of a line are the coordinates of its start point
 * @author Jrg Rdenauer
 */
public class Line extends Container implements fdda.graphics.Line {

  /** norm value for the speed of messages */
  public final static double NORM_DIST_PER_CYCLE = 3.0;

  /** x coordinate of the line end */
  private double endX;

  /** y coordinate of the line end */
  private double endY;

  /** color of the line */
  private Color color = Component.COLOR_NORMAL;

  /** is the line directed? */
  private boolean directed;

  /** is the line working? */
  private boolean working = true;

  /** the connection represented by this line */
  private fdda.model.Connection connection;

  /** shape of the red lines that indicate the line is not working */
  private Area redLines = new Area();

  /**
   * Constructor. Line isn't directed.
   * @param startX x coordinate of start point
   * @param startY y coordinate of start point
   * @param endX x coordinate of end point
   * @param endY y coordinate of end point
   */
  public Line(double startX, double startY, double endX, double endY) {
    super();
    this.endX = endX;
    this.endY = endY;
    this.x = startX;
    this.y = startY;
    // one segment first
    this.addElement(new LineSegment(startX, startY, endX, endY));
    this.setDirected(false);
  }

  /**
   * Overwritten again since the container class changes default implementation.
   * @see fdda.graphics.Component#getX()
   */
  public double getX() {
    return x;
  }
  
  /**
   * Overwritten again since the container class changes default implementation.
   * @see fdda.graphics.Component#getY()
   */
  public double getY() {
    return y;
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.graphics.Line#getLength()
   */
  public double getLength() {
    double length = 0.0;
    ListIterator it = getIterator();
    while (it.hasNext()) {
      Component component = (Component) it.next();
      if (component instanceof LineSegment) {
        length += ((LineSegment) component).getLength();
      }
    }
    return length;
  }

  /*
   *  (non-Javadoc)
   * @see fdda.graphics.Line#getEndX()
   */
  public double getEndX() {
    return endX;
  }

  /*
   *  (non-Javadoc)
   * @see fdda.graphics.Line#getEndY()
   */
  public double getEndY() {
    return endY;
  }

  /*
   *  (non-Javadoc)
   * @see fdda.graphics.Line#setConnection(fdda.model.Connection)
   */
  public void setConnection(fdda.model.Connection connection) {
    this.connection = connection;
  }

  /*
   *  (non-Javadoc)
   * @see fdda.graphics.Line#getConnection()
   */
  public fdda.model.Connection getConnection() {
    return connection;
  }

  /*
   *  (non-Javadoc)
   * @see fdda.graphics.WorkingPart#setWorking(boolean)
   */
  public void setWorking(boolean working) {
    boolean oldWorking = this.working;
    this.working = working;
    if (working != oldWorking) {
      // refresh the graphics
      GraphicalSystem system = (GraphicalSystem) this.getGraphicalSystem();
      if (system != null) {
        GraphicHolder holder = system.getGraphicHolder();
        if (holder != null) {
          Painter painter
            = new Painter(holder);
          painter.setClippingShape(this.getShape());
          painter.repaintSystem();
          painter.updateScreen();
        }
      }
    }      
  }

  /*
   *  (non-Javadoc)
   * @see fdda.graphics.WorkingPart#isWorking()
   */
  public boolean isWorking() {
    return working;
  }

  /**
   * Adds a point to the line. The segment nearest to the new point coordinates 
   * is split.
   * @param x the x coordinate of the new point
   * @param y the y coordinate of the new point
   * @return the newly created point
   */
  public Point addPoint(double x, double y) {
    // look for the nearest segment, also store its position in the list
    LineSegment nearestLs = null;
    double minDistance = Double.POSITIVE_INFINITY;
    double lsDistance = Double.POSITIVE_INFINITY;
    Component component = null;
    int positionOfLs = 0;
    int itPos = 0;
    ListIterator it = getIterator();
    while (it.hasNext()) {
      component = (Component) it.next();
      if (component instanceof LineSegment) {
        lsDistance = distanceLinePoint((LineSegment) component, x, y);
        if (lsDistance < minDistance) {
          nearestLs = (LineSegment) component;
          minDistance = lsDistance;
          positionOfLs = itPos;
        }
      }
      itPos++;
    }
    assert nearestLs != null;
    // adding is done in extra method
    Point newPt = addPointInLS(nearestLs, positionOfLs, x, y);
    dirtyShape = true;
    return newPt;
  }

  /**
   * Removes a point from the line. The adjacent segments are merged.
   * @param point the point
   */
  public void removePoint(Point point) {
    // search the point
    Component component = null;
    ListIterator it = getIterator();
    while (it.hasNext()) {
      component = (Component) it.next();
      if (component instanceof Point && (Point) component == point) {
        // change the previous ls. The _next_ ls is in the list directly
        // _before_ the point; before _that_ is the previous ls, but there
        // may be another point between them
        it.previous();
        // now we are in the list before the point
        LineSegment nextLs = (LineSegment) it.previous();
        component = (Component) it.previous();
        if (component instanceof Point) {
          component = (Component) it.previous();
        }
        LineSegment prevLs = (LineSegment) component;
        prevLs.setEndXY(nextLs.getEndX(), nextLs.getEndY());
        // remove the next ls and the point
        elements.remove(nextLs);
        elements.remove(point);
        dirtyShape = true;
        break;
      }
    }
  }

  /**
   * Selects only the line segements
   * @see fdda.graphics.Component#select()
   */
  public void select() {
    Component component = null;
    ListIterator it = getIterator();
    while (it.hasNext()) {
      component = (Component) it.next();
      if (component instanceof LineSegment) {
        component.select();
      }
    }
    this.selected = true;
  }

  /**
   * Deselects only the line segments
   * @see fdda.graphics.Component#deselect()
   */
  public void deselect() {
    Component component = null;
    ListIterator it = getIterator();
    while (it.hasNext()) {
      component = (Component) it.next();
      if (component instanceof LineSegment) {
        component.deselect();
      }
    }
    this.selected = false;
  }

  /*
   *  (non-Javadoc)
   * @see fdda.graphics.Line#setColor(java.awt.Color)
   */
  public void setColor(Color color) {
    this.color = color;
  }

  /*
   *  (non-Javadoc)
   * @see fdda.graphics.Line#getColor()
   */
  public Color getColor() {
    return color;
  }

  /**
   * Moves all line segments and all points. The start and end point have to be
   * moved separately.
   * @see fdda.graphics.Component#move(double, double)
   */
  public void move(double x, double y) {
    Component component = null;
    ListIterator it = this.getIterator();
    while (it.hasNext()) {
      component = (Component) it.next();
      component.move(x, y);
    }
    // reset the start and end point to previous values
    setXY(getX(), getY());
    setEndXY(getEndX(), getEndY());
    dirtyShape = true;
  }

  /*
   *  (non-Javadoc)
   * @see fdda.graphics.Line#moveStartPoint(double, double)
   */
  public void moveStartPoint(double x, double y) {
    this.x += x;
    this.y += y;
    // the start point is at the first line segment
    ((LineSegment) elements.getFirst()).moveIn(x, y);
    dirtyShape = true;
  }

  /*
   *  (non-Javadoc)
   * @see fdda.graphics.Line#moveEndPoint(double, double)
   */
  public void moveEndPoint(double x, double y) {
    this.endX += x;
    this.endY += y;
    // the end point is at the last line segment. This is either the last 
    // component in the list or the next-to-last, if the last is a Point.
    ListIterator it = elements.listIterator(this.getSize());
    Component component = (Component) it.previous();
    if (!(component instanceof LineSegment)) {
      component = (Component)it.previous();
    }
    ((LineSegment) component).moveOut(x, y);
    dirtyShape = true;
  }

  /**
   * Informs the line that a point has been moved. Adjusts the corresponding 
   * segments
   * @param point the point
   */
  public void pointMoved(Point point) {
    // look for hte point
    ListIterator it = elements.listIterator();
    boolean found = false;
    while (it.hasNext() && !found) {
      if (it.next() == point) {
        found = true;
        it.previous();
      }
    }
    // the next ls is directly before the point in the list
    LineSegment followingLS = (LineSegment) it.previous();
    followingLS.moveIn(point.getX() - followingLS.getX(),
                       point.getY() - followingLS.getY());
    // the previous ls is either one or two elements before it
    LineSegment precedingLS = null;
    assert it.hasPrevious();
    if (it.previous() instanceof LineSegment) {
      precedingLS = (LineSegment) it.next();
    }
    else {
      assert it.hasPrevious();
      precedingLS = (LineSegment) it.previous();
    }
    precedingLS.moveOut(point.getX() - precedingLS.getEndX(),
                        point.getY() - precedingLS.getEndY());
    dirtyShape = true;
  }


  /*
   *  (non-Javadoc)
   * @see fdda.graphics.Line#setDirected(boolean)
   */
  public void setDirected(boolean directed) {
    boolean oldValue = this.directed;
    this.directed = directed;
    if (oldValue != directed) {
      // refresh the graphics
      GraphicalSystem system = (GraphicalSystem) this.getGraphicalSystem();
      if (system != null) {
        GraphicHolder holder = system.getGraphicHolder();
        if (holder != null) {
          Painter painter
            = new Painter(holder);
          painter.setClippingShape(this.getShape());
          painter.repaintSystem();
          painter.updateScreen();
        }
      }
    }
  }

  /*
   *  (non-Javadoc)
   * @see fdda.graphics.Line#isDirected()
   */
  public boolean isDirected() {
    return directed;
  }

  /*
   *  (non-Javadoc)
   * @see java.lang.Object#clone()
   */
  public Object clone() {
    Line newLine = new Line(x, y, endX, endY);
    // the constructor has added a line segment automatically. This must be 
    // removed, since we copy all our segments
    LineSegment falseLS = (LineSegment) newLine.getIterator().next();
    newLine.removeElement(falseLS);
    Component component = null;
    ListIterator it = getIterator();
    while (it.hasNext()) {
      component = (Component) it.next();
      newLine.addElement((Component) component.clone());
    }
    return newLine;
  }

  /*
   *  (non-Javadoc)
   * @see fdda.graphics.java2d.Component#draw(java.awt.Graphics2D)
   */
  public void draw(Graphics2D graphContext) {
    // super.draw draws all segments and points
    super.draw(graphContext);
    // arrowhead
    // don't draw arrowhead while line is constructed by user
    if (this.isDirected() && getConnection() != null) {
      // calculate direction of last segment
      ListIterator it = this.elements.listIterator(this.getSize());
      LineSegment lastSegment = null;
      while (it.hasPrevious() && lastSegment == null) {
        Component comp = (Component) it.previous();
        if (comp instanceof LineSegment) {
          lastSegment = (LineSegment) comp;
        }
      }
      double xv = lastSegment.getEndX() - lastSegment.getX();
      double yv = lastSegment.getEndY() - lastSegment.getY();
      // normalize the vector
      double length = Math.sqrt(xv * xv + yv * yv);
      xv /= length;
      yv /= length;
      // point of the arrowhead: end point - v * radius of the node
      double radius = ((Node)
        getConnection().getEndNode().getGraphicalRepresentation()).getRadius();
      double px = getEndX() - xv * radius;
      double py = getEndY() - yv * radius;
      final double angle = 20.0 * Math.PI / 180.0; // opening angle of the arrow
      final double plength = 10.0; // length of the head lines
      // angle of the last segment
      double alpha = xv != 0.0 ? Math.atan(yv / xv) : yv > 0 ? 0.0 : Math.PI;
      double beta1 = alpha + angle; // betas: angles of the head lines
      // calculate the end points of the head lines. Some geometrics here.
      double wx1 = xv == 0.0 ? -Math.sin(beta1) :
                   xv > 0 ? Math.cos(beta1) : -Math.cos(beta1);
      double wy1 = xv == 0.0 ? Math.cos(beta1) :
                   xv >= 0 ? Math.sin(beta1) : -Math.sin(beta1);
      double qx1 = px - wx1 * plength; // q: end point
      double qy1 = py - wy1 * plength;
      double beta2 = alpha - angle;
      double wx2 = xv == 0.0 ? -Math.sin(beta2) :
                   xv > 0 ? Math.cos(beta2) : -Math.cos(beta2);
      double wy2 = xv == 0.0 ? Math.cos(beta2) :
                   xv > 0 ? Math.sin(beta2) : -Math.sin(beta2);
      double qx2 = px - wx2  * plength;
      double qy2 = py - wy2 * plength;
      Color oldColor = graphContext.getColor();
      graphContext.setColor(this.isSelected() ? COLOR_SELECTED : COLOR_NORMAL);
      graphContext.draw(new Line2D.Double(px, py, qx1, qy1));
      graphContext.draw(new Line2D.Double(px, py, qx2, qy2));
      graphContext.setColor(oldColor);
    }
    // drawing Symbol for showing that Line isn't working
    // these are two short red line crossing out the line
    if (!isWorking()) {
      // find 'middle' LS
      double halfLength = getLength() / 2;
      ListIterator it = getIterator();
      LineSegment theSegment = null;
      double length = 0.0;
      while (it.hasNext() && (length < halfLength)) {
        Component component = (Component) it.next();
        if (component instanceof LineSegment) {
          theSegment = (LineSegment) component;
          length += theSegment.getLength();
        }
      }
      assert theSegment != null;
      double xv = theSegment.getEndX() - theSegment.getX();
      double yv = theSegment.getEndY() - theSegment.getY();
      // normalize the vector
      double l = Math.sqrt(xv * xv + yv * yv);
      xv /= l;
      yv /= l;
      // point between the two red lines
      double restLength = halfLength - length + theSegment.getLength();
      double xp = theSegment.getX() + restLength * xv;
      double yp = theSegment.getY() + restLength * yv;
      // constants
      double RED_LINE_DIST = 3.0;
      double RED_LINE_LENGTH = 20.0;
      double distFactor = Math.sqrt(RED_LINE_DIST * RED_LINE_DIST / l * l);
      double redLineFactor = 
        Math.sqrt(RED_LINE_LENGTH * RED_LINE_LENGTH / l * l);
      Line2D line1 = 
        new Line2D.Double(xp + distFactor * xv - redLineFactor / 2 * yv,
                          yp + distFactor * yv + redLineFactor / 2 * xv,
                          xp + distFactor * xv + redLineFactor / 2 * yv,
                          yp + distFactor * yv - redLineFactor / 2 * xv);
      Line2D line2 = 
        new Line2D.Double(xp - distFactor * xv - redLineFactor / 2 * yv,
                          yp - distFactor * yv + redLineFactor / 2 * xv,
                          xp - distFactor * xv + redLineFactor / 2 * yv,
                          yp - distFactor * yv - redLineFactor / 2 * xv);
      redLines = new Area(line1.getBounds2D());
      redLines.add(new Area(line2.getBounds2D()));
      Color oldColor = graphContext.getColor();
      graphContext.setColor(Color.RED);
      graphContext.draw(line1);
      graphContext.draw(line2);
      graphContext.setColor(oldColor);
    }
  }

  /*
   *  (non-Javadoc)
   * @see fdda.graphics.Line#setEndXY(double, double)
   */
  public void setEndXY(double x, double y) {
    endX = x;
    endY = y;
    // the line segment is either the last or the next-to-last element
    ListIterator it = elements.listIterator(this.getSize());
    Component component = (Component) it.previous();
    if (!(component instanceof LineSegment)) {
      component = (Component) it.previous();
    }
    ((LineSegment) component).setEndXY(x, y);
    dirtyShape = true;
  }
  
  /*
   *  (non-Javadoc)
   * Overwritten to update the line segment
   * @see fdda.graphics.Component#setXY(double, double)
   */
  public void setXY(double x, double y) {
    ((LineSegment) elements.getFirst()).setXY(x, y);
    super.setXY(x, y);
    dirtyShape = true;
  }

  /**
   * Returns false.
   * @see fdda.graphics.Container#isFlatteningAllowed()
   */
  public boolean isFlatteningAllowed() {
    return false;
  }

  /**
   * Returns SEL_EQUALS
   * @see fdda.graphics.Component#selectableWith()
   */
  public int selectableWith() {
    return Component.SEL_EQUALS;
  }

  /**
   * Returns false.
   * @see fdda.graphics.Component#isMovable()
   */
  public boolean isMovable() {
    return false;
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.graphics.java2d.Component#calcShape()
   */
  public void calcShape() {
    super.calcShape();
    if (!isWorking()) {
      Area newShape = new Area(this.shape);
      newShape.add(redLines);
      this.shape = newShape;
    }
  }

  /**
   * Returns the x coordinate of start end point for messages
   * @return the x coordinate of start end point for messages
   */
  public double getMessageStartX() {
    return getX();
  }

  /**
   * Returns the x coordinate of the end point for messages
   * @return the x coordinate of the end point for messages
   */
  public double getMessageEndX() {
    return getEndX();
  }

  /**
   * Returns the y coordinate of the start point for messages
   * @return the y coordinate of the start point for messages
   */
  public double getMessageStartY() {
    return getY();
  }

  /**
   * Returns the y coordinate of the end point for messages
   * @return the y coordinate of the end point for messages
   */
  public double getMessageEndY() {
    return getEndY();
  }

  /**
   * Returns how far a message moves in one cycle.
   * @return the distance a message moves in one cycle
   */  
  public double getMessageDistancePerCycle() {
    double speed = 
      ((Double) getConnection().getProperty(Connection.SPEED_PROPERTY_NAME)
      .getValue()).doubleValue();
    double normSpeed = Preferences.userNodeForPackage(getClass()).getDouble(
      "messageSpeed", NORM_DIST_PER_CYCLE);
    return normSpeed * speed;
  }
  
  /**
   * Moves a message on the line. How far it is moved is determined by the 
   * general speed of messages and the special line speed.
   * @param message the message
   */
  public void moveMessage(Message message) {
    if (message.doesMoveInLineDirection()) {
      moveMessageForward(message);
    }
    else {
      moveMessageBackward(message);
    }
  }

  // PropertyHolder methods - proxy
  
  /*
   *  (non-Javadoc)
   * @see fdda.model.PropertyHolder#addProperty(fdda.model.Property)
   */
  public void addProperty(Property property) {
    connection.addProperty(property);
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.model.PropertyHolder#removeProperty(fdda.model.Property)
   */
  public void removeProperty(Property property) {
    connection.removeProperty(property);
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.model.PropertyHolder#changeProperty(fdda.model.Property)
   */
  public void changeProperty(Property property) {
    connection.changeProperty(property);
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.model.PropertyHolder#getProperty(java.lang.String)
   */
  public Property getProperty(String name) {
    return connection.getProperty(name);
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.model.PropertyHolder#getProperties()
   */
  public Iterator getProperties() {
    return connection.getProperties();
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.model.PropertyHolder#addPropertyListener(fdda.model.PropertyListener)
   */
  public void addPropertyListener(PropertyListener pl) {
    connection.addPropertyListener(pl);
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.model.PropertyHolder#removePropertyListener(fdda.model.PropertyListener)
   */
  public void removePropertyListener(PropertyListener listener) {
    connection.removePropertyListener(listener);
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.model.PropertyHolder#firePropertyChanged(fdda.model.Property)
   */
  public void firePropertyChanged(Property property) {
    connection.firePropertyChanged(property);
  }


  /**
   * Calculates the distance between a line segment and a point. This is done by
   * putting an imaginary line orthogonally through to the segment through the 
   * point. If this line cuts the segment, return the distance to the 
   * intersection point; else return the distance to the nearest end point of 
   * the segment.
   * @param ls the line segment
   * @param pX the x coordinate of the point
   * @param pY the y coordinate of the point
   * @return distance between point and line segment
   */
  private double distanceLinePoint(LineSegment ls, double pX, double pY) {
    double distance = 0.0; // return value
    // short for the end points A and B
    double aX = ls.x;
    double aY = ls.y;
    double bX = ls.getEndX();
    double bY = ls.getEndY();
    // vector of the segment
    double vX = bX - aX;
    double vY = bY - aY;
    // length of the vector
    double l = Math.sqrt(vX * vX + vY * vY);
    // safety check
    if (l == 0.0) {
      distance = Math.sqrt((pX - aX) * (pX - aX) + (pY - aY) * (pY - aY));
    }
    else {
      // parameter: t * v + a = intersection point
      // formula can be derived easily with a (2,2)-LES
      double t = (pX * vX + pY * vY - aX * vX - aY * vY) / (vX * vX + vY * vY);
      // is the intersection point on th ls?
      if (t > 0.0 && t < 1.0) {
        // return the distance - derived by the same LES
        // d is the parameter for the imaginary line
        double d = vX == 0.0 ? ((aX + pX) / -vY) : ((t * vY + aY - pY) / vX);
        // normalize, take the absolute
        distance = Math.abs(d * Math.sqrt(vX * vX + vY * vY));
      }
      else {
        double d1 = Math.sqrt((pX - aX) * (pX - aX) + (pY - aY) * (pY - aY));
        double d2 = Math.sqrt((pX - aX) * (pX - aX) + (pY - aY) * (pY - aY));
        distance = (d1 < d2) ? d1 : d2;
      }
    }
    return distance;
  }

  /**
   * Adds a point in a segment. Splits this segment
   * @param lineSegment the segment
   * @param lsPos the position of the segment in the list
   * @param x the x coordinate of the new point
   * @param y the y coordinate of the new point
   * @return the newly created point
   */
  private Point addPointInLS(LineSegment lineSegment,
                             int lsPos, double x, double y) {
    // create the new components
    LineSegment newLs = new LineSegment(x, y, lineSegment.getEndX(),
      lineSegment.getEndY());
    Point newPt = new Point(x, y);
    newPt.setGraphicalSystem(this.getGraphicalSystem());
    newPt.setLine(this);
    // change the old ls
    lineSegment.setEndXY(x, y);
    // Insert into the list
    ListIterator it = elements.listIterator(lsPos + 1);
    // if the next component in the list is a point, insert after it
    if (it.hasNext() && !(it.next() instanceof Point)) {
      // there was a following component, but not a point -> go back
      it.previous();
    }
    it.add(newLs);
    it.add(newPt);
    return newPt;
  }
  
  /**
   * Moves a message in the direction of the line
   * @param message the message
   */
  private void moveMessageForward(Message message) {
    // look for current LineSegment. Fastest way: check how far the 
    // message has already come
    double dist = getMessageDistancePerCycle();
    double msgDist = message.getRealMessage().getCyclesSinceSending() * dist;
    ListIterator it = elements.listIterator();
    double length = 0.0; 
    LineSegment currentLS = null;
    while ((currentLS == null) && it.hasNext()) {
      Object element = it.next();
      if (element instanceof LineSegment) {
        length += ((LineSegment) element).getLength();
        if (length >= msgDist) currentLS = (LineSegment) element;
      }
    }
    if (currentLS == null) return;
    double msgX = message.getX();
    double msgY = message.getY();
    // will the message still be on this segment after moving?
    while (msgDist + dist > length) {
      // if not: move it virtually to the end of the segment
      dist -= (length - msgDist);
      msgDist = length;
      msgX = currentLS.getEndX();
      msgY = currentLS.getEndY();
      if (!it.hasNext()) return;
      Object element = it.next();
      if (element instanceof Point) {
        if (!it.hasNext()) return; // end of line reached
        element = it.next();
      }
      currentLS = (LineSegment) element;
      length += currentLS.getLength();
    }
    // determine next Point: direction of LS * remaining distance
    double lsX = currentLS.getEndX() - currentLS.getX();
    double lsY = currentLS.getEndY() - currentLS.getY();
    double nextMsgXOffset =  dist * lsX / currentLS.getLength();
    double nextMsgYOffset =  dist * lsY / currentLS.getLength();
    message.move(msgX + nextMsgXOffset - message.getX(), 
                 msgY + nextMsgYOffset - message.getY());
  }
  
  /**
   * Moves a message against the direction of the line
   * @param message the message
   */
  private void moveMessageBackward(Message message) {
    // look for current LineSegment. Fastest way: check how far the 
    // message has already come
    double dist = getMessageDistancePerCycle();
    double msgDist = message.getRealMessage().getCyclesSinceSending() * dist;
    ListIterator it = elements.listIterator(elements.size());
    double length = 0.0;
    LineSegment currentLS = null;
    while ((currentLS == null) && it.hasPrevious()) {
      Object element = it.previous();
      if (element instanceof LineSegment) {
        length += ((LineSegment) element).getLength();
        if (length >= msgDist) currentLS = (LineSegment) element;
      }
    }
    if (currentLS == null) return;
    double msgX = message.getX();
    double msgY = message.getY();
    // will the message still be on this segment after moving?
    while (msgDist + dist > length) {
      // if not: move it virtually to the start of the segment
      dist -= (length - msgDist);
      msgDist = length;
      msgX = currentLS.getX();
      msgY = currentLS.getY();
      if (!it.hasPrevious()) return;
      Object element = it.previous();
      if (element instanceof Point) element = it.previous();
      currentLS = (LineSegment) element;
      length += currentLS.getLength();
    }
    // determine next Point: anti-direction of LS * remaining distance
    double lsX = currentLS.getX() - currentLS.getEndX();
    double lsY = currentLS.getY() - currentLS.getEndY();
    double nextMsgXOffset =  dist * lsX / currentLS.getLength();
    double nextMsgYOffset =  dist * lsY / currentLS.getLength();
    message.move(msgX + nextMsgXOffset - message.getX(), 
                 msgY + nextMsgYOffset - message.getY());
  }

}
