/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/

package fdda.graphics.java2d;

import java.awt.Graphics2D;

import java.awt.Paint;
import java.awt.Stroke;
import java.awt.BasicStroke;

import java.awt.geom.Line2D;
import java.awt.geom.Area;
import java.awt.geom.GeneralPath;

/**
 * A LineSegment is a segment of a line, bordered either by a point or by a 
 * node. The coordinates are the coordinates of the start point. 
 * @author Jrg Rdenauer
 */
public class LineSegment extends Component {

  /** x coordinate of the end point */
  private double endX;

  /** y coordinate of the end point */
  private double endY;

  /**
   * Constructor
   * @param startX x coordinate of start point
   * @param startY y coordinate of start point
   * @param endX x coordinate of end point
   * @param endY y coordinate of end point
   */
  public LineSegment(double startX, double startY, double endX, double endY) {
    super(startX, startY);
    this.endX = endX;
    this.endY = endY;
    calcShape();
  }

  /*
   *  (non-Javadoc)
   * @see java.lang.Object#clone()
   */
  public Object clone() {
    LineSegment newLS = (LineSegment) super.clone();
    return newLS;
  }

  /*
   *  (non-Javadoc)
   * @see fdda.graphics.java2d.Component#draw(java.awt.Graphics2D)
   */
  public void draw(Graphics2D graphContext) {
    final float width = 1.0f;
    Line2D.Double line = new Line2D.Double(getX(), getY(),
        getEndX(), getEndY());
    Paint oldPaint = graphContext.getPaint();
    Stroke oldStroke = graphContext.getStroke();
    graphContext.setStroke(new BasicStroke(width, BasicStroke.CAP_ROUND,
                                           BasicStroke.JOIN_ROUND));
    graphContext.setPaint(this.isSelected() ? COLOR_SELECTED : COLOR_NORMAL);
    graphContext.draw(line);
    graphContext.setPaint(oldPaint);
    graphContext.setStroke(oldStroke);
  }

  /**
   * Returns the x coordinate of the end point
   * @return the x coordinate of the end point
   */
  public double getEndX() {
    return endX;
  }

  /**
   * Returns the y coordinate of the end point
   * @return the y coordinate of the end point
   */
  public double getEndY() {
    return endY;
  }
  
  /**
   * Returns the length of the segment
   * @return the length of the segment
   */
  public double getLength() {
    double xdiff = getX() - getEndX();
    double ydiff = getY() - getEndY();
    return Math.sqrt(xdiff * xdiff + ydiff * ydiff);
  }

  /**
   * Moves both start and end point.
   * @see fdda.graphics.Component#move(double, double)
   */
  public void move(double x, double y) {
    this.x += x;
    this.y += y;
    this.endX += x;
    this.endY += y;
    dirtyShape = true;
  }

  /**
   * Moves the start point.
   * @param x offset for x coordinate
   * @param y offset for y coordinate
   */
  public void moveIn(double x, double y) {
    this.x += x;
    this.y += y;
    dirtyShape = true;
  }

  /**
   * Moves the end point.
   * @param x offset for x coordinate
   * @param y offset for y coordinate
   */
  public void moveOut(double x, double y) {
    this.endX += x;
    this.endY += y;
    dirtyShape = true;
  }

  /*
   *  (non-Javadoc)
   * @see fdda.graphics.Component#setXY(double, double)
   */
  public void setXY(double x, double y) {
    super.setXY(x, y);
    dirtyShape = true;
  }

  /**
   * Resets the end point.
   * @param x the new x coordinate of the end point
   * @param y the new y coordinate of the end point
   */
  public void setEndXY(double x, double y) {
    endX = x;
    endY = y;
    dirtyShape = true;
  }

  /*
   *  (non-Javadoc)
   * @see fdda.graphics.java2d.Component#calcShape()
   */
  protected void calcShape() {
    // the shape is a rectangle enclosing the line segment
    final double D = 4.0; // distance to the line
    // line vector
    double lineVectorX = endX - this.getX();
    double lineVectorY = endY - this.getY();
    // its length
    double lineVectorLength = Math.sqrt(lineVectorX * lineVectorX
                                        + lineVectorY * lineVectorY);
    if (lineVectorLength == 0.0) {
      this.shape = new Area();
    }
    else {
      // normalize the vector
      lineVectorX /= lineVectorLength;
      lineVectorY /= lineVectorLength;
      // and stretch it to the distance
      lineVectorX *= D;
      lineVectorY *= D;
      // calculate the norm vector
      double normVectorX = lineVectorY;
      double normVectorY = -lineVectorX;
      // shape as a 'general path'
      GeneralPath gp = new GeneralPath(GeneralPath.WIND_EVEN_ODD, 4);
      gp.moveTo((float)(this.getX() - lineVectorX - normVectorX),
                (float)(this.getY() - lineVectorY - normVectorY));
      gp.lineTo((float)(this.getX() - lineVectorX + normVectorX),
                (float)(this.getY() - lineVectorY + normVectorY));
      gp.lineTo((float)(endX + lineVectorX + normVectorX),
                (float)(endY + lineVectorY + normVectorY));
      gp.lineTo((float)(endX + lineVectorX - normVectorX),
                (float)(endY + lineVectorY - normVectorY));
      gp.closePath();
      this.shape = gp;
    }
    dirtyShape = false;
  }

  /**
   * Returns false.
   * @see fdda.graphics.Component#isMovable()
   */
  public boolean isMovable() {
    return false;
  }

  /**
   * Returns SEL_EQUALS
   * @see fdda.graphics.Component#selectableWith()
   */
  public int selectableWith() {
    return Component.SEL_EQUALS;
  }

  /**
   * Calls visitLineSegment according to visitor pattern.
   * @see fdda.graphics.Component#visit(fdda.graphics.ComponentVisitor)
   */
  public Object visit(fdda.graphics.ComponentVisitor visitor) {
    if (visitor instanceof Java2DComponentVisitor) {
      return ((Java2DComponentVisitor) visitor).visitLineSegment(this);
    }
    else return null;
  }

}
