/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.graphics.java2d;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Shape;
import java.awt.geom.Rectangle2D;

import fdda.graphics.ComponentVisitor;

/**
 * Implementation of the message interface. The message is shown as a small 
 * rectangle.
 * @author Jrg Rdenauer
 */
public class Message extends Component implements fdda.graphics.Message {

  /** half the length of the message rectangle */  
  public static final double HALF_LENGTH = 16.0;
  
  /** half the width of the message rectangle */  
  public static final double HALF_WIDTH = 8.0;

  /** the default color for the border */  
  private static final Color DEFAULT_COLOR = Color.BLUE;

  /** the color */
  private Color color;
  
  /** the real message of the algorithm */
  private fdda.algorithm.Message realMessage;
  
  /** the line on which the message moves */
  private Line theLine;
  
  /** whether the message moves in the direction of the line */
  private boolean forward;
  
  /** the shape that must be drawn */
  private Shape shape2draw;

  /**
   * Constructor
   * @param line the line on which the message moves
   * @param forward whether the message moves forward
   */
  public Message(Line line, boolean forward) {
    super(forward ? line.getMessageStartX() : line.getMessageEndX(),
          forward ? line.getMessageStartY() : line.getMessageEndY());
    this.theLine = line;
    this.forward = forward;
    this.color = DEFAULT_COLOR;
  }
  
  /* (non-Javadoc)
   * @see fdda.graphics.java2d.Component#calcShape()
   */
  protected void calcShape() {
    double x = getX(); double y = getY();
    final double SAFETY = 8.0;
    this.shape = new Rectangle2D.Double(x - HALF_LENGTH - SAFETY, 
      y - HALF_WIDTH - SAFETY,
      2 * (HALF_LENGTH + SAFETY), 2 * (HALF_WIDTH + SAFETY));
    this.shape2draw = new Rectangle2D.Double(x - HALF_LENGTH, y - HALF_WIDTH,
      2 * HALF_LENGTH, 2 * HALF_WIDTH);
    dirtyShape = false;
  }

  /* (non-Javadoc)
   * @see fdda.graphics.java2d.Component#draw(java.awt.Graphics2D)
   */
  public void draw(Graphics2D graphContext) {
    Color oldColor = graphContext.getColor();
    // white inside
    graphContext.setColor(Color.WHITE);
    graphContext.fill(shape2draw);
    // colored border
    graphContext.setColor(color);
    graphContext.draw(shape2draw);
    // black text, centered
    String text = getRealMessage().getLabel();
    double height = ((GraphicalSystem) this.getGraphicalSystem()).
      getGraphicHolder().stringHeight();
    double width = ((GraphicalSystem) this.getGraphicalSystem()).
      getGraphicHolder().stringWidth(text);
    // cut text until it is short enough
    while ((width > 2 * (HALF_LENGTH - 2)) && (text.length() > 1)) {
      text = text.substring(0, text.length() - 2);
      width = ((GraphicalSystem) this.getGraphicalSystem()).
        getGraphicHolder().stringWidth(text);
    }
    graphContext.setColor(Color.BLACK);
    graphContext.drawString(text, (float) (getX() - (width / 2)), 
      (float) (getY() + (height / 2) - 2));
    graphContext.setColor(oldColor);
  }

  /* (non-Javadoc)
   * @see fdda.graphics.Message#getRealMessage()
   */
  public fdda.algorithm.Message getRealMessage() {
    return realMessage;
  }

  /* (non-Javadoc)
   * @see fdda.graphics.Message#setRealMessage(fdda.algorithm.Message)
   */
  public void setRealMessage(fdda.algorithm.Message message) {
    realMessage = message;
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.graphics.Message#doesMoveInLineDirection()
   */
  public boolean doesMoveInLineDirection() {
    return forward;
  }

  /* (non-Javadoc)
   * @see fdda.graphics.Message#moveForward()
   */
  public void moveForward() {
    theLine.moveMessage(this);
  }

  /**
   * Returns false.
   * @see fdda.graphics.Component#isMovable()
   */
  public boolean isMovable() {
    return false;
  }

  /* (non-Javadoc)
   * @see fdda.graphics.Component#move(double, double)
   */
  public void move(double xOffset, double yOffset) {
    this.x += xOffset;
    this.y += yOffset;
    dirtyShape = true;
  }

  /**
   * Returns SEL_NOTHING. But messages can't be selected anyway.
   * @see fdda.graphics.Component#selectableWith()
   */
  public int selectableWith() {
    return SEL_NOTHING;
  }

  /**
   * Calls visitMessage according to visitor pattern
   * @see fdda.graphics.Component#visit(fdda.graphics.ComponentVisitor)
   */
  public Object visit(ComponentVisitor visitor) {
    return visitor.visitMessage(this);
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.graphics.Message#getLine()
   */
  public fdda.graphics.Line getLine() {
    return theLine;
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.graphics.Message#setColor(java.awt.Color)
   */
  public void setColor(Color color) {
    this.color = color;
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.graphics.Message#getColor()
   */
  public Color getColor() {
    return color;
  }
}
