/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/

package fdda.graphics.java2d;

import fdda.dialog.java2d.GraphicHolder;
import fdda.graphics.ComponentVisitor;

import fdda.model.Property;
import fdda.model.PropertyListener;

import java.awt.Shape;
import java.awt.Graphics2D;
import java.awt.Color;

import java.awt.geom.Area;
import java.awt.geom.Ellipse2D;
import java.awt.geom.Line2D;

import java.util.Iterator;

/**
 * Implementation of the graphical representation of nodes for Java2D. A node is
 * drawn as a filled circle, the label inside. Not working nodes are crossed 
 * out.
 * @author Jrg Rdenauer
 */
public class Node extends Component implements fdda.graphics.Node {

  /** default radius for nodes */
  final static double RADIUS = 20.0;
  
  /** radius for repeaters */
  final static double REPEATER_RADIUS = 6.0;

  /** text color for selected nodes */
  private static final Color TEXT_SELECTED = Color.RED;

  /** normal text color */
  private static final Color TEXT_NORMAL = Color.BLACK;
  
  /** color for repeaters */
  private static final Color COLOR_REPEATER = Color.BLACK;

  /** color of the cross that shows not working nodes */
  public static final Color RED_CROSS_COLOR = new Color(239, 111, 0);

  /** the circle that is drawn (smaller than the shape */
  private Shape shapeToDraw;

  /** the model node */
  private fdda.model.Node modelNode;

  /** the primary color (with which the node is filled) */
  private Color primaryColor = Color.WHITE;

  /** the color of the border */
  private Color secondaryColor = Component.COLOR_NORMAL;

  /** shape of the red cross */
  private Area redCross = new Area();
  
  /** radius of the node */
  private double radius = RADIUS;

  /**
   * Default connstructor. Node is not selected.
   * @param x the x coordinate
   * @param y the y coordinate
   */
  public Node(double x, double y) {
    this(x, y, false);
  }

  /**
   * Constructor
   * @param x the x coordinate
   * @param y the y coordinate
   * @param selected whether the node shall be selected
   */
  public Node(double x, double y, boolean selected) {
    super(x, y, selected);
    dirtyShape = true;
  }

  /*
   *  (non-Javadoc)
   * @see java.lang.Object#clone()
   */
  public Object clone() {
    Node clone = (Node) super.clone();
    return clone;
  }

  /**
   * Returns true.
   * @see fdda.graphics.Component#isMovable()
   */
  public boolean isMovable() {
    return true;
  }

  /*
   *  (non-Javadoc)
   * @see fdda.graphics.Component#move(double, double)
   */
  public void move(double x, double y) {
    this.x += x;
    this.y += y;
    dirtyShape = true;
  }

  /**
   * Returns SEL_MOVABLES
   * @see fdda.graphics.Component#selectableWith()
   */
  public int selectableWith() {
    return SEL_MOVABLES;
  }

  /*
   *  (non-Javadoc)
   * @see fdda.graphics.java2d.Component#draw(java.awt.Graphics2D)
   */
  public void draw(Graphics2D graphContext) {
    Color oldColor = graphContext.getColor();
    if (dirtyShape) calcShape();
    // special case: repeaters
    boolean repeater = (modelNode != null) && modelNode.isRepeater();
    if (repeater) {
      graphContext.setColor(isSelected() ? COLOR_SELECTED : COLOR_REPEATER);
      graphContext.fill(this.shapeToDraw);
    }
    else {
      if (isSelected()) {
        graphContext.setColor(COLOR_SELECTED);
        graphContext.fill(this.shapeToDraw);
      }
      else {
        graphContext.setColor(primaryColor);
        graphContext.fill(this.shapeToDraw);
        graphContext.setColor(secondaryColor);
        graphContext.draw(this.shapeToDraw);
      }
      double height = ((GraphicalSystem) this.getGraphicalSystem()).
        getGraphicHolder().stringHeight();
      double width = ((GraphicalSystem) this.getGraphicalSystem()).
        getGraphicHolder().stringWidth(modelNode.getLabel());
      graphContext.setColor(isSelected() ? TEXT_SELECTED : TEXT_NORMAL);
      graphContext.drawString(modelNode.getLabel(), (float) (x - width / 2),
                              (float) (y + height / 2 - 3) );
    }
    if ((modelNode != null) && !modelNode.isWorking()) {
      // constant: line offset out of circle
      final double CROSS_LENGTH = repeater ? 1.0 : -2.0; 
      Line2D line1 = new Line2D.Double(x - radius - CROSS_LENGTH,
                                       y - radius - CROSS_LENGTH,
                                       x + radius + CROSS_LENGTH,
                                       y + radius + CROSS_LENGTH);
      Line2D line2 = new Line2D.Double(x + radius + CROSS_LENGTH,
                                       y - radius - CROSS_LENGTH,
                                       x - radius - CROSS_LENGTH,
                                       y + radius + CROSS_LENGTH);
      redCross = new Area(line1.getBounds2D());
      redCross.add(new Area(line2.getBounds2D()));
      graphContext.setColor(RED_CROSS_COLOR);
      graphContext.draw(line1);
      graphContext.draw(line2);
    }
    graphContext.setColor(oldColor);
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.graphics.Node#repaint()
   */
  public void repaint() {
    Area clippingShape = this.shape != null ? new Area(this.shape) : new Area();
    dirtyShape = true;
    clippingShape.add(new Area(this.getShape()));
    GraphicalSystem system = (GraphicalSystem) this.getGraphicalSystem();
    if (system != null) {
      GraphicHolder holder = system.getGraphicHolder();
      if (holder != null) {
        Painter painter
          = new Painter(holder);
        painter.setClippingShape(clippingShape);
        painter.repaintSystem();
        painter.updateScreen();
      }
    }
  }    

  /*
   *  (non-Javadoc)
   * @see fdda.graphics.Node#getLineStartX()
   */
  public double getLineStartX() {
    return getX();
  }

  /*
   *  (non-Javadoc)
   * @see fdda.graphics.Node#getLineStartY()
   */
  public double getLineStartY() {
    return getY();
  }
  
  /**
   * Returns the radius of the node
   * @return the radius
   */
  public double getRadius() {
    return radius;
  }

  /**
   * Calls visitNode according to visitor pattern
   * @see fdda.graphics.Component#visit(fdda.graphics.ComponentVisitor)
   */
  public Object visit(ComponentVisitor visitor) {
    return visitor.visitNode(this);
  }

  /*
   *  (non-Javadoc)
   * @see fdda.graphics.Node#setModelNode(fdda.model.Node)
   */
  public void setModelNode(fdda.model.Node modelNode) {
    this.modelNode = modelNode;
  }

  /*
   *  (non-Javadoc)
   * @see fdda.graphics.Node#getModelNode()
   */
  public fdda.model.Node getModelNode() {
    return modelNode;
  }

  /*
   *  (non-Javadoc)
   * @see fdda.graphics.Node#getPrimaryColor()
   */
  public Color getPrimaryColor() {
    return primaryColor;
  }

  /*
   *  (non-Javadoc)
   * @see fdda.graphics.Node#setPrimaryColor(java.awt.Color)
   */
  public void setPrimaryColor(Color color) {
    primaryColor = color;
    repaint();
  }

  /*
   *  (non-Javadoc)
   * @see fdda.graphics.Node#getSecondaryColor()
   */
  public Color getSecondaryColor() {
    return secondaryColor;
  }

  /*
   *  (non-Javadoc)
   * @see fdda.graphics.Node#setSecondaryColor(java.awt.Color)
   */
  public void setSecondaryColor(Color color) {
    secondaryColor = color;
    repaint();
  }
  
  // PropertyHolder methods - proxy

  /*
   *  (non-Javadoc)
   * @see fdda.model.PropertyHolder#addProperty(fdda.model.Property)
   */  
  public void addProperty(Property property) {
    modelNode.addProperty(property);
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.model.PropertyHolder#removeProperty(fdda.model.Property)
   */
  public void removeProperty(Property property) {
    modelNode.removeProperty(property);
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.model.PropertyHolder#changeProperty(fdda.model.Property)
   */
  public void changeProperty(Property property) {
    modelNode.changeProperty(property);
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.model.PropertyHolder#getProperty(java.lang.String)
   */
  public Property getProperty(String name) {
    return modelNode.getProperty(name);
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.model.PropertyHolder#getProperties()
   */
  public Iterator getProperties() {
    return modelNode.getProperties();
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.model.PropertyHolder#addPropertyListener(fdda.model.PropertyListener)
   */
  public void addPropertyListener(PropertyListener pl) {
    modelNode.addPropertyListener(pl);
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.model.PropertyHolder#removePropertyListener(fdda.model.PropertyListener)
   */
  public void removePropertyListener(PropertyListener listener) {
    modelNode.removePropertyListener(listener);
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.model.PropertyHolder#firePropertyChanged(fdda.model.Property)
   */
  public void firePropertyChanged(Property property) {
    modelNode.firePropertyChanged(property);
  }

  /*
   *  (non-Javadoc)
   * @see fdda.graphics.WorkingPart#isWorking()
   */
  public boolean isWorking() {
    return modelNode.isWorking();
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.graphics.WorkingPart#setWorking(boolean)
   */
  public void setWorking(boolean working) {
    if (working != modelNode.isWorking()) {
      modelNode.setWorking(working);
    }
  }

  /*
   *  (non-Javadoc)
   * @see fdda.graphics.java2d.Component#calcShape()
   */
  protected void calcShape() {
    if (getModelNode() == null) radius = RADIUS;
    else radius = (getModelNode().isRepeater()) ? REPEATER_RADIUS : RADIUS;
    final int OFFSET = 5; // safety: shape has a radius bigger than the circle
    this.shape =
      new Ellipse2D.Double(x - radius - OFFSET,
                           y - radius - OFFSET,
                           2 * (radius + OFFSET),
                           2 * (radius + OFFSET));
    if ((modelNode != null) && !modelNode.isWorking()) {
      Area newShape = new Area(this.shape);
      newShape.add(redCross);
      this.shape = newShape;
    }
    this.shapeToDraw =
      new Ellipse2D.Double(x - radius, y - radius, 2 * radius, 2 * radius);
    dirtyShape = false;
  }

}
