/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/

package fdda.graphics.java2d;

import fdda.dialog.java2d.GraphicHolder;
import fdda.utils.Logging;

import java.util.prefs.Preferences;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Paint;
import java.awt.Rectangle;
import java.awt.Shape;

import java.awt.geom.Area;
import java.awt.geom.Ellipse2D;

/**
 * Paints the graphical systems. There are two different general ways to do 
 * this: paint in the normal graphics thread of Java; or paint on a buffered 
 * image that is then drawn on the screen in this graphics thread. Currently, 
 * the first method is used. However, to ensure fast switching and 
 * changeability, there are two drawing methods: one that has a graphics object 
 * as a parameter, and one that doesn't. This last one does _nothing_ currently.
 * 
 * Still, whenever you want to redraw some part of the system, determine its 
 * shape and call - in this order - repaintSystem() and updateScreen(). At 
 * least, that's the _normal_ way of doing it ;-)
 * 
 * All important methods are synchronized; this prevents e.g. the clipping shape
 * from being changed while the painting is still in progress.
 * @author Jrg Rdenauer
 */
public class Painter {

  /** the color of the grid points */
  private static final Color GRID_COLOR = Color.BLACK;

  /** the radius of the grid points */
  private static final double GRID_POINT_RADIUS = 0.3;

  /** the object holding the graphics */
  private GraphicHolder graphicHolder;

  /** the old clipping shape */
  private Shape oldClippingShape;

  /** the actual clipping shape */
  private Shape clippingShape;

  /** the background color */
  private Color bgColor = Color.WHITE;

  /**
   * Constructor.
   * @param graphicHolder the object holding the graphic
   */
  public Painter(GraphicHolder graphicHolder) {
    this.graphicHolder = graphicHolder;
  }

  /**
   * Defines the clipping shape.
   * @param shape the clipping shape
   */
  public synchronized void setClippingShape(Shape shape) {
    clippingShape = shape;
  }
  
  /**
   * Adds another shape to the clipping shape.
   * @param shape the additional clipping shape
   */
  public synchronized void addClippingShape(Shape shape) {
    Area newShape = new Area(clippingShape);
    newShape.add(new Area(shape));
    clippingShape = newShape;
  }
  
  /**
   * Return the current clipping shape
   * @return the clipping shape
   */
  public synchronized Shape getClippingShape() {
    return clippingShape;
  }

  /**
   * Repaints the graphical system, using the current clipping shape.
   */
  public synchronized void repaintSystem() {
    clearBackground();
    drawGraphicalSystem();
  }
  
  /**
   * Repaints the graphical system, using the current clipping shape and 
   * the given graphics object.
   * @param graphics the graphics
   */
  public synchronized void repaintSystem(Graphics2D graphics) {
    clearBackground(graphics);
    drawGraphicalSystem(graphics);
  }
  
  /**
   * Sets the background color
   * @param color the color
   */
  public synchronized void setBGColor(Color color) {
    this.bgColor = color;
  }

  /**
   * Initiates a repainting of the screen.
   */
  public void updateScreen() {
    Shape tempShape = null;
    synchronized (this) {
      tempShape = clippingShape;
    }
    graphicHolder.redraw(tempShape.getBounds(), this);
  }

  /**
   * Clears the background of the system. Does nothing in the current
   * implementation.
   */
  protected void clearBackground() {
  }

  /**
   * Clears the background of the system, using a specified graphics object
   * @param graphics the graphics
   */
  protected void clearBackground(Graphics2D graphics) {
    // fill the background with the bg color
    // save old attributes
    Paint oldPaint = graphics.getPaint();
    oldClippingShape = graphics.getClip();
    // set new attributes
    graphics.setPaint(bgColor);
    graphics.setClip(clippingShape);
    // paint
    graphics.fill(clippingShape);
    // restore attributes
    graphics.setPaint(oldPaint);
    graphics.setClip(oldClippingShape);
  }

  /**
   * Draws the graphical system. Does nothing in the current implementation.
   */
  protected void drawGraphicalSystem() {
  }
  

  /**
   * Draws the graphical system, using a specified graphics object
   * @param graphics the graphics
   */
  protected void drawGraphicalSystem(Graphics2D graphics) {
    oldClippingShape = graphics.getClip();
    graphics.setClip(clippingShape);
    // Draws the grid (if needed)
    if (Preferences.userRoot().node(
      "fdda/graphics").getBoolean("showGrid", false)) {
      drawGrid(graphics);
    }
    try {
      graphicHolder.getGraphicalSystem().draw(graphics);
    }
    catch (NullPointerException e) { // sometimes thrown by java method
      Logging.exceptionCaught(e);
    }
    graphics.setClip(oldClippingShape);
  }

  /**
   * Paints the grid.
   * @param graphics the graphics
   */
  private void drawGrid(Graphics2D graphics) {
    int GRID_X_WIDTH = Preferences.userNodeForPackage(getClass()).getInt(
      "GridXWidth", 10);
    int GRID_Y_WIDTH = GRID_X_WIDTH;
    Color oldColor = graphics.getColor();
    graphics.setColor(GRID_COLOR);
    // calculate the bounds
    Rectangle bounds = graphics.getClip().getBounds();
    int startx = (int) Math.round(Math.floor(bounds.getX()));
    startx -= startx % GRID_X_WIDTH;
    int endx = (int) Math.round(Math.ceil(bounds.getX() + bounds.getWidth()));
    endx += GRID_X_WIDTH - endx % GRID_X_WIDTH;
    int starty = (int) Math.round(Math.floor(bounds.getY()));
    starty -= starty % GRID_Y_WIDTH;
    int endy = (int) Math.round(Math.ceil(bounds.getY() + bounds.getHeight()));
    endy += GRID_Y_WIDTH - endy % GRID_Y_WIDTH;
    // draw - left to right, top to buttom...
    for (int y = starty; y <= endy; y += GRID_Y_WIDTH) {
      for (int x = startx; x <= endx; x += GRID_X_WIDTH) {
        // just a small circle
        double dx = (double) x;
        double dy = (double) y;
        Ellipse2D point = new Ellipse2D.Double(dx - GRID_POINT_RADIUS,
          dy - GRID_POINT_RADIUS, 2 * GRID_POINT_RADIUS, 2 * GRID_POINT_RADIUS);
        graphics.fill(point);
      }
    }
    graphics.setColor(oldColor);
  }

}
