/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/

package fdda.graphics.java2d;

import fdda.graphics.ComponentVisitor;

import java.awt.Graphics2D;

import java.awt.geom.Rectangle2D;

/**
 * Represents a point on a line. The point appears as a small black rectangle; 
 * the coordinates are the middle point of this rectangle.
 * @author Jrg Rdenauer
 */
public class Point extends Component {

  /** the line that contains the point */
  private Line line;

  /** height and width of the rectangle */
  private final static int REC_HEIGHT_WIDTH = 6;

  /**
   * Default constructor. Point is not selected.
   * @param x the x coordinate
   * @param y the y coordinate
   */
  public Point(double x, double y) {
    super(x, y);
    this.line = null;
    this.shape = new Rectangle2D.Double();
    calcShape();
  }

  /**
   * Constructor
   * @param x the x coordinate
   * @param y the y coordinate
   * @param selected whether the point shall be selected
   */
  public Point(double x, double y, boolean selected) {
    super(x, y, selected);
    this.line = null;
    this.shape = new Rectangle2D.Double();
    calcShape();
  }
  
  /**
   * Returns the line that contains the point.
   * @return the line
   */
  public Line getLine() {
    return this.line;
  }

  /**
   * Sets the line that contains the point.
   * @param line the line
   */
  public void setLine(Line line) {
    this.line = line;
  }

  /*
   *  (non-Javadoc)
   * @see fdda.graphics.Component#move(double, double)
   */
  public void move(double x, double y) {
    this.x += x;
    this.y += y;
    dirtyShape = true;
  }

  /*
   *  (non-Javadoc)
   * @see fdda.graphics.java2d.Component#draw(java.awt.Graphics2D)
   */
  public void draw(Graphics2D graphContext){
    graphContext.setColor(isSelected() ? COLOR_SELECTED : COLOR_NORMAL);
    graphContext.fill(new Rectangle2D.Double(x - REC_HEIGHT_WIDTH / 2,
                                             y - REC_HEIGHT_WIDTH / 2,
                                             REC_HEIGHT_WIDTH,
                                             REC_HEIGHT_WIDTH));
  }

  /**
   * Returns true.
   * @see fdda.graphics.Component#isMovable()
   */
  public boolean isMovable() {
    return true;
  }

  /**
   * Returns SEL_ALL.
   * @see fdda.graphics.Component#selectableWith()
   */
  public int selectableWith() {
    return Component.SEL_ALL;
  }

  /**
   * Calls visitPoint according to visitor pattern.
   * @see fdda.graphics.Component#visit(fdda.graphics.ComponentVisitor)
   */
  public Object visit(ComponentVisitor visitor) {
    if (visitor instanceof Java2DComponentVisitor) {
      return ((Java2DComponentVisitor) visitor).visitPoint(this);
    }
    else return null;
  }

  /*
   *  (non-Javadoc)
   * @see fdda.graphics.java2d.Component#calcShape()
   */
  protected void calcShape() {
    ((Rectangle2D.Double)this.shape).setRect(
        this.getX() - REC_HEIGHT_WIDTH / 2 - 3,
        this.getY() - REC_HEIGHT_WIDTH / 2 - 3,
        REC_HEIGHT_WIDTH + 6,
        REC_HEIGHT_WIDTH + 6);
    this.dirtyShape = false;
  }

}