/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.model;

import java.util.List;

import fdda.core.projects.Project;

/**
 * A node in a network. This is the encapsulating Object, containing both a 
 * graphical representation of the node and an algorithmic representation. A 
 * node has some properties and may be not working.
 * 
 * @see fdda.graphics.Node
 * @see fdda.algorithm.Node
 * @author Jrg Rdenauer
 */
public interface Node extends PropertyHolder, WorkingPart {

  /** name of the nodeType property */
  String NODE_TYPE_PROPERTY_NAME = "type"; 

  /** Name of the 'label' Property */
  String LABEL_PROPERTY_NAME = "label";
  
  /** Name of the 'active' Property */
  String ACTIVE_PROPERTY_NAME = "active";
  
  /**
   * Initializes the node. Calls using the created instance of the node (this)
   * are not possible in the constructor. They must be made here.
   * This method is called immediately after node creation.
   */
  void initialize();

  /**
   * Returns the graphical representation of the node.
   * 
   * @return the graphical representation
   */
  fdda.graphics.Node getGraphicalRepresentation();

  /**
   * Returns the algorithmic representation of the node.
   * 
   * @return the algorithmic representation
   */
  fdda.algorithm.Node getAlgorithmicNode();

  /**
   * Specifies the algorithmic representation of the node.
   * 
   * @param algorithmicNode the algorithmic representation
   */
  void setAlgorithmicNode(fdda.algorithm.Node algorithmicNode);

  /**
   * Returns the internal number of the node, which is unique.
   * 
   * @return the internal number
   */
  int getNr();
  
  /**
   * Returns the label of the node, which can identify the node
   * for an user.
   * 
   * @return the label of the node
   */
  String getLabel();
  
  /**
   * Sets the label of the node.
   * 
   * @param label the new label.
   */
  void setLabel(String label);

  /**
   * Returns all the connections starting or ending at the node. 
   * 
   * @return the connections of the node
   */
  List getConnections();

  /**
   * Returns all directed connections starting at the node and all undirected
   * connections of the node.
   * 
   * @return all connections on which a message can leave the node
   */
  List getOutgoingConnections();

  /**
   * Returns all directed connections ending at the node and all undirected 
   * connections of the node.
   * 
   * @return all connections on which a message can reach the node
   */
  List getIncomingConnections();

  /**
   * Returns all nodes that are directly connected to this node through a 
   * connection (either directed or undirected).
   * 
   * @return all neighbours of the node
   */
  List getNeighbours();

  /**
   * Returns whether the node is a neighbour of the node in the parameter.
   * 
   * @param node a node
   * @return whether the two nodes are neighboured
   */
  boolean isNeighbouredTo(Node node);

  /**
   * Returns whether the node can send a message through (exactly) one 
   * connections to another node. This is the case iff there exists an outgoing
   * connection between this node and the node in the parameter, and the 
   * connection is working.
   * 
   * @param node a node
   * @return whether this node can send a message directly to the other node
   */
  boolean canSendMessageDirectlyTo(Node node);
  
  /**
   * Returns whether there exists a connection that may transport messages 
   * directly between this node and another node. The working property is <emph>
   * not</emph> regarded here.
   * @param node a node
   * @return true, iff a connection exists between the node that has the right
   * direction.
   */
  boolean isConnectedTo(Node node);

  /**
   * Adds a connection to the node. No check whether the connection already 
   * was added; in this case, it gets added a second time.
   * 
   * @param connection a connection
   */
  void addConnection(Connection connection);

  /**
   * Removes a connection from the node. If the connection isn't attached to the
   * node, nothing happens. If the connection is attached to the node more than
   * once, only the first attachement is removed. 
   * 
   * @param connection a connection
   */
  void removeConnection(Connection connection);
  
  /** 
   * Returns the project that contains this node.
   * @return the project
   */
  Project getProject();
  
  /**
   * Returns whether this node is a repeater. A repeater is a special node
   * (it can also be shown differently in the graphics) that is passive 
   * normally and forwards any incoming message to all other neighbours.
   * 
   * @return true iff the node is a repeater
   */
  boolean isRepeater();
  
}