/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.model;

import java.util.Iterator;

/**
 * A property of a part of a model. Properties have a certain <emph>Type</emph>.
 * Each property can hold one or more <emph>Values</emph> of this type. This 
 * type can e.g. be Integer, String, Boolean, Date, or anything more complex. 
 * A type can only be used, though, if a related class exists that implements
 * the interface fdda.propertyTypes.PropertyType and is in the package 
 * fdda.propertyTypes. E.g., for integers, the class 
 * fdda.propertyTypes.IntegerType is needed, for booleans, the class
 * fdda.propertyTypes.BooleanType.<br />
 * <br />
 * A property can also have one or more <emph>Facets</emph>. These facets 
 * usually place constraints on the values. Several facets are predefined by 
 * constants in this class - e.g. a facet that determines whether the property 
 * can have multiple values, one that gives a default value, one that defines 
 * a maximum value, etc. These constraints are only used by the dialog, though
 * - if values are changed directly by add/setValue, they are not checked.<br />
 * <br />
 * Finally, a property has some attributes pertaining to the view the user gets:
 * whether he sees it at all, whether he can modify it, and a description. 
 * 
 * @see fdda.propertyTypes.PropertyType
 * @see PropertyHolder
 * @author Jrg Rdenauer
 */
public interface Property extends Cloneable {

  /** Name of the facet that holds the minimal value of the property. */
  String MIN_VALUE = "MIN_VALUE";

  /** Name of the facet that holds the maximal value of the property. */
  String MAX_VALUE = "MAX_VALUE";

  /**
   * Name of the facet that holds the minimum number of values of the property.
   * Is of no effect if LINE_COUNT_LINK is set.
   */
  String MIN_COUNT_VALUES = "MIN_COUNT_VALUES";

  /**
   * Name of the facet that holds the maximum number of values of the property.
   * Is of no effect if LINE_COUNT_LINK is set.
   */
  String MAX_COUNT_VALUES = "MAX_COUNT_VALUES";

  /**
   * Name of the facet that specifies whether the property shall have as many
   * values as the node that holds is has connections. This is the case if the
   * value of the facet is Boolean.TRUE.
   */
  String LINE_COUNT_LINK = "LINE_COUNT_LINK";


  /** Name of the facet that specifies the default value of the property */
  String DEFAULT_VALUE = "DEFAULT_VALUE";

  /**
   * Name of the facet that specifies whether the property has more than one
   * value.
   */
  String MULTI_VALUED = "MULTI_VALUED";
  
  /**
   * Name of the facet that specifies whether the property influences the 
   * graphical representation of the property holder.
   */
  String GRAPHIC_RELEVANT = "GRAPHIC_RELEVANT";
  
  /**
   * Name of the facet that specifies that this property can't be modified while
   * a demonstration is running. Set to Boolean.True to prevent changes in demo 
   * mode.
   */
  String FIXED_WHILE_RUNNING = "FIXED_WHILE_RUNNING";
  
  /**
   * Returns the name of the property. This name
   * must be unique within a PropertyHolder.
   * 
   * @return the name of the property
   */
  String getName();
  
  /**
   * Returns the displayed name of the property.
   * @return the displayed name
   */
  String getDisplayedName();

  /**
   * Returns the value of the property, or the first value in case of a 
   * multi-valued property. If no values exist, null is returned.
   * 
   * @return the value of the property
   */
  Object getValue();

  /**
   * Returns a specific value of the property. If no such value exists, null is
   * returned (especially, if the given number is not 0 and the property is not
   * multi-valued).
   * 
   * @param nr the number of the value
   * @return the specified value of the property
   */
  Object getValue(int nr);

  /**
   * Changes the specified value to the new value. If the property hasn't a 
   * value of that number, addProperty is called.
   * 
   * @param nr the nr of the value
   * @param value the new value
   */
  void setValueAt(int nr, Object value);

  /**
   * Adds a new value at the specified nr. Not existent intermediate values are
   * set to null.
   * 
   * @param nr the nr of the new value
   * @param value the new value
   */
  void addValueAt(int nr, Object value);

  /**
   * Adds a new value at the end of the value list.
   * 
   * @param value the new value
   */
  void addValue(Object value);

  /** 
   * Removes a value from the list. If the value number doesn't exit, nothing
   * happens.
   * 
   * @param nr the number of the value which shall be deleted
   */
  void deleteValue(int nr);

  /**
   * Returns the number of values.
   * 
   * @return the number of values of the properties
   */
  int getCountOfValues();

  /**
   * Returns the type of the property (e.g. Integer).
   * 
   * @return the type of the property
   */
  String getType();

  /**
   * Returns the value of the facet with the specified name. If there is no 
   * facet of that name, null is returned.
   * 
   * @param name the name of the facet
   * @return the value of the facet
   */
  Object getFacet(String name);

  /**
   * Sets the facet of the specified name to the specified value.
   * 
   * @param name the name of the facet
   * @param value the new value of the facet
   */
  void setFacet(String name, Object value);
  
  /**
   * Returns an Iterator over the names of the facets of the property.
   * 
   * @return an Iterator over the facet names
   */
  Iterator facets();

  /**
   * Returns the description of the property
   * 
   * @return the description
   */
  String getDescription();

  /**
   * Returns whether the property is visible to the user.
   * 
   * @return whether the property is visible
   */
  boolean isVisible();

  /**
   * Specifies whether the property shall be visible to the user.
   * 
   * @param visible whether the property shall be visible
   */
  void setVisible(boolean visible);

  /**
   * Returns whether the property shall be modifiable by the user
   * 
   * @return whether the property shall be modifiable
   */
  boolean isModifiable();

  /**
   * Specifies whether the property shall be modifiable by the user
   * 
   * @param modifiable whether the property shall be modifiable
   */
  void setModifiable(boolean modifiable);

  /**
   * Sets the PropertyHolder that holds the property. Is only called by the
   * PropertyHolder itself.
   * 
   * @param holder the PropertyHolder
   */
  void setPropertyHolder(PropertyHolder holder);

}