/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.model;

import fdda.propertyTypes.*;

import fdda.utils.ObjectCache;
import fdda.utils.Notification;
import fdda.utils.Logging;
import fdda.utils.Localization;

/**
 * A factory class that creates Properties.
 * @author Jrg Rdenauer
 */
public final class PropertyFactory {
  
  /** Private constructor to prevent instanciation. */
  private PropertyFactory() {
  }

  /**
   * Creates a PropertyType. If such a PropertyType was previously created, the
   * already created instance is returned (Singleton-like). All exceptions are
   * caught; if an exception occurs, null is returned.
   * 
   * @param typeName the name of the type, e.g. 'String' or 'Integer'.
   * @return the new PropertyType
   * @see fdda.propertyTypes.PropertyType
   */
  public static PropertyType createPropertyType(String typeName) {
    String cellPropTyp = "fdda.propertyTypes." + typeName + "Type";
    try {
      return (PropertyType) ObjectCache.getInstance().getObject(cellPropTyp);
    }
    catch (NoClassDefFoundError e) {
      Logging.errorCaught(e);
      Notification.message(Notification.M_ERROR, Notification.S_PROCESSING,
        Localization.formatMessage(Localization.getLocalizedString(
          PropertyFactory.class, "noClassDefError", 
          "Fehler beim Laden der Klasse {0}: \nKlassendefinition wurde nicht "
          + "gefunden!\nFehlermeldung: {1}"), cellPropTyp, e.getMessage()));
    }
    catch (ClassNotFoundException e) {
      Logging.exceptionCaught(e);
      Notification.message(Notification.M_ERROR, Notification.S_PROCESSING,
        Localization.formatMessage(Localization.getLocalizedString(
          PropertyFactory.class, "classNotFound", 
          "Fehler beim Laden der Klasse {0}: \nKlasse wurde nicht "
          + "gefunden!\nFehlermeldung: {1}"), cellPropTyp, e.getMessage()));
    }
    catch (IllegalAccessException e) {
      Logging.exceptionCaught(e);
      Notification.message(Notification.M_ERROR, Notification.S_PROCESSING,
        Localization.formatMessage(Localization.getLocalizedString(
          PropertyFactory.class, "illegalAccess", 
          "Fehler beim Laden der Klasse {0}: \nKein public default-Konstruktor "
          + "gefunden!\nFehlermeldung: {1}"), cellPropTyp, e.getMessage()));
    }
    catch (InstantiationException e) {
      Logging.exceptionCaught(e);
      Notification.message(Notification.M_ERROR, Notification.S_PROCESSING,
        Localization.formatMessage(Localization.getLocalizedString(
          PropertyFactory.class, "instantiation", 
          "Fehler beim Laden der Klasse {0}: \nKein parameterloser Konstruktor "
          + "gefunden!\nFehlermeldung: {1}"), cellPropTyp, e.getMessage()));
    }
    catch (ClassCastException e) {
      Logging.exceptionCaught(e);
      Notification.message(Notification.M_ERROR, Notification.S_PROCESSING,
        Localization.formatMessage(Localization.getLocalizedString(
          PropertyFactory.class, "notPropertyType", 
          "Fehler beim Laden der Klasse {0}: \nSie implementiert nicht das "
          + "Interface PropertyType!\nFehlermeldung: {1}"), cellPropTyp, 
        e.getMessage()));
    }
    return null;
  }
  
  /**
   * Creates a new property with the specified name of the specified type.
   * The property initially has no values.
   * 
   * @param name the name of the property
   * @param type the type of the property
   * @return the new Property
   */
  public static Property createProperty(String name, String type) {
    return new fdda.model.impl.Property(name, type);
  }
  
  /**
   * Creates a new property with the specified name of the specified type.
   * The property initially has no values.
   * 
   * @param name the name of the property
   * @param type the type of the property
   * @param displayedName the displayed name of the property
   * @param description a description of the property, user-readable.
   * @return the new Property
   */
  public static Property createProperty(String name, String type, 
                                    String displayedName, String description) {
    return new fdda.model.impl.Property(name, type, displayedName, description);
  }
  
}