/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.model;

import java.util.Iterator;

/**
 * A part of the model that holds properties. 
 * 
 * @see Property
 * @author Jrg Rdenauer
 */
public interface PropertyHolder {

  /**
   * Returns an Iterator over the properties of the holder.
   * 
   * @return an Iterator over the properties
   */
  Iterator getProperties();

  /**
   * Returns a property of the holder. If no property with the specified name
   * exists, null is returned.
   * 
   * @param name the name of the property
   * @return the property
   */
  Property getProperty(String name);

  /**
   * Adds a property to the holder. If a property with the same name already
   * exists, it is removed.
   * 
   * @param property, must not be null.
   */
  void addProperty(Property property);

  /**
   * Removes a property from the holder. The key for removal is the name of the
   * property; if no property with that name exists, nothing happens.
   * 
   * @param property the property to be removed, must not be null
   */
  void removeProperty(Property property);
  
  /**
   * Changes a property of the holder - i.e., replaces the property with the 
   * name of the parameter property with the parameter. If no such property 
   * exists, the property is simply added to the holder; in this case, 
   * a propertyAdded event instead of a propertyChanged event is fired.
   * 
   * @param property the new property, must not be null
   */
  void changeProperty(Property property);

  /**
   * Adds a PropertyListener (Observer pattern). 
   * 
   * @param listener the listener to be added
   * @see javax.swing.event.EventListenerList
   */
  void addPropertyListener(PropertyListener listener);

  /**
   * Removes a PropertyListener (Observer pattern).
   * 
   * @param listener the listener to be removed
   * @see javax.swing.event.EventListenerList
   */
  void removePropertyListener(PropertyListener listener);

  /**
   * Informs the listeners that a property has been changed. Note that the 
   * property holder can't determine by himself whether a property is changed
   * internally; therefore, the property must inform its holder of any changes.
   * This method must <strong>only</strong> be called by implementing classes
   * of either PropertyHolder or Property.
   * 
   * @param property the property that was changed.
   */
  void firePropertyChanged(Property property);
}


