/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.model.impl;

import fdda.graphics.Line;
import fdda.utils.Localization;

/**
 * The implementation of the Connection interface.
 * 
 * @see fdda.model.Connection
 * @author Jrg Rdenauer
 */
public class Connection extends WorkingPart
  implements fdda.model.Connection, fdda.model.PropertyListener {
    
  /** description of the directed property */
  private static String DIRECTED_PROP_DESC = null;
  
  /** displayed name of the directed property */
  private static String DIRECTED_PROP_DISPLAY = null;
  
  /** description of the speed property */
  private static String SPEED_PROP_DESC = null;
  
  /** displayed name of the speed property */
  private static String SPEED_PROP_DISPLAY = null;

  /** the start node */
  private fdda.model.Node startNode;
  
  /** the end node */
  private fdda.model.Node endNode;

  /** whether the connection is directed. Redundant, but faster. */
  private boolean directed;
  
  /** the ID (internal number) of the connection */
  private int id;

  /**
   * The constructor. Start and end node as well as the graphical representation
   * of the connection must exist before the connection can be created.
   * 
   * @param nr the internal number (ID) of the connection
   * @param directed whether the connection shall be directed
   * @param startNode the start node
   * @param endNode the end node
   * @param graphicalRepresentation the graphical representation
   */
  public Connection(int nr, boolean directed, fdda.model.Node startNode,
                    fdda.model.Node endNode, Line graphicalRepresentation) {
    super(true, graphicalRepresentation);
    this.id = nr;
    this.startNode = startNode;
    this.endNode = endNode;
    this.directed = directed;
    // create the properties
    if (DIRECTED_PROP_DESC == null) {
      DIRECTED_PROP_DESC = Localization.getLocalizedString(getClass(), 
        "directed_prop_desc", "Soll die Verbindungen Nachrichten nur in eine"
        + " Richtung transportieren knnen?");
      DIRECTED_PROP_DISPLAY = Localization.getLocalizedString(getClass(), 
        "directed_prop_name", "gerichtet");
      SPEED_PROP_DESC = Localization.getLocalizedString(getClass(),
        "speed_prop_desc", "Geschwindigkeit der Verbindung. 1,0 ist normale" +
          " Geschwindigkeit, 2.0 die doppelte, usw.");
      SPEED_PROP_DISPLAY = Localization.getLocalizedString(getClass(), 
        "speed_prop_name", "Geschwindigkeit");
    }
    fdda.model.Property directedProperty = new Property(DIRECTED_PROPERTY_NAME,
        "Boolean", DIRECTED_PROP_DISPLAY, DIRECTED_PROP_DESC);
    directedProperty.setValueAt(0, directed ? Boolean.TRUE : Boolean.FALSE);
    directedProperty.setVisible(true);
    directedProperty.setModifiable(true);
    directedProperty.setFacet(Property.FIXED_WHILE_RUNNING, Boolean.TRUE);
    addProperty(directedProperty);
    fdda.model.Property speedProperty = new Property(SPEED_PROPERTY_NAME,
        "Double", SPEED_PROP_DISPLAY, SPEED_PROP_DESC);
    speedProperty.setValueAt(0, new Double(1.0));
    speedProperty.setVisible(true);
    speedProperty.setModifiable(true);
    speedProperty.setFacet(Property.DEFAULT_VALUE, new Double(1.0));
    speedProperty.setFacet(Property.MAX_VALUE, new Double(100.0));
    speedProperty.setFacet(Property.MIN_VALUE, new Double(0.01));
    speedProperty.setFacet(Property.FIXED_WHILE_RUNNING, Boolean.TRUE);
    addProperty(speedProperty);
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.model.Connection#getNr()
   */
  public int getNr() {
    return id;
  }

  /*
   *  (non-Javadoc)
   * @see fdda.model.Connection#getStartNode()
   */
  public fdda.model.Node getStartNode() {
    return startNode;
  }

  /*
   *  (non-Javadoc)
   * @see fdda.model.Connection#getEndNode()
   */
  public fdda.model.Node getEndNode() {
    return endNode;
  }

  /*
   *  (non-Javadoc)
   * @see fdda.model.Connection#getGraphicalRepresentation()
   */
  public Line getGraphicalRepresentation() {
    return (Line) graphicalRepresentation;
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.model.Connection#isDirected()
   */
  public boolean isDirected() {
    return directed;
  }

  /*
   *  (non-Javadoc)
   * @see fdda.model.Connection#setDirected(boolean)
   * Doesn't change the attribute directly. Instead, changes the property and 
   * listens for the property change. This is because the user can change the
   * 'directed' property without use of this method, and to avoid endless
   * recursion ;-)
   */
  public void setDirected(boolean directed) {
    fdda.model.Property directedProperty = getProperty(DIRECTED_PROPERTY_NAME);
    directedProperty.setValueAt(0, directed ? Boolean.TRUE : Boolean.FALSE);
    // Property calls propertyChanged, this calls internalSetDirected
  }
  
  public double getSpeed() {
    fdda.model.Property speedProperty = getProperty(SPEED_PROPERTY_NAME);
    return ((Double) speedProperty.getValue()).doubleValue();
  }

  /*
   *  (non-Javadoc)
   * @see fdda.model.PropertyListener#propertyChanged(fdda.model.PropertyEvent)
   * Listens for a change to the 'directed' property and adapts the attribute.
   */
  public void propertyChanged(fdda.model.PropertyEvent pe) {
    super.propertyChanged(pe);
    fdda.model.Property property = pe.getProperty();
    if (property.getName().equals(DIRECTED_PROPERTY_NAME)) {
      internalSetDirected(((Boolean) property.getValue()).booleanValue());
    }
  }
  
  /**
   * Sets the directed attribute. Also informs the graphical representation of
   * that change.
   * 
   * @param directed whether the connection shall be directed.
   */
  private void internalSetDirected(boolean directed) {
    this.directed = directed;
    ((Line) graphicalRepresentation).setDirected(directed);    
  }

}