/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.model.impl;

import java.util.List;
import java.util.LinkedList;
import java.util.Iterator;

import fdda.algorithm.Repeater;
import fdda.core.projects.Project;
import fdda.utils.Localization;

/**
 * The implementation of the Node interface.
 * @author Jrg Rdenauer
 */
public class Node extends WorkingPart implements fdda.model.Node {
  
  /** description of the node type property */
  private static String NODETYPE_PROP_DESC = null;
  
  /** displayed name of the node type property */
  private static String NODETYPE_PROP_DISPLAY = null;
  
  /** description of the 'active' property */
  private static String ACTIVE_PROP_DESC = null;
  
  /** displayed name of the 'active' property */
  private static String ACTIVE_PROP_DISPLAY = null;
  
  /** description of the 'label' property */
  private static String LABEL_PROP_DESC = null;
  
  /** displayed name of the 'label' property */
  private static String LABEL_PROP_DISPLAY = null;
  
  /** the algorithm node */
  private fdda.algorithm.Node algorithmNode;

  /** the connections */
  private List connections;

  /** the ID / internal number of the node */
  private int nr;
  
  /** the label of the node */
  private String label;
  
  /** the project of the node */
  private Project project;

  /**
   * Constructor. The graphical representation must exist before the node can be
   * created. 
   * 
   * @param nr the ID / internal number of the node
   * @param graphicalRepresentation the graphical representation
   */
  public Node(int nr, Project project, 
              fdda.graphics.Node graphicalRepresentation) {
    super(true, graphicalRepresentation);
    this.project = project;
    this.nr = nr;
    this.label = "" + nr;
    // use a linked list for the connections, since iterations are much more
    // common than a specific access to a connection.
    this.connections = new LinkedList();
    // load localized Strings (one-time only per class)
    if (NODETYPE_PROP_DESC == null) {
      NODETYPE_PROP_DESC = Localization.getLocalizedString(getClass(),
        "nodetype_prop_desc", 
        "Die Art oder Funktion des Knotens im Algorithmus");
      NODETYPE_PROP_DISPLAY = Localization.getLocalizedString(getClass(), 
        "nodetype_prop_name", "Knotentyp");
      ACTIVE_PROP_DESC = Localization.getLocalizedString(getClass(),
        "active_prop_desc",
        "Soll am Anfang der Demonstration die Methode start() des Knotens" +
        " aufgerufen werden?");
      ACTIVE_PROP_DISPLAY = 
       Localization.getLocalizedString(getClass(), "active_prop_name", "Label");
      LABEL_PROP_DESC = Localization.getLocalizedString(getClass(),
        "label_prop_desc", "Ein (kurzer) Label fr den Knoten");
      LABEL_PROP_DISPLAY = 
        Localization.getLocalizedString(getClass(), "label_prop_name", "Aktiv");
    }
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.model.Node#initialize()
   */
  public void initialize() {
    // property for activity of the node
    Property activeProperty = new Property(ACTIVE_PROPERTY_NAME, "Boolean",
      ACTIVE_PROP_DISPLAY, ACTIVE_PROP_DESC);
    activeProperty.setModifiable(true);
    activeProperty.setVisible(true);
    activeProperty.setFacet(Property.DEFAULT_VALUE, Boolean.FALSE);
    activeProperty.setFacet(Property.FIXED_WHILE_RUNNING, Boolean.TRUE);
    activeProperty.addValue(Boolean.FALSE);
    this.addProperty(activeProperty);
    // show the ID to the user. This will be changed later to another number.
    Property labelProperty = new Property(LABEL_PROPERTY_NAME, "String", 
                                          LABEL_PROP_DISPLAY, LABEL_PROP_DESC);
    labelProperty.setModifiable(true);
    labelProperty.setVisible(true);
    labelProperty.addValue(new Integer(nr));
    labelProperty.setFacet(Property.GRAPHIC_RELEVANT, Boolean.TRUE);
    labelProperty.setFacet(Property.MAX_VALUE, new Integer(5));
    this.addProperty(labelProperty);
    algorithmNode = project.getAlgorithm().createNode(
      project.getAlgorithm().getDefaultType(), this);
    // property for the node type
    Property nodeTypeProperty = 
      new Property(NODE_TYPE_PROPERTY_NAME, "NodeType",
                   NODETYPE_PROP_DISPLAY, NODETYPE_PROP_DESC);
    nodeTypeProperty.setModifiable(true);
    nodeTypeProperty.setVisible(true);
    nodeTypeProperty.setFacet(Property.GRAPHIC_RELEVANT, Boolean.TRUE);
    nodeTypeProperty.setFacet(Property.FIXED_WHILE_RUNNING, Boolean.TRUE);
    nodeTypeProperty.setFacet(Property.DEFAULT_VALUE, algorithmNode.getType());
    nodeTypeProperty.addValue(algorithmNode.getType());
    this.addProperty(nodeTypeProperty);
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.model.Node#getGraphicalRepresentation()
   */
  public fdda.graphics.Node getGraphicalRepresentation() {
    return (fdda.graphics.Node) graphicalRepresentation;
  }

  /*
   *  (non-Javadoc)
   * @see fdda.model.Node#getAlgorithmicNode()
   */
  public fdda.algorithm.Node getAlgorithmicNode() {
    return algorithmNode;
  }

  /*
   *  (non-Javadoc)
   * @see fdda.model.Node#setAlgorithmicNode(fdda.algorithm.Node)
   */
  public void setAlgorithmicNode(fdda.algorithm.Node algorithmicNode) {
    this.algorithmNode = algorithmicNode;
  }

  /*
   *  (non-Javadoc)
   * @see fdda.model.Node#getNr()
   */
  public int getNr() {
    return nr;
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.model.Node#getLabel()
   */
  public String getLabel() {
    return label;
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.model.Node#setLabel(java.lang.String)
   */
  public void setLabel(String label) {
    fdda.model.Property labelProperty = getProperty(LABEL_PROPERTY_NAME);
    labelProperty.setValueAt(0, label);
  }

  /*
   *  (non-Javadoc)
   * @see fdda.model.Node#getConnections()
   */
  public List getConnections() {
    return connections;
  }

  /*
   *  (non-Javadoc)
   * @see fdda.model.Node#getOutgoingConnections()
   */
  public List getOutgoingConnections() {
    List outgoings = new LinkedList();
    Iterator it = connections.iterator();
    while (it.hasNext()) {
      fdda.model.Connection connection = (fdda.model.Connection) it.next();
      if (!connection.isDirected() || connection.getStartNode() == this) {
        outgoings.add(connection);
      }
    }
    return outgoings;
  }

  /*
   *  (non-Javadoc)
   * @see fdda.model.Node#getIncomingConnections()
   */
  public List getIncomingConnections() {
    List incomings = new LinkedList();
    Iterator it = connections.iterator();
    while (it.hasNext()) {
      fdda.model.Connection connection = (fdda.model.Connection) it.next();
      if (!connection.isDirected() || connection.getEndNode() == this) {
        incomings.add(connection);
      }
    }
    return incomings;
  }

  /*
   *  (non-Javadoc)
   * @see fdda.model.Node#getNeighbours()
   */
  public List getNeighbours() {
    List neighbours = new LinkedList();
    Iterator it = connections.iterator();
    while (it.hasNext()) {
      fdda.model.Connection connection = (fdda.model.Connection) it.next();
      neighbours.add((connection.getStartNode() == this) ?
                     connection.getEndNode() : connection.getStartNode());
    }
    return neighbours;
  }

  /*
   *  (non-Javadoc)
   * @see fdda.model.Node#isNeighbouredTo(fdda.model.Node)
   */
  public boolean isNeighbouredTo(fdda.model.Node node) {
    Iterator it = connections.iterator();
    while (it.hasNext()) {
      fdda.model.Connection connection = (fdda.model.Connection) it.next();
      if (connection.getStartNode() == node ||
          connection.getEndNode() == node) {
        return true;
      }
    }
    return false;
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.model.Node#canSendMessageDirectlyTo(fdda.model.Node)
   */
  public boolean canSendMessageDirectlyTo(fdda.model.Node node) {
    Iterator it = connections.iterator();
    while (it.hasNext()) {
      fdda.model.Connection connection = (fdda.model.Connection) it.next();
      if (connection.getEndNode() == node) {
        return connection.isWorking();
      }
      else if (connection.getStartNode() == node && !connection.isDirected()) {
        return connection.isWorking();
      }
      else if (connection.getStartNode() == node && connection.isDirected()) {
        return false; // only other direction
      }
    }
    return false; // no suitable Connection found
  }

  /*
   *  (non-Javadoc)
   * @see fdda.model.Node#canSendMessageDirectlyTo(fdda.model.Node)
   */
  public boolean isConnectedTo(fdda.model.Node node) {
    Iterator it = connections.iterator();
    while (it.hasNext()) {
      fdda.model.Connection connection = (fdda.model.Connection) it.next();
      if (connection.getEndNode() == node) {
        return true;
      }
      else if (connection.getStartNode() == node && !connection.isDirected()) {
        return true;
      }
      else if (connection.getStartNode() == node && connection.isDirected()) {
        return false; // only other direction
      }
    }
    return false; // no suitable Connection found
  }

  /*
   *  (non-Javadoc)
   * @see fdda.model.Node#addConnection(fdda.model.Connection)
   */
  public void addConnection(fdda.model.Connection connection) {
    connections.add(connection);
  }

  /*
   *  (non-Javadoc)
   * @see fdda.model.Node#removeConnection(fdda.model.Connection)
   */
  public void removeConnection(fdda.model.Connection connection) {
    connections.remove(connection);
  }

  /*
   *  (non-Javadoc)
   * @see fdda.model.PropertyListener#propertyChanged(fdda.model.PropertyEvent)
   * Overwritten because of own properties and graphical updates
   */
  public void propertyChanged(fdda.model.PropertyEvent pe) {
    super.propertyChanged(pe);
    if (pe.getProperty().getName().equals(LABEL_PROPERTY_NAME)) {
      label = pe.getProperty().getValue().toString();
    }
    else if (pe.getProperty().getName().equals(NODE_TYPE_PROPERTY_NAME)) {
      algorithmNode = project.getAlgorithm().createNode(
        pe.getProperty().getValue().toString(), this);
    }
    if (pe.getProperty().getFacet(Property.GRAPHIC_RELEVANT) != null) {
      ((fdda.graphics.Node) this.getGraphicalRepresentation()).repaint();
    }
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.model.PropertyListener#propertyAdded(fdda.model.PropertyEvent)
   */
  public void propertyAdded(fdda.model.PropertyEvent pe) {
    super.propertyAdded(pe);
    if (pe.getProperty().getName().equals(LABEL_PROPERTY_NAME)) {
      label = pe.getProperty().getValue().toString();
    }
    else if (pe.getProperty().getName().equals(NODE_TYPE_PROPERTY_NAME)) {
      algorithmNode = project.getAlgorithm().createNode(
        pe.getProperty().getValue().toString(), this);
    }
  }   
  
  /*
   *  (non-Javadoc)
   * @see fdda.model.Node#getProject()
   */
  public Project getProject() {
    return project;
  }
  
  public boolean isRepeater() {
    return (algorithmNode.getType().equals(Repeater.TYPE_NAME));
  }
  
}