/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.model.impl;

import fdda.model.PropertyFactory;
import fdda.model.PropertyHolder;
import fdda.propertyTypes.TypeGenerator;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Set;
import java.util.Iterator;

/**
 * The implementation of the Property interface. Facets are held in a HashMap, 
 * Values are held in an ArrayList.
 * @author Jrg Rdenauer
 */
public class Property implements fdda.model.Property {

  /** name of the property */
  private String name;
  
  /** displayed name of the property */
  private String displayedName;
  
  /** type of the property */
  private String type;

  /** description of the property */
  private String description = "n.a.";

  /** whether the property is visible for the user */
  private boolean visible = true;
  
  /** whether the property is modifiable by the user */
  private boolean modifiable = true;

  /** values of the property */
  private ArrayList valueList;

  /** facets of the property */
  private HashMap facetList = new HashMap();

  /** the object that has the property */
  private PropertyHolder holder = null;

  /**
   * Constructor. 
   * 
   * @param name the name of the property
   * @param type the type of the property
   */
  public Property(String name, String type) {
    this(name, type, null, "");
  }

  /**
   * Constructor with description.
   * 
   * @param name the name of the property
   * @param type the type of the property
   * @param displayedName the displayed name
   * @param description the description of the property
   */
  public Property(String name, String type, 
                 String displayedName, String description) {
    this.name = name;
    this.type = type;
    this.displayedName = displayedName;
    this.description = description;
    valueList = new ArrayList(1);
  }

  /*
   *  (non-Javadoc)
   * @see fdda.model.Property#setPropertyHolder(fdda.model.PropertyHolder)
   */
  public void setPropertyHolder(PropertyHolder holder) {
    this.holder = holder;
  }
    
  /**
   * 'Clones' the property, making a 'deep' copy - i.e., also makes copies of
   * the values and the facets. Note that fdda.model.Property extends 
   * Cloneable.
   * 
   * {@inheritDoc}
   */
  public Object clone() {
    Property newProperty = null;
    try {
      newProperty = (Property) super.clone();
    }
    catch (CloneNotSupportedException e) {
      // can't happen
      throw new InternalError();
    }
    // copy (and clone) the values
    newProperty.valueList.clear();
    TypeGenerator generator = 
      PropertyFactory.createPropertyType(type).getTypeGenerator();
    if (generator != null) for (int i = 0; i < valueList.size(); i++) {
      newProperty.addValue(generator.cloneValue(valueList.get(i)));
    }
    // copy (and clone) the facets
    newProperty.facetList.clear();
    Set keySet = facetList.keySet();
    Iterator it = keySet.iterator();
    String newKey = null;
    String newValue = null;
    while (it.hasNext()) {
      newKey = it.next().toString();
      newValue = facetList.get(newKey).toString();
      newProperty.setFacet(newKey, newValue);
    }
    return newProperty;
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.model.Property#getName()
   */
  public String getName() {
    return name;
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.model.Property#getDisplayedName()
   */
  public String getDisplayedName() {
    return displayedName;
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.model.Property#getType()
   */
  public String getType() {
    return type;
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.model.Property#getDescription()
   */
  public String getDescription() {
    return description;
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.model.Property#addValueAt(int, java.lang.Object)
   */  
  public void addValueAt(int nr, Object value) {
    valueList.ensureCapacity(nr + 1);
    valueList.add(nr, value);
    if (holder != null) {
      holder.firePropertyChanged(this);
    }
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.model.Property#addValue(java.lang.Object)
   */
  public void addValue(Object value) {
    valueList.add(value);
    if (holder != null) {
      holder.firePropertyChanged(this);
    }
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.model.Property#setFacet(java.lang.String, java.lang.Object)
   */
  public void setFacet(String name, Object value) {
    facetList.put(name, value);
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.model.Property#facets()
   */
  public Iterator facets() {
    return facetList.keySet().iterator();
  }

  /*
   *  (non-Javadoc)
   * @see fdda.model.Property#getFacet(java.lang.String)
   */
  public Object getFacet(String name) {
    Object returned = null;
    if (name != null && facetList.containsKey(name)) {
      returned = facetList.get(name);
    }
    return returned;
  }

  /*
   *  (non-Javadoc)
   * @see fdda.model.Property#getValue()
   */
  public Object getValue() {
    return getValue(0);
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.model.Property#getValue(int)
   */
  public Object getValue(int nr)  {
    if ((nr < 0) || (nr > valueList.size() - 1)) {
      return null;
    }
    return valueList.get(nr);
  }

  /**
   * 
   */
  public void deleteValue(int nr) {
    if (nr >= 0 && nr < getCountOfValues()) {
      valueList.remove(nr);
      if (holder != null) {
        holder.firePropertyChanged(this);
     }
    }
  }

  /*
   *  (non-Javadoc)
   * @see fdda.model.Property#setValueAt(int, java.lang.Object)
   */
  public void setValueAt(int nr, Object value) {
    if (nr < 0) return;
    if (valueList.size() - 1 < nr) {
      addValueAt(nr, value);
      if (holder != null) {
        holder.firePropertyChanged(this);
      }
    }
    else {
      Object oldValue = valueList.get(nr);
      valueList.set(nr, value);
      if ((holder != null) && !(oldValue.equals(value))) {
        holder.firePropertyChanged(this);
      }
    }
  }

  /*
   *  (non-Javadoc)
   * @see fdda.model.Property#getCountOfValues()
   */
  public int getCountOfValues() {
    return valueList.size();
  }

  /*
   *  (non-Javadoc)
   * @see fdda.model.Property#setVisible(boolean)
   */
  public void setVisible(boolean visible) {
    this.visible = visible;
  }

  /*
   *  (non-Javadoc)
   * @see fdda.model.Property#setModifiable(boolean)
   */
  public void setModifiable(boolean modifiable) {
    this.modifiable = modifiable;
  }

  /*
   *  (non-Javadoc)
   * @see fdda.model.Property#isModifiable()
   */
  public boolean isModifiable() {
    return modifiable;
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.model.Property#isVisible()
   */
  public boolean isVisible() {
    return visible;
  }

}
