/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.model.impl;

import fdda.model.PropertyListener;
import fdda.model.PropertyEvent;
import fdda.utils.Localization;

import java.util.HashMap;
import java.util.Iterator;

import javax.swing.event.EventListenerList;

/**
 * The implementation of WorkingPart and PropertyHolder, superclass of both
 * Node and Connection. Is also a PropertyListener to react to user changes of
 * the 'working' property.
 * @see fdda.model.PropertyHolder
 * @author Jrg Rdenauer
 */
abstract class WorkingPart
  implements fdda.model.PropertyHolder, fdda.model.PropertyListener,
             fdda.model.WorkingPart {
               
  /** description of 'working' property */
  private static String WORKING_PROP_DESC = null;
  
  /** displayed name of 'working' property */
  private static String WORKING_PROP_DISPLAY = null;

  /** the properties */
  private HashMap properties;
  
  /** whether the part is currently working (redundant, but faster) */
  private boolean working;
  
  /** the graphical representation of the part */
  protected fdda.graphics.WorkingPart graphicalRepresentation;

  /** the listeners for property events */
  private EventListenerList propertyListeners;

  /** 
   * Whether the part shall be currently listening for property events itself. 
   * Sometimes it's necessary to stop listening for a short time ;-)
   */  
  private boolean listening;

  /**
   * Constructor. The graphical representation must exist before the part is 
   * created.
   * 
   * @param working whether the part is currently working
   * @param graphicalRepresentation the graphical representation
   */
  public WorkingPart(boolean working,
                     fdda.graphics.WorkingPart graphicalRepresentation) {
    this.working = working;
    this.properties = new HashMap();
    this.graphicalRepresentation = graphicalRepresentation;
    this.propertyListeners = new EventListenerList();
    // add a property for the 'working' attribute
    if (WORKING_PROP_DESC == null) {
      WORKING_PROP_DESC = Localization.getLocalizedString(getClass(), 
        "working_prop_desc", 
        "Legt fest, ob der Knoten / die Verbindung funktioniert.");
      WORKING_PROP_DISPLAY = 
        Localization.getLocalizedString(getClass(), "working_prop_name",
          "funktionierend");
    }
    Property workingProperty = 
      new Property(WORKING_PROPERTY_NAME, "Boolean", 
                   WORKING_PROP_DISPLAY, WORKING_PROP_DESC);
    workingProperty.setVisible(true);
    workingProperty.setModifiable(true);
    workingProperty.setValueAt(0, working ? Boolean.TRUE : Boolean.FALSE);
    workingProperty.setFacet(Property.GRAPHIC_RELEVANT, Boolean.TRUE);
    addProperty(workingProperty);
    // also listen for property changes to detect changes to the 'working' prop.
    this.addPropertyListener(this);
    this.listening = true;
  }

  /*
   *  (non-Javadoc)
   * @see fdda.model.WorkingPart#isWorking()
   */
  public boolean isWorking() {
    return working;
  }

  /*
   *  (non-Javadoc)
   * @see fdda.model.WorkingPart#setWorking(boolean)
   * Stops listening for the property while changing it to avoid endless 
   * recursion. Notifies the graphical representation of the change.
   */
  public final void setWorking(boolean working) {
    this.working = working;
    listening = false;
    getProperty(WORKING_PROPERTY_NAME).setValueAt(0, Boolean.valueOf(working));
    listening = true;
    graphicalRepresentation.setWorking(working);
  }

  /*
   *  (non-Javadoc)
   * @see fdda.model.PropertyHolder#getProperties()
   */
  public Iterator getProperties() {
    return properties.values().iterator();
  }

  /*
   *  (non-Javadoc)
   * @see fdda.model.PropertyHolder#getProperty(java.lang.String)
   */
  public fdda.model.Property getProperty(String name) {
    if (properties.containsKey(name)) {
      return (fdda.model.Property) properties.get(name);
    }
    else {
      return null;
    }
  }

  /*
   *  (non-Javadoc)
   * @see fdda.model.PropertyHolder#addProperty(fdda.model.Property)
   */
  public void addProperty(fdda.model.Property property) {
    if (properties.containsKey(property.getName())) {
      removeProperty(getProperty(property.getName()));
    }
    properties.put(property.getName(), property);
    property.setPropertyHolder(this);
    firePropertyAdded(property);
  }

  /*
   *  (non-Javadoc)
   * @see fdda.model.PropertyHolder#removeProperty(fdda.model.Property)
   */
  public void removeProperty(fdda.model.Property property) {
    Object returned = properties.remove(property.getName());
    if (returned != null) {
      firePropertyRemoved(property);
    }
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.model.PropertyHolder#changeProperty(fdda.model.Property)
   */
  public void changeProperty(fdda.model.Property property) {
    if (properties.containsKey(property.getName())) {
      properties.put(property.getName(), property);
      property.setPropertyHolder(this); // safety reasons only
      firePropertyChanged(property);
    }
    else {
      addProperty(property);
    }
  }

  /*
   *  (non-Javadoc)
   * @see fdda.model.PropertyHolder#addPropertyListener
   * (fdda.model.PropertyListener)
   */
  public void addPropertyListener(PropertyListener listener) {
    propertyListeners.add(PropertyListener.class, listener);
  }

  /*
   *  (non-Javadoc)
   * @see fdda.model.PropertyHolder#removePropertyListener
   * (fdda.model.PropertyListener)
   */
  public void removePropertyListener(PropertyListener listener) {
    propertyListeners.remove(PropertyListener.class, listener);
  }

  /*
   *  (non-Javadoc)
   * @see fdda.model.PropertyHolder#firePropertyChanged(fdda.model.Property)
   * @see javax.swing.event.EventListenerList
   */
  public void firePropertyChanged(fdda.model.Property property) {
    Object[] listeners = propertyListeners.getListenerList();
    PropertyEvent propertyEvent = null;
    for (int i = listeners.length - 2; i >= 0; i -= 2) {
      if (listeners[i] == PropertyListener.class) {
        if (propertyEvent == null) 
          propertyEvent = new PropertyEvent(this, property);
        ((PropertyListener) listeners[i + 1]).propertyChanged(propertyEvent);
      }
    }
  }

  /**
   * Notifies the listeners that a property has been added.
   * 
   * @param property the added property
   * @see javax.swing.event.EventListenerList
   */
  protected void firePropertyAdded(fdda.model.Property property) {
    Object[] listeners = propertyListeners.getListenerList();
    PropertyEvent propertyEvent = null;
    for (int i = listeners.length - 2; i >= 0; i -= 2) {
      if (listeners[i] == PropertyListener.class) {
        if (propertyEvent == null) propertyEvent = 
          new PropertyEvent(this, property);
        ((PropertyListener) listeners[i + 1]).propertyAdded(propertyEvent);
      }
    }
  }

  /**
   * Notifies the listeners that a property has been removed.
   * 
   * @param property the added property
   * @see javax.swing.event.EventListenerList
   */
  protected void firePropertyRemoved(fdda.model.Property property) {
    Object[] listeners = propertyListeners.getListenerList();
    PropertyEvent propertyEvent = null;
    for (int i = listeners.length - 2; i >= 0; i -= 2) {
      if (listeners[i] == PropertyListener.class) {
        if (propertyEvent == null) 
          propertyEvent = new PropertyEvent(this, property);
        ((PropertyListener) listeners[i + 1]).propertyRemoved(propertyEvent);
      }
    }
  }

  /*
   *  (non-Javadoc)
   * @see fdda.model.PropertyListener#propertyChanged(fdda.model.PropertyEvent)
   * Listening for changes of the 'working' property made by the user.
   */
  public void propertyChanged(fdda.model.PropertyEvent pe) {
    if (listening && pe.getProperty().getName().equals(WORKING_PROPERTY_NAME)) {
      setWorking(((Boolean) pe.getProperty().getValue()).booleanValue());
    }
  }

  /*
   *  (non-Javadoc)
   * @see fdda.model.PropertyListener#propertyAdded(fdda.model.PropertyEvent)
   * Not used.
   */
  public void propertyAdded(fdda.model.PropertyEvent pe) {
  }

  /*
   *  (non-Javadoc)
   * @see fdda.model.PropertyListener#propertyRemoved(fdda.model.PropertyEvent)
   * Not used.
   */
  public void propertyRemoved(fdda.model.PropertyEvent pe) {
  }

}