/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/

package fdda.plugins;

import fdda.utils.Localization;
import fdda.utils.Notification;

import java.util.LinkedList;
import java.util.Iterator;

/**
 * Default implementation of ErrorHandler for convenience. Collects all errors
 * and warnings in two lists and shows them on explicit request
 * @author Jrg Rdenauer
 */
public class DefaultErrorHandler implements ErrorHandler {
  
  /** warning message */
  private static String WARNING_MESSAGE = null;
  
  /** error message */
  private static String ERROR_MESSAGE = null;

  /** the warnings */
  private LinkedList warnings;

  /** the errors */
  private LinkedList errors;

  /** the action descriptions */
  private String action;
  
  /**
   * Constructor
   * @param action a description of the action where errors occured. Should be
   * localized.
   */
  public DefaultErrorHandler(String action) {
    this.action = action;
    warnings = new LinkedList();
    errors = new LinkedList();
    if (WARNING_MESSAGE == null) {
      WARNING_MESSAGE = Localization.getLocalizedString(getClass(), 
        "warning_message", "Beim {0} gab es folgende Warnungen:\n");
      ERROR_MESSAGE = Localization.getLocalizedString(getClass(),
        "error_message", "Beim {0} sind folgende Fehler aufgetreten:\n");
    }
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.plugins.ErrorHandler#handleError(int, java.lang.String, java.lang.String)
   */
  public void handleError(int type, final String title, final String message) {
    if (type == Notification.M_ERROR) {
      errors.addLast(message);
    }
    else if (type == Notification.M_WARNING) {
      warnings.addLast(message);
    }
    else {
      Notification.message(type, Notification.S_PLUGIN, message);
    }
  }
  
  /**
   * Shows the warnings to the user in a dialog.
   */
  public void printWarnings() {
    Iterator it = warnings.iterator();
    if (it.hasNext()) {
      String message = Localization.formatMessage(WARNING_MESSAGE, action);
      while (it.hasNext()) {
        message += "- " + (String) it.next();
        if (it.hasNext()) {
          message += "\n";
        }
      }
      Notification.message(
        Notification.M_WARNING, Notification.S_PLUGIN, message);
    }
  }

  /**
   * Shows the errors to the user in a dialog.
   */
  public void printErrors() {
    Iterator it = errors.iterator();
    if (it.hasNext()) {
      String message = Localization.formatMessage(ERROR_MESSAGE, action);
      while (it.hasNext()) {
        message += "- " + (String) it.next();
        if (it.hasNext()) {
          message += "\n";
        }
      }
      Notification.message(
        Notification.M_ERROR, Notification.S_PLUGIN, message);
    }
  }
  
  /**
   * Returns whether an error occured.
   * @return true, iff at least one error occured.
   */
  public boolean hadError() {
    return errors.size() > 0;
  }

}
