/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.plugins.xmlreader;

import fdda.plugins.AlgorithmLoader;
import fdda.plugins.ComponentNotSupportedException;
import fdda.plugins.ErrorHandler;
import fdda.plugins.Plugin;

import fdda.algorithm.Algorithms;
import fdda.algorithm.Algorithm;
import fdda.algorithm.IllegalNodeException;

import java.io.*;
import java.net.URL;

import java.util.Enumeration;
import java.util.jar.JarFile;
import java.util.prefs.Preferences;
import java.util.zip.ZipEntry;

import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.NodeList;

import fdda.utils.Localization;
import fdda.utils.Logging;
import fdda.utils.Notification;

/**
 * Reads information about an algorithm from an XML file according to 
 * 'algorithm.dtd'. Loads the associated classes.
 * @author Jrg Rdenauer
 */
public class AlgorithmReader implements AlgorithmLoader {

  /** preference option for the default path to algorithms */  
  public static final String ALGORITHM_PATH_OPTION 
    = "berlin.plugins.xmlreader.AlgorithmPath";

  /** the default default path to algorithms */
  public static final String ALGORITHM_DEFAULT_PATH 
    = System.getProperty("user.home") + File.separator + "fdda" 
    + File.separator + "algorithms";

  /** the object handling errors */
  private ErrorHandler handler;

  /* (non-Javadoc)
   * @see fdda.plugins.Plugin#getName()
   */
  public String getName() {
    return Localization.getLocalizedString(getClass(), "alg_loader_name", 
      "Simple XML input");
  }

  /* (non-Javadoc)
   * @see fdda.plugins.Plugin#getType()
   */
  public int getType() {
    return Plugin.ALGORITHM_READER;
  }

  /* (non-Javadoc)
   * @see fdda.plugins.Plugin#hasOptions()
   */
  public boolean hasOptions() {
    return true;
  }

  /* (non-Javadoc)
   * @see fdda.plugins.Plugin#useDialog(java.lang.String)
   */
  public void useDialog(String component) 
  throws ComponentNotSupportedException {
    if (!"swing".equals(component)) 
      throw new ComponentNotSupportedException(component);
  }

  /* (non-Javadoc)
   * @see fdda.plugins.Plugin#configure()
   */
  public void configure() {
    SwingAlgReaderOptions optionsDialog = new SwingAlgReaderOptions();
    if (optionsDialog.showDialog()) {
      Preferences.userNodeForPackage(getClass()).put(
        ALGORITHM_PATH_OPTION, optionsDialog.getDefaultProjDir());
    }
  }

  /*
   *  (non-Javadoc)
   * @see fdda.plugins.AlgorithmLoader#searchForAlgorithms(fdda.plugins.ErrorHandler)
   */
  public void searchForAlgorithms(ErrorHandler errorHandler) {
    this.handler = errorHandler;
    String topDirPath = System.getProperty("user.home") + File.separator
      + "fdda" + File.separator + "algorithms";
    File topDir = new File(topDirPath);
    recursiveSearch(topDir);
  }
  
  /**
   * Searches recursively in the directories.
   * @param file the directory or file
   */
  private void recursiveSearch(File file) {
    if ((file == null) || !file.exists()) return;
    if (file.isDirectory()) {
      File[] files = file.listFiles();
      for (int i = 0; i < files.length; ++i) {
        recursiveSearch(files[i]);
      }
    }
    else {
      String fileName = file.getName();
      String extension = "";
      if (fileName.indexOf('.') != -1) {
        extension = fileName.substring(fileName.lastIndexOf('.') + 1,
          fileName.length());
      }
      if (extension.equals("xml")) {
        try {
          String path = file.getCanonicalPath();
          if (path.indexOf(File.separator) != -1) {
            path = path.substring(0, path.lastIndexOf(File.separator) + 1);
          }
          loadAlgorithm(new FileInputStream(file), (new File(path)).toURL(), 
            file.getName());
        }
        catch (IOException e) {
          Logging.exceptionCaught(e);
          Notification.message(Notification.M_ERROR, Notification.S_SYSTEM,
            Localization.formatMessage(Localization.getLocalizedString(
              getClass(), "file_load_error", "Beim Laden der Algorithmus-"
              + "Datei {0} ist ein Fehler aufgetreten:\n {1})"), new Object[] {
              file.toString(), e.getLocalizedMessage()}));
        }
      }
      else if (extension.equals("jar")) {
        try {
          JarFile jarFile = new JarFile(file);
          Enumeration entries = jarFile.entries();
          while (entries.hasMoreElements()) {
            ZipEntry entry = (ZipEntry) entries.nextElement();
            String name = entry.getName();
            // name includes all subdirectories
            extension = "";
            if (name.lastIndexOf('.') != -1) {
              extension = name.substring(name.lastIndexOf('.') + 1,
                name.length());
            }
            if (extension.equals("xml")) {
              try {
                loadAlgorithm(jarFile.getInputStream(entry), file.toURL(), 
                  entry.getName());
              }
              catch (IOException e) {
                Logging.exceptionCaught(e);
                handler.handleError(Notification.M_ERROR, "System",
                  Localization.formatMessage(Localization.getLocalizedString(
                    getClass(), "jar_file_alg_error", "Beim Laden der "
                    + "Datei {0} im Archiv {1} ist ein Fehler aufgetreten:"
                    + "\n {2})"), new Object[] {
                    file.toString(), name, e.getLocalizedMessage()}));
              }
            }
          }
        }
        catch (IOException e) {
          Logging.exceptionCaught(e);
          handler.handleError(Notification.M_ERROR, "System", 
            Localization.formatMessage(Localization.getLocalizedString(
              getClass(), "jar_file_load_error", "Beim Laden der Archivdatei"
              + "{0} ist ein Fehler aufgetreten:\n {1})"), new Object[] {
              file.toString(), e.getLocalizedMessage()}));
        }
      }
    }
  }
  
  /**
   * Loads an algorithm from an input stream.
   * @param is the stream
   * @param location the location of classes of this algorithm
   * @param fileName the name of the file (or stream source)
   * @throws IOException if an I/O error occurs
   */
  private void loadAlgorithm(InputStream is, URL location, String fileName) 
  throws IOException {
    // using the error handling mechanism from the plugin architecture
    XMLParser parser = new XMLParser(handler);
    String dtd = System.getProperty("system.home") + File.separator + "DTDs" + 
      File.separator + "algorithm.dtd";
    Document doc = 
      parser.parse(new BufferedInputStream(is), dtd, "algorithm", fileName);
    if (doc != null) {
      org.w3c.dom.Node algorithmNode = doc.getDocumentElement();
      NamedNodeMap algorithmAtts = algorithmNode.getAttributes();
      // errors here don't give messages: they are very probably already
      // put out when the XML stream was parsed. In fact, these checks are
      // mostly additional safety only.
      if (algorithmAtts == null) return;
      org.w3c.dom.Node nameNode = algorithmAtts.getNamedItem("name");
      if (nameNode == null) return; 
      String algorithmName = nameNode.getNodeValue();
      org.w3c.dom.Node descNode = algorithmAtts.getNamedItem("description");
      String algDescription = (descNode != null) ? descNode.getNodeValue() : "";
      Algorithm newAlgorithm = new Algorithm(location);
      newAlgorithm.setDescription(algDescription);
      newAlgorithm.setName(algorithmName);
      boolean hasDefault = false;
      // parse the node types
      NodeList nodeTypeNodes = algorithmNode.getChildNodes();
      for (int i = 0; i < nodeTypeNodes.getLength(); i++) {
        org.w3c.dom.Node nodeTypeNode = nodeTypeNodes.item(i);
        NamedNodeMap nodeTypeAtts = nodeTypeNode.getAttributes();
        if (nodeTypeAtts == null) continue;
        org.w3c.dom.Node typeNameNode = nodeTypeAtts.getNamedItem("name");
        org.w3c.dom.Node typeClassNode = nodeTypeAtts.getNamedItem("className");
        if ((typeNameNode == null) || (typeClassNode == null)) continue; 
        String nodeTypeName = typeNameNode.getNodeValue();
        String nodeTypeClass = typeClassNode.getNodeValue();
        try {
          newAlgorithm.addNodeType(nodeTypeName, nodeTypeClass);
          org.w3c.dom.Node defaultNode = nodeTypeAtts.getNamedItem("default");
          if ((defaultNode != null) && 
            (Boolean.valueOf(defaultNode.getNodeValue()).booleanValue())) {
            hasDefault = true;
            newAlgorithm.setDefaultType(nodeTypeName);
          }
        }
        catch (IllegalNodeException e) {
          Logging.exceptionCaught(e);
          Throwable t = e.getCause();
          if (t instanceof NoClassDefFoundError) {
            handler.handleError(Notification.M_ERROR, 
              "System", Localization.formatMessage(
              Localization.getLocalizedString(getClass(), "noclassdeffound",
              "Die KnotenTyp-Klasse {0} konnte nicht instantiiert werden, "
              + "da keine Klassendefinition gefunden wurde."), nodeTypeClass));
          }
          else if (t instanceof ClassNotFoundException) {
            handler.handleError(Notification.M_ERROR, 
              "System", Localization.formatMessage(
              Localization.getLocalizedString(getClass(), "classnotfound",
              "Die KnotenTyp-Klasse {0} konnte nicht instantiiert werden, "
              + "da sie nicht gefunden wurde."), nodeTypeClass));
          }
          else if (t instanceof InstantiationException) {
            handler.handleError(Notification.M_ERROR, 
              "System", Localization.formatMessage(
              Localization.getLocalizedString(getClass(), "instantiation",
              "Die KnotenTyp-Klasse {0} konnte nicht instantiiert werden, "
              + "da kein parameterloser Konstruktor gefunden wurde."), 
              nodeTypeClass));
          }   
          else if (t instanceof IllegalAccessException) {
            handler.handleError(Notification.M_ERROR, 
              "System", Localization.formatMessage(
              Localization.getLocalizedString(getClass(), "illegalaccess",
              "Die KnotenTyp-Klasse {0} konnte nicht instantiiert werden, "
              + "da der Konstruktor nicht public ist."), nodeTypeClass));
          }    
          else if (t instanceof ClassCastException) {
            handler.handleError(Notification.M_ERROR, 
              "System", Localization.formatMessage(
              Localization.getLocalizedString(getClass(), "classcast",
              "Die KnotenTyp-Klasse {0} konnte nicht instantiiert werden, "
              + "da sie keine Unterklasse von fdda.algorithm.Node ist."), 
              nodeTypeClass));
          }
          else {
            String message = (t != null) ? 
              t.getLocalizedMessage() : e.getLocalizedMessage();
            handler.handleError(Notification.M_ERROR, 
              "System", Localization.formatMessage(
                Localization.getLocalizedString(getClass(), "illegal_node", 
                "Der KnotenTyp-Klasse {0} konnte "
                + "nicht instantiiert werden. Fehlermeldung:\n{1}"), 
                nodeTypeClass, message));
          } 
        }
      }
      // add standard node type as default if the algorithm didn't specify one
      if (!hasDefault) newAlgorithm.setDefaultType("Passive");
      boolean success = Algorithms.getInstance().addAlgorithm(newAlgorithm);
      if (!success) {
        handler.handleError(Notification.M_ERROR, "System",
          Localization.formatMessage(Localization.getLocalizedString(getClass(),
            "algorithm_name_error", "Algorithmus {0} konnte nicht hinzugefgt" +
              " werden, da der Name schon belegt ist."), algorithmName));
      }
      else {
        Logging.getPackageLogger(getClass()).fine(
          Localization.formatMessage(Localization.getLocalizedString(getClass(),
            "algorithm_added", "Algorithmus {0} geladen."), algorithmName));
      }
    }
  }

}
