/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.plugins.xmlreader;

import fdda.graphics.java2d.*;
import fdda.plugins.ErrorHandler;
import fdda.utils.Localization;
import fdda.utils.Notification;

import org.w3c.dom.*;

/**
 * Provides static helper methods to read the graphical components for the 
 * java2d component out of an XML document.
 * @author Jrg Rdenauer
 */
final class GraphicsReader {
  
  /** Private constructor to prevent instanciation */
  private GraphicsReader() {
  }

  /**
   * Reads a node
   * @param xmlNode the node in the DOM that contains the graphical node info
   * @param handler the error handler
   * @param id ID for the new node
   * @return the new node
   */
  public static fdda.graphics.Node readNode(Element xmlNode, 
                                            ErrorHandler handler, String id) {
    org.w3c.dom.Node xmlGraphicsNode = 
      xmlNode.getElementsByTagName("graphNode").item(0);
    double x = 0.0;
    double y = 0.0;
    NamedNodeMap attributes = xmlGraphicsNode.getAttributes();
    try {
      x = Double.parseDouble(attributes.getNamedItem("x").getNodeValue());
      y = Double.parseDouble(attributes.getNamedItem("y").getNodeValue());
    }
    catch (NumberFormatException e) {
      writeError(Localization.formatMessage(
        Localization.getLocalizedString(GraphicsReader.class, 
        "node_illegal_attribute", "Illegal attribute value for node {0}!"), 
        id), handler);
    }
    fdda.graphics.Node  graphicsNode = 
      fdda.graphics.ComponentFactory.getInstance().createNode(x, y, false);
    return graphicsNode;
  }
  
  /**
   * Reads a line from the DOM. Also reads its points.
   * @param element the DOM element containing the data of the line
   * @param handler the error handler
   * @param connID ID of the connection represented by the line
   * @param startNode start node of the line
   * @param endNode end node of the line
   * @param directed whether the connection is directed
   * @return the new line
   */
  public static fdda.graphics.Line readLine(Element element, ErrorHandler handler, 
                              String connID, fdda.graphics.Node startNode,
                              fdda.graphics.Node endNode, boolean directed) {
    Element xmlLine = (Element) element.getElementsByTagName("line").item(0);
    NamedNodeMap lineAttributes = xmlLine.getAttributes();
    // coordinates are not needed, since line is created through 
    // Node.getLineStartX/Y anyway. Nevertheless, they can be used for some
    // error checking...
    double lineX = 0.0, lineY = 0.0, endX = 0.0, endY = 0.0;
    try {
      lineX = 
        Double.parseDouble(lineAttributes.getNamedItem("x").getNodeValue());
      lineY = 
        Double.parseDouble(lineAttributes.getNamedItem("y").getNodeValue());
      endX = 
        Double.parseDouble(lineAttributes.getNamedItem("endx").getNodeValue());
      endY = 
        Double.parseDouble(lineAttributes.getNamedItem("endy").getNodeValue());    
    }
    catch (NumberFormatException e) {
      writeWarning(Localization.formatMessage(Localization.getLocalizedString(
        GraphicsReader.class, "illegal_line_coordinates", 
        "Illegal coordinates for line of connection {0}."), connID), handler);
    }
    // check coordinates
    final double EPSILON = 2.1d;
    double expectedStartX = startNode.getLineStartX();
    double expectedStartY = startNode.getLineStartY();
    // swap start and end node if either x or y coordinate indicates it
    boolean swap = 
      ((Math.abs(lineX - expectedStartX) > EPSILON) &&
      (Math.abs(lineX - endNode.getLineStartX()) <= EPSILON)) ||
      ((Math.abs(lineY - expectedStartY) > EPSILON) &&
      (Math.abs(lineY - endNode.getLineStartY()) <= EPSILON));
    if (swap) {
      expectedStartX = endNode.getLineStartX();
      expectedStartY = endNode.getLineStartY();
    }
    double expectedEndX = swap ? startNode.getLineStartX() 
                               : endNode.getLineStartX();
    double expectedEndY = swap ? startNode.getLineStartY()
                               : endNode.getLineStartY();
    if (Math.abs(lineX - expectedStartX) > EPSILON) {
      writeWarning(Localization.formatMessage(Localization.getLocalizedString(
        GraphicsReader.class, "line_start_x_corrected",
        "Corrected line start x coordinate for line {0}"), connID), handler);
    }
    if (Math.abs(lineY - expectedStartY) > EPSILON) {
      writeWarning(Localization.formatMessage(Localization.getLocalizedString(
        GraphicsReader.class, "line_start_y_corrected",
        "Corrected line start y coordinate for line {0}"), connID), handler);
    }
    if (Math.abs(endX - expectedEndX) > EPSILON) {
      writeWarning(Localization.formatMessage(Localization.getLocalizedString(
        GraphicsReader.class, "line_end_x_corrected",
        "Corrected line end x coordinate for line {0}"), connID), handler);
    }
    if (Math.abs(endY - expectedEndY) > EPSILON) {
      writeWarning(Localization.formatMessage(Localization.getLocalizedString(
        GraphicsReader.class, "line_end_y_corrected",
        "Corrected line end y coordinate for line {0}"), connID), handler);
    }
    Line line = (Line) fdda.graphics.ComponentFactory.getInstance().createLine(
      startNode, endNode, false, directed);
    NodeList points = xmlLine.getElementsByTagName("point");
    for (int i = 0; i < points.getLength(); i++) {
      org.w3c.dom.Node point = points.item(i);
      NamedNodeMap pointAttributes = point.getAttributes();
      try {
        double pointX = 
          Double.parseDouble(pointAttributes.getNamedItem("x").getNodeValue());
        double pointY = 
          Double.parseDouble(pointAttributes.getNamedItem("y").getNodeValue());
        line.addPoint(pointX, pointY);
      }
      catch (NumberFormatException e) {
        writeError(Localization.formatMessage(Localization.getLocalizedString(
          GraphicsReader.class, "illegal_point_coordinates", 
          "Illegal coordinates in point for line {0}. Point was ignored."), 
          connID), handler);
      }
    }
    return line;
  }

  /**
   * Writes an error message to the handler
   * @param error the error message
   * @param handler the handler
   */
  private static void writeError(String error, ErrorHandler handler) {
    handler.handleError(Notification.M_ERROR, "XML", error);
  }
  
  /**
   * Writes a warning message to the handler
   * @param warning the warning message
   * @param handler the handler
   */
  private static void writeWarning(String warning, ErrorHandler handler) {
    handler.handleError(Notification.M_WARNING, "XML", warning);
  }


}
