/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.plugins.xmlreader;

import fdda.algorithm.Algorithms;

import fdda.core.Core;
import fdda.core.projects.Project;

import fdda.model.Connection;
import fdda.model.Property;
import fdda.model.PropertyHolder;
import fdda.model.PropertyFactory;

import fdda.propertyTypes.PropertyType;
import fdda.propertyTypes.TypeGenerator;
import fdda.propertyTypes.WrongFacetException;

import fdda.plugins.ErrorHandler;
import fdda.utils.Localization;
import fdda.utils.Notification;

import fdda.graphics.Line;

import java.util.HashMap;
import java.util.HashSet;

import org.w3c.dom.*;

/**
 * Reads data from an XML document and fills a project with that data.
 * @author Jrg Rdenauer
 */
class ProjectFiller {
  
  /** error handling object */
  private ErrorHandler handler;
  /** map of IDs to start nodes of connections */
  private HashMap connectionStartNodes;
  /** map of IDs to end nodes of connections */
  private HashMap connectionEndNodes;
  /** the connections that are not directed */
  private HashSet undirectedConnections;

  /**
   * Constructor for ProjectFiller.
   * @param handler the handler
   */
  public ProjectFiller(ErrorHandler handler) {
    this.handler = handler;
    connectionStartNodes = new HashMap(50);
    connectionEndNodes = new HashMap(50);
    undirectedConnections = new HashSet(50);
  }
  
  /**
   * Fills a project with the data from the DOM
   * @param doc the Document
   * @param project the project
   * @return the new name of the project
   */
  public String fillProject(Document doc, Project project) {
    String projectName = 
      Localization.getLocalizedString(getClass(), "unknown", "unknown"); 
    Node systemTag = doc.getDocumentElement();
    Node name = systemTag.getAttributes().getNamedItem("name");
    if (name != null) {
      projectName = name.getNodeValue();
    }
    Node algorithm = systemTag.getAttributes().getNamedItem("algorithm");
    if (algorithm != null) {
      String algorithmName = algorithm.getNodeValue();
      Algorithms algorithms = Algorithms.getInstance();
      if (algorithms.getAvailableAlgorithms().contains(algorithmName)) {
        project.setAlgorithm(algorithms.getAlgorithm(algorithmName));
      }
      else {
        handler.handleError(Notification.M_WARNING, "Laden", 
          Localization.formatMessage(Localization.getLocalizedString(getClass(),
           "algorithm_not_known", "Algorithm {0} is unknown."), algorithmName));
      }
    }
    NodeList nodes = ((Element)systemTag).getElementsByTagName("node");
    for (int i = 0; i < nodes.getLength(); i++) {
      fdda.model.Node node = parseNode((Element)nodes.item(i), project);
      if (node != null) {
        project.addNode(node);
        node.getGraphicalRepresentation().setGraphicalSystem(
          project.getGraphicalSystem());
        project.getGraphicalSystem().addNode(node.getGraphicalRepresentation());
      }
    }
    NodeList connections = 
      ((Element) systemTag).getElementsByTagName("connection");
    for (int i = 0; i < connections.getLength(); i++) {
      Connection connection = parseConnection((Element) connections.item(i));
      if (connection != null) {
        project.addConnection(connection);
        connection.getGraphicalRepresentation().setGraphicalSystem(
          project.getGraphicalSystem());
        project.getGraphicalSystem().addLine(
          connection.getGraphicalRepresentation());
      }
    }
    if (!connectionStartNodes.isEmpty()) {
      writeWarning(Localization.getLocalizedString(getClass(), 
        "too_much_outgoing", "There were unreferenced outgoing connections!"));
    }
    if (!connectionEndNodes.isEmpty()) {
      writeWarning(Localization.getLocalizedString(getClass(),
        "too_much_incoming", "There were unreferenced incoming connections!"));
    }
    connectionStartNodes.clear();
    connectionEndNodes.clear();
    undirectedConnections.clear();
    return projectName;
  }
  
  /**
   * Parses a node.
   * @param xmlNode Element of the DOM containing the data
   * @param project project of the node
   * @return the new node
   */
  private fdda.model.Node parseNode(Element xmlNode, Project project) {
    String id = xmlNode.getAttributes().getNamedItem("id").getNodeValue();
    Node labelNode = xmlNode.getAttributes().getNamedItem("label");
    Node workingNode = xmlNode.getAttributes().getNamedItem("working");
    fdda.graphics.Node graphicsNode = null;
    graphicsNode = GraphicsReader.readNode(xmlNode, handler, id);
    int newID = Core.getModelManager().getNextID();
    fdda.model.Node theNode 
      = new fdda.model.impl.Node(newID, project, graphicsNode);
    theNode.initialize();
    graphicsNode.setModelNode(theNode);
    if (labelNode != null) {
      theNode.setLabel(labelNode.getNodeValue());
    }
    else {
      theNode.setLabel(id.substring(1));
    }
    if (workingNode != null) {
      theNode.setWorking(
        Boolean.valueOf(workingNode.getNodeValue()).booleanValue());
    }
    readProperties(theNode, xmlNode);
    // incoming connections
    NodeList incomings = xmlNode.getElementsByTagName("incoming");
    for (int i = 0; i < incomings.getLength(); i++) {
      Element incoming = (Element) incomings.item(i);
      NamedNodeMap incomingAttributes = incoming.getAttributes();
      String connID = incomingAttributes.getNamedItem("id").getNodeValue();
      if (undirectedConnections.contains(connID) &&
          connectionStartNodes.containsKey(connID) &&
          connectionStartNodes.get(connID) != theNode &&
          connectionStartNodes.containsKey(connID + "_2") &&
          connectionStartNodes.get(connID + "_2") != theNode) { 
        writeError(Localization.formatMessage(Localization.getLocalizedString(
          getClass(), "connection_inconsistent", 
          "Connection {0} is inconsistent!"), connID));
      }
      if (connectionEndNodes.containsKey(connID)) {
        fdda.model.Node oldNode = 
          (fdda.model.Node) connectionEndNodes.get(connID);
        if (oldNode == theNode) {
          writeWarning(Localization.formatMessage(
            Localization.getLocalizedString(getClass(), "connection_doubled", 
              "Connection {0} mentioned 2 times at node {1}"), 
            new Integer(connID), new Integer(theNode.getNr())));
        }
        else if (connectionStartNodes.containsKey(connID)) {
          fdda.model.Node startNode = 
            (fdda.model.Node) connectionStartNodes.get(connID);
          if (startNode != theNode && startNode != oldNode) {
            writeError(Localization.formatMessage(
              Localization.getLocalizedString(getClass(), 
                "connection_inconsistent", "Connection {0} is inconsistent!"), 
              connID));
          }
          else {
            undirectedConnections.add(connID);
            connectionEndNodes.put(connID + "_2", theNode);
          }
        }        
        else {
          undirectedConnections.add(connID);
          connectionEndNodes.put(connID + "_2", theNode);
        }
      }
      else {
        connectionEndNodes.put(connID, theNode);
      }
    }
    // similar for outgoing connections
    NodeList outgoings = xmlNode.getElementsByTagName("outgoing");
    for (int i = 0; i < outgoings.getLength(); i++) {
      Element outgoing = (Element) outgoings.item(i);
      NamedNodeMap outgoingAttributes = outgoing.getAttributes();
      String connID = outgoingAttributes.getNamedItem("id").getNodeValue();
      if (undirectedConnections.contains(connID) &&
          connectionEndNodes.containsKey(connID) &&
          connectionEndNodes.get(connID) != theNode &&
          connectionEndNodes.containsKey(connID + "_2") &&
          connectionEndNodes.get(connID + "_2") != theNode) { 
            writeError(Localization.formatMessage(
              Localization.getLocalizedString(getClass(), 
                "connection_inconsistent", "Connection {0} is inconsistent!"), 
              connID));
      }
      if (connectionStartNodes.containsKey(connID)) {
        fdda.model.Node oldNode = 
          (fdda.model.Node) connectionStartNodes.get(connID);
        if (oldNode == theNode) {
          Localization.formatMessage(
          Localization.getLocalizedString(getClass(), "connection_doubled", 
            "Connection {0} mentioned 2 times at node {1}"), 
          new Integer(connID), new Integer(theNode.getNr()));
        }
        else if (connectionEndNodes.containsKey(connID)) {
          fdda.model.Node endNode = 
            (fdda.model.Node) connectionEndNodes.get(connID);
          if (endNode != theNode && endNode != oldNode) {
            writeError(Localization.formatMessage(
              Localization.getLocalizedString(getClass(), 
                "connection_inconsistent", "Connection {0} is inconsistent!"), 
              connID));
          }
          else {
            undirectedConnections.add(connID);
            connectionStartNodes.put(connID + "_2", theNode);
          }
        }        
        else {
          undirectedConnections.add(connID);
          connectionStartNodes.put(connID + "_2", theNode);
        }
      }
      else {
        connectionStartNodes.put(connID, theNode);
      }
    }
    return theNode; 
  }
  
  /**
   * Removes a connection from the data structures of this object.
   * @param connID the ID of the connection
   */
  private void removeConnection(String connID) {
    connectionStartNodes.remove(connID);
    connectionStartNodes.remove(connID + "_2");
    connectionEndNodes.remove(connID);
    connectionEndNodes.remove(connID + "_2");
    undirectedConnections.remove(connID);
  }
  
  /**
   * Parses a connection from the DOM. Start and end node must already be 
   * stored in connectionStart/EndNodes
   * @param element the DOM element containing the data of the connection
   * @return the new connection
   */
  private Connection parseConnection(Element element) {
    NamedNodeMap attributes = element.getAttributes();
    String connID = attributes.getNamedItem("id").getNodeValue();
    Node workingNode = attributes.getNamedItem("working");
    // consistency checks
    fdda.model.Node startNode = 
      (fdda.model.Node) connectionStartNodes.get(connID);
    if (startNode == null) {
      writeError(Localization.formatMessage(Localization.getLocalizedString(
        getClass(), "no_start_node", "No start node for connection {0}!"), 
        connID));
      removeConnection(connID);
      return null;
    }
    fdda.model.Node endNode = (fdda.model.Node) connectionEndNodes.get(connID);
    if (endNode == null) {
      writeError(Localization.formatMessage(Localization.getLocalizedString(
        getClass(), "no_end_node", "No end node for connection {0}!"), 
        connID));
      removeConnection(connID);
      return null;
    }
    boolean directed = !undirectedConnections.contains(connID);
    if (!directed) {
      fdda.model.Node startNode2 = 
        (fdda.model.Node) connectionStartNodes.get(connID + "_2");
      fdda.model.Node endNode2 = 
        (fdda.model.Node) connectionEndNodes.get(connID + "_2");
      if ((startNode != endNode && startNode != endNode2) ||
          (startNode2 != endNode && startNode2 != endNode2) ||
          (endNode != startNode && endNode != startNode2) ||
          (endNode2 != startNode && endNode2 != startNode2)) {
        writeError(Localization.formatMessage(Localization.getLocalizedString(
          getClass(), "mismatched_nodes", 
          "Nodes for undirected connection {0} don't match."), connID));
        removeConnection(connID);
        return null;
      }
      if (startNode == endNode) {
        startNode = startNode2;
      }
    }
    if (startNode == endNode) {
      writeError(Localization.formatMessage(Localization.getLocalizedString(
        getClass(), "connection_loop", "Connection {0} is a loop!"), connID));
      removeConnection(connID);
      return null;
    }
    Line line = null;
    line = GraphicsReader.readLine(element, handler, connID, 
      startNode.getGraphicalRepresentation(), 
      endNode.getGraphicalRepresentation(), directed);
    int newID = Core.getModelManager().getNextID();
    Connection connection = new fdda.model.impl.Connection(
      newID, directed, startNode, endNode, line);
    readProperties(connection, element);
    // check the 'directed' property
    Property directedProperty = 
      connection.getProperty(Connection.DIRECTED_PROPERTY_NAME);
    if (((Boolean) directedProperty.getValue()).booleanValue() != directed) {
      writeWarning(Localization.formatMessage(Localization.getLocalizedString(
          getClass(), "directed_property_error", 
          "'Directed' property doesn't match real direction in line {0}"),
        connID));
      directedProperty.setValueAt(0, directed ? Boolean.TRUE : Boolean.FALSE);
    }
    // clear connection from temporary caches
    removeConnection(connID);
    line.setConnection(connection);
    startNode.addConnection(connection);
    endNode.addConnection(connection);
    if (workingNode != null) {
      connection.setWorking(
        Boolean.valueOf(workingNode.getNodeValue()).booleanValue());
    }
    return connection;
  }
  
  /**
   * Reads the properties of an element.
   * @param holder the property holder (node or connection)
   * @param xmlElement the element of the DOM holding the data
   */
  private void readProperties(PropertyHolder holder, Element xmlElement) {
    NodeList properties = xmlElement.getElementsByTagName("property");
    for (int i = 0; i < properties.getLength(); i++) {
      Element xmlProperty = (Element) properties.item(i);
      NamedNodeMap attributes = xmlProperty.getAttributes();
      String name = attributes.getNamedItem("name").getNodeValue();
      String type = attributes.getNamedItem("type").getNodeValue();
      Node displayNode = attributes.getNamedItem("displayName");
      String displayName = 
        (displayNode == null) ? name : displayNode.getNodeValue(); 
      Node descNode = attributes.getNamedItem("description");
      String desc = (descNode == null) ? "" : descNode.getNodeValue();
      Property property = 
        PropertyFactory.createProperty(name, type, displayName, desc);
      PropertyType propType = PropertyFactory.createPropertyType(type);
      if (propType == null) {
        writeError(Localization.formatMessage(Localization.getLocalizedString(
          getClass(), "property_type_missing", 
          "No class for property type {0} found!"), type));
        continue;
      }
      TypeGenerator generator = propType.getTypeGenerator();
      NodeList facets = xmlProperty.getElementsByTagName("facet");
      for (int j = 0; j < facets.getLength(); j++) {
        Node facet = facets.item(j);
        NamedNodeMap facetAtts = facet.getAttributes();
        String facetName = facetAtts.getNamedItem("name").getNodeValue();
        String facetValue = facetAtts.getNamedItem("value").getNodeValue();
        Object value = null;
        if (facetName.equals("ENTRY_COUNT_LINK") ||
            facetName.equals("EXIT_COUNT_LINK") ||
            facetName.equals("MULTI_VALUED") ||
            facetName.equals("FIXED_WHILE_RUNNING")) {
          value = Boolean.valueOf(facetValue);
        }
        else if (facetName.equals("MIN_VALUE") ||
                  facetName.equals("MAX_VALUE") ||
                  facetName.equals("DEFAULT_VALUE")) {
          try {
            value = generator.parseString(facetValue);            
          }
          catch (java.text.ParseException e) {
            writeWarning(Localization.formatMessage(
              Localization.getLocalizedString(getClass(), "facet_parse_error", 
                "Parse Error in value for facet {0}: {1}"), facetName, 
              e.getMessage()));
            value = null;
          }
        }
        else if (facetName.equals("MIN_COUNT_VALUES") ||
                  facetName.equals("MAX_COUNT_VALUES")) {
          try {
            value = new Integer(facetValue);
          }
          catch (NumberFormatException e) {
            writeWarning(Localization.formatMessage(
              Localization.getLocalizedString(getClass(), "facet_int_error", 
                "Parse Error in value for facet {0}: {1} is not an Integer"),
              facetName, facetValue));
            value = null;
          }
        }
        else {
          value = facetValue;
        }
        if (value != null) {
          property.setFacet(facetName, value);
        }
      }
      int returnCode = generator.correctFacets(property);
      if (returnCode != TypeGenerator.WELL_DONE) {
        writeWarning(generator.describeAction(returnCode));
      }
      NodeList values = xmlProperty.getElementsByTagName("value");
      for (int j = 0; j < values.getLength(); j++) {
        Element valueElement = (Element) values.item(j);
        String valueString = "";
        if (valueElement.hasChildNodes()) {
          valueString = valueElement.getFirstChild().getNodeValue();
        }
        try {
           Object value = generator.parseString(valueString);
           value = generator.correctValue(value, property);
           int retCode2 = generator.getlastCorrectAction();
           if (retCode2 != TypeGenerator.WELL_DONE) {
             writeWarning(generator.describeAction(retCode2));
           }
           property.addValue(value);
        }
        catch (java.text.ParseException e) {
          writeWarning(Localization.formatMessage(
            Localization.getLocalizedString(getClass(), "value_parse_error", 
              "Parse error in value {0}: {1}"), valueString, e.getMessage()));
        }
        catch (WrongFacetException e) {
          writeWarning("Internal Error: WrongFacetException thrown by generator"
          + " for type: " + type + ", but facets are corrected.");
        }
      }
      holder.addProperty(property);
    }
  }
  
  /** 
   * Writes an error message to the handler
   * @param error the error message
   */
  private void writeError(String error) {
    handler.handleError(Notification.M_ERROR, "XML", error);
  }
  
  /**
   * Writes a warning message to the handler
   * @param warning the warning message
   */
  private void writeWarning(String warning) {
    handler.handleError(Notification.M_WARNING, "XML", warning);
  }

}
