/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.plugins.xmlreader;

import fdda.plugins.Plugin;
import fdda.plugins.ProjectInformation;
import fdda.plugins.SystemReader;
import fdda.plugins.ErrorHandler;
import fdda.plugins.MissingOptionsException;
import fdda.plugins.ComponentNotSupportedException;
import fdda.utils.Localization;
import fdda.utils.Notification;

import fdda.core.projects.Project;
import fdda.core.Core;

import java.io.File;
import java.io.FileInputStream;
import java.io.BufferedInputStream;
import java.io.FileNotFoundException;
import java.util.Date;
import java.util.Vector;
import java.util.prefs.Preferences;

import org.w3c.dom.Document;

/**
 * Plug-in that reads projects from XML files according to the project.dtd. 
 * Works with the 'swing' and 'java2d' components. 
 * @author Jrg Rdenauer
 */
public class ProjectReader implements SystemReader {

  /** name of the option for the default storage path */
  public static final String PROJECT_PATH_OPTION 
    = "berlin.plugins.xmlreader.ProjectPath";

  /** the default default storage path */
  public static final String PROJECT_DEFAULT_PATH 
    = System.getProperty("user.home") + File.separator + "fdda" 
    + File.separator + "projects";

  /** number of the last project informations stored */
  private static final int PROJECT_INFO_COUNT = 4;

  /** the file that is read */
  private File file = null;

  /*
   *  (non-Javadoc)
   * @see fdda.plugins.Plugin#useDialog(java.lang.String)
   */
  public void useDialog(String component) 
  throws ComponentNotSupportedException {
    if (!"swing".equals(component)) {
      throw new ComponentNotSupportedException(component);
    }
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.plugins.SystemReader#useGraphic(java.lang.String)
   */
  public void useGraphic(String component) 
  throws ComponentNotSupportedException {
    if (!"java2d".equals(component)) {
      throw new ComponentNotSupportedException(component);
    } 
  }

  /**
   * @see fdda.plugins.SystemReader#performUI()
   */
  public boolean performUI() {
    file = SwingReaderOptions.chooseFile();
    return file != null;
  }

  /**
   * @see fdda.plugins.SystemReader#setSystemLocation(java.lang.String)
   */
  public boolean setSystemLocation(String location) {
    file = new File(location);
    if (file.exists()) {
      return true;
    }
    else {
      file = null;
      return false;
    }
  }

  /**
   * @see fdda.plugins.SystemReader#loadProject(fdda.plugins.ErrorHandler, fdda.core.projects.Project)
   */
  public void loadProject(final ErrorHandler errorHandler, Project project)
    throws MissingOptionsException {
    if (file == null) {
      throw new MissingOptionsException("Keine Datei angegeben!");
    }
    XMLParser parser = new XMLParser(errorHandler);
    String dtd = System.getProperty("system.home") + File.separator + "DTDs" + 
      File.separator + "project.dtd";
    Document doc = null;
    try {
      doc = parser.parse(new BufferedInputStream(new FileInputStream(file)), 
                         dtd, "project", file.getName());
    }
    catch (FileNotFoundException e) {
      doc = null;
      errorHandler.handleError(Notification.M_ERROR, "Plug-in", 
        Localization.getLocalizedString(getClass(), "file_not_found", 
        "File wasn't found."));
    }    
    if (doc != null) {
      ProjectFiller filler = new ProjectFiller(errorHandler);
      String projectName = filler.fillProject(doc, project);
      Core.getProjectManager().changeProjectName(project, projectName);
      Preferences prefs = Preferences.userNodeForPackage(getClass());
      int moveFrom = PROJECT_INFO_COUNT - 1;
      // is the project already there?
      boolean done = false;
      for (int i = 0; i < PROJECT_INFO_COUNT && !done; i++) {
        String location = prefs.get("lastProject" + i, "");
        if (location.equals(file.toString())) {
          // then push the entry to the front
          moveFrom = i - 1;
          done = true;
        }
      }
      // shift the old entries
      for (int i = moveFrom; i >= 0; i--) {
        String location = prefs.get("lastProject" + i, "");
        if (location != null && !location.equals("")) {
          prefs.put("lastProject" + (i + 1), location);
          String pname = prefs.get("lastProjectName" + i, "");
          prefs.put("lastProjectName" + (i + 1), pname);
          long ptime = prefs.getLong("lastProjectTime" + i, 
            (new Date()).getTime());
          prefs.putLong("lastProjectTime" + (i + 1), ptime); 
        }
      }
      // add the new entry at the first location
      prefs.put("lastProject0", file.toString());
      prefs.put("lastProjectName0", project.getName());
      prefs.putLong("lastProjectTime0", 
        java.util.Calendar.getInstance().getTimeInMillis());
      // save last used project
      Preferences.userRoot().node("fdda").put(
        "lastProjectLocation", file.toString());  
    }
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.plugins.Plugin#getName()
   */
  public String getName() {
    return Localization.getLocalizedString(getClass(), "project_loader_name", 
      "Simple XML input");
  }

  /*
   *  (non-Javadoc)
   * @see fdda.plugins.Plugin#getType()
   */
  public int getType() {
    return Plugin.SYSTEM_READER;
  }

  /**
   * Returns true.
   * @see fdda.plugins.Plugin#hasOptions()
   */
  public boolean hasOptions() {
    return true;
  }

  /*
   *  (non-Javadoc)
   * @see fdda.plugins.Plugin#configure()
   */
  public void configure() {
    SwingReaderOptions optionsDialog = new SwingReaderOptions();
    if (optionsDialog.showDialog()) {
      Preferences.userNodeForPackage(getClass()).put(
        PROJECT_PATH_OPTION, optionsDialog.getDefaultProjDir());
    }
  }

  /*
   *  (non-Javadoc)
   * @see fdda.plugins.SystemReader#getLastLoadedProjectsInformation()
   */
  public ProjectInformation[] getLastLoadedProjectsInformation() {
    Date[] dates = getLastProjectTimes();
    String[] names = getLastProjectNames();
    String[] locations = getLastProjectLocations();
    int count = Math.min(dates.length, Math.min(names.length, Math.min(
      locations.length, PROJECT_INFO_COUNT)));
    ProjectInformation[] infos = new ProjectInformationImpl[count];
    for (int i = 0; i < infos.length; i++) {
      ProjectInformationImpl info = new ProjectInformationImpl();
      info.setLocation(locations[i]);
      info.setName(names[i]);
      info.setProcessingTime(dates[i]);
      infos[i] = info;
    }
    return infos;
  }
  
  /**
   * Loads the times of the last projects from the preferences.
   * @return the times
   */
  private Date[] getLastProjectTimes() {
    Vector dates = new Vector();
    for (int i = 0; i < PROJECT_INFO_COUNT; i++) {
      Date date = new Date(Preferences.userNodeForPackage(getClass()).getLong(
        "lastProjectTime" + i, (new Date()).getTime()));
      dates.add(date);
    }
    Date[] datesArray = new Date[dates.size()];
    dates.toArray(datesArray);
    return datesArray;
  }

  /**
   * Loads the locations of the last projects from the preferences
   * @return the locations
   */
  private String[] getLastProjectLocations() {
    Vector locations = new Vector();
    for (int i = 0; i < PROJECT_INFO_COUNT; i++) {
      String location = Preferences.userNodeForPackage(getClass()).get(
        "lastProject" + i, "");
      if (location != null && !location.equals("")) {
        locations.add(location);
      }
    }
    String[] locationsArray = new String[locations.size()];
    locations.toArray(locationsArray);
    return locationsArray;
  }
  
  /**
   * Loads the names of the last projects from the preferences
   * @return the names
   */
  private String[] getLastProjectNames() {
    Vector names = new Vector();
    for (int i = 0; i < PROJECT_INFO_COUNT; i++) {
      String name = Preferences.userNodeForPackage(getClass()).get(
        "lastProjectName" + i, "");
      if (name != null && !name.equals("")) {
        names.add(name);
      }
    }
    String[] namesArray = new String[names.size()];
    names.toArray(namesArray);
    return namesArray;
  }

}
