/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.plugins.xmlreader;

import fdda.utils.Localization;
import fdda.utils.Notification;
import fdda.utils.Logging;

import java.io.*;

import javax.xml.parsers.*;

import fdda.plugins.ErrorHandler;

import org.w3c.dom.Document;
import org.xml.sax.*;

/**
 * Parses an XML file - wrapper for the java functions. SAX parser is used.
 * @author Jrg Rdenauer
 */
class XMLParser {
  
  /** object that handles errors */
  private ErrorHandler handler;
  
  /**
   * Constructor
   * @param handler the handler
   */
  public XMLParser(ErrorHandler handler) {
    this.handler = handler;
  }
  
  /**
   * Parses a document. This document is given as an InputStream. Since the DTD
   * will be fixed, its location and the name of the root element must also be
   * given.
   * 
   * @param is the InputStream for reading the document
   * @param dtd location of the dtd (a file path)
   * @param rootElem the name of the root element
   * @param sourceInfo info about the source, for error messages
   * @return the parsed document or null if unrecoverable errors occur
   */
  public Document parse(InputStream is, String dtd, String rootElem, 
                        final Object sourceInfo) {
    DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
    factory.setValidating(true);
    factory.setIgnoringComments(true);
    factory.setCoalescing(false);
    factory.setIgnoringElementContentWhitespace(true);
    try {
      DocumentBuilder builder = factory.newDocumentBuilder();
      builder.setErrorHandler(new org.xml.sax.ErrorHandler() {
          public void warning(SAXParseException e) {
            writeWarning(Localization.formatMessage(
              Localization.getLocalizedString(getClass(), "warning_msg", 
               "Warnung: in der Datei {0}, Zeile {2}: \n{4}"), 
               new Object[] {sourceInfo, e.getSystemId(),
                 new Integer(e.getLineNumber()), 
                 new Integer(e.getColumnNumber()), e.getMessage()}));
          }
          public void error(SAXParseException e) {
            writeError(Localization.formatMessage(
            Localization.getLocalizedString(getClass(), "error_msg", 
             "Fehler n der Datei {0}, Zeile {2}: \n{4}"), 
             new Object[] {sourceInfo, e.getSystemId(),
               new Integer(e.getLineNumber()), 
               new Integer(e.getColumnNumber()), e.getMessage()}));
          }
          public void fatalError(SAXParseException e) {
            writeError(Localization.formatMessage(
            Localization.getLocalizedString(getClass(), "severe_error_msg", 
             "Schwerer Fehler n der Datei {0}, Zeile {2}: \n{4}"), 
             new Object[] {sourceInfo, e.getSystemId(),
               new Integer(e.getLineNumber()), 
               new Integer(e.getColumnNumber()), e.getMessage()}));
          }
        }
      );
      // fix the DOCTYPE
      File testFile = new File(dtd);
      if (testFile.exists()) {
        dtd = testFile.toURI().toURL().toString();
        DOCTYPEChangerStream dcs = new DOCTYPEChangerStream(is);
        dcs.setRootElement(rootElem);
        dcs.setSystemIdentifier(dtd);
        Document doc = builder.parse(dcs);
        return doc;
      }
      else {
        writeError(Localization.formatMessage(Localization.getLocalizedString(
          getClass(), "dtd_not_found", "DTD {0} wurde nicht gefunden!"), dtd));
      }
    } 
    catch (SAXException e) {
      Logging.exceptionCaught(e);
      writeError(Localization.formatMessage(Localization.getLocalizedString(
        getClass(), "parse_exception", 
      "Fehler beim Einlesen der Datei {0}:\n{1}"), sourceInfo, e.getMessage()));
    } 
    catch (ParserConfigurationException e) {
      Logging.exceptionCaught(e);
      writeError(Localization.formatMessage(Localization.getLocalizedString(
        getClass(), "parse_exception", 
      "Fehler beim Einlesen der Datei {0}:\n{1}"), sourceInfo, e.getMessage()));
    } 
    catch (IOException e) {
      Logging.exceptionCaught(e);
      writeError(Localization.formatMessage(Localization.getLocalizedString(
        getClass(), "parse_exception", 
      "Fehler beim Einlesen der Datei {0}:\n{1}"), sourceInfo, e.getMessage()));
    } 
    catch (IllegalArgumentException e) {
      Logging.exceptionCaught(e);
      writeError(Localization.formatMessage(Localization.getLocalizedString(
        getClass(), "parse_exception", 
      "Fehler beim Einlesen der Datei {0}:\n{1}"), sourceInfo, e.getMessage()));
    } 
    catch (NullPointerException e) {
      Logging.exceptionCaught(e);
      writeError(Localization.formatMessage(Localization.getLocalizedString(
        getClass(), "parse_exception", 
      "Fehler beim Einlesen der Datei {0}:\n{1}"), sourceInfo, e.getMessage()));
    }    
    return null;
  }
  
  /**
   * Writes an error message to the user
   * @param error the error message
   */
  private void writeError(String error) {
    handler.handleError(Notification.M_ERROR, "XML-Parsing", error);
  }
  
  /**
   * Writes a warning message to the user
   * @param warning the warning message
   */
  private void writeWarning(String warning) {
    handler.handleError(Notification.M_WARNING, "XML-Parsing", warning);
  }

}
