/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/

package fdda.plugins.xmlwriter;

import fdda.plugins.ErrorHandler;
import fdda.plugins.MissingOptionsException;
import fdda.plugins.ComponentNotSupportedException;
import fdda.plugins.Plugin;
import fdda.plugins.ProjectInformation;

import fdda.core.projects.Project;
import fdda.core.Core;

import fdda.utils.Localization;
import fdda.utils.Logging;
import fdda.utils.Notification;

import java.io.*;

import java.util.Date;
import java.util.Vector;
import java.util.Iterator;
import java.util.prefs.Preferences;

/**
 * SystemWriter plug-in that writes projects into XML files according to the
 * project.dtd. Works with the components 'swing' and 'java2d'.
 * @author Jrg Rdenauer
 */
public class ProjectWriter implements fdda.plugins.SystemWriter {

  /** the option for the default save path */
  static final String PROJECT_PATH_OPTION =
    "fdda.plugins.xmlwriter.ProjectPath";
  
  /** the default save path */
  static final String PROJECT_DEFAULT_PATH = 
    System.getProperty("user.home") + File.separator + "fdda" 
    + File.separator + "projects";

  /** how many project informations are stored? */
  private static final int PROJECT_INFO_COUNT = 4;

  /** the file to write */
  private File file;

  /**
   * Constructor
   */
  public ProjectWriter() {
    file = null;
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.plugins.Plugin#useDialog(java.lang.String)
   */
  public void useDialog(String component) 
  throws ComponentNotSupportedException {
    if (!"swing".equals(component)) {
      throw new ComponentNotSupportedException(component);
    }
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.plugins.SystemWriter#useGraphic(java.lang.String)
   */
  public void useGraphic(String component) 
  throws ComponentNotSupportedException {
    if (!"java2d".equals(component)) {
      throw new ComponentNotSupportedException(component);
    } 
  }

  /**
   * Uses a dialog to determine the default directory for saved files.
   * @see fdda.plugins.Plugin#configure()
   */
  public void configure() {
    SwingWriterOptions optionsDialog = new SwingWriterOptions();
    if (optionsDialog.showDialog()) {
      Preferences.userNodeForPackage(getClass()).put(
        PROJECT_PATH_OPTION, optionsDialog.getDefaultProjDir());
    }
  }

  /*
   *  (non-Javadoc)
   * @see fdda.plugins.Plugin#getName()
   */
  public String getName() {
    return Localization.getLocalizedString(getClass(), "project_writer_name",
      "Einfache XML-Ausgabe");
  }

  /*
   *  (non-Javadoc)
   * @see fdda.plugins.Plugin#getType()
   */
  public int getType() {
    return Plugin.SYSTEM_WRITER;
  }

  /**
   * Returns true.
   * @see fdda.plugins.Plugin#hasOptions()
   */
  public boolean hasOptions() {
    return true;
  }

  /**
   * Shows a dialog to select the file that is written.
   * @see fdda.plugins.SystemWriter#performUI()
   */
  public boolean performUI() {
    file = SwingWriterOptions.chooseFile();
    return file != null;
  }

  /**
   * {@inheritDoc}
   * The location must be a file name or path and end with ".xml".
   */
  public boolean setSystemLocation(String location) {
    if (!location.endsWith(".xml")) {
      return false;
    }
    file = new File(location);
    return true; // Dateiname kann halt vielleicht komisch aussehen....
  }

  /*
   *  (non-Javadoc)
   * @see fdda.plugins.SystemWriter#getLastSavedProjectsInformation()
   */
  public ProjectInformation[] getLastSavedProjectsInformation() {
    Date[] dates = getLastProjectTimes();
    String[] names = getLastProjectNames();
    String[] locations = getLastProjectLocations();
    int count = Math.min(dates.length, Math.min(names.length, Math.min(
      locations.length, PROJECT_INFO_COUNT)));
    ProjectInformation[] infos = new ProjectInformationImpl[count];
    for (int i = 0; i < infos.length; i++) {
      ProjectInformationImpl info = new ProjectInformationImpl();
      info.setLocation(locations[i]);
      info.setName(names[i]);
      info.setProcessingTime(dates[i]);
      infos[i] = info;
    }
    return infos;
  }
  
  /**
   * Reads the times the last projects were saved from the preferences API
   * @return the dates
   */
  private Date[] getLastProjectTimes() {
    Vector dates = new Vector();
    for (int i = 0; i < PROJECT_INFO_COUNT; i++) {
      Date date = new Date(Preferences.userNodeForPackage(getClass()).getLong(
        "lastProjectTime" + i, (new Date()).getTime()));
      dates.add(date);
    }
    Date[] datesArray = new Date[dates.size()];
    dates.toArray(datesArray);
    return datesArray;
  }

  /**
   * Reads the last project locations from the preferences API
   * @return the last project locations
   */
  private String[] getLastProjectLocations() {
    Vector locations = new Vector();
    for (int i = 0; i < PROJECT_INFO_COUNT; i++) {
      String location = Preferences.userNodeForPackage(getClass()).get(
        "lastProject" + i, "");
      if (location != null && !location.equals("")) {
        locations.add(location);
      }
    }
    String[] locationsArray = new String[locations.size()];
    locations.toArray(locationsArray);
    return locationsArray;
  }

  /**
   * Reads the last project names from the preferences API
   * @return the last project names
   */
  private String[] getLastProjectNames() {
    Vector names = new Vector();
    for (int i = 0; i < PROJECT_INFO_COUNT; i++) {
      String name = Preferences.userNodeForPackage(getClass()).get(
        "lastProjectName" + i, "");
      if (name != null && !name.equals("")) {
        names.add(name);
      }
    }
    String[] namesArray = new String[names.size()];
    names.toArray(namesArray);
    return namesArray;
  }

  /*
   *  (non-Javadoc)
   * @see fdda.plugins.SystemWriter#writeProject(fdda.core.projects.Project, fdda.plugins.ErrorHandler)
   */
  public void writeProject(Project project,
                              ErrorHandler errorHandler)
  throws MissingOptionsException {
    if (file == null) {
      throw new MissingOptionsException("Keine Datei angegeben!");
    }
    try {
      String name = null;
      // Namen des Systems auf Dateinamen setzen
      name = file.getName();
      Core.getProjectManager().changeProjectName(project, name);
      int dotIndex = name.lastIndexOf('.');
      if (dotIndex > 0 && dotIndex < name.length()) {
        name = name.substring(0, dotIndex);
      }
      PrintWriter w = new PrintWriter(new BufferedWriter(new FileWriter(file)));
      w.println("<?xml version=\"1.0\" encoding=\"ISO-8859-15\"?>");
      w.println();
      w.println("<!DOCTYPE project SYSTEM \"" + 
        System.getProperty("system.home") + File.separator + "DTDs" + 
        File.separator + "project.dtd" + "\">");
      w.println();
      w.println("<project name=\"" + name + "\" algorithm=\"" + 
         project.getAlgorithm().getName() + "\">");
      Iterator nodes = project.nodes();
      while (nodes.hasNext()) {
        w.println();
        fdda.model.Node node = (fdda.model.Node) nodes.next();
        ModelWriter.writeNode(node, w);
      }
      Iterator connections = project.connections();
      while (connections.hasNext()) {
        w.println();
        fdda.model.Connection connection 
          = (fdda.model.Connection) connections.next();
        ModelWriter.writeConnection(connection, w);
      }
      w.println("</project>");
      w.flush();
      Core.getProjectManager().changeProjectName(project, name);
      project.setLocation(file.toString());
      project.setChanged(false);
      boolean done = false;
      Preferences prefs = Preferences.userNodeForPackage(getClass());
      int moveFrom = PROJECT_INFO_COUNT - 1;
      // is it already an entry?
      for (int i = 0; i < PROJECT_INFO_COUNT && !done; i++) {
        String location = prefs.get("lastProject" + i, "");
        if (location.equals(file.toString())) {
          // then move the entry to the front
          moveFrom = i - 1;
          done = true;
        }
      }
      // move the old entries
      for (int i = moveFrom; i >= 0; i--) {
        String location = prefs.get("lastProject" + i, "");
        if (location != null && !location.equals("")) {
          prefs.put("lastProject" + (i + 1), location);
          String pname = prefs.get("lastProjectName" + i, "");
          prefs.put("lastProjectName" + (i + 1), pname);
          long ptime = prefs.getLong("lastProjectTime" + i, 
            (new Date()).getTime());
          prefs.putLong("lastProjectTime" + (i + 1), ptime); 
        }
      }
      // add the new entry
      prefs.put("lastProject0", file.toString());
      prefs.put("lastProjectName0", name);
      prefs.putLong("lastProjectTime0", 
        java.util.Calendar.getInstance().getTimeInMillis());
      // save last used project preference
      Preferences.userRoot().node("fdda").put(
        "lastProjectLocation", file.toString());  
    }
    catch (java.io.IOException e) {
      Logging.exceptionCaught(e);
      errorHandler.handleError(Notification.M_ERROR, 
        "Speichern", Localization.formatMessage(
        Localization.getLocalizedString(getClass(), "save_error_msg", 
        "Fehler beim Schreiben der Datei. Fehlermeldung:\n{0})"),  
        e.getLocalizedMessage()));
    }
  }

}
