/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.propertyTypes;

import fdda.utils.Localization;
import fdda.utils.Logging;

import java.util.MissingResourceException;
import java.util.ResourceBundle;

/**
 * Abstract superclass for type generators. Provides implementations of 
 * getLastCorrectAction and describeAction, which are the same for many type
 * generators.
 * 
 * @author Jrg Rdenauer
 */
public abstract class AbstractGenerator implements TypeGenerator {

  /** der Returncode der letzten Ausfhrung von correctValue()*/
  protected int correctValueRetCode = TypeGenerator.WELL_DONE;

  /** the descriptions for various actions */
  private String[] descriptions = null;

  /*
   *  (non-Javadoc)
   * @see fdda.propertyTypes.TypeGenerator#getlastCorrectAction()
   */
  public int getlastCorrectAction(){
    return correctValueRetCode;
  }

  /*
   *  (non-Javadoc)
   * @see fdda.propertyTypes.TypeGenerator#describeAction(int)
   */
  public String describeAction(int actionCode){
    // 'lazy' loading of the descriptions
    if (descriptions == null) {
      loadDescriptions();
    }
    String clearText = "";
    if (actionCode >= TypeGenerator.FACET_OF_WRONG_TYPE) {
      clearText += descriptions[8] + "\n";
      actionCode -= TypeGenerator.FACET_OF_WRONG_TYPE;
    }
    if (actionCode >= TypeGenerator.FACET_DEFAULT_IS_LOWER_THAN_MIN) {
      clearText+= descriptions[7] + "\n";
      actionCode -= TypeGenerator.FACET_DEFAULT_IS_LOWER_THAN_MIN;
    }
    if (actionCode >= TypeGenerator.FACET_DEFAULT_IS_GREATER_THAN_MAX) {
      clearText += descriptions[6] + "\n";
      actionCode -= TypeGenerator.FACET_DEFAULT_IS_GREATER_THAN_MAX;
    }
    if (actionCode >= TypeGenerator.FACET_MIN_IS_GREATER_THAN_MAX) {
      clearText += descriptions[5] + "\n";
      actionCode -= TypeGenerator.FACET_MIN_IS_GREATER_THAN_MAX;
    }
    if (actionCode >= TypeGenerator.VALUE_SET_TO_USER_DEFAULT) {
      clearText += descriptions[4] + "\n";
      actionCode -= TypeGenerator.VALUE_SET_TO_USER_DEFAULT;
    }
    if (actionCode >= TypeGenerator.VALUE_SET_TO_CLASS_DEFAULT) {
      clearText += descriptions[3] + "\n";
      actionCode -= TypeGenerator.VALUE_SET_TO_CLASS_DEFAULT;
    }
    if (actionCode >= TypeGenerator.VALUE_SET_TO_MIN) {
      clearText += descriptions[2] + "\n";
      actionCode -= TypeGenerator.VALUE_SET_TO_MIN;
    }
    if (actionCode >= TypeGenerator.VALUE_SET_TO_MAX) {
      clearText += descriptions[1] + "\n";
      actionCode -= TypeGenerator.VALUE_SET_TO_MAX;
    }
    if (actionCode >= TypeGenerator.WRONG_CLASS_OBJECT) {
      clearText += descriptions[0] + "\n";
      actionCode -= TypeGenerator.WRONG_CLASS_OBJECT;
    }
    return clearText;
  }

  /**
   * Loads the localized descriptions of the Strings.
   * 
   */
  private void loadDescriptions() {
    descriptions = new String[9];
    try {
      ResourceBundle rb = Localization.getResourceBundleForPackage(
        AbstractGenerator.class);
      descriptions[0] = Localization.getLocalizedString(rb, "error0", 
        "Das Objekt war nicht vom erwarteten Typ.");
      descriptions[1] = Localization.getLocalizedString(rb, "error1",
        "Der Wert des Objektes war zu hoch. Er wurde auf das " 
        + "erlaubte Maximum gesetzt.");
      descriptions[2] = Localization.getLocalizedString(rb, "error2",
        "Der Wert des Objects war zu niedrig. Er wurde auf das "
        + "erlaubte Minimum gesetzt.");
      descriptions[3] = Localization.getLocalizedString(rb, "error3",
        "Der Wert des Objektes war falsch. Er wurde auf den "
        + "Standardwert des Typs gesetzt.");
      descriptions[4] = Localization.getLocalizedString(rb, "error4",
        "Der Wert des Objektes war falsch. Er wurde auf den "
        + "Standardwert der Eigenschaft gesetzt.");
      descriptions[5] = Localization.getLocalizedString(rb, "error5",
        "Das erlaubte Minimum war grer als das erlaubte Maximum.");
      descriptions[6] = Localization.getLocalizedString(rb, "error6",
        "Der Default-Wert war grer als das erlaubte Maximum.");
      descriptions[7] = Localization.getLocalizedString(rb, "error7",
        "Der Default-Wert war kleiner als das erlaubte Minimum.");
      descriptions[8] = Localization.getLocalizedString(rb, "error8",
        "Die Facette war nicht vom erwarteten Typ.");
    }
    catch (MissingResourceException e) {
      Logging.exceptionCaught(e);
      descriptions[0] = "Das Objekt war nicht vom erwarteten Typ.";
      descriptions[1] = "Der Wert des Objektes war zu hoch. Er wurde auf das " 
        + "erlaubte Maximum gesetzt.";
      descriptions[2] = "Der Wert des Objects war zu niedrig. Er wurde auf das "
        + "erlaubte Minimum gesetzt.";
      descriptions[3] = "Der Wert des Objektes war falsch. Er wurde auf den "
        + "Standardwert des Typs gesetzt.";
      descriptions[4] = "Der Wert des Objektes war falsch. Er wurde auf den "
        + "Standardwert der Eigenschaft gesetzt.";
      descriptions[5] = "Das erlaubte Minimum war grer als das erlaubte "
        + "Maximum.";
      descriptions[6] = "Der Default-Wert war grer als das erlaubte Maximum.";
      descriptions[7] 
        = "Der Default-Wert war kleiner als das erlaubte Minimum.";
      descriptions[8] = "Die Facette war nicht vom erwarteten Typ.";
    }
  }


}
