/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.propertyTypes;

import fdda.model.Property;

import java.text.ParseException;

/**
 * A TypeGenerator can generate values for properties of a certain type. It also
 * corrects facets that don't fit with the type and values that don't fit with
 * the type or with the constraints imposed by the facets. Those last abilities
 * are especially used when parsing properties from external files.
 * 
 * @author Jrg Rdenauer
 */
public interface TypeGenerator {

  /** Return Code: No corrections were necessary */
  int WELL_DONE = 0;

  /** RC: The Object was of the wrong type. A correction was impossible. */
  int WRONG_CLASS_OBJECT = 1;

  /** RC: the value was too high. It was set to the maximum allowed value. */
  int VALUE_SET_TO_MAX = 2;

  /** RC: the value was too low. It was set to the minimum allowed value. */
  int VALUE_SET_TO_MIN = 4;

  /** RC: the value was wrong. It was set to the type's default value. */
  int VALUE_SET_TO_CLASS_DEFAULT = 8;

  /** RC: the value was wrong. It was set to the property's default value. */
  int VALUE_SET_TO_USER_DEFAULT = 16;

  /** 
   * RC: the minimum allowed value was higher than the maximum allowed value.
   * They were switched.
   */
  int FACET_MIN_IS_GREATER_THAN_MAX = 32;

  /**
   * RC: the default value for the property was higher than the maximum allowed
   * value. It was set to the maximum value.
   */
  int FACET_DEFAULT_IS_GREATER_THAN_MAX = 64;

  /**
   * RC: the default value for the property was lower than the minimum allowed
   * value. It was set to the minimum value. 
   */
  int FACET_DEFAULT_IS_LOWER_THAN_MIN = 128;

  /**
   * RC: a facet value is of the wrong type. No corrections are possible.
   */
  int FACET_OF_WRONG_TYPE = 256;
  
  /**
   * RC: the default value for the property was not allowed. It was set to the
   * class default value.
   */
  int FACET_DEFAULT_NOT_ALLOWED = 512;

  /**
   * Returns a 'deep' copy of a value.
   * 
   * @param valueObject the value
   * @return a copy of the value
   */
  Object cloneValue(Object valueObject);
  
  /**
   * Returns a value parsed from a String representation.
   * 
   * @param string the representation of the value
   * @return the parsed value
   * @throws ParseException if the String doesn't represent a value of the type
   */
  Object parseString(String string) throws ParseException;

  /**
   * Returns the last action made by correctValue
   * 
   * @return the code of the last correcting action
   * @see fdda.propertyTypes.TypeGenerator#correctValue(Object, Property)
   */
  int getlastCorrectAction();

  /**
   * Returns a String describing a correcting action, user-friendly
   * 
   * @param actionCode the code of the action
   * @return the description of that action
   */
  String describeAction(int actionCode);


  /**
   * Corrects a value of a property so that it fits to the type and facets. 
   * It is a precondition that the facets of the property are correct; if 
   * necessary, that should be ensured by calling correctFacets(Property). The
   * action taken by the method can afterwards be retrieved by a call to 
   * getLastCorrectAction().
   * 
   * @param inValue the value
   * @param property the property (to get the facets)
   * @return the corrected value
   * @throws WrongFacetException if the facets of the property are erroneous.
   */
  Object correctValue(Object inValue, Property property)
    throws WrongFacetException;

  /**
   * Checks whether the facets of a property fit to the type and corrects them, 
   * if necessary. 
   * 
   * @param property the property that holds the facets
   * @return an error code that contains the action taken
   */
  int correctFacets(Property property);

}
