/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.propertyTypes.implementations;

import fdda.model.Property;

import fdda.propertyTypes.TypeGenerator;
import fdda.propertyTypes.AbstractGenerator;
import fdda.propertyTypes.WrongFacetException;

import java.text.ParseException;

/**
 * Generator for double values.
 * @author Jrg Rdenauer
 */
public class DoubleGenerator extends AbstractGenerator {

  /** class default value */
  private static final Double CLASS_DEFAULT_VALUE = new Double(0.00);

  /**
   * Default constructor.
   */
  public DoubleGenerator() {
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.propertyTypes.TypeGenerator#cloneValue(java.lang.Object)
   */
  public Object cloneValue(Object valueObject) {
    return new Double(valueObject.toString());
  }

  /*
   *  (non-Javadoc)
   * @see fdda.propertyTypes.TypeGenerator#parseString(java.lang.String)
   */
  public Object parseString(String string) throws ParseException {
    Double value = null;
    if (string != null) {
      try {
        value = Double.valueOf(string);
      }
      catch (NumberFormatException e) {
        throw new ParseException(string, 0);
      }
      return value;
    } 
    else {
      throw new ParseException("null", 0);
    }
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.propertyTypes.TypeGenerator#correctValue(java.lang.Object, fdda.model.Property)
   */
  public Object correctValue(Object inValue, Property property)
  throws WrongFacetException {
    correctValueRetCode = TypeGenerator.WELL_DONE;
    // object to return
    Object retObject = inValue;
    // check whether type is correct
    if (!(inValue instanceof Double)) {
      correctValueRetCode = TypeGenerator.WRONG_CLASS_OBJECT;
      return retObject;
    }
    Double checkValue = (Double) inValue;

    // check facets
    correctValueRetCode = correctFacets(property);
    if (correctValueRetCode != 0) {
      throw new WrongFacetException();
    }

    Double inMinValue = (Double) property.getFacet(Property.MIN_VALUE);
    Double inMaxValue = (Double) property.getFacet(Property.MAX_VALUE);
    Double inDefaultValue = (Double) property.getFacet(Property.DEFAULT_VALUE);

    if (inMinValue != null || inMaxValue != null) {
      if (checkValue == null) {
        // null not allowed, since min or max is given
        if (inDefaultValue != null) {
          checkValue = inDefaultValue;
          correctValueRetCode = TypeGenerator.VALUE_SET_TO_USER_DEFAULT;
        } 
        else {
          checkValue = CLASS_DEFAULT_VALUE;
          correctValueRetCode = TypeGenerator.VALUE_SET_TO_CLASS_DEFAULT;
        }
      } 
      else {
        if (inMinValue != null
            && checkValue.doubleValue() < inMinValue.doubleValue()) {
          checkValue = inMinValue;
          correctValueRetCode = TypeGenerator.VALUE_SET_TO_MIN;
        }
        if (inMaxValue != null
            && checkValue.doubleValue() > inMaxValue.doubleValue()) {
          checkValue = inMaxValue;
          correctValueRetCode = TypeGenerator.VALUE_SET_TO_MAX;
        }
      }
    }
    return checkValue;
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.propertyTypes.TypeGenerator#correctFacets(fdda.model.Property)
   */
  public int correctFacets(Property property) {
    int retCode = TypeGenerator.WELL_DONE;
    if (property != null) {
      Double inMinValue = (Double) property.getFacet(Property.MIN_VALUE);
      Double inMaxValue = (Double) property.getFacet(Property.MAX_VALUE);
      Double inDefaultValue = 
        (Double) property.getFacet(Property.DEFAULT_VALUE);
      // Min bigger than Max? -> swap
      if (inMinValue != null && inMaxValue != null
          && inMinValue.doubleValue() > inMaxValue.doubleValue()) {
        Double dummyValue = new Double(inMinValue.doubleValue());
        inMinValue = inMaxValue;
        inMaxValue = dummyValue;
        retCode = TypeGenerator.FACET_MIN_IS_GREATER_THAN_MAX;
      }
      // default smaller than minimum?
      if (inDefaultValue != null && inMinValue != null
          && inDefaultValue.doubleValue() < inMinValue.doubleValue()) {
        inDefaultValue = inMinValue;
        retCode = TypeGenerator.FACET_DEFAULT_IS_LOWER_THAN_MIN;
      }
      // default bigger than maximum?
      if (inDefaultValue != null && inMaxValue != null
        && inDefaultValue.doubleValue() > inMaxValue.doubleValue()) {
        inDefaultValue = inMaxValue;
        retCode = TypeGenerator.FACET_DEFAULT_IS_GREATER_THAN_MAX;
      }
      if (inMinValue != null) {
        property.setFacet(Property.MIN_VALUE, inMinValue);
      }
      if (inMaxValue != null) {
        property.setFacet(Property.MAX_VALUE, inMaxValue);
      }
      if (inDefaultValue != null) {
        property.setFacet(Property.DEFAULT_VALUE, inDefaultValue);
      }
    }
    return retCode;
  }

}
