/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/

package fdda.propertyTypes.implementations;

import fdda.propertyTypes.AbstractGenerator;
import fdda.propertyTypes.TypeGenerator;
import fdda.propertyTypes.WrongFacetException;

import fdda.model.Property;

import java.text.ParseException;

/**
 * Generator for Integer values.
 * @author Jrg Rdenauer
 */
public class IntegerGenerator extends AbstractGenerator {

  /** class default value */
  private final static Integer classDefaultValue = new Integer(0);

  /**
   * Default constructor.
   */
  public IntegerGenerator() {
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.propertyTypes.TypeGenerator#cloneValue(java.lang.Object)
   */
  public Object cloneValue(Object valueObject) {
    return new Integer(valueObject.toString());
 }

  /*
   *  (non-Javadoc)
   * @see fdda.propertyTypes.TypeGenerator#parseString(java.lang.String)
   */
  public Object parseString(String string) throws ParseException {
    Integer value = null;
    if (string != null) {
      try {
        value = Integer.valueOf(string);
      }
      catch (NumberFormatException e) {
        throw new ParseException(string, 0);
      }
      return value;
    } 
    else {
      throw new ParseException("null", 0);
    }
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.propertyTypes.TypeGenerator#correctValue(java.lang.Object, fdda.model.Property)
   */
  public Object correctValue(Object inValue, Property property)
    throws WrongFacetException {
    correctValueRetCode = TypeGenerator.WELL_DONE;
    
    // check for correct type first
    if (!(inValue instanceof Integer)) {
      correctValueRetCode = TypeGenerator.WRONG_CLASS_OBJECT;
      return inValue;
    }

    Integer checkValue = (Integer) inValue;

    // check facets
    correctValueRetCode = correctFacets(property);
    if (correctValueRetCode != 0) {
      throw new WrongFacetException();
    }

    if (property != null) {
      Integer inMinValue = (Integer) property.getFacet(Property.MIN_VALUE);
      Integer inMaxValue = (Integer) property.getFacet(Property.MAX_VALUE);
      Integer inDefaultValue = 
        (Integer) property.getFacet(Property.DEFAULT_VALUE);
      
      if (inMinValue != null || inMaxValue != null) {
        if (checkValue == null) {
          // null is not allowed, since min or max are given
          if (inDefaultValue != null) {
            checkValue = inDefaultValue;
            correctValueRetCode = TypeGenerator.VALUE_SET_TO_USER_DEFAULT;
          } 
          else {
            checkValue = classDefaultValue;
            correctValueRetCode = TypeGenerator.VALUE_SET_TO_CLASS_DEFAULT;
          }
        } 
        else {
          // value smaller than min?
          if (inMinValue != null
              && checkValue.intValue() < (int) inMinValue.intValue()) {
            checkValue = inMinValue;
            correctValueRetCode = TypeGenerator.VALUE_SET_TO_MIN;
          }
          // value bigger than max?
          if (inMaxValue != null
              && checkValue.intValue() > inMaxValue.intValue()) {
            checkValue = inMaxValue;
            correctValueRetCode = TypeGenerator.VALUE_SET_TO_MAX;
          }
        }
      }
    }
    return checkValue;
  }

  /*
   *  (non-Javadoc)
   * @see fdda.propertyTypes.TypeGenerator#correctFacets(fdda.model.Property)
   */
  public int correctFacets(Property property) {

    int retCode = TypeGenerator.WELL_DONE;

    Integer inMinValue = (Integer) property.getFacet(Property.MIN_VALUE);
    Integer inMaxValue = (Integer) property.getFacet(Property.MAX_VALUE);
    Integer inDefaultValue = 
      (Integer) property.getFacet(Property.DEFAULT_VALUE);
    // min bigger than max?
    if (inMinValue != null && inMaxValue != null
        && inMinValue.intValue() > inMaxValue.intValue()) {
      Integer dummyValue = new Integer(inMinValue.intValue());
      inMinValue = inMaxValue;
      inMaxValue = dummyValue;
      retCode = TypeGenerator.FACET_MIN_IS_GREATER_THAN_MAX;
    }
    // default smaller than min?
    if (inDefaultValue != null  && inMinValue != null
        && inDefaultValue.intValue() < inMinValue.intValue()) {
      inDefaultValue = inMinValue;
      retCode = TypeGenerator.FACET_DEFAULT_IS_LOWER_THAN_MIN;
    }
    // default bigger than max?
    if (inDefaultValue != null && inMaxValue != null
        && inDefaultValue.intValue() > inMaxValue.intValue()) {
      inDefaultValue = inMaxValue;
      retCode = TypeGenerator.FACET_DEFAULT_IS_GREATER_THAN_MAX;
    }
    if (inMinValue != null) {
      property.setFacet(Property.MIN_VALUE, inMinValue);
    }
    if (inMaxValue != null) {
      property.setFacet(Property.MAX_VALUE, inMaxValue);
    }
    if (inDefaultValue != null) {
      property.setFacet(Property.DEFAULT_VALUE, inDefaultValue);
    }
    return retCode;
  }

}
