/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.propertyTypes.implementations;

import fdda.propertyTypes.TypeGenerator;
import fdda.propertyTypes.AbstractGenerator;
import fdda.propertyTypes.WrongFacetException;
import fdda.utils.Logging;

import fdda.model.Property;

/**
 * Generator for string values.
 * @author Jrg Rdenauer
 */
public class StringGenerator extends AbstractGenerator {

  /** class default value */
  private static final String CLASS_DEFAULT_VALUE = "";

  /**
   * Default constructor.
   */
  public StringGenerator() {
  }
  
  /*
   *  (non-Javadoc)
   * @see fdda.propertyTypes.TypeGenerator#cloneValue(java.lang.Object)
   */
  public Object cloneValue(Object valueObject) {
    if (valueObject == null) {
      return null;
    }
    if (valueObject instanceof String) return valueObject;
    else return valueObject.toString();
  }

  /*
   *  (non-Javadoc)
   * @see fdda.propertyTypes.TypeGenerator#parseString(java.lang.String)
   */
  public Object parseString(String string) {
    return string;
  }

  /*
   *  (non-Javadoc)
   * @see fdda.propertyTypes.TypeGenerator#correctValue(java.lang.Object, fdda.model.Property)
   */
  public Object correctValue(Object inValue, Property property)
    throws WrongFacetException {
    correctValueRetCode = TypeGenerator.WELL_DONE;
    // check for correct type
    if (!(inValue instanceof String)) {
      correctValueRetCode = TypeGenerator.WRONG_CLASS_OBJECT;
      return inValue;
    }
    // check facets
    correctValueRetCode = correctFacets(property);
    if (correctValueRetCode != 0) {
      throw new WrongFacetException();
    }
    
    Integer inMinValue = null;
    Integer inMaxValue = null;
    try {
      Object temp = property.getFacet(Property.MIN_VALUE);
      if (temp instanceof Integer) inMinValue = (Integer) temp;
      else if (temp instanceof String) {
        inMinValue = new Integer(Integer.parseInt((String) temp));
      } 
      temp = property.getFacet(Property.MAX_VALUE);
      if (temp instanceof Integer) inMaxValue = (Integer) temp;
      else if (temp instanceof String) {
        inMaxValue = new Integer(Integer.parseInt((String) temp));
      } 
    }
    catch (NumberFormatException e) {
      Logging.exceptionCaught(e);
    }
    String inDefaultValue = (String) property.getFacet(Property.DEFAULT_VALUE);
    String checkValue = (String) inValue;

    if (inMinValue != null || inMaxValue != null) {
      if (checkValue == null) {
        // null not allowed, since min or max are given
        if (inDefaultValue != null) {
          checkValue = inDefaultValue;
          correctValueRetCode = TypeGenerator.VALUE_SET_TO_USER_DEFAULT;
        } 
        else {
          checkValue = CLASS_DEFAULT_VALUE;
          correctValueRetCode = TypeGenerator.VALUE_SET_TO_CLASS_DEFAULT;
        }
      } 
      else {
        // string shorter than min length?
        if (inMinValue != null && checkValue.length() < inMinValue.intValue()) {
          // fill with spaces
          for (int i = 1; i < inMinValue.intValue() - checkValue.length(); i++){
            checkValue = checkValue + " ";
          }
          correctValueRetCode = TypeGenerator.VALUE_SET_TO_MIN;
        }
        // string longer than max length?
        if (inMaxValue != null && checkValue.length() > inMaxValue.intValue()) {
          // cut to correct length
          checkValue = checkValue.substring(0, inMaxValue.intValue());
          correctValueRetCode = TypeGenerator.VALUE_SET_TO_MAX;
        }
      }
    }
    return checkValue;
  }

  /*
   *  (non-Javadoc)
   * @see fdda.propertyTypes.TypeGenerator#correctFacets(fdda.model.Property)
   */
  public int correctFacets(Property property){
    Integer inMinValue = null;
    Integer inMaxValue = null;
    try {
      Object temp = property.getFacet(Property.MIN_VALUE);
      if (temp instanceof Integer) inMinValue = (Integer) temp;
      else if (temp instanceof String) {
        inMinValue = new Integer(Integer.parseInt((String) temp));
      } 
      temp = property.getFacet(Property.MAX_VALUE);
      if (temp instanceof Integer) inMaxValue = (Integer) temp;
      else if (temp instanceof String) {
        inMaxValue = new Integer(Integer.parseInt((String) temp));
      } 
    }
    catch (NumberFormatException e) {
      Logging.exceptionCaught(e);
    }
    String inDefaultValue = (String) property.getFacet(Property.DEFAULT_VALUE);
    int retCode = TypeGenerator.WELL_DONE;
    // min bigger than max? -> swap
    if (inMinValue != null && inMaxValue != null
      && inMinValue.intValue() > inMaxValue.intValue()) {
      Integer dummyValue = new Integer(inMinValue.intValue());
      inMinValue = inMaxValue;
      inMaxValue = dummyValue;
      retCode = TypeGenerator.FACET_MIN_IS_GREATER_THAN_MAX;
    }
    // default too short?
    if (inDefaultValue != null && inMaxValue != null && inMinValue != null
        && inDefaultValue.length() < inMinValue.intValue()) {
      char[] dummyCharArray = new char[] {' '};
      inDefaultValue = inDefaultValue + new String(dummyCharArray, 0,
          inMinValue.intValue() - inDefaultValue.length());
      retCode = TypeGenerator.FACET_DEFAULT_IS_LOWER_THAN_MIN;
    }
    // default too long?
    if (inDefaultValue != null && inMaxValue != null && inMinValue != null
        && inDefaultValue.length() > inMaxValue.intValue()) {
      inDefaultValue = inDefaultValue.substring(0, inMaxValue.intValue());
      retCode = TypeGenerator.FACET_DEFAULT_IS_GREATER_THAN_MAX;
    }
    if (inMinValue != null) {
      property.setFacet(Property.MIN_VALUE, inMinValue);
    }
    if (inMaxValue != null) {
      property.setFacet(Property.MAX_VALUE, inMaxValue);
    }
    if (inDefaultValue != null) {
      property.setFacet(Property.DEFAULT_VALUE, inDefaultValue);
    }
    return retCode;
  }

}
