/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.utils;

import java.util.Locale;
import java.util.ResourceBundle;
import java.util.MissingResourceException;

import java.text.MessageFormat;

import javax.swing.JComponent;

/**
 * Provides some helping methods pertaining to the output of localized Strings,
 * either in the dialog or for messages. The locale can be specified if the 
 * default locale shall not be used.
 * 
 * @author Jrg Rdenauer
 */
public final class Localization {

  /** the locale that is used */
  private static Locale locale = Locale.getDefault(); 
  
  /**
   * Private constructor to prevent instantiation.
   */
  private Localization() {
  }
  
  /**
   * Sets the locale that is used. If null is given, it is ignored.
   * 
   * @param aLocale the new locale
   */
  public static void setLocale(Locale aLocale) {
    if (aLocale != null) {
      locale = aLocale;
      JComponent.setDefaultLocale(aLocale);
    }
  }
  
  /**
   * Returns the used locale.
   * 
   * @return the locale
   */
  public static Locale getLocale() {
    return locale;
  }
  
  /**
   * Formats a message using MessageFormat.format. Catches possibly occuring
   * exceptions.
   * 
   * @param message the message
   * @param params the parameters
   * @return the formatted message
   * @see java.text.MessageFormat
   */
  public static String formatMessage(String message, Object[] params) {
    String formattedMessage = message;
    try {
      formattedMessage = MessageFormat.format(message, params);
    }
    catch (IllegalArgumentException e) {
      Logging.exceptionCaught(e);
    }
    return formattedMessage;
  }
  
  /**
   * Formats a message using MessageFormat.format, version with 1 parameter. 
   * Catches possibly occuring exceptions.
   * 
   * @param message the message
   * @param o1 the first parameter
   * @return the formatted message
   * @see java.text.MessageFormat
   */
  public static String formatMessage(String message, Object o1) {
    return formatMessage(message, new Object[] {o1});
  }

  /**
   * Formats a message using MessageFormat.format, version with 2 parameters. 
   * Catches possibly occuring exceptions.
   * 
   * @param message the message
   * @param o1 the first parameter
   * @param o2 the second parameter
   * @return the formatted message
   * @see java.text.MessageFormat
   */  
  public static String formatMessage(String message, Object o1, Object o2) {
    return formatMessage(message, new Object[] {o1, o2});
  }
  
  /**
   * Formats a message using MessageFormat.format, version with 3 parameter. 
   * Catches possibly occuring exceptions.
   * 
   * @param message the message
   * @param o1 the first parameter
   * @param o2 the second parameter
   * @param o3 the third parameter
   * @return the formatted message
   * @see java.text.MessageFormat
   */
  public static String formatMessage(String message, 
                                     Object o1, Object o2, Object o3) {
    return formatMessage(message, new Object[] {o1, o2, o3});
  }
  
  /**
   * Loads a resource bundle for a certain package. The package is determined
   * by a class in it. Packages should be roughly equivalent to components and
   * should therefore use different resource bundles. <br />
   * The resources must either be in a ResourceBundle class in the package with
   * the name 'LocaleResources[_language etc.]' or in a resource file in the 
   * package directory with the name 'LocalResources[_language etc.].properties.
   * 
   * @param aClass a class in the package, must not be null
   * @return the resource bundle for the package
   * @throws MissingResourceException if the resource bundle isn't found
   * @see java.util.ResourceBundle
   */
  public static ResourceBundle getResourceBundleForPackage(Class aClass) {
    ResourceBundle bundle = ResourceBundle.getBundle(
      aClass.getPackage().getName() + ".LocaleResources", locale);
    return bundle;
  } 
  
  /**
   * Loads a localized version of a String out of a resource bundle. All 
   * exceptions are caught; in case of an exception, the default String is 
   * returned.
   * 
   * @param bundle the resource bundle, must not be null
   * @param key the key for the String
   * @param aDefault the default String
   * @return the localized String
   */
  public static String getLocalizedString(ResourceBundle bundle, String key, 
                                          String aDefault) {
    String localizedString = aDefault;
    try {
      localizedString = bundle.getString(key);
    }
    catch (NullPointerException e) {
      Logging.exceptionCaught(e);
    }
    catch (MissingResourceException e) {
      Logging.exceptionCaught(e);
    }
    catch (ClassCastException e) {
      Logging.exceptionCaught(e);
    }
    return localizedString;   
  }
  
  /**
   * Loads a localized version of a String out of a resource bundle. The 
   * resource bundle is determined by a class and is the resource bundle for the
   * package of the class. Catches all exceptions; if an exception occurs, the 
   * default String is returned.
   * 
   * @param aClass a class
   * @param key the key for the String
   * @param aDefault the default String
   * @return the localized String
   * @see fdda.utils.Localization#getResourceBundleForPackage(Class)
   */
  public static String getLocalizedString(Class aClass, String key, 
                                          String aDefault) {
    try {
      ResourceBundle bundle = getResourceBundleForPackage(aClass);
      return getLocalizedString(bundle, key, aDefault);
    }
    catch (NullPointerException e) {
      Logging.exceptionCaught(e);
      return aDefault;
    }
    catch (MissingResourceException e) {
      Logging.exceptionCaught(e);
      return aDefault;
    }
  }

}
