/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.utils;

import java.io.IOException;
import java.text.DateFormat;

import java.util.logging.*;

/**
 * Provides some small help functions for easy logging.
 * 
 * @author Jrg Rdenauer
 */
public final class Logging {
  
  /** error logger */
  private static Logger errorLogger;

  /**
   * Private constructor to prevent instantiation of the class.
   */
  private Logging() {
  }
  
  /**
   * Initializes the logging component
   */
  public static void initialize() {
    Logger fddaLogger = 
      Logger.getLogger("fdda", "fdda.LocaleResources");
    Handler[] handlers = fddaLogger.getHandlers();
    for (int i = 0; i < handlers.length; ++i) {
      fddaLogger.removeHandler(handlers[i]);
    }
    ConsoleHandler consoleHandler = new ConsoleHandler();
    consoleHandler.setLevel(Level.INFO);
    consoleHandler.setFormatter(new LogFormatter());
    
    boolean fileOk = true;
    FileHandler fileHandler = null;
    try {
      fileHandler = new FileHandler("%h/fdda/fdda.log", true);
      fileHandler.setLevel(Level.ALL);
      fileHandler.setFormatter(new LogFormatter());
    }
    catch (IOException e) {
      fileOk = false;
    }
    
    fddaLogger.setLevel(Level.ALL);
    fddaLogger.addHandler(consoleHandler);
    if (fileOk) {
      fddaLogger.addHandler(fileHandler);
    }
    
    errorLogger = Logger.getLogger("fdda.errors", 
      "fdda.utils.LocaleResources");
    errorLogger.setLevel(Level.ALL);
  }
  
  /**
   * Logs the event that an exception has been caught. Should be used if no
   * further exception is thrown (neither the same nor an other, perhaps 
   * encapsulating exception) for debugging purposes, because the exception 
   * event is not necessarily conveyed to the user.<br />
   * Logs the exact place where the exception was caught and the localized 
   * message of the exception with the logger level 'fine' on the 'global' 
   * logger.
   * 
   * @param e the caught Exception, must not be null
   */
  public static void exceptionCaught(Exception e) {
    // get the caller
    Exception ex = new Exception("");
    ex.fillInStackTrace();
    StackTraceElement caller = ex.getStackTrace()[1];
    // log a warning message
    String className = e.getClass().getName();
    if (className.indexOf('.') != -1) {
      className = 
        className.substring(className.lastIndexOf('.') + 1, className.length());
    }
    errorLogger.log(Level.WARNING, "exception_caught", new Object[] {
      caller.getFileName(), caller.getMethodName(), 
      new Integer(caller.getLineNumber()), e.getLocalizedMessage(), 
      className});
  }

  /**
   * Logs the event that an error has been caught. Should be used if no
   * further exception / error is thrown (neither the same nor an other, perhaps
   * encapsulating exception) for debugging purposes, because the exception 
   * event is not necessarily conveyed to the user.<br />
   * Logs the exact place where the error was caught and the localized 
   * message of the error with the logger level 'fine' on the 'global' 
   * logger.
   * 
   * @param e the caught Error, must not be null
   */
  public static void errorCaught(Error e) {
    // get the caller
    Exception ex = new Exception("");
    ex.fillInStackTrace();
    StackTraceElement caller = ex.getStackTrace()[1];
    // log a warning message
    String className = e.getClass().getName();
    if (className.indexOf('.') != -1) {
      className = 
        className.substring(className.lastIndexOf('.') + 1, className.length());
    }
    errorLogger.log(Level.SEVERE, "error_caught", new Object[] {
      caller.getFileName(), caller.getMethodName(), 
      new Integer(caller.getLineNumber()), e.getLocalizedMessage(), 
      className});
  }

  
  /**
   * Returns a logger for a certain package (determined from a class). Packages
   * should roughly be equivalent to components, so they should each use 
   * different loggers. For each package, always the same logger is returned.
   * 
   * @param aClass a class in the package, must not be null
   * @return the logger for that package
   */
  public static Logger getPackageLogger(Class aClass) {
    Logger logger = Logger.getLogger(aClass.getPackage().getName(), 
      aClass.getPackage().getName() + ".LocaleResources");
    logger.setLevel(Level.ALL);
    return logger;
  }
  
  /**
   * Class that formats log messages
   */
  private static class LogFormatter extends Formatter {
    
     private static DateFormat df = 
       DateFormat.getDateTimeInstance(DateFormat.SHORT, DateFormat.MEDIUM,
         Localization.getLocale());
     
     private static String lf = System.getProperty("line.separator");
     
     private static String debug = null;
     private static String config = null;
     private static String info = null;
     private static String warning = null;
     private static String error = null;
     
     private static String getLevelString(Level logLevel) {
       if (logLevel.intValue() <= Level.FINE.intValue()) {
         if (debug == null) {
           debug = 
             Localization.getLocalizedString(Logging.class, "debug", 
              "Debug: ");
         }
         return debug; 
       }
       else if (logLevel == Level.CONFIG) { 
         if (config == null) {
           config = 
             Localization.getLocalizedString(Logging.class, "config",
               "Konfiguration:");
         }
         return config;
       }
       else if (logLevel == Level.INFO) {
         if (info == null) {
           info = 
             Localization.getLocalizedString(Logging.class, "info", "Info: ");
         }
         return info;
       }
       else if (logLevel == Level.WARNING) {
         if (warning == null) {
           warning = Localization.getLocalizedString(Logging.class, "warning",
             "Warnung: ");
         }
         return warning;
       }
       else if (logLevel == Level.SEVERE) {
         if (error == null) {
           error = Localization.getLocalizedString(Logging.class, "error",
             "Fehler: ");
         }
         return error;
       }
       else {
         return "";
       }
     }
     
     public String format(LogRecord record) {
       String msg = formatMessage(record);
       String s = df.format(new java.util.Date()) + " ";
       return s + getLevelString(record.getLevel()) + lf + msg + lf;
     }
  }
  
}
