/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.utils;

import java.util.ArrayList;

/**
 * Provides an abstract interface for communicating messages to the user. The
 * way these messages are displayed depends on the dialog component, where a 
 * subclass of Notification must be specified to do the real work. Each 
 * message has a source (roughly spoken the subsystem that triggers it) and a 
 * type. <br />
 * Notification messages are not to be confused with logging; these are messages
 * that the user shall definitely see; in some cases these are even questions
 * the user must answer. <br />
 * The 'message'-methods are all synchronized and it's therefore safe to use 
 * them from several concurrent threads (though no order of showing the 
 * messages is garanteed).
 * 
 * @author Jrg Rdenauer
 */
public abstract class Notification {

  /**
   * Type 'warning'.
   */
  public static final int M_WARNING = 1003;

  /**
   * Type 'question'. A question usually has some custom (abstract) buttons for
   * the user to answer it.
   */
  public static final int M_QUESTION = 1002;

  /**
   * Type 'confirmation'
   */
  public static final int M_CONFIRM = 1001;

  /**
   * Type 'error'
   */
  public static final int M_ERROR = 1004;

  /**
   * Source 'XML-Processing'
   */
  public static final int S_XML = 2;

  /**
   * Source 'Operating System'
   */
  public static final int S_SYSTEM = 4;

  /**
   * Source 'Algorithm'
   */
  public static final int S_ALGORITHM = 3;

  /**
   * Source 'Modelling'
   */
  public static final int S_MOD = 1;

  /**
   * Source 'Processing'
   */
  public static final int S_PROCESSING = 5;

  /**
   * Source 'Plug-in'
   */
  public static final int S_PLUGIN = 6;

  /**
   * The instance of a Nofification subclass used as a delegate.
   */
  private static Notification notification = null;

  /**
   * List of Strings describing the sources of the messages.
   */
  private static ArrayList sources;
  
  // fills the sources list with the localized names
  static {
    sources = new ArrayList(10);
    setSourceName(S_MOD, Localization.getLocalizedString(
      Notification.class, "modelling", "Modellierung"));
    setSourceName(S_XML, Localization.getLocalizedString(
      Notification.class, "xml", "XML-Verarbeitung"));
    setSourceName(S_ALGORITHM, Localization.getLocalizedString(
      Notification.class, "algorithm", "Algorithmus"));
    setSourceName(S_SYSTEM, Localization.getLocalizedString(
      Notification.class, "os", "Betriebssystem"));
    setSourceName(S_PROCESSING, Localization.getLocalizedString(
      Notification.class, "processing", "Verarbeitung"));
    setSourceName(S_PLUGIN, Localization.getLocalizedString(
      Notification.class, "plugin", "Plug-In"));
  }

  /**
   * Shows a message to the user. Returns the number of the (abstract) button 
   * that was used to confirm the message.
   * 
   * @param type the type of the message, one of the constants
   * @param source the source of the message, probably one of the constants
   * @param title the title of the message, e.g. used as title in a dialog
   * @param message the message text
   * @param buttons the inscriptions for (abstract) buttons, e.g. {"yes", "no"}
   * @return the number of the button the user pressed (count starts with 0)
   */
  public static final synchronized int message(int type, int source,
      final String title, final String message, final String[] buttons) {
    if (notification == null) {
      throw new ComponentNotInitializedDisaster("Notification");
    }
    return notification.showMessage(type, source, title, message, buttons);
  }

  /**
   * Shows a message to the user, with a simple "Ok" button for confirmation.
   * 
   * @param type the type of the message. It doesn't make sense to use this for
   * a question
   * @param source the source of the message, probably one of the constants
   * @param title the title of the message, e.g. used as title in a dialog
   * @param message the message text
   */
  public static final synchronized void message(int type, int source,
      final String title, final String message) {
    if (notification == null) {
      throw new ComponentNotInitializedDisaster("Notification");
    }
    notification.showMessage(type, source, title, message, new String[] {"Ok"});
  }

  /**
   * Shows a message to the user, with the source description as the title.
   * 
   * @param type the type of the message.
   * @param source the source of the message, probably one of the constants
   * @param message the message text
   * @param buttons the inscriptions for (abstract) buttons, e.g. {"yes", "no"}
   * @return the number of the button the user pressed (count starts with 0)
   */
  public static final synchronized int message(int type, int source,
      final String message, final String[] buttons) {
    if (notification == null) {
      throw new ComponentNotInitializedDisaster("Notification");
    }
    String title = "";
    if (source >= sources.size()) {
      title = "";
    }
    else {
      title = "" + sources.get(source);
    }
    return notification.showMessage(type, source, title, message, buttons);
  }

  /**
   * Shows a message to the user, with a simple "Ok" button for confirmation and
   * the source description as a title.
   * 
   * @param type the type of the message. It doesn't make sense to use this for
   * a question
   * @param source the source of the message, probably one of the constants
   * @param message the message text
   */
  public static final synchronized void message(int type, int source,
      String message) {
    if (notification == null) {
      throw new ComponentNotInitializedDisaster("Notification");
    }
    String title = "";
    if (source >= sources.size()) {
      title = "";
    }
    else {
      title = "" + sources.get(source);
    }
    notification.showMessage(type, source, title, message, new String[] {"Ok"});
  }
  
  /**
   * Indicates to the user that the application is occupied (e.g. by changing
   * the cursor).
   */
  public static final synchronized void indicateOccupationState() {
    if (notification == null) {
      throw new ComponentNotInitializedDisaster("Notification");
    }
    notification.indicateState(true);
  }
  
  /**
   * Indicates to the user that the application is not occupied (e.g. by 
   * changing the cursor).
   */
  public static final synchronized void indicateNormalState() {
    if (notification == null) {
      throw new ComponentNotInitializedDisaster("Notification");
    }
    notification.indicateState(false);
  }

  /**
   * Delegate the showing of the message to a subclass.
   * 
   * @param type the type of the message, one of the constants
   * @param source the source of the message, probably one of the constants
   * @param title the title of the message, e.g. used as title in a dialog
   * @param message the message text
   * @param buttons the inscriptions for (abstract) buttons, e.g. {"yes", "no"}
   * @return the number of the button the user pressed (count starts with 0)
   */
  protected abstract int showMessage(int type, int source, final String title,
    final String message, final String[] buttons);
    
  /**
   * Delegate the occupation state indication to a subclass.
   * 
   * @param occupation whether the application is occupied
   */
  protected abstract void indicateState(boolean occupation);

  /**
   * Initializes the component by specifying the subclass that shows the 
   * messages.
   * 
   * @param aNotification the (instance of) the subclass that shows the messages
   */
  public static final void setNotification(Notification aNotification) {
    notification = aNotification;
  }

  /**
   * Sets the description / name for a certain source. Old source names get 
   * replaced.
   * 
   * @param nr the number of the source
   * @param name the name / description of the source
   */
  public static final void setSourceName(int nr, String name) {
    while (sources.size() <= nr) {
      sources.add("");
    }
    sources.set(nr, name);
  }

}