/*
    Copyright (C) 2004  [Jrg Rdenauer]
    
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
    
*/
package fdda.utils;

import java.util.HashMap;

/**
 * The instance of this class (Singleton) caches instances of the classes that 
 * have to be loaded dynamically (via Class.forName()). The loaded class itself
 * is also cached, in case of the need for several new instances.
 * The cache is <emph>not</emph> thread-safe.
 *
 * @see java.lang.Class 
 * @author Jrg Rdenauer
 */
public class ObjectCache {

  /**
   * The instance of the class (Singleton pattern).
   */
  private static ObjectCache instance = null;

  /**
   * Saves the dynamically loaded classes.
   */
  private HashMap classes;

  /**
   * Saves the instances of the classes.
   */
  private HashMap instances;

  /**
   * The default constructor.
   */
  protected ObjectCache() {
    instances = new HashMap();
    classes = new HashMap();
  }

  /**
   * Returns the instance of ObjectCache (Singleton-Pattern).
   * 
   * @return the single instance of the class.
   */
  public static ObjectCache getInstance() {
    if (instance == null) {
      instance = new ObjectCache();
    }
    return instance;
  }

  /**
   * Loads a class dynamically, if it isn't cached. If it is cached (i.e., was
   * already loaded once), the cached class is returned.
   * 
   * @param className the name of the class 
   * @return the loaded class
   * @throws ClassNotFoundException if the class doesn't exist in the CLASSPATH
   */
  public final Class loadClass(String className) 
  throws ClassNotFoundException  {
    Class classObject = null;
    if (classes.containsKey(className)) {
      classObject = (Class) classes.get(className);
    }
    else {
      classObject = Class.forName(className);
      classes.put(className, classObject);
      Logging.getPackageLogger(getClass()).fine(Localization.formatMessage(
        Localization.getLocalizedString(getClass(), "loaded_class", 
        "Loaded class {0}"), className));
    }
    return classObject;
  }

  /**
   * Returns an instance of a dynamically loaded class. If an instance of the 
   * class is cached, the cached instance is returned; otherwise, a new instance
   * is created and cached. That means that this method should only be used for
   * classes that are used Singleton-like in the application. The class must 
   * have a public constructor without parameters.
   * 
   * @param className the name of the class
   * @return an instance of the class - may already be used!
   * @throws ClassNotFoundException if the class doesn't exist in the CLASSPATH
   * @throws InstantiationException if no instance of the class can be created
   * @throws IllegalAccessException if no default constructor is available
   */
  public final Object getObject(String className) throws ClassNotFoundException,
  InstantiationException, IllegalAccessException {
    Object newInstance = null;
    if (instances.containsKey(className)) {
      newInstance = instances.get(className);
    }
    else {
      Class typClass = loadClass(className);
      newInstance = typClass.newInstance();
      instances.put(className, newInstance);
    }
    return newInstance;
  }

}
